/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
dojo.provide("com.ibm.rdm.web.attributes.AttributeGroupSection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("com.ibm.rdm.web.common.CollapseableTitleSection");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.rdm.web.attributes", "AttributeGroupSection");

dojo.declare(
    "com.ibm.rdm.web.attributes.AttributeGroupSection",
    com.ibm.rdm.web.common.CollapseableTitleSection,
{
    ///////////////////////////////////////////////////////////////////////////////////
    // Public fields
    
    ///////////////////////////////////////////////////////////////////////////////////
    // Private fields
    _fieldsTable: null,
    // _assignedFieldValues: an array of property lists
    //    Here's an example value:
    //        [ { id:"Cost_ID", value:"333" } ]
    _assignedFieldValues: null,
    // _fieldProperties: an array (and a hashmap)
    //    See _fieldProperties in AttributeGroupStyleParser for a description.
    _fieldProperties: null,
    _messages: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // API

    postCreate: function() {
        this.inherited(arguments);
        this._fieldsTable = document.createElement("table");
        this._fieldsTable.setAttribute("tabIndex", "0");
        this.setContent(this._fieldsTable);
    },
    
	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.attributes", "AttributeGroupSection");
	},

    startup: function() {
        this.inherited(arguments);
    },
    
    destroy: function() {
        this.inherited(arguments);
    },

    update: function(fieldProperties, assignedFieldValues) {
        var mapFieldIdToFieldValueProperties = new Object;
        // Define the default display values for properties of
        // different types.
        var arrayDefaultValue = [];
        arrayDefaultValue["Integer"] = 0;
        arrayDefaultValue["Boolean"] = false;
        arrayDefaultValue["Decimal"] = "";
        arrayDefaultValue["String"] = "";
        arrayDefaultValue["Enum"] = "";
        arrayDefaultValue["Date"] = "";
        arrayDefaultValue["URI"] = "";        
        if (assignedFieldValues) {
            for (var i = 0; i < assignedFieldValues.length; i++) {
                var fieldValueProperties = assignedFieldValues[i];
                mapFieldIdToFieldValueProperties[fieldValueProperties.id] =
                    fieldValueProperties;
            }
        }
        // Clear out the table.
        if (this._fieldsTable.firstChild) {
            this._fieldsTable.removeChild(this._fieldsTable.firstChild);
        }
        // Add a new tbody and add appropriate rows.
        var tbody = document.createElement("tbody");
        this._fieldsTable.appendChild(tbody);
        for (var i = 0; i < fieldProperties.length; i++) {
            attributeGroupField = fieldProperties[i];
            var fieldValueProperties =
                mapFieldIdToFieldValueProperties[attributeGroupField.id];
            var value = (fieldValueProperties ? fieldValueProperties.value : 
            				arrayDefaultValue[fieldProperties[i].type]);
            // If the value pertains to an enum, this value is actually an
            // id such as "Should_ID", but we want the enum value's display name
            // which in this case would be "Should".
            if (value && attributeGroupField.type === "Enum") {
                value = attributeGroupField.enumValues[value].displayName;
            }
            this._addFieldTableRow(tbody, attributeGroupField, value);
        }
    },

    ///////////////////////////////////////////////////////////////////////////////////
    // Helpers

    _addFieldTableRow: function(tbody, attributeGroupField, value) {
        var tr = document.createElement("tr");
        var td = document.createElement("td");
        tr.appendChild(td);
        var fieldName = dojo.string.substitute(this._messages.fieldName, [attributeGroupField.displayName]); 
        td.appendChild(com.ibm.rdm.web.util.domutils.createSpannedTextNode("field-name", fieldName));
        td = document.createElement("td");
        tr.appendChild(td);
        td.appendChild(
            com.ibm.rdm.web.util.domutils.createSpannedTextNode(
                "field-value", value));
        tbody.appendChild(tr);
    },

    _updateField: function(parentNode, textToAdd) {
//         var textNode = document.createTextNode(textToAdd);
//         var firstChild = parentNode.firstChild;
//         if (firstChild) {
//             parentNode.replaceChild(textNode, firstChild);
//         } else {
//             parentNode.appendChild(textNode);
//         }
    }
});
