/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.attributes.AttributeGroupsCache");

dojo.require("com.ibm.rdm.web.attributes.AttributeGroupsRetriever");


com.ibm.rdm.web.attributes.AttributeGroupsCache = {
    // summary:
    //    This static class is used to obtain information about attribute
    //    groups.  It caches this information so that subsequent requests
    //    for the same information do not needlessly tax the server (and
    //    client as we avoid needlessly reparsing the same document
    //    information).
    //    This cache makes use of the use HEAD requests for documents that
    //    have previously been obtained (via GET).
    //    Only if the HEAD request states that the previously obtained
    //    document has changed since last GET'ing it do we issue a new
    //    GET request and parse the document.

    ///////////////////////////////////////////////////////////////////////////////////
    // Private fields

    // _mapProjectURIToRetriever: an object that serves as a hashmap
    //    An object containing properties in which each property represents
    //    a projectURI and each property value represents an
    //    AttributeGroupsRetriever for that particular project.
    _mapProjectURIToRetriever: new Object,
    _lastRetrieverUsedInAFetch: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // API

    fetchAttributeGroups: function(projectURI, scopedAttributeGroupNamespaces, onAttributeGroupComplete, callbackScope) {
        // projectURI:
        //    The project in which we're attempting to obtain attribute group
        //    information.
        // scopedAttributeGroupNamespaces:
        //    An array of attribute group style URIs, representing a subset
        //    of the project's attribute group styles that you wish to receive
        //    information about (e.g. ["http://com.ibm.rdm.attribute.group.FEAT"]).
        //    If passed as null, you will be informed about ALL of the project's
        //    attribute group styles.
        // onAttributeGroupComplete:
        //    A function that will be called back whenever the current version
        //    attribute group has been parsed
        // callbackScope:
        //    The scope in which to call the passed 'onAttributeGroupComplete'.
        var retriever = this._getRetriever(projectURI);
        this._lastRetrieverUsedInAFetch = retriever;
        retriever.fetchAttributeGroups(
            scopedAttributeGroupNamespaces,
            onAttributeGroupComplete,
            callbackScope);
    },

    cancelLastFetch: function() {
        if (this._lastRetrieverUsedInAFetch) {
            this._lastRetrieverUsedInAFetch.cancelOutstandingFetches();
        }
    },

    ///////////////////////////////////////////////////////////////////////////////////
    // Helpers

    _getRetriever: function(projectURI) {
        var retriever = this._mapProjectURIToRetriever[projectURI];
        if (!retriever) {
            retriever = new com.ibm.rdm.web.attributes.AttributeGroupsRetriever({
                projectURI: projectURI
            });
            this._mapProjectURIToRetriever[projectURI] = retriever;
        }
        return retriever;
    }
};

})();
