/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.collections.CollectionViewer");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.layout.ContentPane");
dojo.require("com.ibm.rdm.web.project.DashboardSection");
dojo.require("com.ibm.rdm.web.project.DashboardTabPage");
dojo.require("com.ibm.rdm.web.project.projectutils");
dojo.require("com.ibm.rdm.web.search.results.ResourceInfo");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.project.DashboardToolbarWidget");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.project", "ProjectDashboard");

var projectutils = com.ibm.rdm.web.project.projectutils;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var queryutils = com.ibm.rdm.web.util.queryutils;

dojo.declare(
    "com.ibm.rdm.web.collections.CollectionViewer",
    [dijit._Widget, dijit._Templated],
{
    // summary:
    //    Shows a collection resource similar a project dashboard view
    //    This code is based on the ProjectDashboard.js, and a lot of its code is reused, so some object names may not be relevant
	
	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "collections/templates/CollectionViewer.html"),		    	
    widgetsInTemplate: true,

    ///////////////////////////////////////////////////////////////////////////////////
    // Attach points
    _rootNode: null,
    _toolbarTable: null,
    _numArtifactsSpan: null,
    _toolbarNode: null,
    _contentPane: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // Public fields
	
    ///////////////////////////////////////////////////////////////////////////////////
    // Private fields
    resourceURI: null,
	_collectionView: null,
	_collectionArtifactURIs: [],
	_documentHasBeenLoaded: false,
	projectSnapshotURI: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // NLS fields requiring translation
	messages: dojo.i18n.getLocalization("com.ibm.rdm.web.project", "ProjectDashboard"),
	
    ///////////////////////////////////////////////////////////////////////////////////
    // API
    
    constructor: function() {
		this.resourceURI = null;
		this._collectionView = {};
		this._collectionArtifactURIs = [];
		this.projectSnapshotURI = null;
		this._documentHasBeenLoaded = false;	
	},

    postCreate: function() {
		this._numArtifactsSpan.appendChild(document.createTextNode(""));
		this._toolbar = new com.ibm.rdm.web.project.DashboardToolbarWidget({}, this._toolbarNode);
    },

    setDimensions: function(width, height) {
    	this._contentPane.domNode.style.overflow = "hidden";
        this._contentPane.resize({
			w: width,
			h: height - this._getTabContainerOffsetTop()
		});
		this._toolbarTable.width = width + "px";
    },
    
    loadResource: function(contentType, revision) {
    	this.projectSnapshotURI = this.projectSnapshotURI?
		        							this.projectSnapshotURI:
		        							(this.resourceViewer?this.resourceViewer.projectSnapshotURI:null);
		this.revision = revision;
    	this._loadCollectionDeferred = RDMClient.doGet({
				uri: this.resourceURI,
				revision: revision,
		        projectSnapshotURI: this.projectSnapshotURI
			});
		return this._loadCollectionDeferred;
    },
    
    //starts up the widget
    setProjectURI: function(projectURI) {
    	this._loadCollectionDeferred.addCallback(this, 
	    	function(response) {
		    	this.projectURI = projectURI;
		        this._collectionView = new com.ibm.rdm.web.project.DashboardTabPage(
		            {
		            	resourceURI: this.resourceURI,
		                projectURI: this.projectURI,
		                projectSnapshotURI: this.projectSnapshotURI,
		                revision: this.revision,
		                viewType: com.ibm.rdm.web.project.DashboardTabPage.viewType.COLLECTION,
						footerHeight: this.footerHeight,
						dashboardToolbar: this._toolbar,
						numArtifacts: this._numArtifactsSpan,
						resourceViewer: this.resourceViewer
		            },
		            null
		        );
				this._toolbar.setViewType(com.ibm.rdm.web.project.DashboardTabPage.viewType.COLLECTION,
		            		this._collectionView.getResultSetView());
		            		
				this._contentPane.attr('content', this._collectionView.domNode);
				this._collectionView.initIfNeeded();
				//this._collectionView.refresh();
				this._documentHasBeenLoaded = true;
	    	}
	    );
    },

    ///////////////////////////////////////////////////////////////////////////////////
    // Helpers

	_getTabContainerOffsetTop: function() {
		if (!this._tabContainerOffsetTop) {
			this._tabContainerOffsetTop = dojo.coords(this._toolbarTable).h;
		}
		return this._tabContainerOffsetTop;
	}
  
});

})();
