/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function(){
dojo.provide("com.ibm.rdm.web.commenting.Comment"); 
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.common.richtext.RichTextConverter");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("dojo.date");
dojo.require("dojo.date.stamp");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");

var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var RichTextConverter = com.ibm.rdm.web.common.richtext.RichTextConverter;


/**
   Comment object used everywhere with util functions to covert to/from atom.
   All params that does not start with char "_" will be added to the comment
   as a property.
   All properties are String, unless stated otherwise.
 */
dojo.declare("com.ibm.rdm.web.commenting.Comment", null, {
    /**
        An identifier for a location/IFragment/element in the resource to which this comment belongs to.
    */
    locationId: null,
    /**
    Priority of the comment. High, Medium, Low. 
	*/
	title: null,
    /**
        Text of the comment. TODO: Change this to text.
    */
    text: null,
    /**
        Priority of the comment. High, Medium, Low. 
    */
    priority: null,
    /**
        List of user URIs for users to which the comment is directed to    
    */
    directedToUsers: null,
    /**
        Uri path to the immediate parent (replied) comment 
    */
    parentComment: null,
    /**
        Uri path to the non-reply comment; the root of the discussion thread.
    */
    rootComment: null,
    /**
        Boolean. If true, comment is resolved, otherwise not resolved.
    */
    isResolved: null,
    /**
       rrc id of review if comment made for a review. 
    */
    reviewId: null,
    /**
      whether or not the comment is a root comment
    */
    isRoot: null,
    /**
    	comment number
    */
    commentNumber: null,
    /**
    Author of the comment. 
     */
    author: null,     // not persisted
    /**
    Date (ISO String) when this comment was created.
     */
    created: null,    // not persisted
    /**
       Date (ISO String) when this comment was updated.
    */
    updated: null,    // not persisted
    /**
    	Time in ms when this comment was updated.
     */
    updatedTime: null,    // not persisted
    /**
       Date (ISO String) when the comment was last updated, cached value, takes
       replies into consideration    
    */
    lastUpdated: null,    // not persisted
    /**
    	Time in ms when the comment was last updated
     */
    lastUpdatedTime: null,    // not persisted
    /**
      Uri to the comment resource
    */
    linkSelfUri: null, // not persisted
    /**
      Uri to the resource Comment is for
     */
    resourceUri: null,
    isElementDeleted: null,
    hasRichText: null,
    richText: null, // store rich text for updating comments
    flattenedText: null, // used when truncating text
    /**
       Array of Comment objects, which have rootComment as this one.
    */
    replies: null, // not persisted
    parent: null, // not persisted
    isShow: true, // for filtering
    
    projectURI: null,  // set for recentComments

    //  ------------------ Internal state
    _anyErrors: false,
    
    /**
       All params that do not start with char "_" will be added to the comment
       as a property.
    */
    constructor: function(params) {
        this.replies = new Array();
        this.directedToUsers = new Array();
        if (params != undefined) {
        	// Copy all attrs to this comment object, unless it starts with _
        	for (attr in params) {
            	if (attr.charAt(0)!="_") {
               		this[attr] = params[attr];
            	}
        	}
        }
    },
    
    createCommentAtomEntryXmlStr: function() {    
    	var baseURI = com.ibm.rdm.web.FRONTING_SERVICE_COMMENTS;
   		var annotation = this;
       	function xmlNode(attr, encode) {
    	  	var value = annotation[attr];
    	  	if ((attr == "parentComment") || (attr == "rootComment")) {
				value = com.ibm.rdm.web.util.queryutils.getRelativeUriFromFullyQualifiedUriLink(baseURI, value);
    	  	} 
    	  	// reviewId stored as lastSegment in the comment
    	  	if (attr == "reviewId") {
    	  		if (value != null && (value != "")) {
    	  			value = com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL + "/" + value;
    	  			value = com.ibm.rdm.web.util.queryutils.getRelativeUriFromFullyQualifiedUriLink(baseURI, value);
    	  		}
    	  	}
			var returnValue = null;
    	  	if (!value) {
          		returnValue = "<" + attr + "/>";
    	  	} else {
    	  		if (encode) {
          			returnValue = "<" + attr + ">" + xmlEncode(value) + "</" + attr + ">";
    	  		} else {
          			returnValue = "<" + attr + ">" + value + "</" + attr + ">";
    	  		}
    	  	}
          return returnValue + "\n";
       }  

       function xmlEncode(text) {
			return com.ibm.rdm.web.util.domutils.getHTMLEncodedText(text);
       }
      
       var directedToXml = "";
       for (var i=0; i<this.directedToUsers.length; i++) {
       		if (com.ibm.rdm.web.util.queryutils.sameLocation(com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES, com.ibm.rdm.web.FRONTING_SERVICE_JFS_USERS)) {
       			var relativeUser = com.ibm.rdm.web.FRONTING_SERVICE_JFS_USERS + "/" + this.directedToUsers[i];
				relativeUser = com.ibm.rdm.web.util.queryutils.getRelativeUriFromFullyQualifiedUriLink(baseURI, relativeUser);
       			var xmlEntry = "<" + com.ibm.rdm.web.commenting.DIRECTED_TO_TAG + ">" + encodeURI(relativeUser) + "</" + com.ibm.rdm.web.commenting.DIRECTED_TO_TAG + ">";
       		} else {
       			var absoluteUser = com.ibm.rdm.web.FRONTING_SERVICE_JFS_USERS + "/" + this.directedToUsers[i];
       			var xmlEntry = "<" + com.ibm.rdm.web.commenting.DIRECTED_TO_TAG + ">" + encodeURI(absoluteUser) + "</" + com.ibm.rdm.web.commenting.DIRECTED_TO_TAG + ">";
       		}
	        xmlEntry = xmlEntry + "\n";
       		directedToXml = directedToXml + xmlEntry;
       }
      
       var textEncoded = null;
       if (this.hasRichText) {
       		this.text = this.richText;
       		textEncoded = xmlNode("text", true);
       } else {
       		textEncoded = xmlNode("text", true);
			textEncoded = textEncoded.replace(/&nbsp;/g, "&#160;");
       }	
       var xmlStr = '<comment xmlns="' + com.ibm.rdm.web.commenting.XMLNS_COMMENTING +  '">\n'
                    + "<resourceUri>" + com.ibm.rdm.web.util.queryutils.getRelativeUriFromFullyQualifiedUriLink(baseURI, this.resourceUri) + "</resourceUri>"
                    + xmlNode("locationId")
                    + textEncoded
                    + xmlNode("priority")
                    + directedToXml
                    + xmlNode("parentComment")
                    + xmlNode("rootComment")
                    + xmlNode("isResolved")
                    + xmlNode("reviewId")
                    + "</comment>\n";
        
        var titleText = xmlNode("title", true);
		titleText = titleText.replace(/&nbsp;/g, "&#160;");
        
        var all = '<?xml version="1.0" encoding="UTF-8"?>\n'
        	  + '<entry xmlns="http://www.w3.org/2005/Atom">\n'
              + titleText
              + '<category scheme="http://com.ibm.rdm/category/comment/" term="comment"/>\n'
              + '<content lang="english" type="application/xml">\n'
              + xmlStr
              + '</content>\n'
              + '</entry>\n';
        ErrorHandler.debug("Posting: " + all);
        return all;
    },
    
    /**
        Returns true if an error was encountered when initing this comment object
    */
    anyErrors: function() {
        return this._anyErrors;
    },
    
    isDirectedTo: function(forUser) {
        for (var i=0; i<this.directedToUsers.length; i++) {
        	if (forUser == this.directedToUsers[i]) {
        		return true;
        	}
        }
        return false;
    },        
    
    /**
        Based on the updated date, returns the constants that represent
        today, 1-3 days, 4+ days.
 
    */
    getDLMRange: function() {
    	if (!this.isRoot) return null;
		var now = new Date();
        var todayDate = new Date(now.getFullYear(), now.getMonth(), now.getDate());
    	var updatedTime = this.lastUpdatedTime;
    	var updatedDate =  new Date(this.lastUpdated);
        return this._getDateRange(todayDate, updatedTime, updatedDate);
    },
    
    getDateRange: function() {
		var now = new Date();
        var todayDate = new Date(now.getFullYear(), now.getMonth(), now.getDate());
    	var updatedTime = this.updatedTime;
    	var updatedDate =  new Date(this.updated);
        return this._getDateRange(todayDate, updatedTime, updatedDate);
    },
    
    _getDateRange: function(todayDate, updatedTime, updatedDate) {
    	var range = null;
		if (updatedTime > todayDate.getTime()) {
			range = com.ibm.rdm.web.commenting.UPDATED_TODAY;
        }
        else if (updatedTime > (todayDate.getTime() - 1000*60*60*24)) {
        	range = com.ibm.rdm.web.commenting.UPDATED_YESTERDAY;
        }
        else if (Math.abs(dojo.date.difference(todayDate, updatedDate, "day")) < 7) {
        	range = com.ibm.rdm.web.commenting.UPDATED_PAST_WEEK;
        }
        else if ((Math.abs(dojo.date.difference(todayDate, updatedDate, "day")) < 30)) {
        	range = com.ibm.rdm.web.commenting.UPDATED_PAST_MONTH;
        }
        else
        	range = com.ibm.rdm.web.commenting.UPDATED_EARLIER;
        return range;
    },
    
    setDateLastModified: function() {
    	var allReplies = this.getAllReplies();
    	if (allReplies.length == 0) {
            this.lastUpdated = this.updated;
            this.lastUpdatedTime = this.updatedTime;
    		return;
    	}
    	var lastReplyTime = -1;
    	var dlm = null;
        for (var i=0; i<allReplies.length; i++) {
            var reply = allReplies[i];
            if (lastReplyTime < reply.updatedTime) {
            	lastReplyTime = reply.updatedTime;
            	dlm = reply.updated;
           	}
        }
        if (lastReplyTime < this.updatedTime) dlm = this.updated;
        this.lastUpdated = dlm;
        this.lastUpdatedTime = (new Date(this.lastUpdated)).getTime();
    },
    
    getAllReplies: function() {
    	var allReplies = new Array();
        dojo.forEach(this.replies, function(reply) {
        	allReplies.push(reply);
        	var childReplies = reply.getAllReplies()
        	dojo.forEach(childReplies, function(childReply) {
        		allReplies.push(childReply);
         	});
        });
        return allReplies;
    },
    
    isReviewComment: function() {
    	return this.reviewId != null && this.reviewId != "";
    },
    
    // Return whether or not to show the comment for filtering, show the comment
    // if the isShow flag is true for the comment or any of its replies
    isShowForFilter: function() {
    	if (this.isShow) return true;
    	if (!this.isRoot) return false;
    	// we want to show the root comment if any of its replies will be shown
    	var allReplies = this.getAllReplies();
        for(var i=0; i<allReplies.length; i++) {
        	var reply = allReplies[i];
        	if (reply.isShowForFilter()) return true;
        };    
        return false;
    },
    
    toString: function() {
        var comment = "(linkSelfUri: " + this.linkSelfUri;
        for (a in this) {
            if (a!="linkSelfUri") {
                var v = this[a];
                if ( dojo.isString(v) && v.length>30 ) {
                    v = (v+"").substring(0,30);
                }
                if (! (v instanceof Function) )
                    comment = comment + ", " + a + ": " + v;
            }
        }
        return comment + ")";
    },
    
    _parseCommentNode: function(entry, baseURI) {
		var recentCommentsFeedURI = com.ibm.rdm.web.FRONTING_SERVICE_RECENT_FEEDS + "/Recent%20Comments";
        var commentNode = entry.getElementsByTagName("comment")[0];
        for (var i=0; i<commentNode.childNodes.length; i++) {
           	var childNode = commentNode.childNodes[i];
           	if (childNode != undefined) {
           		var childNodeValue = dojo.isIE?(childNode.text):(childNode.textContent);
           		if (childNode.tagName == com.ibm.rdm.web.commenting.DIRECTED_TO_TAG) {
           			if (childNodeValue) {
                		var lastSegment = childNodeValue.substring(childNodeValue.lastIndexOf("/")+1);
           				this.directedToUsers.push(decodeURIComponent(lastSegment));
           			}
           		} else {
           			this[childNode.tagName] = (childNodeValue)?childNodeValue:"";
           		}
        	}
        }
      	this.isResolved = (this.isResolved=="true")?true:false;
        var text = this.text;
        this.hasRichText = (text.search("<rte:text") == 0);
        if (this.hasRichText) {
        	// cache the rich text so that when we update rich client comments
        	// the text is properly preserved, do not use the transformed browser
        	// displayabe text on update
        	this.richText = text;
        	text = RichTextConverter.convertRrcXhtmlToHtml(text);
        	
        	//create a flattened version of the html text
        	var div = document.createElement("DIV");
        	div.innerHTML = text;
        	this.flattenedText = dojo.isIE?(div.innerText):(div.textContent);
        	this.flattenedText = dojo.string.trim(this.flattenedText);
        }
        // Expand the relative URIs to fully qualified URIs
        if (baseURI == recentCommentsFeedURI) {
        	// TODO (LPK) - remove this special test when server returns everything properly for recent feed entries
        	this.resourceUri = com.ibm.rdm.web.util.queryutils.getFullyQualifiedUriFromRelativeLink(com.ibm.rdm.web.FRONTING_SERVICE_RECENT_FEEDS,  this.resourceUri);
        } else {
        	this.resourceUri = com.ibm.rdm.web.util.queryutils.getFullyQualifiedUriFromRelativeLink(baseURI, this.resourceUri);
        }
        if (baseURI == recentCommentsFeedURI) {
        	if (this.parentComment != "") {
        		this.parentComment = com.ibm.rdm.web.FRONTING_SERVICE_COMMENTS + "/" +  this.parentComment;
        	}
        } else {
        	if (this.parentComment != "") {
				this.parentComment = com.ibm.rdm.web.util.queryutils.getFullyQualifiedUriFromRelativeLink(baseURI, this.parentComment);
        	}
		}
        if (baseURI == recentCommentsFeedURI) {
        	if (this.rootComment != "") {
        		this.rootComment = com.ibm.rdm.web.FRONTING_SERVICE_COMMENTS + "/" +  this.rootComment;
        	}
        } else {
        	if (this.rootComment != "") {
				this.rootComment = com.ibm.rdm.web.util.queryutils.getFullyQualifiedUriFromRelativeLink(baseURI, this.rootComment);
        	}
		}
		if (this.isReviewComment()) {
            var lastSegment = this.reviewId.substring(this.reviewId.lastIndexOf("/")+1);
			this.reviewId = lastSegment;
		}
        this.text = (text)?text:"";
        this.isRoot = this.parentComment == "";    
    },
    
    parseFeedEntry: function(entry, baseURI) {
        this.title = this._getTextValue(entry, "title");
        if (baseURI.indexOf(com.ibm.rdm.web.FRONTING_SERVICE_RECENT_FEEDS) == 0) {
        	this.updated = dojo.date.stamp.fromISOString(this._getTextValue(entry, "updated"));
        	this.updated = new Date(this.updated);
        } else {
        	this.updated = new Date(this._getTextValue(entry, "updated"));
        }    
		this.updatedTime = this.updated.getTime();
        var authorNode = entry.getElementsByTagName("author").item(0);
        var author = this._getTextValue(authorNode, "name");
        var i = author.lastIndexOf("/");
        this.author = decodeURIComponent(author.substring(i + 1));
        var linkNode = entry.getElementsByTagName("link").item(0);
		this.linkSelfUri = linkNode.getAttribute("href");
		// TODO (LPK) - workaround until server correctly returns linkSelfUri as absolute
		// in recent comment entries
		if (this.linkSelfUri.indexOf("..") == 0) {
            var lastSegment = this.linkSelfUri.substring(this.linkSelfUri.lastIndexOf("/")+1);
        	this.linkSelfUri = com.ibm.rdm.web.FRONTING_SERVICE_COMMENTS + "/" +  lastSegment;
		}
        this._parseCommentNode(entry, baseURI);       
    },
    
    parseGetEntry: function(entry) {
        this.title = this._getTextValue(entry, "title");
        this._parseCommentNode(entry, com.ibm.rdm.web.FRONTING_SERVICE_COMMENTS);       
    },
    
    _getTextValue: function(node, attr) {
        try {
        	var tags = node.getElementsByTagName(attr);
        	if (tags.length == 0) return "";
        	if (tags.item(0).firstChild) {
                return tags.item(0).firstChild.nodeValue;
        	} else {
        		return "";
        	}
        } catch (e) {
            return "";
        }
    }

});


}());