/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function(){

dojo.provide("com.ibm.rdm.web.commenting.CommentingClient");
dojo.require("dojo.DeferredList");
dojo.require("com.ibm.rdm.web.commenting.Comment");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("dojox.string.Builder");

var UriHelper = com.ibm.rdm.web.common.UriHelper;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;

com.ibm.rdm.web.commenting.CommentingClient = {
	
    /**
        Retrieves all the comments for a resource for the specified user. 
        Returns dojo.Deferred
    */
    getComments: function(resourceURI) {
	    var feedURI = com.ibm.rdm.web.FRONTING_SERVICE_COMMENTS + "?resourceURI=" + encodeURIComponent(resourceURI);		
	    var deferred = RDMClient.doGet({uri: feedURI});
        return deferred;
    },
    /**
        Retrieves all the comments for the given review and artifacts. 
        Returns dojo.Deferred
    */
    getReviewComments: function(reviewId) {
    	var self = this;
    	var allReviewCommentsDeferred = new dojo.Deferred();
    	// get uris of comments to retrieve
    	var queryURL = this._getReviewQueryString(reviewId);
    	queryURL = com.ibm.rdm.web.FRONTING_SERVICE_QUERY + "?" + queryURL;
		var deferred = RDMClient.doGet({
			uri: queryURL
		});
		deferred.addCallback(this, function(response){
 			var multiGetArgs = [];
	       	var entries = response.doc.getElementsByTagName("entry");
			for (var i = 0; i < entries.length; i++) {
				var entry = entries[i];
        		var node = entry.getElementsByTagName("content")[0];
	        	node = node.getElementsByTagName("result")[0];
	        	node = node.getElementsByTagName("binding")[0];
    			var uri = dojo.isIE?(node.text):(node.textContent);
	        	multiGetArgs.push({
					uri: uri
				});
			};
			var multiGetDeferred = self._multiCommentGet(multiGetArgs);					
			multiGetDeferred.addCallback(this, function(multiGetResponse) {
				allReviewCommentsDeferred.callback(multiGetResponse);
			});
			multiGetDeferred.addErrback(this, function(multiGetError) {
	 			var errorSectionWidget = ErrorHandler.getErrorSectionWidget();
				if (errorSectionWidget) {
					errorSectionWidget.showErrorMessage(multiGetError);
				}
				allReviewCommentsDeferred.errback(multiGetError);
	 		});
			return response;
		});
		// retrieve the comments
 		return allReviewCommentsDeferred;
    },
    
	_multiCommentGet: function(argsArray) {
		var deferred = new dojo.Deferred();
		var multiGetDeferred = RDMClient.doMultiGet(argsArray);
		multiGetDeferred.addCallback(this, function(multiGetResponse){
			var comments = [];
			var multiGetResponsesNodes = domutils.getElementsByTagNameNS(multiGetResponse.doc, "MultiResponse", 'http://com.ibm.rdm/multi-request#', 'rrmMulti');
			for (var i=0; i<multiGetResponsesNodes.length; i++) {
				var comment = new com.ibm.rdm.web.commenting.Comment();
				var responseNode = multiGetResponsesNodes[i];
				comment.linkSelfUri = responseNode.getAttribute("rdf:about");
				var headerNodes = domutils.getElementsByTagNameNS(responseNode, "httpHeader", 'http://com.ibm.rdm/multi-request#', 'rrmMulti');
				for (var j=0; j<headerNodes.length; j++) {
					var header = headerNodes[j];
					var headerChildren = header.childNodes;
					var headerNameNode = null;
					var headerValueNode = null;
					for (var k=0; k<headerChildren.length; k++) {
						var child = headerChildren[k];
						if (child.tagName == "rrmMulti:httpHeaderName") {
							headerNameNode = child;
						}
						if (child.tagName == "rrmMulti:httpHeaderValue") {
							headerValueNode = child;
						}
					};
					var headerNameValue = dojo.isIE?(headerNameNode.text):(headerNameNode.textContent);
					if (headerNameValue == com.ibm.rdm.web.commenting.MODIFIED_BY_PROPERTY) {
						var headerValueValue = dojo.isIE?(headerValueNode.text):(headerValueNode.textContent);
						comment.author = headerValueValue; 
        				var index = comment.author.lastIndexOf("/");
        				comment.author = decodeURIComponent(comment.author.substring(index + 1));
					}
					if (headerNameValue == com.ibm.rdm.web.commenting.MODIFIED_PROPERTY) {
						var headerValueValue = dojo.isIE?(headerValueNode.text):(headerValueNode.textContent);
						var updated = headerValueValue;
						comment.updated = new Date(updated);
						comment.updatedTime = comment.updated.getTime();
					}
				};
				var responseContentNode = domutils.getElementsByTagNameNS(responseNode, "responseContent", 'http://com.ibm.rdm/multi-request#', 'rrmMulti')[0];
				var responseXML = dojo.isIE?responseContentNode.childNodes[0].text:responseContentNode.childNodes[0].textContent;
				responseXML = dojo.trim(responseXML);
				var xmlDoc = dojox.data.dom.createDocument(responseXML);
    			comment.parseGetEntry(xmlDoc);
    			comments.push(comment);
			};
			deferred.callback(comments);
		});
		multiGetDeferred.addErrback(this, function(multiGetErrorResponse){
			deferred.callback(multiGetErrorResponse);
		});
		return deferred;
	},
    
    _getReviewQueryString: function(reviewId) {
    	var fullReviewId = com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL + "/" + reviewId;
		var queryString = "commenting:reviewId=";
    	queryString = queryString + "%3C" + fullReviewId + "%3E";
    	queryString = queryString + "&namespaces=commenting=http://com.ibm.rdm.comments";
    	queryString = queryString + encodeURIComponent("#");
		return "query=" + queryString;
    },
    
    /**
        Saves a NEW comment to the specified repository for a resource using the provided
        user credentials.
        comment: The comment that will be saved.
    */
    saveNewComment: function(comment, projectURI) {
    	var commentSaved = new dojo.Deferred();
	    var postURL = com.ibm.rdm.web.FRONTING_SERVICE_COMMENTS + "?resourceURI=" + encodeURIComponent(comment.resourceUri);
		var xmlStr = comment.createCommentAtomEntryXmlStr();
    	var requestHeaders = null;
    	if (projectURI) {
    		requestHeaders = {
		       	"Content-Type": com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_COMMENT,
			 	"Content-Length": xmlStr.length
		    };
    	} else {
    		requestHeaders = {
		       	"Content-Type": com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_COMMENT,
			 	"Content-Length": xmlStr.length
		    };
    	}
		var deferred = RDMClient.doPost({
				uri: postURL,
				projectURI: projectURI,
		        postData: xmlStr,
		        xhrArgs: {
		        	headers: requestHeaders
		        }
			});
		deferred.addCallback(this, function(response){
			var onCommentRetrieved = function(response) {
				response.commentNumber = commentNumber;
	 			commentSaved.callback(response);
	 		};	
			this._setResponseData(comment, response.ioArgs.xhr);
			// get not returning commentNumber
			var commentNumber = comment.commentNumber;
            var commentDeferred = this.getComment(comment.linkSelfUri);
            commentDeferred.addCallback(onCommentRetrieved);
		    return response;
		});
		return commentSaved;
		
    },
    
    deleteComment: function(comment) {
    	var requestHeaders = {
		    "Content-Type": com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_COMMENT
		};
		var args = {
	    	uri: comment.linkSelfUri,
			xhrArgs: {
		    	headers: requestHeaders
			}
		};
		var deferred = RDMClient.doDelete(args);		
		deferred.addCallback(this, function(response){
			return comment;
		});
		return deferred;
    },
    
    updateComment: function(comment, updateProperties, projectURI) {
    	var commentUpdated = new dojo.Deferred();
    	// Pass in the comment changes so that the comment is actually updated only 
    	// if the operation is successful.
        var freshComment = comment;
        for (attribute in updateProperties) {
            if (!dojo.isFunction(updateProperties[attribute])) {
            	freshComment[attribute] = updateProperties[attribute];
            }
        }
	    var putURL = comment.linkSelfUri; 
		var xmlStr = freshComment.createCommentAtomEntryXmlStr();
    	var self = this;
    	var requestHeaders = null;
    	if (projectURI) {
    		requestHeaders = {
		       	"Content-Type": com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_COMMENT,
			 	"Content-Length": xmlStr.length
		    };
    	} else {
    		requestHeaders = {
		       	"Content-Type": com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_COMMENT,
			 	"Content-Length": xmlStr.length
		    };
    	}
		var args = {
	            uri: putURL,
	            projectURI: projectURI,
		        putData: xmlStr,  
		        xhrArgs: {
		        	headers: requestHeaders
		        }
		};
		var deferred = RDMClient.doPut(args);		
		deferred.addCallback(this, function(response){
			var onCommentRetrieved = function(response) {
				response.commentNumber = commentNumber;
				// get only returns modified by user in response headers,
				// so reset this field to the comment author, ideally JFS
				// should be returning both modifiedBy and creator in the 
				// response headers for a GET, workaround for work item 22459
				response.author = comment.author;
	 			commentUpdated.callback(response);
	 		};	
			// get not returning commentNumber
			var commentNumber = freshComment.commentNumber;
            var commentDeferred = this.getComment(freshComment.linkSelfUri, true);
            commentDeferred.addCallback(onCommentRetrieved);
		    return response;
		});
		return commentUpdated;
    },
    _setEditResponseData: function(comment, xhr) {
		comment.author = xhr.getResponseHeader(com.ibm.rdm.web.commenting.MODIFIED_BY_PROPERTY); 
        var i = comment.author.lastIndexOf("/");
        comment.author = decodeURIComponent(comment.author.substring(i + 1));
		var updated = xhr.getResponseHeader(com.ibm.rdm.web.commenting.MODIFIED_PROPERTY);
		comment.updated = new Date(updated);
		comment.updatedTime = comment.updated.getTime();
    },
    _setResponseData: function(comment, xhr) {
		comment.linkSelfUri = xhr.getResponseHeader(com.ibm.rdm.web.commenting.LOCATION_PROPERTY); 
		comment.author = xhr.getResponseHeader(com.ibm.rdm.web.commenting.MODIFIED_BY_PROPERTY); 
        var i = comment.author.lastIndexOf("/");
        comment.author = decodeURIComponent(comment.author.substring(i + 1));
		var updated = xhr.getResponseHeader(com.ibm.rdm.web.commenting.MODIFIED_PROPERTY);
		comment.updated = new Date(updated);
		comment.updatedTime = comment.updated.getTime();
		var start = xhr.responseText.indexOf("<commentsfeed:commentNumber>") + 28;
		var stop = xhr.responseText.indexOf("</commentsfeed:commentNumber>");
		comment.commentNumber = xhr.responseText.substring(start, stop);
    },
    /**
        Gets a fresh copy of the provided Comment
    */
    getComment: function(commentURL, skipCache) {
    	var deferred = null;
    	if (skipCache) {
    		deferred = RDMClient.doGet({
    			uri: commentURL,
				skipCache: skipCache
			});
    	} else {
    		deferred = RDMClient.doGet({uri: commentURL});
    	};
    	deferred.addCallback(this, function(response){
			var comment = new com.ibm.rdm.web.commenting.Comment();
	        this._setResponseData(comment, response.ioArgs.xhr);
    		comment.parseGetEntry(response.doc);
    		comment.linkSelfUri = commentURL;
	        return comment;
		});
        return deferred;
    }
};

}());

