/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function(){

dojo.provide("com.ibm.rdm.web.commenting.CommentingController");

dojo.require("com.ibm.rdm.web.commenting.CommentingClient");
dojo.require("com.ibm.rdm.web.commenting.EventTopics");
dojo.require("com.ibm.rdm.web.review.ReviewCache");
dojo.require("dojo.DeferredList");
dojo.require("com.ibm.rdm.web.commenting.Comment");
dojo.require("com.ibm.rdm.web.commenting.Comments");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");

// Do not require widgets, except its own view. Only use this to update owning widgets state with data

var CommentingClient = com.ibm.rdm.web.commenting.CommentingClient;
var EventTopics = com.ibm.rdm.web.commenting.EventTopics;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var queryutils = com.ibm.rdm.web.util.queryutils;
var domutils = com.ibm.rdm.web.util.domutils;

dojo.declare("com.ibm.rdm.web.commenting.CommentingController", null,  {
	
	_reviews: null,
	_deferredArray: null,
	_annotations: null,
	_revisionDate: null,
	projectURI: null,
	reviewURI: null,
	
    constructor: function(view) {
        this._view = view;
        this._deferredArray = [];
		this._reviews = new dojox.collections.Dictionary();
    },
    destroy: function() {
        this.inherited(arguments);
        if (this._deferredArray) {
            for (var i = 0; i < this._deferredArray.length; i++) {
                var deferred = this._deferredArray[i];
                if (deferred.fired == -1) {
                    deferred.cancel();
                }
            }
        }
    },
    setReviewURI: function(uri) {
    	this.reviewURI = uri;
    },
    getReviewDescription: function(reviewId) {
    	var review = this._reviews.item(reviewId);
    	if (review == null) ErrorHandler.debug("CommentingController - review " + reviewId + " not found");
    	return review;
    },
    _getComments: function(revisionDate) {
    	this._annotations = new Array();
    	this._revisionDate = revisionDate;
    	var deferred = CommentingClient.getComments(this._view.resourceURI);
		this._deferredArray.push(deferred);
        deferred.addCallback(this, this._onCommentsRetrieved );
        return deferred;
    },
    saveNewComment: function(annotation) {
        var deferred = CommentingClient.saveNewComment(annotation, this.projectURI);
		this._deferredArray.push(deferred);
	    deferred.addCallback(this, this._onAnnotationSaved); 
	    return deferred; 
	},
	updateComment: function(comment, updateProperties) {
        var deferred = CommentingClient.updateComment(comment, updateProperties, this.projectURI);
		this._deferredArray.push(deferred);
        deferred.addCallback(this, this._onCommentUpdated); 
        return deferred; 
	},
	deleteComment: function(comment) {
        var deferred = CommentingClient.deleteComment(comment);
		this._deferredArray.push(deferred);
        deferred.addCallback(this, this._onCommentDeleted); 
        return deferred; 
	},
	_onCommentUpdated: function(updatedComment) {
       for (var i=0; i<this._annotations.length; i++) {
    	   if (this._annotations[i].linkSelfUri == updatedComment.linkSelfUri) {
    		   this._annotations[i] = updatedComment;
    		   break;
    	   }
       }
       this._view._onCommentUpdated(updatedComment);
	   return updatedComment;
	},
	_onCommentDeleted: function(deletedComment) {
		var index = 0;
       	for (index; index<this._annotations.length; index++) {
       		var annotation = this._annotations[index];
    	   	if (annotation.linkSelfUri == deletedComment.linkSelfUri) {
    		   break;
    	   	}
       	}
	   	this._annotations.splice(index, 1);
       	this._view._onCommentDeleted(deletedComment);
	   	return deletedComment;
	},
    _onAnnotationSaved: function(comment) {
        ErrorHandler.debug("Annotation saved successfully");
        this._view._addNewAnnotation(comment);
    },
    _onCommentsRetrieved: function(feed) {
    	this._count = 0;
    	var self = this;
	   	this._reviews = new dojox.collections.Dictionary();
	   	if (this.reviewURI) this._reviews.add(this.reviewURI, null);
       	if (feed && feed.doc) {  
	       	var timer = (new Date()).getTime();
	       	var entries = feed.doc.getElementsByTagName("entry");
			for (var i = 0; i < entries.length; i++) {
				var comment = new com.ibm.rdm.web.commenting.Comment();
				comment.parseFeedEntry(entries[i], com.ibm.rdm.web.FRONTING_SERVICE_COMMENTS);
	          	if (!comment.anyErrors()) {
	        	  this._annotations.push(comment);
	          	}
			}
			
        	function commentComparator(comment1, comment2) {
           		var a = parseInt(comment1.commentNumber);
           		var b = parseInt(comment2.commentNumber);
           		return (a<b)?-1:((a>b)?1:0);
        	};
	    	this._annotations.sort(commentComparator);
	    	
	    	// need to gather review information for groupByReview and filtering
	    	dojo.forEach(this._annotations, function(comment) {
	    		if (comment.isReviewComment()) {
	    			var uri = com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL + "/" + comment.reviewId;
	    			self._reviews.add(uri, null);
	    		}
	    	});
	    	var reviewsKeyList = this._reviews.getKeyList();
	    	if (reviewsKeyList.length > 0) {
        		// Use a query instead of multifetch describe to get the data we need since describe data can be quite
        		// large for reviews if a lot of artifacts and/or participants exist for the review
				var query = (
					"query=dc:isPartOf=<" + com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL + ">"
				);
				query = query + " and resource in [";
				for (var i=0; i<reviewsKeyList.length-1; i++) {
					query = query + "<" + reviewsKeyList[i] + ">,"
				}
				query = query + "<" + reviewsKeyList[reviewsKeyList.length-1] + ">"
				query = query + "]&properties=name,review:reviewTimestamp,review:reviewStatus";
				query = query + "&namespaces=dc=" + queryutils.DCTERMS_NAMESPACE + ",review=" + queryutils.REVIEW_NAMESPACE;
				var deferred = RDMClient.doPost({
					uri: com.ibm.rdm.web.FRONTING_SERVICE_QUERY,
					projectURI: this._projectURI,
		        	postData: query
				});
        		var onReviewsGetInfo = function(response) {
					var entries = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(response.doc, "entry");
					for(var i=0; i<entries.length; i++){
						// resource, title, reviewTimestamp, reviewStatus
						var description = {
							name: domutils.getEntryBindingValueByName(entries[i], "title"),
							reviewTimestamp: domutils.getEntryBindingValueByName(entries[i], "reviewTimestamp"),
							reviewStatus: domutils.getEntryBindingValueByName(entries[i], "reviewStatus"),
							resourceURI: domutils.getEntryBindingValueByName(entries[i], "resource")	
						};
						self._reviews.add(description.resourceURI, description);
					}
	    			self._processComments();
				};
        		var onReviewsGetInfoError = function(error) {
            		com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorNumber(
            			7216, [], error
    				);
        		};
            	deferred.addCallback(onReviewsGetInfo);
            	deferred.addErrback(onReviewsGetInfoError);
	    	} else {
				self._processComments();
	    	}

	    	ErrorHandler.debug("created comments objects in : " + ((new Date()).getTime() - timer) + "ms");
	       
	       var nextPageURI = com.ibm.rdm.web.util.queryutils.getNextPageURI(feed.doc);
	       if (nextPageURI) {
	    	   var deferred = RDMClient.doGet({uri: nextPageURI});
	    	   this._deferredArray.push(deferred);
	    	   return deferred.addCallback(this, this._onCommentsRetrieved); 
	       }
       } else {
    	   // need to do these calls when no comments exist (i.e., feed null)
	    	if (this._reviews.getKeyList().length > 0) {
        		var deferred = com.ibm.rdm.web.common.ProjectCache.getBatchResourceDescriptionsMultiFetch(this._reviews.getKeyList());
        		var onReviewsGetInfo = function(resourceDescriptions) {
					for (resourceURI in resourceDescriptions) {
						self._reviews.add(resourceURI, resourceDescriptions[resourceURI]);
					}
	    			self._processComments();
				};
        		var onReviewsGetInfoError = function(error) {
            		com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorNumber(
            			7216, [], error
    				);
        		};
            	deferred.addCallback(onReviewsGetInfo);
            	deferred.addErrback(onReviewsGetInfoError);
	    	} else {
				self._processComments();
	    	}
       }
     },
     _processComments: function() {
		this._createComments();
		this._view._showAnnotations();
		this._view._filterWidget.updateCurrentFilterMessage();
     },
    _createComments: function() {
    	this._view._annotations = null;
    	this._view._annotations = new com.ibm.rdm.web.commenting.Comments (
        {
        	"commentingWidget": this._view,
        	"resourceViewer": this._view.viewer,
            "jsAnnotations": this._annotations,
            "orderedDocElementIds": this._view.orderedDocElementIds,
            "groupCommentsBy": this._view._groupCommentsBy,
            "sortCommentsBy": this._view._sortCommentsBy,
            "revisionDate": this._revisionDate
        });
    }

});


}());