/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
dojo.provide("com.ibm.rdm.web.commenting.CommentingWidget");

dojo.require("com.ibm.rdm.web.commenting.CommentingController");
dojo.require("com.ibm.rdm.web.commenting.CommentPane");
dojo.require("com.ibm.rdm.web.commenting.EditCommentForm");
dojo.require("com.ibm.rdm.web.commenting.FilterWidget");
dojo.require("com.ibm.rdm.web.commenting.utils");
dojo.require("com.ibm.rdm.web.commenting.EventTopics");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.common.UsersSelectWidget");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.ConfirmForm");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.review.ReviewCache");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.util.domutils");

dojo.require("dijit.layout.ContentPane");
dojo.require("dojox.collections.Dictionary");
dojo.require("dijit.form.DateTextBox");
dojo.require("dojo.i18n");
dojo.require("dojo.date");
dojo.require("dojo.date.stamp");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.Tooltip");
dojo.require("dijit.form.Form");
dojo.require("dijit.Menu");
dojo.require("dijit.form.Button");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.SimpleTextarea")

dojo.requireLocalization("com.ibm.rdm.web.commenting", "commenting");

var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var EventTopics = com.ibm.rdm.web.commenting.EventTopics;
var domutils = com.ibm.rdm.web.util.domutils;
var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;

/**
 * Main commenting widget. 
 * Clients must call _initAnnotations() after ctor, which will return dojo.Defered to 
 * signal completion XHR to get comments from server (would it be possible/OK to change
 * postCreate() and return Deferred?)
*/
dojo.declare("com.ibm.rdm.web.commenting.CommentingWidget", [dijit._Widget, dijit._Templated], {
    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "commenting/templates/CommentingWidget.html"),
    viewer: null,
    /* The value of this attribute must be one of the values found in the
       com.ibm.rdm.web.commenting.CommentingWidget.displayContext
       property list. */
    displayContext: null,
    widgetsInTemplate: true, 
    selectCommentURI: null,
    // Internals
    // GUI states 
    _allCommentPanes: new dojox.collections.Dictionary(),
    _allLocationPanes: new dojox.collections.Dictionary(),
    _highlightedLocationPane: null,
    _highlightedCommentPane: null,
    _currentDocSelectedLocationId: null,
    _controller: null,
    // Set by controller, for info about the resource, such as owning-team and displayName
    _resourceInfo: null,
    // All subscribed events
    _subs: [],
    _editCommentFormTitle: null,
    _sortCommentsBy: 0,
    _groupCommentsBy: 0,
    _isDeletedElementsUpdated: false,
    _messages: null,
    _annotations: null,
    resourceURI: null,
    hasStarted: false,
    // used for filtering comments in history mode
    revisionDate: null,
    regroupComments: false,
    
    postCreate: function() {
        var self = this;
        this.inherited("postCreate", arguments);
        this._controller = new com.ibm.rdm.web.commenting.CommentingController(this);
        if (this.viewer.isInReviewMode()) {
        	var uri = com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL + "/" + this.viewer.reviewId;
        	this._controller.setReviewURI(uri);
        }
        this._initSubscriptions();
        this._resourceLocationPane = dijit.byId(this._resourceLocationPaneNode.id);       

        // FilterWidget, hidden
        this._filterWidget = new com.ibm.rdm.web.commenting.FilterWidget({commentingWidget: self}, this._filterWidgetNode);        
        this._filterWidget.hide();
        
        // Toolbar init
        // this._filterToolbarIconNode.setAttribute("src", dojo.moduleUrl("com.ibm.rdm.web.commenting", "images/icons/etool16/filter.gif").uri);
        //this._refreshToolbarIconNode.setAttribute("src", dojo.moduleUrl("com.ibm.rdm.web.commenting", "images/icons/etool16/refresh.gif").uri);
        //this._addAllResourceCommentIconNode.setAttribute("src", dojo.moduleUrl("com.ibm.rdm.web.commenting", "images/icons/etool16/add_comment.gif").uri);
        
        // this._locateInDocIconNode.setAttribute("src", dojo.moduleUrl("com.ibm.rdm.web.commenting", "images/icons/etool16/show-comment.gif").uri);
        this._filterToggleWidget = new com.ibm.rdm.web.commenting.IconDropdown(
            {menu: null,
             iconClass: "filter",
             toggleOnClick: true,
             onClick: function() {
                if (self._filterToggleWidget.isActive()) { // it will be active after click, so do the reverse
                    self._onFilterClick(); // open filter widget
                }
                else {
                    self._filterWidget._onCloseClicked();
                }
             }
             });
        //this._filterToggleNode.appendChild(this._filterToggleWidget.domNode);
        this._locateInDocToggleWidget = new com.ibm.rdm.web.commenting.IconDropdown(
            {menu: null,
             iconClass: "locateInDoc",
             toggleOnClick: true,
             onClick: function() {
                self._onLocateInDocClick(); 
             }
             });
        
        //default sorting and grouping
        this._sortCommentsBy = com.ibm.rdm.web.commenting.CommentingWidget.Ordering.SORTBY_DATE;
        this._groupCommentsBy = com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_NOGROUPING;
        this._previousSortingSelection = this._sortByDateMenuItem;
		this._previousGroupingSelection = this._noGroupingMenuItem;
    	if (this.isReviewArtifact()) {
    		this._groupByReviewMenuItem.attr("disabled", true);
    	}
		if (this.viewer.isStoryboardViewer()) {
        	this._groupByArtifactMenuItem.attr("disabled", true);
        } else {
        	this._commentingToolbarOptionsMenu.removeChild(this._commentingToolbarCreateForFrame);
        }  
   	},
   	postMixInProperties: function() {
	   	this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.commenting", "commenting");
   	},
    /**
     * Total destruction of this widget, including all peer/children widgets and domNodes
     *
    */
    destroy: function() {
        for (var i = 0; i < this._subs.length; i++) {
            dojo.unsubscribe(this._subs[i]);
            this._subs[i] = null;
        }
        this._subs = [];
        var locationPanes = this._allLocationPanes.getValueList();
        for (var i = 0; i < locationPanes.length; i++) {
        	locationPanes[i].destroyRecursive();
        	locationPanes[i] = null;
        }
        this._allLocationPanes = new dojox.collections.Dictionary();
        // TODO: Destroy this._filterWidget
        this.inherited("destroy", arguments);
    },
    /**
     * Refreshes the comments shown by this widget. The location and comment panes
     * are destroyed and recreated. 
     * Returns dojo.Deferred object.
     
    */
    startup: function() {
    	this.hasStarted = true;
    	// workaround for 20520 and history viewer
    	if (this._groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_ARTIFACT) {
    		this._groupCommentsBy = com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_NOGROUPING;
    		this._regroupComments = true;
    	} 
    	this._initAnnotations();
    },
    setResourceInfo: function(resourceInfo) {
    	this._resourceInfo = resourceInfo;
    	this._setFilterWidgetProjectURI();
    	this._controller.projectURI = this.getProjectURI();
    },
    subscribe: function(topic, method) {
    	var topic = EventTopics.getTopic(topic, this.resourceURI);
    	ErrorHandler.debug("CommentingWidget: subscribe to event -> " + topic);
        this._subs.push(
        		dojo.subscribe(
        				topic, 
        				this, 
        				method));
    },
    publish: function(topic, params) {
    	var topic = EventTopics.getTopic(topic, this.resourceURI);
    	//console.debug("CommentingWidget: fire event -> " + topic);
		dojo.publish(
				topic, 
				params);
    },
    isReviewArtifact: function() {
    	return this.viewer.getContentType() == com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REVIEW;
    },
	
    isExpanded: function() {
    	return this._sidebarSection.isExpanded();
    },
    isHistoryViewer: function() {
		return this.revisionDate != null;
    },
    _initSubscriptions: function() {
    	this._subs = new Array(); 
    	this.subscribe(EventTopics.COMMENT_SELECTED_TOPIC, "_toggleResourceLocationSelection");
    	this.subscribe(EventTopics.STORYBOARD_FRAME_LOADED_TOPIC, "_highlightAllCommentElements");
    },
    
    _highlightAllCommentElements: function() {
    	this.locateInDoc();
    },
    
    expandSidebar: function() {
    	this._sidebarSection.expandSection();
    },
    
    containerDimensionsHaveChanged: function() {
        // summary:
        //     This function is called when the containing SidebarSection has
        //     changed its dimensions.  We handle this by resetting the
        //     scrolling viewport, as needed.
        this._setHeightOfScrollingPaneElement();
    },
    onCreateNewCommentForArtifact: function() {
    	if (!this.hasStarted)
    		this._sidebarSection.expandSection();
    	this._onCreateNewCommentForArtifactMenuClick();
    },
    createNewCommentForArtifact: function() {
    	this._onCreateNewCommentForArtifactMenuClick();
    },
    _onCreateNewCommentForFrameMenuClick: function() {
    	var id = this.viewer._resourceViewerWidget.getActiveFrameLocationId();
        this.showEditCommentForm(
        	{
        		formTitle: dojo.isIE?(this._commentingToolbarCreate.containerNode.innerText):(this._commentingToolbarCreate.containerNode.textContent),
        		commentTitle: "",
        		resourceLocationId: id,
       			locationPane: null 				
        	}
        );
    },
    showMessage: function(message) {
    	this._showMessage(message);
    },
    getProjectURI: function() {
    	if(this._resourceInfo){
			return this._resourceInfo.owningTeam;
    	}
    },
    _setFilterWidgetProjectURI: function() {
    	this._filterWidget.setProjectURI(this.getProjectURI());
    },
    _onFilterClick: function(event) {
    	if (this._filterToggleNode.checked) {
	        this._filterWidget.unhide();
	        this._setHeightOfScrollingPaneElement();
    	}
	    else {
	    	this._filterWidget._onCloseClicked()
	    }
    },
    getReviewDescription: function(reviewId) {
    	return this._controller.getReviewDescription(reviewId);
    },
    isCommentsCreated: function() {
    	return this._annotations != null;
    },
    _onCreateNewCommentForArtifactMenuClick: function(event) {
		var title = null;
    	if (this.isReviewArtifact()) {
			title = dojo.string.substitute(this._messages.reviewFormTitle, [this._resourceInfo.displayName]);
		} else {
			title = dojo.string.substitute(this._messages.artifactFormTitle, [this._resourceInfo.displayName]);
		}
    	this.showEditCommentForm(
    		{
    			formTitle: title,
    			commentTitle: "",
    			resourceLocationId: ""			
    		}
    	);
    },
    
    resolveButtonClick: function(comment) {
    	var resolvedValue = comment.isResolved
    	var deferred = this._controller.updateComment(comment, {isResolved:!resolvedValue});
    	deferred.addErrback(this, function(error) {
            com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorNumber(
            	7208, [], error
    		);
        });
    },
    
    replyToButtonClick: function(comment) {
    	var title = dojo.string.substitute(this._messages.re, [comment.title]);
        this.showEditCommentForm(
        	{
        		formTitle: this._messages.replyToComment,
        		resourceLocationId: comment.locationId,
        		commentTitle: title,
        		directedToUsers: new Array(com.ibm.rdm.web.util.queryutils.getTrimmedUserName(comment.author)),
        		comment: comment 				
        	}
        )
    },
    
    editButtonClick: function(comment) {
        this.showEditCommentForm(
        	{
        		formTitle: this._messages.editComment,
        		resourceLocationId: comment.locationId,
        		commentTitle: comment.title,
        		directedToUsers: comment.directedToUsers,
        		priority: comment.priority,
    			text: comment.text,
        		comment: comment,
        		isUpdateComment: true
        	}
        )
    },
    _onEditCommentMenuClick: function(event) {
    	var comment = this._highlightedCommentPane.annotation;
    	if (this._highlightedCommentPane) {
            this.showEditCommentForm(
        		{
        			formTitle: dojo.isIE?(this._commentingToolbarEdit.containerNode.innerText):(this._commentingToolbarEdit.containerNode.textContent),
        			resourceLocationId: this._highlightedCommentPane.getResourceLocationId(),
        			commentTitle: this._getCommentPaneTitle(this._highlightedCommentPane),
        			directedToUsers: comment.directedToUsers,
        			priority: comment.priority,
    				text: comment.text,
        			comment: this._highlightedCommentPane.annotation,
        			isUpdateComment: true
        		}
        	)
        }
    },
    
    _onDeleteCommentMenuClick: function(event) {
    	var comment = this._highlightedCommentPane.annotation;
    	var self = this;
    	if (this._highlightedCommentPane) {
			var form = this._createConfirmForm(this._messages.confirmDeleteTitle);
			form.connect(form._okButtonNode, "onClick", function(event) {
				self._deleteComment(comment);
			});
			form.show();
        }
    },
    
    _deleteComment: function(comment) {
    	var deferred = this._controller.deleteComment(comment);
    	deferred.addErrback(this, function(error) {
            com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorNumber(
            	7218, [], error
    		);
        });
    },
    
	_createConfirmForm: function(message) {
		var form = new com.ibm.rdm.web.common.ConfirmForm(
		   {
				id: "com.ibm.rdm.web.commenting.ConfirmForm",
				dialogTitle: message,
				confirmMessage: this._messages.confirmDelete
		   }
		);   
		var dialogWidget = new dijit.Dialog(
			{
				id: "com.ibm.rdm.web.commenting.ConfirmDialog",
				duration: 1 // popup immediately
			},
			form.domNode
		);
		form.setDialogWidget(dialogWidget);
		return form;
	},
	
    /**
        If a location pane was selected, add comment to it
        If a locationId is currently selected in assoc doc, add a new comment for it
        If nothing was selected, add to the entire document
    */
    _onCreateNewCommentMenuClick: function(event) {
    	if (this._currentDocSelectedLocationId != null) {
            // add a comment to the last selected location within the doc
            this.showEditCommentForm(
        		{
        			formTitle: dojo.isIE?(this._commentingToolbarCreate.containerNode.innerText):(this._commentingToolbarCreate.containerNode.textContent),
        			commentTitle: "",
        			resourceLocationId: this._currentDocSelectedLocationId,
       				locationPane: null 				
        		}
        	);
        }
    },
    _onReplyToCommentMenuClick: function(event) {
    	if (this._highlightedCommentPane) {
        	var title = dojo.string.substitute(this._messages.re, [this._getCommentPaneTitle(this._highlightedCommentPane)]);
            this.showEditCommentForm(
        		{
        			formTitle: dojo.isIE?(this._commentingToolbarReply.containerNode.innerText):(this._commentingToolbarReply.containerNode.textContent),
        			resourceLocationId: this._highlightedCommentPane.getResourceLocationId(),
        			commentTitle: title,
        			directedToUsers: new Array(com.ibm.rdm.web.util.queryutils.getTrimmedUserName(this._highlightedCommentPane.annotation.author)),
        			comment: this._highlightedCommentPane.annotation 				
        		}
        	)
        }
    },
    _onResolveCommentMenuClick: function(event) {
    	if (this._highlightedCommentPane) {
    		var resolvedValue = this._highlightedCommentPane.annotation.isResolved
    		var deferred = this._controller.updateComment(this._highlightedCommentPane.annotation, {isResolved:!resolvedValue});
    		deferred.addErrback(this, function(error) {
            	com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorNumber(
            		7208, [], error
    			);
            });
        }
    },
    _onAddAllResourceCommentClick: function(event) {
    },
    
    _onCommentingToolbarOptionsMenuOpen: function(event) {
    	var self = this;
    	var frameOption = null;	// storyboard viewer create frame comment 
    	var setMenuState = function() {
    		frameOption = self._commentingToolbarOptionsMenu.getIndexOfChild(self._commentingToolbarCreateForFrame);
    		if (self.viewer.isStoryboardViewer()) {
    			// add option to create comment for frame when in frame view in storyboard viewer
    			var frameOption = self._commentingToolbarOptionsMenu.getIndexOfChild(self._commentingToolbarCreateForFrame);
    			var storyboardViewer = self.viewer._resourceViewerWidget;
        		if (storyboardViewer.isFrameDisplayed()) {
        			if (frameOption == -1) {
        				self._commentingToolbarOptionsMenu.addChild(self._commentingToolbarCreateForFrame, 1);
        			}
    				var fName = self.truncateTitle(storyboardViewer.getActiveFrameName());
    				var msg = dojo.string.substitute(self._messages.createElementComment,[fName]);
					self._commentingToolbarCreateForFrame.attr("label", msg);
        		} else {
        			if (frameOption != -1) {
        				self._commentingToolbarOptionsMenu.removeChild(self._commentingToolbarCreateForFrame);
        			}
        		}
    		}   
    		var menuItemText = null;
    		if (self.isReviewArtifact()) {
				menuItemText = self._messages.createReviewComment;
			} else {
				menuItemText = self._messages.createArtifactComment;
			}
    		self._commentingToolbarCreateForArtifact.attr("label", menuItemText);
    		self._editCommentFormTitle = "";   	
    		var commentTitleText = "";
    		if (self._highlightedCommentPane) {
    			var commentTitleText = self.getTruncatedElementTitle(self._highlightedCommentPane.annotation.locationId);
        	} else if (self._currentDocSelectedLocationId) {
    			var commentTitleText = self.getTruncatedElementTitle(self._currentDocSelectedLocationId);
    		}
    		// comment title will not display if xml chars not escaped
			commentTitleText = com.ibm.rdm.web.util.domutils.getXMLEncodedText(commentTitleText);
    		var menuMessage = null;
    		if ((self._currentDocSelectedLocationId || (self._highlightedCommentPane && self._getGlobalResourceId()!=self._highlightedCommentPane.annotation.locationId)) &&
    			!(self._highlightedCommentPane && self._highlightedCommentPane.annotation.isElementDeleted)) {
    			self._commentingToolbarCreate.attr("disabled", false);
    			menuMessage = dojo.string.substitute(self._messages.createElementComment,[commentTitleText]);
    		} else {
    			self._commentingToolbarCreate.attr("disabled", true);
    			menuMessage = dojo.string.substitute(self._messages.createElementComment,[""]);
    		}
        	self._commentingToolbarEdit.attr("disabled", false);
        	self._commentingToolbarDelete.attr("disabled", false);
			self._commentingToolbarCreate.attr("label", menuMessage);
    		self._commentingToolbarCreateForArtifact.attr("disabled", false);
    		if (frameOption) {
    			self._commentingToolbarCreateForFrame.attr("disabled", false);
    		}
			if (self._highlightedCommentPane && !self._highlightedCommentPane.annotation.isElementDeleted) {
    			self._commentingToolbarReply.attr("disabled", false);
    			self._commentingToolbarResolve.attr("disabled", false);
    			if (self._highlightedCommentPane.annotation.isResolved) {
        			self._commentingToolbarEdit.attr("disabled", true);
    				self._commentingToolbarResolve.attr("label", self._messages.unresolveComment);
    			} else {
    				self._commentingToolbarResolve.attr("label", self._messages.resolveComment);
    			}
    			// only allow delete if the comment does not have replies
    			var comment = self._highlightedCommentPane.annotation;
    			if (comment.replies.length == 0) {
        			self._commentingToolbarDelete.attr("disabled", false);
    			} else {
        			self._commentingToolbarDelete.attr("disabled", true);
    			}
    			var commentTitleText = self.truncateTitle(self._getCommentPaneTitle(self._highlightedCommentPane));
    			var menuMessage = dojo.string.substitute(self._messages.replyComment,[commentTitleText]);
				self._commentingToolbarReply.attr("label", menuMessage);
			} else if (self._highlightedCommentPane && self._highlightedCommentPane.annotation.isElementDeleted) {
    			// only allow delete if the comment does not have replies
    			var comment = self._highlightedCommentPane.annotation;
    			if (comment.replies.length == 0) {
        			self._commentingToolbarDelete.attr("disabled", false);
    			} else {
        			self._commentingToolbarDelete.attr("disabled", true);
    			}
    			var menuMessage = dojo.string.substitute(self._messages.replyComment,[""]);
				self._commentingToolbarReply.attr("label", menuMessage);
    			self._commentingToolbarReply.attr("disabled", true);
    			self._commentingToolbarResolve.attr("disabled", true);
    			self._commentingToolbarEdit.attr("disabled", true);
    		} else {
    			var menuMessage = dojo.string.substitute(self._messages.replyComment,[""]);
				self._commentingToolbarReply.attr("label", menuMessage);
    			self._commentingToolbarReply.attr("disabled", true);
    			self._commentingToolbarResolve.attr("disabled", true);
    			self._commentingToolbarEdit.attr("disabled", true);
    			self._commentingToolbarDelete.attr("disabled", true);
    		}  
     		if (self._highlightedCommentPane && self._highlightedCommentPane.annotation.author != com.ibm.rdm.web.AUTHENTICATED_USER.userId) {
    			self._commentingToolbarEdit.attr("disabled", true);
    			self._commentingToolbarDelete.attr("disabled", true);
    		}    		  	    	
    		// disable comment creation/editing/resolving    	
    		if (self.isNoCreateCommentMode()) {
    			var menuMessage = dojo.string.substitute(self._messages.replyComment,[""]);
				if (frameOption) {
					self._commentingToolbarCreateForFrame.attr("disabled", true);
				}
    			self._commentingToolbarCreateForArtifact.attr("disabled", true);
				self._commentingToolbarCreate.attr("disabled", true);
				self._commentingToolbarReply.attr("label", menuMessage);
    			self._commentingToolbarReply.attr("disabled", true);
    			self._commentingToolbarResolve.attr("disabled", true);
    			self._commentingToolbarEdit.attr("disabled", true);
    			self._commentingToolbarDelete.attr("disabled", true);
    		}
    	};
    	    	
    	if (this.viewer.isInReviewMode()) {
        	var uri = com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL + "/" + this.viewer.reviewId;
			var review = this.getReviewDescription(uri);
			// The review should be found, but be safe here so that the menu displays properly
			var doDisable = !review || review.reviewStatus == com.ibm.rdm.web.review.reviewTypes.REVIEW_STATE_TYPES.Paused
					|| review.reviewStatus == com.ibm.rdm.web.review.reviewTypes.REVIEW_STATE_TYPES.Finalized;
			if (doDisable) {
    			var menuMessage = dojo.string.substitute(self._messages.replyComment,[""]);
    			this._commentingToolbarCreateForArtifact.attr("disabled", true);
				this._commentingToolbarCreate.attr("disabled", true);
				this._commentingToolbarReply.attr("label", menuMessage);
    			this._commentingToolbarReply.attr("disabled", true);
    			this._commentingToolbarResolve.attr("disabled", true);
    			this._commentingToolbarEdit.attr("disabled", true);
    			self._commentingToolbarDelete.attr("disabled", true);
			} else {
				setMenuState();
			}
		} else {
			setMenuState();
		}
    },
    
    isNoCreateCommentMode: function() {
    	// test for history mode
		if (this.revisionDate) return true;
		// test whether user has permission to create comments
		var projectOperations = com.ibm.rdm.web.AUTHENTICATED_USER.permissions.item(this.getProjectURI());	
		var isAllowed = projectOperations.item(com.ibm.rdm.web.common.ProjectCache.MODIFY_COMMENT);	
		return !isAllowed;
    },
        
    getCommentReviewId: function() {
    	if (this.viewer._contentType == com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REVIEW) {
    		// only comments for artifacts in the review are flagged with a reviewId, comments
    		// for the review itself are not
    		return "";
    	} else {
    		return this.viewer.reviewId;
    	}
    },
    
    setButtonStates: function(comment, replyToAnchor, editAnchor, resolveAnchor) {
    	var self = this;
    
    	var setStates = function() {
    		if (comment.isElementDeleted || self.isNoCreateCommentMode()) {
    			dojo.removeClass(replyToAnchor, "enabled");
    			dojo.removeClass(resolveAnchor, "enabled");
    			dojo.addClass(replyToAnchor, "disabled");
    			dojo.addClass(resolveAnchor, "disabled");
    		} else {
    			dojo.removeClass(replyToAnchor, "disabled");
    			dojo.removeClass(resolveAnchor, "disabled");
    			dojo.addClass(replyToAnchor, "enabled");
    			dojo.addClass(resolveAnchor, "enabled");
    		};
    		if (comment.isResolved || (comment.author != com.ibm.rdm.web.AUTHENTICATED_USER.userId) || self.isNoCreateCommentMode() || comment.isElementDeleted) {
    			dojo.removeClass(editAnchor, "enabled");
    			dojo.addClass(editAnchor, "disabled");
    		} else {
    			dojo.removeClass(editAnchor, "disabled");
    			dojo.addClass(editAnchor, "enabled");
    		}
    	};
    	    	
    	if (this.viewer.isInReviewMode()) {
			// get associated review, if review paused or finalized disable comment creation
        	var uri = com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL + "/" + this.viewer.reviewId;
			var review = this.getReviewDescription(uri);
			var doDisable = review.reviewStatus == com.ibm.rdm.web.review.reviewTypes.REVIEW_STATE_TYPES.Paused
				|| review.reviewStatus == com.ibm.rdm.web.review.reviewTypes.REVIEW_STATE_TYPES.Finalized;
			if (doDisable) {
    			dojo.removeClass(replyToAnchor, "enabled");
    			dojo.removeClass(resolveAnchor, "enabled");
    			dojo.removeClass(editAnchor, "enabled");
    			dojo.addClass(replyToAnchor, "disabled");
    			dojo.addClass(resolveAnchor, "disabled");
    			dojo.addClass(editAnchor, "disabled");
 			} else {
				setStates();
			}
		} else {
			setStates();
		}
    },
    
    _getCommentPaneTitle: function(commentPaneWidget) {
    	return commentPaneWidget.annotation.title;
    },
    _onCollapseAllClick: function(event) {
    	// just collapse the comments, not the groups
		var commentPanes = this._allCommentPanes.getValueList();
		for (var i=0; i<commentPanes.length; i++) {
    		var commentPane = commentPanes[i];
    		commentPane.collapse();
		}
    },
    _onExpandAllClick: function(event) {
    	var locationPanes = this._allLocationPanes.getValueList();
		for (var i=0; i<locationPanes.length; i++) {
    		var resourceLocationPaneWidget = locationPanes[i];
    		if (!resourceLocationPaneWidget.open)
    			resourceLocationPaneWidget.toggle();	
		}
		var commentPanes = this._allCommentPanes.getValueList();
		for (var i=0; i<commentPanes.length; i++) {
    		var commentPane = commentPanes[i];
    		commentPane.expand();
		}
    },
    _onLocateInDocClick: function(event) {
        this.locateInDoc();
    },
    locateInDoc: function() {
        if (this._locateInDocToggleNode.checked) {
            this._locateCommentsInDocument();
        } else {
            this._clearCommentsInDocument();
        }   
    },
    _onWindowResize: function(event) {
        this._setHeightOfScrollingPaneElement();
    },    
    _onCommentDeleted: function(comment) {
    	this._showMessage(this._messages.loadingComments);
    	this._highlightedCommentPane = null;
    	this._resetState(false);
    	// Get rid of the comment pane
       	var pane = this._allCommentPanes.item(comment.linkSelfUri);
       	this._allCommentPanes.remove(comment.linkSelfUri);
    	pane.destroyRecursive();
        this._controller._createComments();
        this._showAnnotations();
        // Update the current filter message
        this._filterWidget.updateCurrentFilterMessage();
    },
    _onCommentUpdated: function(comment) {
       // Refresh the pane of this comment with new comment, hide if necessary
       /*var cp = this._getCommentPaneOf(comment);
       cp.setComment(comment);
       this._filterWidget.applyFilter([cp]);
        // Update editor highlight
        if (this._commentsLocated) {
            this._locateCommentsInDocument();
        }*/
    	this._showMessage(this._messages.loadingComments);
    	this._resetState(false);
    	
    	// comment was updated, so we need to refresh the widget
    	// remember the expanded state of the comment
    	var expanded = false; 
    	if (this._allCommentPanes.containsKey(comment.linkSelfUri)) {
    		expanded = this._allCommentPanes.item(comment.linkSelfUri).isExpanded;
    		this._allCommentPanes.item(comment.linkSelfUri).destroyRecursive();
    	}
    	this._allCommentPanes.remove(comment.linkSelfUri);

        this._controller._createComments();
        this._showAnnotations();
        
        this._openLocationPane(comment.locationId);
        
        // Update editor highlight
        if (this._commentsLocated) {
            this._locateCommentsInDocument();
        }
        
        this._selectCommentPane(comment.linkSelfUri, true);
        if (expanded) {
        	this._allCommentPanes.item(comment.linkSelfUri).expand();
        } else {
        	this._allCommentPanes.item(comment.linkSelfUri).collapse();
        }
        
        // Update the current filter message
        this._filterWidget.updateCurrentFilterMessage();
    },
    _onNewCommentSaved: function(comment) {
        this._addNewAnnotation(comment);
    },    
    _onAutoScrollClicked: function(event) {
        if (this._autoScrollNode.checked) {
            var locPane = this._highlightedLocationPane;
            if (!locPane)
               return;
            if (locPane.isHidden())
                return;
            var locId = locPane.resourceLocationId;   
            this._highlightPaneById(locId, false);
        }
    },
    _onSortByDateMenuItemClick: function(event) {
		this._updateSortingGrouping(com.ibm.rdm.web.commenting.CommentingWidget.Ordering.SORTBY_DATE, undefined, this._sortByDateMenuItem);
    },
    _onSortByUserMenuItemClick: function(event) {
    	this._updateSortingGrouping(com.ibm.rdm.web.commenting.CommentingWidget.Ordering.SORTBY_USER, undefined, this._sortByUserMenuItem);
    },
    _updateSortingGrouping: function(sortBy, groupBy, menuItem) {
    	this._showMessage(this._messages.sortingComments);
		this._resetState(false);
		var previousSelection;
				
		if (sortBy) {
			this._sortCommentsBy = sortBy;
			previousSelection = this._previousSortingSelection;
			if (menuItem != null) this._previousSortingSelection = menuItem;
		}
		
		if (groupBy) {
			this._groupCommentsBy = groupBy;
			previousSelection = this._previousGroupingSelection;
			if (menuItem != null) this._previousGroupingSelection = menuItem;
			
			//need to disable sorting if grouping is set to "author" or "date"
			var isSortingDisabled = (groupBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_DATE || groupBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_AUTHOR);
			this._sortByDateMenuItem.attr("disabled", isSortingDisabled);
			this._sortByUserMenuItem.attr("disabled", isSortingDisabled);
			if (isSortingDisabled) {
				if (groupBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_DATE) {
					dojo.removeClass(this._sortByDateMenuItem.iconNode, "commentingToolbarDropdownBullet");
					dojo.addClass(this._sortByUserMenuItem.iconNode, "commentingToolbarDropdownBullet");
					this._previousSortingSelection = this._sortByUserMenuItem;
				}
				if (groupBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_AUTHOR) {
					dojo.removeClass(this._sortByUserMenuItem.iconNode, "commentingToolbarDropdownBullet");
					dojo.addClass(this._sortByDateMenuItem.iconNode, "commentingToolbarDropdownBullet");
					this._previousSortingSelection = this._sortByDateMenuItem;
				}
			}
		}
	    this._controller._createComments();
	    this._showAnnotations();
	    if (menuItem != null) {
	    	dojo.removeClass(previousSelection.iconNode, "commentingToolbarDropdownBullet");
			dojo.addClass(menuItem.iconNode, "commentingToolbarDropdownBullet");
		}
	},
	_onNoGroupingMenuItemClick: function(event) {
		this._updateSortingGrouping(undefined, com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_NOGROUPING, this._noGroupingMenuItem);
	},
	_onGroupByArtifactMenuItemClick: function(event) {
		this._updateSortingGrouping(undefined, com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_ARTIFACT, this._groupByArtifactMenuItem);
	},
	_onGroupByDateItemClick: function(event) {
		this._updateSortingGrouping(com.ibm.rdm.web.commenting.CommentingWidget.Ordering.SORTBY_USER, com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_DATE, this._groupByDateMenuItem);
	},
	_onGroupByAuthorMenuItemClick: function(event) {
		this._updateSortingGrouping(com.ibm.rdm.web.commenting.CommentingWidget.Ordering.SORTBY_DATE, com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_AUTHOR, this._groupByAuthorMenuItem);
	},
	_onGroupByReviewMenuItemClick: function(event) {
		this._updateSortingGrouping(undefined, com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_REVIEW, this._groupByReviewMenuItem);
	},
    /**
        Publishes com.ibm.rdm.web.commenting.EventTopics.LOCATE_COMMENTED_LOCATIONS_IN_RESOURCE
        which is an object whose attributes are locationIds which has unfiltered
        comments based on the current filter, with corresponding walues equal to the
        unfiltered comment count for that location.
        
    */
    _locateCommentsInDocument: function() {
          var result = {};
          var commentPanes = this._allCommentPanes.getValueList();
          for (var i=0; i<commentPanes.length; i++) {
              var commentPane = commentPanes[i];
              if (!commentPane.isHidden() && commentPane.annotation.isRoot) {
                  var locationId = commentPane.annotation.locationId;
            	  if (result[locationId])
                	  result[locationId]++;
                  else
                	  result[locationId] = 1;
               	  result[locationId] += commentPane.annotation.getAllReplies().length;
              }              
          }
          this.publish(EventTopics.LOCATE_COMMENTED_LOCATIONS_IN_RESOURCE, [result]);
    },
    /**
        Undo/clear of _locateCommentsInDocument
    */
    _clearCommentsInDocument: function() {
          var result = {};
          this.publish(EventTopics.LOCATE_COMMENTED_LOCATIONS_IN_RESOURCE, [result]);
    },
    _setHeightOfScrollingPaneElement: function() {
    	if (!this.hasStarted)
    		return;
        var self = this;
        var windowBasedResizeContentPane = function() {
            var menuHeight =  dojo.marginBox(self._commentingMenuNode).h; // parseInt(self._commentingMenuNode.style.height, 10)
            // TODO: In IE msgHeight height shows 700, when it is zero.
            // var msgHeight  =  dojo.marginBox(self._messageNode).h ; // parseInt(self._messageNode.style.height, 10);
            var filterHeight = dojo.marginBox(self._filterWidget.domNode).h ;
            var topHeight = (isNaN(menuHeight)?0:menuHeight) + /*(isNaN(msgHeight)?0:msgHeight) + */(isNaN(filterHeight)?0:filterHeight);
            ErrorHandler.debug("menu: " +  menuHeight + ", filter: " + filterHeight + ", w: " + com.ibm.rdm.web.commenting.utils.getWindowHeight());
            var newHeight = (com.ibm.rdm.web.commenting.utils.getWindowHeight() - topHeight);
            if (newHeight>0) { 
                // dijit.byId("_annotationsWidgetPane").domNode.style.height = newHeight + "px";
                dijit.byId(self._annotationsWidgetNode).domNode.style.height = newHeight + "px";
            } else {
                ErrorHandler.debug("Did not resize window. Height is too small");        
            }
        };
        var domBasedResizeContentPane = function() {
            if (!self.domNode.parentNode) { return; }
            var height = self.domNode.parentNode.clientHeight;
            var width = self.domNode.parentNode.clientWidth;
            var contentPaneTop = (
                self._annotationsWidgetNode.domNode.offsetTop
                - self._annotationsWidgetNode.domNode.parentNode.offsetTop);
            var scrollHeightToUse = (height - contentPaneTop);
            if (scrollHeightToUse > 0)
            	dijit.byId(self._annotationsWidgetNode).domNode.style.height = scrollHeightToUse + "px";
            dijit.byId(self._annotationsWidgetNode).domNode.style.width = width + "px";
        };
        if (this.displayContext
            == com.ibm.rdm.web.commenting.CommentingWidget.displayContext.SIDEBAR_SECTION) {
            domBasedResizeContentPane();
        } else {
            windowBasedResizeContentPane();
        }
    },
    _showMessage: function(message) {
    	this._clearMessage();
        this._sidebarSection.setSectionTitleInfo(message);
    },
    _clearMessage: function() {
    	this._sidebarSection.clearSectionTitleInfo();
    },
    /**
        Keeps CommentingWidget but resets its state and destroys subwidgets related to
        resetted state. 
    */
    _resetState: function(destoryCommentPanes) {
    	if (destoryCommentPanes){
    		var locationPanes = this._allLocationPanes.getValueList();
	    	for (var i=0; locationPanes && i<locationPanes.length; i++) {
	            var lp = locationPanes[i];
            	lp.destroyCommentPanes();
	            lp.destroyRecursive();
	        }
        	this._allCommentPanes = new dojox.collections.Dictionary();
        	this._allLocationPanes = new dojox.collections.Dictionary();
        	
        	this._highlightedLocationPane = null;
            this._highlightedCommentPane = null; 
    	}
    	else {
    		var locationPanes = this._allLocationPanes.getValueList();
	    	for (var i=0; locationPanes && i<locationPanes.length; i++) {
	            var lp = locationPanes[i];
            	lp.hide();
	        }
    	}

        //this._refreshToolbarIconNode.parentNode.blur();
        this._annotations = null;     
    	this._isDeletedElementsUpdated = false;
    },
    _initToolbarButtons: function() {
    
    },
    _initAnnotations: function() {
    	this._showMessage(this._messages.loadingComments);
    	domutils.createHourGlassCursor();
    	this._isDeletedElementsUpdated = false;
        // Reset all widget state and get some comments from repo
        this._resetState(true);
        var deferred = this._controller._getComments(this.revisionDate);
        deferred.addCallback(this, this._doPostGetComments);
        deferred.addErrback(this, function(error) {
        	this._clearMessage();
        	domutils.createDefaultCursor();
        	this._resetState(true);
            com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorNumber(
            	7209, [], error
    		);
        });
        return deferred;
    },
    
    _doPostGetComments: function() {
    	this._initSubscriptions();
     	if (this.selectCommentURI != null) {
     		// TODO - need to test this when have rich client running again, test for 
     		// an artifact that has a lot of elements to render
     		// see if we can select the comment, if it is an artifact-level comment
     		// we do not need to wait until the resource is loaded since no element
     		// has to be selected in the viewer
//     		var commentPane = this._allCommentPanes.item(this.selectCommentURI);
//     		if (commentPane) {
//     			if (commentPane.annotation.locationId == "") {
//       				this._selectCommentPane(this.selectCommentURI, true);
//       				this.selectCommentURI = null;
//       			}
//       		}
       	}     		
    	
    },
    
    _showAnnotations: function(response) {
    	var timer = (new Date()).getTime();
       var self = this;
       if (this._groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_NOGROUPING) {
    	   var resourceLocationPaneWidget = self._createPaneForLocation(com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_NOGROUPING);
    	   if (resourceLocationPaneWidget) {
    		   try {
    		   		resourceLocationPaneWidget.domNode.removeChild(resourceLocationPaneWidget.focusNode);
    		   } catch(e){
    		   		// continue processing
    		   }
		       self._resourceLocationPane.domNode.appendChild(resourceLocationPaneWidget.domNode);
		       resourceLocationPaneWidget.addMoreButton();
		       if (!resourceLocationPaneWidget.open)
	    			resourceLocationPaneWidget.toggle();
    	   }
       }
       else {             
	       	var commentGroupsKeys = this._annotations._dictionary.getKeyList();
			if (self._groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_AUTHOR) {
      			var onComplete = function(userMap) {
   					function getUserInfo(userId, userInfos) {
    					for (var userURL in userInfos) {
        					if (userInfos[userURL].userId === userId) {
        						return userInfos[userURL];
        					}
        				}
        				return null;
        			};
		    	   	dojo.forEach(commentGroupsKeys, function(locElementId) {
						var p = self._createPaneForLocation(locElementId);
						if (p) {
							self._resourceLocationPane.domNode.appendChild(p.domNode);
							p.addMoreButton();
        					var userInfo = getUserInfo(locElementId, userMap);
        					var author = (userInfo ? userInfo.name : messages.unknownUser);
							p.setTitle(author);
						}
		    	   	});
       				self._filterWidget._filterCommentPanes();       
     			};
        		ProjectCache.getProjectUsers(self.getProjectURI()).addCallback(this, onComplete);
			} else {
	    	   	dojo.forEach(commentGroupsKeys, function(locElementId) {
					var p = self._createPaneForLocation(locElementId);
					if (p) {
						self._resourceLocationPane.domNode.appendChild(p.domNode);
						p.addMoreButton();
						if (self._groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_DATE) {
							p.setTitle(com.ibm.rdm.web.commenting.utils.getUpdatedRangeTitle(locElementId));
						} else if (self._groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_REVIEW) {
							if (locElementId == com.ibm.rdm.web.commenting.NON_REVIEW_COMMENTS) {
								p.setTitle(self._messages.allOtherComments);
							} else {
								var review = self.getReviewDescription(locElementId);
								if (review == null) {
									p.setTitle(self._messages.unknownReview);
								} else {
									p.setTitle(review.name);
								}
							}
						}
					}
	    	   });
	   		}
       }
       this.updateCommentingAfterResourceViewer();
       
       this._clearMessage();
       domutils.createDefaultCursor();
       
       
        // Size widget to scroll below the toolbar, after postCreate() so the domNodes are set
        // TODO: Assumes widget occupies 100% of the clientWindow
        this._setHeightOfScrollingPaneElement();        
       
       this.publish(EventTopics.COMMENTING_WIDGET_LOADED_TOPIC, [this]);
       
       ErrorHandler.debug("created comments widgets in : " + ((new Date()).getTime() - timer) + "ms");
       
       return response;
    },
    _createPaneForAnnotation: function(comment) {
    		if (this._allCommentPanes.containsKey(comment.linkSelfUri))
    			return this._allCommentPanes.item(comment.linkSelfUri);
    		
    		var commentPane = new com.ibm.rdm.web.commenting.CommentPane({ annotation: comment, commentingWidget: this});
            this._allCommentPanes.add(comment.linkSelfUri, commentPane);
			return commentPane;
    },
    _createPaneForLocation: function(groupId) {
             var anns = this._annotations._dictionary.item(groupId);
             
             if (!anns) {
             	// no comments exist
                return;
             }
             var commentPanes = new Array(/* CommentPane */ );
             var self = this;
             
             function createPane(c) {
                var p = self._createPaneForAnnotation(c);
                commentPanes.push(p);
                return p;
             }
             
             function createReplyPane(c, parentPane) {
                 var p = self._createPaneForAnnotation(c);
                 parentPane.replyPanes.push(p);
                 commentPanes.push(p);
                 p.parentPane = parentPane;
              }
              
             dojo.forEach(anns, function(ann) {
                var parentPane = createPane(ann);
                dojo.forEach(ann.getAllReplies(), function(reply) {
                    createReplyPane(reply, parentPane);
                });    
             });
             
             var p = this._createEmptyResourceLocationPane(groupId, commentPanes, false);
             return p;
    },
    _createEmptyResourceLocationPane: function(resourceLocationId, commentPanes, isOpen, locationDescription) {		
		if (this._allLocationPanes.containsKey(resourceLocationId)) {
			var locationPane = this._allLocationPanes.item(resourceLocationId);
			locationPane.initializeCommentPanes(commentPanes);
			locationPane.unhide();
			return locationPane;
		}
        var p = new com.ibm.rdm.web.commenting.ResourceLocationPane(
           {   open: (this._groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_NOGROUPING), 
        	   "commentPanes":(commentPanes==null)?new Array():commentPanes,
               "resourceLocationId": resourceLocationId,
               "commentingWidget": this });
        this._allLocationPanes.add(resourceLocationId, p);
        return p;
    },
    _byLocationPaneId: function(paneId) {
    	return this._allLocationPanes.item(paneId);
    },    
    _byCommentPaneId: function(paneId) {
        var commentPanes = this._allCommentPanes.getKeyList();
    	return this._allCommentPanes.item(paneId);
    },    
    _addNewAnnotation: function(comment) {
    	// comment has been saved already; just updating the GUI
    	this._showMessage(this._messages.loadingComments);
    	this._resetState(false);
    	this._controller._annotations.push(comment),
        this._controller._createComments();
        this._showAnnotations();
        
        this._openLocationPane(comment.locationId);
        
        // Update editor highlight
        if (this._commentsLocated) {
            this._locateCommentsInDocument();
        }
    	var commentPane = this._getCommentPaneOf(comment.linkSelfUri);
    	if (!commentPane.isRootPane()) {
    		commentPane.getRootPane().updateReplyDetails();
    	} else {
    		commentPane.expand();
    	}
        this._selectCommentPane(comment.linkSelfUri, true);
        
        // Update the current filter message
        this._filterWidget.updateCurrentFilterMessage();
    },
    _isGlobalResourceId: function(locationId) {
        return (locationId == this._getGlobalResourceId());
    },
    _getGlobalResourceId: function() {
        return "";
    },
    _trackAddedDocSelection: function(docLocationId) {
        this._currentDocSelectedLocationId = docLocationId;
    },
    _trackRemovedDocSelection: function(docLocationId) {
        this._currentDocSelectedLocationId = null;
    },
    _openLocationPane: function(docLocationId) {
        var locPane = this._byLocationPaneId(docLocationId);
        if (!locPane) {
        	return;
        }
        if (!locPane.open) {
            locPane.toggle();
        }
    },
    /*
       Highlights location ResourceLocationPane, unhighlights the currently highlighted one,
       if there was one.
    */
    _highlightPaneById: function(docLocationId, stayPut) {
    	var locPane = this._byLocationPaneId(docLocationId);
        if (locPane) {
        	this._highlightPane(locPane);
        }
    },
    _highlightPane: function(locPane, stayPut) {
    	if (this._groupCommentsBy != com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_ARTIFACT)
    		return;
    	
        /*if (!locPane || locPane.isHidden()) {
            // Clear previous highlights
        	this.unselectAllHighlightedPanes();
            return;
        }*/

        // scroll to top
        //if (!stayPut && this._autoScrollNode.checked) {
        //    this._scrollLocationPaneToTop(docLocationId, event);
        //}    

        // Unselect the selected comment pane, if there is one
        //this._unselectHighlightedCommentPane();
        
        // 5671: If the pane is already selected, do not do anything 
        if (locPane == this._highlightedLocationPane) {
            return;
        }
                
        this._unselectHighlightedLocationPane();
        
        // TODO: Make these class names constants. 
        locPane.select();
        this._highlightedLocationPane = locPane;
        this._trackAddedDocSelection(locPane.getResourceLocationId());
        
        var topic = EventTopics.RESOURCE_LOCATION_PANE_SELECTED_TOPIC;
        this.publish(topic, [locPane.getResourceLocationId(), this.resourceURI]); 
    },
    _toggleResourceLocationSelection: function(selectedLocationId, unselectedLocationId) {
    	//unselect
    	this.unselectAllHighlightedPanes();
        this._trackRemovedDocSelection(unselectedLocationId);
        
        //select
        if (selectedLocationId) {
	        this._trackAddedDocSelection(selectedLocationId);
	        this._openLocationPane(selectedLocationId);
	        this._highlightPaneById(selectedLocationId, false);
	        this._scrollLocationPaneToTop(selectedLocationId);
        }
    },
    _selectCommentPane: function(commentUri, isProgrammaticSelection) {
    	var commentPane = this._getCommentPaneOf(commentUri);
    	if (commentPane == null) return;
    	var rootPane = commentPane.getRootPane();
    	var groupPane = this._getGroupPaneOf(rootPane.annotation);
    	if (groupPane != null) {
    		if (!groupPane.open) {
    			groupPane.toggle();
    		}
    	}
    	// to select the comment it's root pane must be expanded
    	if (rootPane == null) return;
    	// user clicks also come through this method, only expand the comment when
    	// this is called programmatically
    	if (isProgrammaticSelection) {
			if (!rootPane.isExpanded) rootPane.expand();
		}
    	this._unselectHighlightedCommentPane();
        this._highlightPane(commentPane.resourceLocationPane);
        commentPane.select();
        this._highlightedCommentPane = commentPane;
        this._trackAddedDocSelection(commentPane.annotation.locationId);
        
        //let the resource viewer know that the comment pane is selected
        var topic = EventTopics.RESOURCE_LOCATION_PANE_SELECTED_TOPIC;
        this.publish(topic, [commentPane.annotation.locationId, this.resourceURI]);
        if (isProgrammaticSelection != undefined && isProgrammaticSelection) {
	        this._scrollCommentPaneToTop(commentUri);
        }
    },
    _unselectHighlightedCommentPane: function() {
    	if (this._highlightedCommentPane) {
            this._highlightedCommentPane.unselect();
            this._highlightedCommentPane = null;
        }
    },
    _unselectHighlightedLocationPane: function() {
    	if (this._highlightedLocationPane) {
            this._highlightedLocationPane.unselect();
            this._highlightedLocationPane = null;
        }
    },
    unselectAllHighlightedPanes: function(commentUri) {
    	this._unselectHighlightedLocationPane();
    	this._unselectHighlightedCommentPane();
    },
    _scrollLocationPaneToTop: function(docLocationId) {
     	var locationPane = this._byLocationPaneId(docLocationId);
     	var allCommentsPane = dijit.byId(this._annotationsWidgetNode);
        if (!locationPane) {
            return;
        }
        com.ibm.rdm.web.util.autoscroller.scrollToNode(locationPane.domNode, allCommentsPane.domNode);
     },
     _scrollCommentPaneToTop: function(commentLocationId) {
      	var locationPane = this._byCommentPaneId(commentLocationId);
      	var allCommentsPane = dijit.byId(this._annotationsWidgetNode);
      	if (!locationPane) {
        	return;
      	}
     	com.ibm.rdm.web.util.autoscroller.scrollToNode(locationPane.domNode, allCommentsPane.domNode);
      },
    _getGroupPaneOf: function(comment) {
    	  if (!comment.isRoot) return null;
    	  var group = null;
    	  switch (this._groupCommentsBy) {
        	case com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_NOGROUPING: 
        		break;
        	case com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_ARTIFACT:
        		group = this._allLocationPanes.item(comment.locationId);
        		break;
        	case com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_DATE:
            	var now = new Date();
        		group = this._allLocationPanes.item(comment.getDLMRange());
        		break;
        	case com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_AUTHOR:
        		group = this._allLocationPanes.item(comment.author);
        		break;
        	case com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_REVIEW:
        		group = this._allLocationPanes.item(comment.reviewId);
        		break;
        	default:
    			// TODO (LPK) log error
        };
        return group;
    },    
    _getCommentPaneOf: function(linkSelfUri /*Comment or String */) {
        return this._allCommentPanes.item(linkSelfUri);
    },    
    getCommentCount: function(callbackFn, callbackScope) {
    	return this._filterWidget.getCommentCount(callbackFn, callbackScope);
    },
    _updateDeletedElements: function() {
    	var commentPanes = this._allCommentPanes.getValueList();
		if (this.viewer.isStoryboardViewer()) {
			// workaround, do not flag any storyboard comments as unreferenced
    		for (var i = 0; i < commentPanes.length; i++) {
	    		commentPanes[i]._commentDeletedElementTextNode.innerHTML = "";
	    	}
 		} else {
    		for (var i = 0; i < commentPanes.length; i++) {
    			if (commentPanes[i].annotation.locationId != "") {
	    			var node = this._resourceViewerWidget.idNodesMap[commentPanes[i].annotation.locationId];
	    			if (!node) {
	    				//var spanNode = document.createElement("SPAN");
	    				commentPanes[i]._commentDeletedElementTextNode.appendChild(document.createTextNode(this._messages.deletedElement));
	    				dojo.addClass(commentPanes[i]._commentDeletedElementTextNode, "rdm-annotation-deletedElementText");
	    				//commentPanes[i]._commentHeadlineNode.appendChild(spanNode);
	    				commentPanes[i].annotation.isElementDeleted = true;
	    				commentPanes[i].annotation.locationId = ""; // group these comments at the artifact level
	    			}
	    			else
	    				commentPanes[i]._commentDeletedElementTextNode.innerHTML = "";
    			}
    		}
        }
    	this._isDeletedElementsUpdated = true;
    },
    updateCommentingAfterResourceViewer: function() {
    	ErrorHandler.debug("CommentingWidget: waiting for resource viewer widget to load contents");
    	if (!this.viewer || !this.viewer._resourceViewerWidget)
    		return;
    	
    	var resourceViewer = (this.viewer._resourceViewerWidget._resourceViewerWidget)?
									this.viewer._resourceViewerWidget._resourceViewerWidget:
				    				this.viewer._resourceViewerWidget;
    	
    	//if (resourceViewer.idNodesMap == undefined || 
    	//		(this.viewer._resourceViewerWidget._resourceViewerWidget && !this.viewer._resourceViewerWidget._currentlyDisplayedFrameIndex))
    	//	return;
    	
    	// if the viewer doesn't define an idNodesMap, no reason to wait for the resource to load
    	// in order to update the commmenting widget, viewer for Review artifact falls under this
    	// category
    	if (resourceViewer.idNodesMap == undefined) {
    		this._doUpdatesAfterResourceViewer();
    		return;
    	}
    	
    	if (resourceViewer && (resourceViewer._documentHasBeenLoaded || this.viewer._resourceViewerWidget._frameInfos)) {
    		ErrorHandler.debug("CommentingWidget: updating commenting with the resource viewer information.");
    		this._resourceViewerWidget = resourceViewer;
    		this._doUpdatesAfterResourceViewer();
     		
            if (this._updateTimeout) {
            	clearTimeout(this._updateTimeout);
            	this._updateTimeout = null;
            }
        }
    	else {
    		if (this._updateTimeout) {
            	clearTimeout(this._updateTimeout);
            	this._updateTimeout = null;
            }
    		this._updateTimeout = setTimeout(
   	             dojo.hitch(this, this.updateCommentingAfterResourceViewer),
   	             500);
    	}
    },
    
    _doUpdatesAfterResourceViewer: function() {
    	if (this._groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_ARTIFACT) {
	    	var locationPanes = this._allLocationPanes.getKeyList();
	        for (var i = 0; i < locationPanes.length; i++) {
	            var title = this.getTruncatedElementTitle(locationPanes[i]);
	            this._allLocationPanes.item(locationPanes[i]).setTitle(title);
	           }
    	}
    	if (!this._isDeletedElementsUpdated)
    		this._updateDeletedElements();
    		
     	if (this.selectCommentURI != null) {
       		this._selectCommentPane(this.selectCommentURI, true);
       		this.selectCommentURI = null;
       	}    
       	
       	if (this._regroupComments) {
       		// workaround for 20520 and history viewer
       		this._regroupComments = false;
    		this._updateSortingGrouping(null, com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_ARTIFACT, null);
       	} else {
       		this.viewer._updateCommentsCounterInHeader();
       		this._filterWidget._filterCommentPanes();       
       	}	
    },
    
    getTruncatedElementTitle: function(elementId) { //TODO: Should this be a method in ResourceViewerWidget?
		if (this._isGlobalResourceId(elementId)) { //this is the main resource comment title pane
			return this.truncateTitle(dojo.string.substitute(this._messages.artifactGroup, [this._resourceInfo.displayName]));
		}
		else if (this._resourceViewerWidget && this._resourceViewerWidget._documentHasBeenLoaded && this._resourceViewerWidget.idNodesMap) {
			var sourceName = this.viewer.getElementDisplayName(elementId);
			if (sourceName == null) {
        		return "";
			} else {
				sourceName = sourceName.replace(/\r\n/g, " ");
				sourceName = sourceName.replace(/\r/g, " ");
				sourceName = sourceName.replace(/\n/g, " ");
				return this.truncateTitle(sourceName);
			}
    	}
		return "";
    },
    truncateTitle: function(titleText) {
    	if (titleText==null || titleText=="")
    		return "";
    	var descriptionSubstingSize = 40;
   		var text = titleText;
    	if (titleText.length > descriptionSubstingSize) {
   			text = titleText.substring(0,descriptionSubstingSize);
     		return dojo.string.substitute(this._messages.valueMore, [text]);
    	} else {
     		return text;
    	}	
    },
    //_onResourceViewerWidgetLoaded: function(resourceViewerWidget) {
    	//this._resourceViewerWidget = resourceViewerWidget;
    //},
    _handleAnchorClicked: function(event) {
        ResourceLinksHandler.handleAnchorClicked(event);
    },
    
    showEditCommentForm: function(formData) {
     	com.ibm.rdm.web.commenting.utils.openEditCommentForm(formData, this.getProjectURI(), this);
    },

    closeFilter: function() {
    	this._filterWidget.hide();
    	this._filterToggleNode.checked = false;
    	this._filterToggleNode.focus();
        this._setHeightOfScrollingPaneElement();
    },
    
    setRevisionDate: function(revisionDate) {
    	this.revisionDate = revisionDate;
    }
});

com.ibm.rdm.web.commenting.CommentingWidget.displayContext = {
    SIDEBAR_SECTION: 0,
    OTHER: 1
};



com.ibm.rdm.web.commenting.CommentingWidget.Ordering = {
	SORTBY_USER: "SORTBY_USER",
	SORTBY_DATE: "SORTBY_DATE",
	GROUPBY_NOGROUPING: "GROUPBY_NOGROUPING",
	GROUPBY_ARTIFACT: "GROUPBY_ARTIFACT",
	GROUPBY_DATE: "GROUPBY_DATE",
	GROUPBY_AUTHOR: "GROUPBY_AUTHOR",
	GROUPBY_REVIEW: "GROUPBY_REVIEW"
};
