/*
 * Copyright (c) 2007, 2010. IBM Corporation
 * All rights reserved.
 */
dojo.provide("com.ibm.rdm.web.commenting.Comments");
dojo.require("com.ibm.rdm.web.commenting.utils");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.util.queryutils");

var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var queryUtils = com.ibm.rdm.web.util.queryutils;
var commentUtils = com.ibm.rdm.web.commenting.utils;

dojo.declare("com.ibm.rdm.web.commenting.Comments", null, {
    /*
        params.jsAnnotations: Array of JS annotation objects
        params.orderedDocElementIds: Location ids in the document in the order of appearance
        // leaving room for AtomXML annotations, in case needed
    */
	
	_dictionary: null,
	_jsAnnotations: null,
	_resourceViewer: null,
	_commentingWidget: null,
	_revisionDate: null,
	
    constructor: function(params) {
		var timer = (new Date()).getTime();
        if (params && params.jsAnnotations) {
        	this._resourceViewer = params.resourceViewer;
            this._jsAnnotations = params.jsAnnotations;
            this.orderedDocElementIds = params.orderedDocElementIds;
            this.sortCommentsBy = params.sortCommentsBy;
            this.groupCommentsBy = params.groupCommentsBy;
            this._removeUnknownLocations();
            this._commentingWidget = params.commentingWidget;
            this._revisionDate = params.revisionDate;
            /* this._dictionary: <docElementId(String); annotations(Object[]> */
            this._createDictionary();
            
            // this._testMe();
            // console.log(this.toString());
        } 
        ErrorHandler.debug("created sorted comments collection in : " + ((new Date()).getTime() - timer) + "ms");
    },
    /**
        Returns a Comment with the provided uri-path (e.g. /jazz/resources/rdm/Web2/comment_2).
        null if not found.
    */
    getComment: function(/*String*/ path) {
        for (var i=0; i<this._jsAnnotations.length; i++) {
            if (this._jsAnnotations[i].linkSelfUri==path) {
                return this._jsAnnotations[i];
            }
        }
        return null;
        
    },
    
    /*
        Returns docLocationId(String) which has comments, that is after the 
        provided docLocationId. Use it with docLocationId which does not have
        a comment to find out where the new location pane should be inserted.
        Returns null, if no commented locs after this location exist.
          
    */
    getFirstCommentedLocationIdAfter: function(docLocationId/*String*/) {
        var self = this;
        var dict = self._dictionary;
        var found = false;
        for (var i=0; (self.orderedDocElementIds && i<self.orderedDocElementIds.length); i++) {
            var l = self.orderedDocElementIds[i];
            if (docLocationId == l) {
                found = true;
            }
            else  if (found) {
                var commented = dict.containsKey(l);
                if (commented) {
                    return l;
                }
            }
        };    
        return null;
    },
    /*
       Returns all Comments(Object[Comment]) 
    */
    getComments: function() {
        return this._jsAnnotations;    
    },
    /*
      Returns(int) no of comments for a given locationId(String), or total comment count for the doc.
    */
    getCommentCount: function(locationId) {
        if (!locationId) {
            return this._jsAnnotations.length;
         }
         else {
            var list = this._dictionary.item(locationId);
            return (list)?list.length:0;
         }
    
    },
    /*
        Returns(int[]) locationIds with 1 or more comments.  
        if isSorted is true, commented locations are returned in the order they appear.
    */
    getCommentedLocations: function(isSorted /*boolean*/) {
        if (!this._dictionary) {
            return new Array();
        }
            
        if (!isSorted) {
            return this._dictionary.getKeyList();
        }
        
        // There is a dictionary and must be sorted by order of appearance
        var dict = this._dictionary;
        var sortedCommLocs = dojo.every(this.orderedDocElementIds, function(l) {
             return (dict.item(l) && dict.item(l).length>0) 
        }); 
        return sortedCommLocs;
        
        
    },
    toString: function() {
        var str = "";
        var self = this;
        dojo.forEach(self._dictionary.getKeyList(), function(k) {
            var m = "docElementId: " + k + " [";
            var annList = self._dictionary.item(k);
            dojo.forEach(annList, function(ann) {
                m = m = ann.toString();
                dojo.forEach(ann.replies, function(r) {
                    m =  m + "(R)" + r.toString(); 
                });
            });
            m += "]";
            str += m + "\n"; 
        });
        return str;
    },
    _testMe: function() {
         ErrorHandler.debug(this.toString());
         ErrorHandler.debug("getCommentedLocations() test: " + this.getCommentedLocations());
         ErrorHandler.debug("getCommentCount(NOT EXIST) test: " + this.getCommentCount(-827737) );
         ErrorHandler.debug("getCommentCount(exist) test: " + this.getCommentCount(this.getCommentedLocations()[0]));
         ErrorHandler.debug("getCommentCount() test: " + this.getCommentCount() );
    },
    _removeUnknownLocations: function() {
    },
    _createDictionary: function() {
        this._dictionary = new dojox.collections.Dictionary();
        var dictionary = this._dictionary;
        var zelf = this;
        var normalizedAnnotations = new Array();
        
        function repliesComparator(comment1, comment2) {
           var a = parseInt(comment1.commentNumber);
           var b = parseInt(comment2.commentNumber);
           return (a<b)?-1:((a>b)?1:0);
        }

        // Date comparator - sort by DLM, most recently modified first - for root comments
        function descendingDateComparator(comment1, comment2) {
           var a = comment1.lastUpdatedTime;
           var b = comment2.lastUpdatedTime;
           return (a<b)?1:((a>b)?-1:0);
        }
              
        //create all groups when in group by date mode regardless of whether or not comments exist for 
        //the group
        if (this.groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_DATE) {
        	var now = new Date();
        	var todayDate = new Date(now.getFullYear(), now.getMonth(), now.getDate());
            this._dictionary.add(com.ibm.rdm.web.commenting.UPDATED_TODAY, new Array());
            this._dictionary.add(com.ibm.rdm.web.commenting.UPDATED_YESTERDAY, new Array());
            this._dictionary.add(com.ibm.rdm.web.commenting.UPDATED_PAST_WEEK, new Array());
            this._dictionary.add(com.ibm.rdm.web.commenting.UPDATED_PAST_MONTH, new Array());
            this._dictionary.add(com.ibm.rdm.web.commenting.UPDATED_EARLIER, new Array());
        }
        // gather comments
        var commentsDictionary = new dojox.collections.Dictionary();
        dojo.forEach(this._jsAnnotations, function(comment) {
        	var includeComment = true;
        	// Filter comments based on the specified revisionDate.  If the comment modified
        	// date is > the revision date, do not include the comment
        	if (zelf._revisionDate) {
        		var commentDate = new Date(comment.updated);
        		var compare = dojo.date.compare(commentDate, zelf._revisionDate);
        		includeComment = compare != 1;        			
        	} 
        	if (includeComment) {
        		commentsDictionary.add(comment.linkSelfUri, comment);
        		comment.replies = new Array();
        		if (comment.isRoot) {
        			normalizedAnnotations.push(comment);
        		}
        	}
        });
        
        // hookup replies
        dojo.forEach(commentsDictionary.getValueList(), function(comment) {
        	var parent = commentsDictionary.item(comment.parentComment);
        	// parent can be undefined if filtered out based on revision date
        	if (parent) {
        		parent.replies.push(comment);
        		comment.parent = parent;
        	}
        });
        
        // group
        dojo.forEach(normalizedAnnotations, function(rootComment) {
    		rootComment.setDateLastModified();  
        	var key;
        	if (zelf.groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_NOGROUPING) {
        		key = com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_NOGROUPING;
        	}
        	else if (zelf.groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_ARTIFACT) {
        		key = rootComment.locationId;
        	}
        	else if (zelf.groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_DATE) {
        		key = rootComment.getDLMRange();
        	}
        	else if (zelf.groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_AUTHOR) {
        		key = queryutils.getTrimmedUserName(rootComment.author);
        	}
        	else if (zelf.groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_REVIEW) {
        		if (rootComment.isReviewComment()) {
	    			key = com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL + "/" + rootComment.reviewId;
        		} else {
        			key = null;
        		}
        	}
        	zelf._addAnnotationToDictionary(rootComment, key);
        });
        
        // sort replies
        dojo.forEach(commentsDictionary.getValueList(), function(comment) {
        	comment.replies.sort(repliesComparator);
        });
        
        this._jsAnnotations = normalizedAnnotations;

		if (this.sortCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.SORTBY_USER) {
        	var groupKeys = dictionary.getKeyList();       
      		var onComplete = function(userMap) {
         		var userDictionary = new dojox.collections.Dictionary();
         		for (var userURL in userMap) {
         			userDictionary.add(decodeURIComponent(queryutils.getTrimmedUserName(userURL)), userMap[userURL].name);
        		}
        		// Author comparator for comments - for root comments - sort by DLM within an author, most 
        		// recently modified first
        		function userComparator(comment1, comment2) {
            		var a = userDictionary.item(comment1.author);
            		if (a == null || a == "") {
            			a = commentUtils.getUnknownUserName();
            		}
            		a = a.toLowerCase();
            		var b = userDictionary.item(comment2.author);
            		if (b == null || b == "") {
            			b = commentUtils.getUnknownUserName();
            		}
            		b = b.toLowerCase();
            		return (a<b)?-1:((a>b)?1:(comment1.lastUpdatedTime<comment2.lastUpdatedTime?1:-1));
        		}
        		dojo.forEach(groupKeys, function(key)  {
            		var commentsList = dictionary.item(key);
            		commentsList.sort(userComparator);
        		});
        		this._sortGroupPanes(groupKeys, dictionary);
     		};
     		// use getAllUsers since users that created comments may be removed
     		// from the project or archived
        	ProjectCache.getAllUsers().addCallback(this, onComplete);
		} else {
        	var groupKeys = dictionary.getKeyList();       
        	// sort root comments
        	dojo.forEach(groupKeys, function(key)  {
            	var commentsList = dictionary.item(key);
            	commentsList.sort(descendingDateComparator);
        	});
        	this._sortGroupPanes(groupKeys, dictionary);
		}        
    },
    
    _sortGroupPanes: function(groupKeys, dictionary) {
        // sort group panes
    	if (this.groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_REVIEW) {
    		this._reviewSortGroupPanes(groupKeys, dictionary);
    	} else if (this.groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_AUTHOR) {
    		this._authorSortGroupPanes(groupKeys, dictionary);
    	} else {
    		this._otherSortGroupPanes(groupKeys, dictionary);
    	}
    },
    
    _authorSortGroupPanes: function(groupKeys, dictionary) {
      	var onComplete = function(userMap) {
         	var userDictionary = new dojox.collections.Dictionary();
         	for (var userURL in userMap) {
         		userDictionary.add(decodeURIComponent(queryutils.getTrimmedUserName(userURL)), userMap[userURL].name);
        	}
        	function userGroupComparator(key1, key2) {
            	var a = userDictionary.item(key1);
            	if (a == null || a == "") {
            		a = commentUtils.getUnknownUserName();
            	}
            	a = a.toLowerCase();
            	var b = userDictionary.item(key2);
            	if (b == null || b == "") {
            		b = commentUtils.getUnknownUserName();
            	}
            	b = b.toLowerCase();
            	return (a<b)?-1:((a>b)?1:0);
        	}
			groupKeys.sort(userGroupComparator);
        	var sortedDictionary = new dojox.collections.Dictionary();
        	dojo.forEach(groupKeys, function(key)  {
				var dictionaryItem = dictionary.item(key);
        		sortedDictionary.add(key, dictionaryItem);
        	});
        	this._dictionary = sortedDictionary;
     	};
     	// use getAllUsers since users that created comments may be removed
     	// from the project or archived
        ProjectCache.getAllUsers().addCallback(this, onComplete);
    },
    
    _reviewSortGroupPanes: function(groupKeys, dictionary) {
    	var self = this;
		// We want the group of comments that are not part of a review to be last
		function reviewGroupComparator(groupId1, groupId2) {
			if (groupId1 == null) return 1;
			if (groupId2 == null) return -1;
			var aReview = self._commentingWidget.getReviewDescription(groupId1);
			var bReview = self._commentingWidget.getReviewDescription(groupId2);
			if (aReview == null) return 1;
			if (bReview == null) return -1;
			var aStart = dojo.date.stamp.fromISOString(aReview.reviewTimestamp);
			var bStart = dojo.date.stamp.fromISOString(bReview.reviewTimestamp);
			return (aStart<bStart)?1:((aStart>bStart)?-1:0);
		}
		groupKeys.sort(reviewGroupComparator);
		var sortedDictionary = new dojox.collections.Dictionary();
		dojo.forEach(groupKeys, function(key)  {
			var review = self._commentingWidget.getReviewDescription(key);
			if (review == null) {
				sortedDictionary.add(com.ibm.rdm.web.commenting.NON_REVIEW_COMMENTS, dictionary.item(key));
			} else {
				sortedDictionary.add(key, dictionary.item(key));
			}
		});
		this._dictionary = sortedDictionary;
    },
    
    _otherSortGroupPanes: function(groupKeys, dictionary) {
        function groupComparator(groupId1, groupId2) {
            var a = groupId1;
            var b = groupId2;
            return (a<b)?-1:((a>b)?1:0);
        }   
        if (this.groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_ARTIFACT) {
			groupKeys.sort(groupComparator);
        } else if (this.groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_DATE) {
    		groupKeys = new Array();;
     		groupKeys.push(com.ibm.rdm.web.commenting.UPDATED_TODAY);
    		groupKeys.push(com.ibm.rdm.web.commenting.UPDATED_YESTERDAY);
    		groupKeys.push(com.ibm.rdm.web.commenting.UPDATED_PAST_WEEK);
     		groupKeys.push(com.ibm.rdm.web.commenting.UPDATED_PAST_MONTH);
    		groupKeys.push(com.ibm.rdm.web.commenting.UPDATED_EARLIER);
        }       
        var sortedDictionary = new dojox.collections.Dictionary();
        dojo.forEach(groupKeys, function(key)  {
        	sortedDictionary.add(key, dictionary.item(key));
        });
 
        this._dictionary = sortedDictionary;
    },
    _addAnnotationToDictionary: function(comment, key) {
        var dictionary = this._dictionary;
        if (!dictionary.containsKey(key) ) {
            var list = new Array();
            dictionary.add(key, list);
        }
        dictionary.item(key).push(comment);
    }
});
