/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
dojo.provide("com.ibm.rdm.web.commenting.EditCommentForm");

dojo.require("com.ibm.rdm.web.commenting.utils");
dojo.require("com.ibm.rdm.web.common.UsersListWidget");
dojo.require("com.ibm.rdm.web.commenting.CommentingClient");

dojo.require("dijit.form.SimpleTextarea")
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.rdm.web.commenting", "EditCommentForm");

dojo.declare("com.ibm.rdm.web.commenting.EditCommentForm", [dijit._Widget, dijit._Templated], {
	formData: null,
    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "commenting/templates/EditCommentForm.html"),
    commentingWidget: null, // required for WidgetMixing 
    widgetsInTemplate: true, 
    projectURI: null,
    _priorityWidget: null,
    _priorityLabelNode: null,
    _directedToLabelNode: null,
    _commentRichTextDiv: null,
    _commentTextAreaDiv: null,
    _messages: null,
    TitleTruncateLength: 32,
    
    postCreate: function() {
        this.inherited("postCreate", arguments);

        this._priorityWidget = new com.ibm.rdm.web.common.IconSelectWidget({
	        widgetmode: "full", 
	        selected: com.ibm.rdm.web.commenting.PRIORITY_DEFAULT,
	        iconSelectOptions: com.ibm.rdm.web.commenting.utils.getFormPriorityOptions()
	        }
        );
        // accessibility, match up label with the widget
		this._priorityWidget._htmlSelectNode.setAttribute("id", this._priorityWidget.id);
		this._priorityLabelNode.setAttribute("for", this._priorityWidget._htmlSelectNode.id);
        this._prioritySelectNode.appendChild(this._priorityWidget.domNode);
        dojo.addClass(this._priorityLabelNode, "comment-priority-label");
        dojo.addClass(this._priorityWidget._htmlSelectNode, "comment-priority-select");
        this.initialize();
        dojo.connect(this.commentTitleArea.domNode, "onkeypress", this, "_onKeyPress");
    },
    postMixInProperties: function() {
 	   this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.commenting", "EditCommentForm");
    },
    initialize: function() {
        if (this._usersSelectWidget) {
        	this._usersSelectWidget.destroyRecursive();
        }
        this._usersSelectWidget = new com.ibm.rdm.web.common.UsersListWidget({
        	projectURI: this.projectURI
        });
        // accessibility, match up label with the widget
		this._usersSelectWidget._htmlListSelectNode.setAttribute("id", this._usersSelectWidget.id);
		this._directedToLabelNode.setAttribute("for", this._usersSelectWidget._htmlListSelectNode.id);
        this._directedToSelectNode.appendChild(this._usersSelectWidget.domNode);
        dojo.addClass(this._usersSelectWidget._htmlListSelectNode, "comment-user-list");
        
        //13654: Comment text persists with every new comment as if it was somehow cached
        //override crappy dojo implementation
        this._editCommentForm._onReset = function(e) {
        	//do nothing, let the form reset itself
        };
    },
    focusTextArea: function() {
    /*
        // TODO: Below does not work with FF and JAF/dojo1, but it may be a temp thing or an iframe issue. Header dissapears, and scrolls get mesed up.
        this.parentCommentContainer.domNode.scrollIntoView(true); */
        //this.commentTextArea.focus();
    },
    _onKeyPress: function(e) {
            var k=e.keyCode,ks=dojo.keys;
            if(e.ctrlKey){
                if(k===83||k===115){ //z
                    dojo.stopEvent(e);
                    this._save();
                    return;
                }
            }
            if (k==ks.ESCAPE) {
                dojo.stopEvent(e);
                this._cancel();
            }
            if (k==ks.ENTER) {
            	dojo.stopEvent(e);
            	this._save();
            	return;
            }
    },
    _onSave: function(e) {
        dojo.stopEvent(e);
        this._save();    
    },
    _setButtonsDisabled: function(isDisabled) {
	    this._okButtonNode.attr("disabled", isDisabled);
		this._cancelButtonNode.attr("disabled", isDisabled);
	},
    _save: function() {
		this._setButtonsDisabled(true);

	    var locationId = this.formData.resourceLocationId;
	    var text = this.commentTextArea.attr("value");
	    var title = this.commentTitleArea.attr("value");
	    var priority = this._priorityWidget.getSelectedValue();
	    var directedToList = this._usersSelectWidget.getSelectedValues();
	    if (text && !title) {
	    	if (text.length <= this.TitleTruncateLength) {
	    		title = text;
	    	} else {
	    		var truncatedTitle = com.ibm.rdm.web.commenting.utils.truncateRichText(text, this.TitleTruncateLength);
	    		title = dojo.string.substitute(this._messages.truncateTitle,[truncatedTitle]);
	    	}
	    }	    
	    var deferred = null;
	    if (this.formData.isUpdateComment) {
	    	this.showMessage(this._messages.savingComment);
	    	var updateProperties = null;
	    	if (this.formData.comment.hasRichText) {
	    		updateProperties = {
	               title: title,
	               priority: priority, 
	               directedToUsers: directedToList,
	               hasRichText: true
	        	};
	        } else {
	    		updateProperties = {
	    		   text: text,
	               title: title,
	               priority: priority, 
	               directedToUsers: directedToList,
	               hasRichText: false,
	               flattenedText: null
	        	};
	        }
	    	deferred = this.commentingWidget._controller.updateComment(this.formData.comment, updateProperties);
	    }
	    else {
	    	// Calculate immediate parent and root parent comment
	    	var parentComment = "";
	    	var rootComment = "";
	    	var reviewId = "";
	    	var resURI = "";
	    	var locId = "";
	    	if (this.formData.comment) {
	    		// reply comment
	       		parentComment = this.formData.comment.linkSelfUri;
	        	rootComment = this.formData.comment.rootComment;
	        	if (rootComment == "") rootComment = parentComment;
	        	// only set the reviewId if the reply is made in the context
	        	// of the review, so get the review id from the commenting widget
	        	if (this.commentingWidget) {
	    			reviewId = this.commentingWidget.getCommentReviewId();
	        	} 
	        	//reviewId = this.formData.comment.reviewId;
	        	resURI = this.formData.comment.resourceUri;
	        	locId = this.formData.comment.locationId;
	    	} else {
	    		reviewId = this.commentingWidget.getCommentReviewId();
	    		resURI = this.commentingWidget.resourceURI;
	        	locId = locationId;
	    	}
	    	this.showMessage(this._messages.creatingComment);
	    	var newComment = new com.ibm.rdm.web.commenting.Comment({
	           locationId: locId, 
	           reviewId: reviewId,
	           text: text,
	           title: title,
	           priority: priority, 
	           directedToUsers: directedToList,
	           parentComment: parentComment,
	           rootComment: rootComment,
	           resourceUri: resURI,
	           isResolved: false,
	           isRoot: parentComment == ""
	      	});
	    	if (this.commentingWidget) {
	    		deferred = this.commentingWidget._controller.saveNewComment(newComment);
	    	} else {
	    		deferred = com.ibm.rdm.web.commenting.CommentingClient.saveNewComment(newComment, this.projectURI);
	    	}
	    } 
	    deferred.addCallback(this, this.hide);
	    deferred.addErrback(this, function(error) {
	    	this.hide();
            com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorNumber(
            	7208, [], error
    		);
        });
    },
    _onCancel: function(event) {
        dojo.stopEvent(event);
        this._cancel();
    },    
    _cancel: function() {
        this.hide();        
    },
    hide: function() {
    	this._setButtonsDisabled(true);
    	this.clearMessage();
    	this.dialogWidget.hide();
    },
    show: function() {
    	this.reset();
    	//sometimes users loaded after the widget is rendered.. make another attempt in postcreate
    	if (this.formData.directedToUsers && this._usersSelectWidget) {
    		this._usersSelectWidget.selectValues(this.formData.directedToUsers);
    	}
    	if (this.formData.priority) {
    		this._priorityWidget.select(this.formData.priority);
    	} else {
    		this._priorityWidget.select(com.ibm.rdm.web.commenting.PRIORITY_DEFAULT);
    	}
    	if (this.formData.isUpdateComment && this.formData.comment.hasRichText) {
        	this._hideCommentTextArea();
        	if (this.formData.text) this._commentRichTextDiv.innerHTML = this.formData.comment.text;
    	} else {
        	this._hideCommentRichTextArea();
        	if (this.formData.text) this.commentTextArea.attr("value", this.formData.text);
    	}
		
    	if (this.formData.commentTitle) {
    		this.commentTitleArea.attr("value", this.formData.commentTitle);
     	}
    	
    	//dojo hack: tooltip does not go away because the dialog maintains focus on the original caller
    	this.dialogWidget._savedFocus = null;
    	
    	this.dialogWidget.show();
    	this._setButtonsDisabled(false);
    	this.focusTextArea();
    },
    _hideCommentTextArea: function() {
    	// just hiding the row doesn't work in IE, so hide/show individual fields
        if (dojo.isIE) {
        	this._commentRichNoEditLabel.innerHTML = " " + this._messages.commentRichText;
        	this._commentRichTextDiv.style.display="";
        	this._commentTextAreaDiv.style.display="none";
        } else {
        	this._commentRichNoEditLabel.style.display="";
        	this._commentRichTextRow.style.display="";
        	this._commentTextAreaRow.style.display="none";
        }
    },
    _hideCommentRichTextArea: function() {
    	// just hiding the row doesn't work in IE, so hide/show individual fields
        if (dojo.isIE) {
        	this._commentRichNoEditLabel.innerHTML = "";
        	this._commentRichTextDiv.style.display="none";
        	this._commentTextAreaDiv.style.display="";
        } else {
        	this._commentRichNoEditLabel.style.display="none";
        	this._commentRichTextRow.style.display="none";
        	this._commentTextAreaRow.style.display="";
        }
    },
    
    reset: function() {
    	this._editCommentForm.reset();
    	this._editCommentForm.domNode.reset();
    	this.clearMessage();
    },
    showMessage: function(msg) {
    	this._statusImageNode.style.display = "";
        this._statusMessageNode.appendChild(document.createTextNode(msg));
    },
    clearMessage: function() {
    	this._statusImageNode.style.display = "none";
    	if (this._statusMessageNode.firstChild != null)
    		this._statusMessageNode.removeChild(this._statusMessageNode.firstChild);
    }
});
