/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
dojo.provide("com.ibm.rdm.web.commenting.ReviewCommentForm");

dojo.require("com.ibm.rdm.web.commenting.utils");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.review.reviewTypes");

dojo.require("dojo.i18n");
dojo.require("dijit.form.Form");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.SimpleTextarea")

dojo.requireLocalization("com.ibm.rdm.web.commenting", "ReviewCommentForm");

dojo.declare("com.ibm.rdm.web.commenting.ReviewCommentForm", [dijit._Widget, dijit._Templated], {
	formData: null,
    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "commenting/templates/ReviewCommentForm.html"),
    widgetsInTemplate: true, 
    dialogWidget: null,
    commentText: null,
    artifactStatus: null,
    isMultipleArtifacts: null,
    _reviewCommentForm: null,
    _commentTextArea: null,
    _commentLabelNode: null,
    _okButtonNode: null,
    _cancelButtonNode: null,
    _messages: null,
    
    postCreate: function() {
        this.inherited("postCreate", arguments);
        this.initialize();
    },
    postMixInProperties: function() {
  	   this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.commenting", "ReviewCommentForm");
     },
    initialize: function() {        
        //13654: Comment text persists with every new comment as if it was somehow cached
        //override poor dojo implementation
    	this.commentText = "";
		switch (this.artifactStatus) {
		case com.ibm.rdm.web.review.reviewTypes.ARTIFACT_STATUS_TYPES.Approved:
			this._okButtonNode.setLabel(this._messages.approve);
			break;
		case com.ibm.rdm.web.review.reviewTypes.ARTIFACT_STATUS_TYPES.Disapproved:
			this._okButtonNode.setLabel(this._messages.disapprove);
			break;
		case com.ibm.rdm.web.review.reviewTypes.ARTIFACT_STATUS_TYPES.Abstained:
			this._okButtonNode.setLabel(this._messages.abstain);
			break;
		case com.ibm.rdm.web.review.reviewTypes.ARTIFACT_STATUS_TYPES.Reviewed:
			this._okButtonNode.setLabel(this._messages.reviewed);
			break;
		};
		var commentLabel = null;
		if (this.isMultipleArtifacts) {
			commentLabel = this._messages.commentLabelMultiple;
		} else {
			commentLabel = this._messages.commentLabel;
		}
 		if (dojo.isIE) {
 			this._commentLabelNode.innerText = commentLabel;
 		} else {
 			this._commentLabelNode.textContent = commentLabel;
 		}
    	this._initializeDialogTitle();
        this._reviewCommentForm._onReset = function(e) {
        	//do nothing, let the form reset itself
        };
    },
    _initializeDialogTitle: function() {
    	if (this.dialogWidget != null) {
        	var dialogTitle = null;
    		switch (this.artifactStatus) {
    		case com.ibm.rdm.web.review.reviewTypes.ARTIFACT_STATUS_TYPES.Approved:
    			if (this.isMultipleArtifacts) {
    				dialogTitle = this._messages.approveArtifacts;
    			} else {
    				dialogTitle = this._messages.approveArtifact;
    			}
    			break;
    		case com.ibm.rdm.web.review.reviewTypes.ARTIFACT_STATUS_TYPES.Disapproved:
    			if (this.isMultipleArtifacts) {
    				dialogTitle = this._messages.disapproveArtifacts;
    			} else {
    				dialogTitle = this._messages.disapproveArtifact;
    			}
    			break;
    		case com.ibm.rdm.web.review.reviewTypes.ARTIFACT_STATUS_TYPES.Abstained:
    			if (this.isMultipleArtifacts) {
    				dialogTitle = this._messages.abstainArtifacts;
    			} else {
    				dialogTitle = this._messages.abstainArtifact;
    			}
    			break;
    		case com.ibm.rdm.web.review.reviewTypes.ARTIFACT_STATUS_TYPES.Reviewed:
    			if (this.isMultipleArtifacts) {
    				dialogTitle = this._messages.reviewedArtifacts;
    			} else {
    				dialogTitle = this._messages.reviewedArtifact;
    			}
    			break;
    		};
    		dojo.isIE?(this.dialogWidget.titleNode.innerText = dialogTitle):(this.dialogWidget.titleNode.textContent = dialogTitle);
    	}
    },
    setDialogWidget: function(dialogWidget) {
    	this.dialogWidget = dialogWidget;
    	this._initializeDialogTitle();
    },
    _onKeyPress: function(e) {
            var k=e.keyCode,ks=dojo.keys;
            if (k==ks.ESCAPE) {
                dojo.stopEvent(e);
                this._cancel();
            }
            if (k==ks.ENTER) {
            	dojo.stopEvent(e);
            	this._ok();
            	return;
            }
    },
    _onOk: function(e) {
        dojo.stopEvent(e);
        this._ok();        
    },
    _onCancel: function(event) {
        dojo.stopEvent(event);
        this._cancel();
    },    
    _ok: function() {
    	this.commentText = this._commentTextArea.attr("value");
        this.hide();        
    },
    _cancel: function() {
        this.hide();        
    },
    hide: function() {
    	this.dialogWidget.hide();
    },
    show: function() {
    	this.reset();
    	// TODO (LPK) setting focus on ok button not working
    	this._okButtonNode.focus();
    	this.dialogWidget.show();
    },
    reset: function() {
    	this._reviewCommentForm.reset();
    	this._reviewCommentForm.domNode.reset();
    }
});
