/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
dojo.provide("com.ibm.rdm.web.commenting.utils");
dojo.require("jazz.util.date");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("jazz.ui.HoverPopup");
dojo.require("com.ibm.rdm.web.commenting.EditCommentForm");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.viewer.viewerutils");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.common.UserHover");

// ---------- COMMENTING CONSTANTS --------
com.ibm.rdm.web.commenting.LOCATION_PANE_ID_PREFIX = "rdm-ui-annotation-annotationpane-location-";
com.ibm.rdm.web.commenting.XMLNS_COMMENTING = "http://com.ibm.rdm.comments";
com.ibm.rdm.web.commenting.XMLNS_RECENT_COMMENTING = "http://com.ibm.rdm/recentevents/comments";
com.ibm.rdm.web.commenting.COMMENT_SLUG = "comment";
com.ibm.rdm.web.commenting.QUERY_URI = "projects/rdm/query";
com.ibm.rdm.web.commenting.MODIFIED_PROPERTY = "Last-Modified";
com.ibm.rdm.web.commenting.MODIFIED_BY_PROPERTY = "X-Last-Modified-User";
com.ibm.rdm.web.commenting.LOCATION_PROPERTY = "Location";
com.ibm.rdm.web.commenting.CONTENT_LOCATION_PROPERTY = "Content-Location";
com.ibm.rdm.web.commenting.DIRECTED_TO_TAG = "directedTo";

// Group key for non-review comments
com.ibm.rdm.web.commenting.NON_REVIEW_COMMENTS = -1;

// Values of priority fields
// NOTE - these values match the stored values for priority
com.ibm.rdm.web.commenting.PRIORITY_LOW = "Low";
com.ibm.rdm.web.commenting.PRIORITY_MEDIUM = "Medium";
com.ibm.rdm.web.commenting.PRIORITY_HIGH = "High";
com.ibm.rdm.web.commenting.PRIORITY_DEFAULT = com.ibm.rdm.web.commenting.PRIORITY_MEDIUM;
com.ibm.rdm.web.commenting.PRIORITY_UNSPECIFIED = "Unspecified";

// Date ranges
com.ibm.rdm.web.commenting.UPDATED_UNSPECIFIED = 0;
com.ibm.rdm.web.commenting.UPDATED_TODAY = 1;
com.ibm.rdm.web.commenting.UPDATED_YESTERDAY = 2;
com.ibm.rdm.web.commenting.UPDATED_PAST_WEEK = 3;
com.ibm.rdm.web.commenting.UPDATED_PAST_MONTH = 4;
com.ibm.rdm.web.commenting.UPDATED_EARLIER = 5;

// if directedTo users for a comment exceeds this value, abbreviate the directedTo names
com.ibm.rdm.web.commenting.ABBREVIATE_DIRECTEDTO_SIZE = 3;
com.ibm.rdm.web.commenting.MAX_DIRECTEDTO_SIZE = 7;

dojo.requireLocalization("com.ibm.rdm.web.commenting", "commenting");
var messages = dojo.i18n.getLocalization("com.ibm.rdm.web.commenting", "commenting");
var closeToolTips = new Array();
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var CommentingUtils = com.ibm.rdm.web.commenting.utils;
var COMMENT_TRUNCATION_LENGTH = 250;

// ---------- COMMENTING UTILS ----------
com.ibm.rdm.web.commenting.utils = {
	_UPDATED_OPTIONS: null,
	_PRIORITY_OPTIONS_FORM: null,
	_REQ_UPDATED_OPTIONS: null,
	_PRIORITY_OPTIONS_FILTER: null,
	
	getFilterPriorityOptions: function() {
		if (this._PRIORITY_OPTIONS_FILTER == null) {
			this._PRIORITY_OPTIONS_FILTER = {};
			this._PRIORITY_OPTIONS_FILTER[com.ibm.rdm.web.commenting.PRIORITY_UNSPECIFIED]= {text: messages.all, iconUri: null, optionClass: "filter-form-defaults"};
    		this._PRIORITY_OPTIONS_FILTER[com.ibm.rdm.web.commenting.PRIORITY_LOW] = {text: messages.low, iconUri: null};
    		this._PRIORITY_OPTIONS_FILTER[com.ibm.rdm.web.commenting.PRIORITY_MEDIUM] = {text: messages.medium, iconUri: null};
    		this._PRIORITY_OPTIONS_FILTER[com.ibm.rdm.web.commenting.PRIORITY_HIGH] =  {text: messages.high, iconUri: null};
		}
		return this._PRIORITY_OPTIONS_FILTER;
	},
	
	getFormPriorityOptions: function() {
		if (this._PRIORITY_OPTIONS_FORM == null) {
			this._PRIORITY_OPTIONS_FORM = {};
    		this._PRIORITY_OPTIONS_FORM[com.ibm.rdm.web.commenting.PRIORITY_LOW] = {text: messages.low, iconUri: null};
    		this._PRIORITY_OPTIONS_FORM[com.ibm.rdm.web.commenting.PRIORITY_MEDIUM] = {text: messages.medium, iconUri: null};
    		this._PRIORITY_OPTIONS_FORM[com.ibm.rdm.web.commenting.PRIORITY_HIGH] =  {text: messages.high, iconUri: null};
		}
		return this._PRIORITY_OPTIONS_FORM;
	},
	
	getUpdatedOptions: function() {
		if (this._UPDATED_OPTIONS == null) {
			this._UPDATED_OPTIONS = {};
			this._UPDATED_OPTIONS[com.ibm.rdm.web.commenting.UPDATED_UNSPECIFIED]= {text: messages.all, iconUri: null};
			this._UPDATED_OPTIONS[com.ibm.rdm.web.commenting.UPDATED_TODAY]= {text: messages.today, iconUri: null};
			this._UPDATED_OPTIONS[com.ibm.rdm.web.commenting.UPDATED_YESTERDAY]= {text: messages.yesterday, iconUri: null};
			this._UPDATED_OPTIONS[com.ibm.rdm.web.commenting.UPDATED_PAST_WEEK]= {text: messages.pastWeek, iconUri: null};
			this._UPDATED_OPTIONS[com.ibm.rdm.web.commenting.UPDATED_PAST_MONTH]= {text: messages.pastMonth, iconUri: null};
			this._UPDATED_OPTIONS[com.ibm.rdm.web.commenting.UPDATED_EARLIER]= {text: messages.earlier, iconUri: null};
		};
		return this._UPDATED_OPTIONS;
	},
	
	getReqUpdatedOptions: function() {
		if (this._REQ_UPDATED_OPTIONS == null) {
			this._REQ_UPDATED_OPTIONS = {};
			this._REQ_UPDATED_OPTIONS[com.ibm.rdm.web.commenting.UPDATED_UNSPECIFIED]= {text: messages.all, iconUri: null};
			this._REQ_UPDATED_OPTIONS[com.ibm.rdm.web.commenting.UPDATED_TODAY]= {text: messages.today, iconUri: null};
			this._REQ_UPDATED_OPTIONS[com.ibm.rdm.web.commenting.UPDATED_YESTERDAY]= {text: messages.yesterday, iconUri: null};
			this._REQ_UPDATED_OPTIONS[com.ibm.rdm.web.commenting.UPDATED_PAST_WEEK]= {text: messages.pastWeek, iconUri: null};
			this._REQ_UPDATED_OPTIONS[com.ibm.rdm.web.commenting.UPDATED_PAST_MONTH]= {text: messages.pastMonth, iconUri: null};
			this._REQ_UPDATED_OPTIONS[com.ibm.rdm.web.commenting.UPDATED_EARLIER]= {text: messages.earlier, iconUri: null};
		};
		return this._REQ_UPDATED_OPTIONS;
	},
	
	getUpdatedRangeTitle: function(type) {
		if (type == com.ibm.rdm.web.commenting.UPDATED_UNSPECIFIED) return messages.all;
		if (type == com.ibm.rdm.web.commenting.UPDATED_TODAY) return messages.today;
		if (type == com.ibm.rdm.web.commenting.UPDATED_YESTERDAY) return messages.yesterday;
		if (type == com.ibm.rdm.web.commenting.UPDATED_PAST_WEEK) return messages.pastWeek;
		if (type == com.ibm.rdm.web.commenting.UPDATED_PAST_MONTH) return messages.pastMonth;
		if (type == com.ibm.rdm.web.commenting.UPDATED_EARLIER) return messages.earlier;
		return messages.unknownRange;
	},
	
	getUnknownUserName: function() {
		// summary:
		//		Returns a name to use for unknown user.
		// description:
		//		Return a string to display for user references that are not found.
		return messages.unknownUser;
	},
	
    getWindowHeight: function() {
        var r;
        try {
            r = window.innerHeight;
        }
        catch (e) {
        	// attempt to handle the error by returning a legitimate value
        	return 100;
        }
        r = document.body.clientHeight;
        return r;
    },
    spanHtml: function(text, className) {
        if (!text)
            return "";
        else 
            return "<span class=\"" + className + "\"" + ">" + text + "</span>";
    },
    getPriorityImgPathOfComment: function(comment) {
        return this.getPriorityImgPath(comment.priority, comment.isResolved);
    },
    getPriorityImgPath: function(priority, isResolved) {
        var fname = "priority-low.gif";
        if (com.ibm.rdm.web.commenting.PRIORITY_MEDIUM == priority) {
            fname = "priority-medium.gif";
        } 
        else if (com.ibm.rdm.web.commenting.PRIORITY_HIGH == priority) {
            fname = "priority-high.gif";
        }
        
        if (isResolved) {
            fname = "res-" + fname;    
        }
        return dojo.moduleUrl("com.ibm.rdm.web.commenting", "images/icons/obj16/" + fname);
        
    },
    fixContentPane: function(contentPane) {
        contentPane._createSubWidgets = dojo.hitch(contentPane, function(){
            return;
        });
    },
    truncateRichText: function(htmlString,len) {
    	// htmlString: String
    	//			     The html string before truncated
    	// len: int
    	//		  The length of the string after truncated	
    	var ind = 0; //index from the front 
    	var l = len; //index from the end
    	var queue = []; // the front part of the result
    	var stack = []; // the end part of the result
    	while (ind < htmlString.length){
    		if (htmlString.charAt(ind) == '<'){
    			if (htmlString.charAt(ind + 1) != '/'){
    				// it is an open tag
    				var p = htmlString.indexOf(">", ind);
    				var openTag = htmlString.substring(ind,p+1);
    				ind = p;
    				var tagName = openTag.match(/\w+/);
    				queue.push(openTag);
    				if (htmlString.charAt(p-1) != '/'){
    					// handle tags such as <br/>
    					stack.push("</" + tagName+">");
    				}
    			}else{
    				// it is a close tag
    				var p = htmlString.indexOf(">", ind);
    				var closeTag = htmlString.substring(ind, p+1);
    				ind = p;
    				var topEle = stack.pop();
    				if (topEle == closeTag){
    					queue.push(topEle);
    				}
    			}
    		}else{
    			queue.push(htmlString.charAt(ind));
    			l--;
    			if (l==0) break;
    		}
    		ind++;
    	}
    	var resultHTML = queue.join("") + stack.join("");
    	return resultHTML;
    },
    
    createCommentByline: function(widget, appendTo, comment, showUserHovers) {
    	var userHovers = [];
    	var userToStyle = "com-ibm-rdm-comment-entry-username";
    	var userFromStyle = "com-ibm-rdm-comment-entry-username";
    	var reviewIconStyle = "com-ibm-rdm-comment-entry-review-icon";
    	var timeStyle = "com-ibm-rdm-comment-entry-time-text";
    	var toTextStyle = "com-ibm-rdm-comment-entry-to-text";
    	var toElementStyle = "com-ibm-rdm-comment-entry-to-field";
		
        var fromNode = document.createElement("span");
        var self = this;
        
   		function getUserInfo(userId, userInfos) {
    		for (var userURL in userInfos) {
        		if (userInfos[userURL].userId === userId) {
        			return userInfos[userURL];
        		}
        	}
        	return null;
        };
        
      	var handleUserInfo = function(mapUserIdToUserInfo) {
        	var userInfo = getUserInfo(comment.author, mapUserIdToUserInfo);
        	var author = (userInfo ? userInfo.name : getUnknownUserName());
        	var authorEmail = (userInfo ? userInfo.email : "");
        	var authorNode = com.ibm.rdm.web.util.domutils.createMailTo(authorEmail,
        		comment.title, author, comment.resourceUri, userFromStyle);
        	fromNode.appendChild(authorNode);
        	
        	if (showUserHovers) {
        		var authorHover = new com.ibm.rdm.web.common.UserHover({projectURI : comment.projectURI}, null);
        		authorHover.createHoverPopup(authorNode, userInfo);
        		userHovers.push(authorHover);
        	}
        	
       		if (comment.isReviewComment()) {
        		var reviewIconURI = com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REVIEW);
        		var commentReviewIcon = document.createElement("img");
        		commentReviewIcon.setAttribute("src", dojo.moduleUrl("com.ibm.rdm.web", "commenting/images/icons/obj16/review-comment.gif"));
    			var reviewURI = com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL + "/" + comment.reviewId;
				widget.connect(commentReviewIcon, "onclick", function(event) {
                    dojo.stopEvent(event);
					ResourceLinksHandler.showInternalResource(reviewURI, false);
				});
				// TODO (LPK) - tooltip like rich client shows
        		//commentReviewIcon.setAttribute("title", "Review comment.  Use click to open the review.");
        		// TODO (LPK) - retrieve review via deferred, put review name in alt tag
    			//var review = this.commentingWidget.getReview(this.annotation.reviewId);
        		//var msg = null;
    			//if (review == null) {
            	//	msg = dojo.string.substitute(this._messages["reviewComment"], [this._messages["reviewUnknown"]]);
    			//} else {
            	//	msg = dojo.string.substitute(this._messages["reviewComment"], [review.getReviewName()]);
    			//}
        		//commentReviewIcon.setAttribute("alt", msg);
        		dojo.addClass(commentReviewIcon, reviewIconStyle);
        		dojo.place(commentReviewIcon, authorNode, "before");
        	}
        	
        	var abbreviateUserNames = comment.directedToUsers.length > com.ibm.rdm.web.commenting.ABBREVIATE_DIRECTEDTO_SIZE;
        	var length = Math.min(comment.directedToUsers.length, com.ibm.rdm.web.commenting.MAX_DIRECTEDTO_SIZE);
	        for (var i = 0; i<length; i++) {
	        	var directedTo = comment.directedToUsers[i];
	        	var directedToUserInfo = getUserInfo(directedTo, mapUserIdToUserInfo);
	        	// skip users who are unidentified
	            var directedToUser = (directedToUserInfo ? directedToUserInfo.name : "");
	            if (directedToUser != "") {
	        		if (abbreviateUserNames) {
	        			directedToUser = this._abbreviatedUserName(directedToUser);
	        		}
	            	var directedToUserEmail = (directedToUserInfo ? directedToUserInfo.email : "");
	            	var directedToNode = com.ibm.rdm.web.util.domutils.createMailTo(directedToUserEmail,
	        			comment.title, directedToUser, comment.resourceUri, userToStyle);
	        		
	        		if (showUserHovers) {
	        			var directedToHover = new com.ibm.rdm.web.common.UserHover({projectURI : comment.projectURI}, null);
        				directedToHover.createHoverPopup(directedToNode, directedToUserInfo);
        				userHovers.push(directedToHover);
        			}
	        			
	        		if (i==0) {
	            		var toTextElement = com.ibm.rdm.web.search.domutils.createSpannedTextNode(toTextStyle, " " + messages["to"] + " ");
						dojo.addClass(toTextElement, toElementStyle);	                	
	            		fromNode.appendChild(toTextElement);
	        		} else {
	            		var commaElement = com.ibm.rdm.web.search.domutils.createSpannedTextNode(toTextStyle, ", ");
						dojo.addClass(commaElement, toElementStyle);	                	
	            		fromNode.appendChild(commaElement);
	        		}
	            	fromNode.appendChild(directedToNode);
	            }
    	    }
    	    
        	if (comment.directedToUsers.length > com.ibm.rdm.web.commenting.MAX_DIRECTEDTO_SIZE) {
	            var moreElement = com.ibm.rdm.web.search.domutils.createSpannedTextNode(toTextStyle, "...");
				dojo.addClass(moreElement, toElementStyle);	                	
	            fromNode.appendChild(moreElement);
        	}
    	    
    	    var timeStr =  jazz.util.date.formatDif(new Date(comment.updated), false, true);
	        var timeElement = com.ibm.rdm.web.search.domutils.createSpannedTextNode(timeStyle, " " + timeStr);
	        fromNode.appendChild(timeElement);
	        return mapUserIdToUserInfo;
        };
        
        ProjectCache.getAllUsers().addCallback(this, handleUserInfo);
              
        appendTo.appendChild(fromNode);
        return userHovers;          
        
    },
    
    _abbreviatedUserName: function(userName) {
    	if (userName.length == 0) return userName;
    	var substrings = new Array();
    	var pos = userName.indexOf(" ");
    	var name = userName;
    	if (pos != -1) {
    		while (pos != -1) {
    			var subName = name.substring(0, pos);
    			if (subName != "") {
    				substrings.push(subName);
    			}
    			name = name.slice(pos + 1);
    			pos = name.indexOf(" ");
    		}
    	}
    	substrings.push(name);
    	var abbreviatedName = null;
    	if (substrings.length == 1) {
    		// take first two characters of the string
    		abbreviatedName = substrings[0].substring(0,2);
    	} else {
    		// take first character of each substring
    		abbreviatedName = "";
    		for (var i=0; i<substrings.length; i++) {
    			abbreviatedName = abbreviatedName + substrings[i].charAt(0);
    		}
    	}
    	return abbreviatedName;
    },
    
    getPriorityImageSrc: function(priority, isResolved) {
    	var imagePath = null;
      	if (priority) {
          	switch (priority) {
            	case "High" :
                	imagePath = (isResolved ?
                    	"commenting/images/icons/obj16/res-priority-high.gif" :
                     	"commenting/images/icons/obj16/priority-high-pd.gif"
                	)
                	break;
            	case "Medium" : 
                 	imagePath = (isResolved ?
                     	"commenting/images/icons/obj16/res-priority-medium.gif" :
                       	"commenting/images/icons/obj16/priority-medium.gif"
                 	);
               		break;
            	case "Low" :
                	imagePath = (isResolved ?
                     	"commenting/images/icons/obj16/res-priority-low.gif" :
                     	"commenting/images/icons/obj16/priority-low.gif"
                   	);
           			break;
           	}
     	}
		return dojo.moduleUrl("com.ibm.rdm.web", imagePath);
	},
	
    _connectCommentLinksHandler: function(widget, commentTextNode, popupWidget) {
    	var self = this;
    	var links = dojo.query("a", commentTextNode);
        for (var i = 0; i < links.length; i++) {
        	var link = links[i];
            var linkHref = link.attributes.getNamedItem("href").value;
            if (ResourceLinksHandler.isLinkInternal(linkHref)) {
				var href = com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES + '/' + com.ibm.rdm.web.util.queryutils.getLastPathSegment(linkHref);
				link.setAttribute("href", href);
			}
			dojo.connect(link, "onclick", widget, function(event) {
	    		self._handleLinkInCommentClicked(event, popupWidget);
	    	});
        }
    },
	
	setCommentText: function(widget, commentTextNode, comment) {
        var truncatedText = comment.text;
        var isTruncated = false;
        if (comment.hasRichText) {
        	if (comment.flattenedText.length > COMMENT_TRUNCATION_LENGTH) {
        		truncatedText = this.truncateRichText(comment.text, COMMENT_TRUNCATION_LENGTH);
        		truncatedText = dojo.string.substitute(messages["valueMore"], [truncatedText]);
        		isTruncated = true;
        	}
        } else {        	
        	if (comment.text.length > COMMENT_TRUNCATION_LENGTH) {
    			truncatedText = comment.text.substring(0, COMMENT_TRUNCATION_LENGTH);
        		truncatedText = dojo.string.substitute(messages["valueMore"], [truncatedText]);
    			isTruncated = true;
        	}
        }
        if (!comment.hasRichText) {
        	truncatedText = domutils.getXMLEncodedText(truncatedText);
        	truncatedText = truncatedText.replace(/\n/g, "<br/>");
        }
    	commentTextNode.innerHTML = truncatedText;
		this._connectCommentLinksHandler(widget, commentTextNode);
        if (comment.isResolved) {
        	dojo.addClass(commentTextNode, "com-ibm-rdm-resolved-comment-text");
        }
		return isTruncated;
	},
	
	_hookupReplyToLink: function(widget, replyNode, comment, commentingWidget) {
		var self = this;
    	var createFunction = function createPopupContents(popupWidget) {
           return self._getReplyTooltipContent(widget, comment, commentingWidget, popupWidget);
        };
    	
    	return new jazz.ui.HoverPopup({
			around: [replyNode],
			createContents: createFunction
        }, null);
	},
	
	hookupCommentMoreLink: function(widget, moreNode, comment, commentingWidget) {
		var self = this;
    	var createFunction = function createPopupContents(popupWidget) {
           return self._getCommentMoreTooltipContent(widget, comment, commentingWidget, popupWidget);
        };
    	
    	return new jazz.ui.HoverPopup({
			around: [moreNode],
			createContents: createFunction
        }, null);
	},
	
	hookupRecentCommentMoreLink: function(widget, moreNode, comment, artifactName, artifactType) {
		var self = this;
    	var createFunction = function createPopupContents(popupWidget) {
           return self._getRecentCommentMoreTooltipContent(widget, comment, artifactName, artifactType, popupWidget);
        };
    	
    	return new jazz.ui.HoverPopup({
			around: [moreNode],
			createContents: createFunction
        }, null);
	},
	
	_getCommentMoreTooltipContent: function(widget, comment, commentingWidget, popupWidget){
		var popup = document.createElement("span");
    	var popupDiv = document.createElement("div");
    	dojo.addClass(popupDiv, "com-ibm-rdm-comment-flyout");
    	popup.appendChild(popupDiv);
    	
    	// links section at top of tooltip
      	var linksDiv = this._createPopupLinksDiv(popupDiv);
        
        // reply to link
    	this._createPopupReplyToLink(widget, linksDiv, comment, popupWidget, commentingWidget);
    	
    	// comment byline
      	com.ibm.rdm.web.commenting.utils.createCommentByline(widget, popupDiv, comment, false);
		dojo.addClass(popupDiv, "info");
      	
      	// comment title
        this._createPopupTitleDiv(comment, popupDiv);
        
        // comment text
        this._createPopupTextDiv(widget, comment, popupDiv, popupWidget);
      	
      	return popup;                 
	},
	
	_getRecentCommentMoreTooltipContent: function(widget, comment, artifactName, artifactType, popupWidget){
		var self = this;
		var popup = document.createElement("span");
    	var popupDiv = document.createElement("div");
    	dojo.addClass(popupDiv, "com-ibm-rdm-comment-flyout");
    	popup.appendChild(popupDiv);
    		
    	// links section at top of tooltip
      	var linksDiv = this._createPopupLinksDiv(popupDiv);
      	
      	// open artifact link
	    var link = ResourceLinksHandler.createResourceLink(comment.resourceUri, messages.openArtifact, false);
       	dojo.connect(link, "onclick", widget, function(event) {
			popupWidget.close();
            self._handleCommentArtifactLinkClicked(event, comment.linkSelfUri)});            
        linksDiv.appendChild(link);
      	linksDiv.appendChild(document.createTextNode(" | "));
        
        // reply to link
    	this._createPopupReplyToLink(widget, linksDiv, comment, popupWidget);
    		
    	// comment byline
      	com.ibm.rdm.web.commenting.utils.createCommentByline(widget, popupDiv, comment, false);
		dojo.addClass(popupDiv, "info");
      	
      	// comment title
        this._createPopupTitleDiv(comment, popupDiv);
           
        // comment text
        this._createPopupTextDiv(widget, comment, popupDiv, popupWidget);
      	     	
       	// artifact link at bottom of tooltip
      	var divArtifact = document.createElement("div");
      	this.createArtifactNode(widget, divArtifact, artifactName, artifactType, comment, false, popupWidget);
     	dojo.addClass(divArtifact, "artifactInfo");
       	popupDiv.appendChild(divArtifact);
     	
      	return popup;                 
	},
	
	_getReplyTooltipContent: function(widget, comment, commentingWidget, popupWidget){
		var self = this;
		var popup = document.createElement("span");
    	var popupDiv = document.createElement("div");
    	dojo.addClass(popupDiv, "com-ibm-rdm-comment-flyout");
    	popup.appendChild(popupDiv);
    		
    	// links section at top of tooltip
      	var linksDiv = this._createPopupLinksDiv(popupDiv);
        
        // go to comment link
		var goToLink = document.createElement("A");
        goToLink.appendChild(document.createTextNode(messages.goToComment));
       	dojo.connect(goToLink, "onclick", widget, function(event) {
			popupWidget.close();
            self._goToCommentClicked(event, comment, commentingWidget)});            
        linksDiv.appendChild(goToLink);
    		
    	// comment byline
      	com.ibm.rdm.web.commenting.utils.createCommentByline(widget, popupDiv, comment, false);
		dojo.addClass(popupDiv, "info");
      	
      	// comment title
        this._createPopupTitleDiv(comment, popupDiv);
          
        // comment text  
        this._createPopupTextDiv(widget, comment, popupDiv, popupWidget);
 
       	return popup;                 
	},
	
	_createPopupLinksDiv: function(popupDiv) {
    	// links section at top of tooltip
      	var linksDiv = document.createElement("div");
      	dojo.addClass(linksDiv, "linksDiv");
       	popupDiv.appendChild(linksDiv);
      	return linksDiv;
    },
     
    _createPopupTextDiv: function(widget, comment, popupDiv, popupWidget) {
      	var divText = document.createElement("div");
       	var contentPaneText = new dijit.layout.ContentPane({style: "height: 200px; width: 350px;"}, divText);
     	dojo.addClass(contentPaneText, "content");            
     	dojo.addClass(divText, "text");
        if (comment.isResolved) {
        	dojo.addClass(divText, "resolvedText");
        }
     	var text = comment.text;     	
        if (!comment.hasRichText) {
        	text = text.replace(/\n/g, "<br/>");
        }
      	divText.innerHTML = "<span>" + text + "</span>";
      	this._connectCommentLinksHandler(widget, divText, popupWidget);
       	popupDiv.appendChild(contentPaneText.domNode);
    },
    
    _createPopupTitleDiv: function(comment, popupDiv) {
      	var divTitle = document.createElement("div");
        if (comment.isResolved) {
        	dojo.addClass(divTitle, "resolvedTitle");
        } else if (comment.priority == com.ibm.rdm.web.commenting.PRIORITY_HIGH) {
     		dojo.addClass(divTitle, "title-high-priority");
      	} else {
     		dojo.addClass(divTitle, "title");
     	}
      	divTitle.appendChild(document.createTextNode(comment.title));
      	popupDiv.appendChild(divTitle);
    },
    
    _createPopupReplyToLink: function(widget, linksDiv, comment, popupWidget, commentingWidget) {
		var self = this;
		var replyLink = document.createElement("A");
        replyLink.appendChild(document.createTextNode(messages.replyToComment));
       	dojo.connect(replyLink, "onclick", widget, function(event) {
			popupWidget.close();
            self._replyToCommentClicked(event, comment, commentingWidget)});            
        linksDiv.appendChild(replyLink);
    },
    
    _replyToCommentClicked: function(event, comment, commentingWidget) {
		if (commentingWidget) {
			commentingWidget.replyToButtonClick(comment);
		} else {
        	this.replyToComment(comment);
        }
    },
	
    _goToCommentClicked: function(event, comment, commentingWidget) {
    	commentingWidget._selectCommentPane(comment.linkSelfUri, true);
    },
	
	replyToComment: function(comment) {
		var self = this;
        var title = dojo.string.substitute(messages.re, [comment.title]);
       	var formData = {
        	formTitle: messages.replyToComment,
        	resourceLocationId: comment.locationId,
        	commentTitle: title,
        	directedToUsers: new Array(com.ibm.rdm.web.util.queryutils.getTrimmedUserName(comment.author)),
        	comment: comment
        };
        // projectURI needs to be set for comment form in order to populate users
        var onResourceDescriptionLoaded = function(response) {
    		self.openEditCommentForm(formData, response.projectURI);
 			return response;
		};
        var onResourceDescriptionError = (this, function(error) {
            com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorNumber(
            	7211, [comment.resourceUri], error
    		);
        });
        var deferred = com.ibm.rdm.web.common.ProjectCache.getResourceDescriptionMultiFetch(comment.resourceUri);
		deferred.addCallback(onResourceDescriptionLoaded);
        deferred.addErrback(onResourceDescriptionError);
	},
	
    createArtifactInReviewNode: function(widget, appendTo, artifactName, artifactType, artifactRevision, comment, createRichText, popupWidget) {
        // artifact image
        var artifactImg = this._createArtifactImage(artifactType);
	    appendTo.appendChild(artifactImg);
	    	
        // artifact name
        if (artifactType == null) {
        	var node = com.ibm.rdm.web.util.domutils.createSpannedTextNode("com-ibm-rdm-comment-artifact-not-exist", messages.artifactNotExist);
            appendTo.appendChild(node);
        } else {
			var href = UriHelper.getResourceInReviewContextJafLink(
				comment.resourceUri,
				artifactRevision,
				comment.reviewId
			);
			var resourceHyperlink = ResourceLinksHandler.createResourceLink(href, artifactName, true);
        	dojo.connect(resourceHyperlink, "onclick", widget, function(event) {
        		if (popupWidget) {
					popupWidget.close();
				}
				dojo.stopEvent(event);
				ResourceLinksHandler.showArtifactInReview(href, comment.linkSelfUri);
			});
        	appendTo.appendChild(resourceHyperlink);
        	dojo.addClass(resourceHyperlink, "com-ibm-rdm-comment-artifact-hyperlink");
        }
    },
	
    createArtifactNode: function(widget, appendTo, artifactName, artifactType, comment, createRichText, popupWidget) {
		var self = this;
        // artifact image
        var artifactImg = this._createArtifactImage(artifactType);
	    appendTo.appendChild(artifactImg);
	    	
        // artifact name
        if (artifactType == null) {
        	var node = com.ibm.rdm.web.util.domutils.createSpannedTextNode("com-ibm-rdm-comment-artifact-not-exist", messages.artifactNotExist);
            appendTo.appendChild(node);
        } else {
	    	var resourceHyperlink = ResourceLinksHandler.createResourceLink(comment.resourceUri, artifactName, createRichText);
        	dojo.connect(resourceHyperlink, "onclick", widget, function(event) {
        		if (popupWidget) popupWidget.close();
            	self._handleCommentArtifactLinkClicked(event, comment.linkSelfUri)});            
        	appendTo.appendChild(resourceHyperlink);
        	dojo.addClass(resourceHyperlink, "com-ibm-rdm-comment-artifact-hyperlink");
        }
    },
    
    _createArtifactImage: function(artifactType) {
       var artifactImg = document.createElement("img");
		var imgURI = null;
        if (artifactType == null) {
			imgURI = "com.ibm.rdm.web/common/images/resources/unknown_obj.gif";
        } else {
			imgURI = com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(artifactType);
			if (imgURI == null) {
				imgURI = "com.ibm.rdm.web/common/images/resources/unknown_obj.gif";
			}
        }
        artifactImg.setAttribute("src", imgURI);
       	dojo.addClass(artifactImg, "com-ibm-rdm-comment-artifact-image");
       	return artifactImg;
    },
    
    
    createTitleNode: function(widget, appendTo, comment, replyComment, commentingWidget) {
        // priority image
        if (comment.priority=="High"){            
            var priorityImg = document.createElement("img");
            var icon = com.ibm.rdm.web.commenting.utils.getPriorityImageSrc(comment.priority, comment.isResolved);
            priorityImg.setAttribute("src", icon);
        	dojo.addClass(priorityImg, "com-ibm-rdm-comment-priority-imag");
    		appendTo.appendChild(priorityImg);
    	}
    		
        // title
		var title = comment.title;
        if (!comment.title || comment.title.length == 0) {
            title = messages.noTitle;
        }
        var commentTitleNode = com.ibm.rdm.web.search.domutils.createSpannedTextNode("com-ibm-rdm-comment-title", title);
		if (comment.priority=="High") {
			dojo.addClass(commentTitleNode, "com-ibm-rdm-comment-title-high-priority");
		} 
		appendTo.appendChild(commentTitleNode);
        if (comment.isResolved) {
        	dojo.addClass(commentTitleNode, "com-ibm-rdm-resolved-comment-text");
        }
	       
	    if (replyComment) {
        	var message = dojo.string.substitute(messages.replyCommentLink, [replyComment.commentNumber]);
 	    	var reElement = com.ibm.rdm.web.search.domutils.createSpannedTextNode("rdm-annotation-commentReplyFor", " " + message);
        	commentTitleNode.appendChild(reElement);
        	var replyHover = this._hookupReplyToLink(widget, reElement, replyComment, commentingWidget);
        	return replyHover;
	    }
	    
		return null;
	},
		
	_handleCommentArtifactLinkClicked: function(event, commentURI) {
        ResourceLinksHandler.handleCommentArtifactAnchorClicked(event, commentURI);
    },
    
    _handleLinkInCommentClicked: function(event, popupWidget) {
		dojo.stopEvent(event);
    	if (popupWidget) popupWidget.close();
		var anchor = event.currentTarget;
		var href = anchor.attributes.getNamedItem("href").value;
		com.ibm.rdm.web.viewer.viewerutils.openResourceUsingAppropriateViewer(href);
    },
    
    openEditCommentForm: function(formData, projectURI, commentingWidget) {
        var self = this;
		var dialogWidget = dijit.byId("com.ibm.rdm.web.commenting.NewCommentDialog");
		var form = dijit.byId("com.ibm.rdm.web.commenting.EditCommentForm");
		if (dialogWidget == null || form == null) {
			
			var form = new com.ibm.rdm.web.commenting.EditCommentForm(
                {
                	id: "com.ibm.rdm.web.commenting.EditCommentForm",
                	commentingWidget: commentingWidget,
                    projectURI: projectURI
                }
            );   
			
			dialogWidget = new dijit.Dialog(
                {
                    id: "com.ibm.rdm.web.commenting.NewCommentDialog",
                    duration: 100
                },
                form.domNode
			);
		} else {
			form.initialize();
		}
		form.commentingWidget = commentingWidget;
		form.dialogWidget = dialogWidget;
		form.formData = formData;
		dojo.isIE?(dialogWidget.titleNode.innerText = formData.formTitle):(dialogWidget.titleNode.textContent = formData.formTitle);
        form.show();
    }
	

};
