/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
dojo.provide("com.ibm.rdm.web.common.CollapseableTitleSection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.common", "common");


dojo.declare(
    "com.ibm.rdm.web.common.CollapseableTitleSection",
    [dijit._Widget, dijit._Templated],
{
    // summary:
    //    Provides a title section that highlights when moused over and
    //    expands/collpses its content when clicked.

    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "common/templates/CollapseableTitleSection.html"),
    widgetsInTemplate: false,

    ///////////////////////////////////////////////////////////////////////////////////
    // Public fields
    sectionTitle: null,
    
    ///////////////////////////////////////////////////////////////////////////////////
    // Private fields
    _titleDiv: null,
    _sectionTitleSpan: null,
    _domNodes: null,
    _isCollapsed: null,
    _messages: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // Attach points
    _contentDiv: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // API

    postCreate: function() {
        this.connect(this._titleDiv, "onmouseover", this._onTitleMouseOver);
        this.connect(this._titleDiv, "onmouseout", this._onTitleMouseOut);
        this.connect(this._titleDiv, "onclick", this._onTitleMouseClick);
        this.connect(this._titleDiv, "onkeypress", this._onKeyPress); 
        this._isCollapsed = false;
    	if (this._isCollapsed) {
    		dojo.removeClass(this._titleDiv, "expanded");
    		dojo.addClass(this._titleDiv, "collapsed");
    	} else {
    		dojo.removeClass(this._titleDiv, "collapsed");
    		dojo.addClass(this._titleDiv, "expanded");
    	};
    	this._setExpandedStateIcon();
    },
    
	postMixInProperties: function(){
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
	},

    startup: function() {
    	this.inherited(arguments);
    },
    
    destroy: function() {
    	this.inherited(arguments);
    },

    setTitle: function(title) {
    	this._sectionTitleSpan.innerHTML = "";
    	this._sectionTitleSpan.appendChild(document.createTextNode(title));
    },
    
    setContent: function(domNode) {
    	// summary:
    	//    Can pass pointers to external domNodes that it hides and collapses
    	//		typically used when wanting to collapse and expand rows in table
    	//		but keep the rows aligned with the header
    	//
    	//    Or can pass domNode that it appends to its content div
    	if (dojo.isArray(domNode)) {
    		this._domNodes = domNode;
    	} else {
    		this._contentDiv.appendChild(domNode);
    	}
    },
    
    setHeaderImage: function(imgURI) {
    	this._headerImg.src = imgURI;
    	this._headerImg.style.display = "";
    },
    
    collapse: function() {
    	this._collapse();
    },
    
    expand: function() {
    	this._expand();
    },
    

    ///////////////////////////////////////////////////////////////////////////////////
    // Helpers

    _onTitleMouseClick: function(event) {
        if (this._isCollapsed) {
            this._expand();
        } else {
            this._collapse();
        }
    },

    _onKeyPress: function(event) {
    	if(event.charCode == dojo.keys.SPACE) { 
    		this._onTitleMouseClick(event); 
    	}
    },
    
	_setExpandedStateIcon: function() {
		this._expandStateImage.setAttribute("src", this._getExpandStateImageSrc(!this._isCollapsed));
		this._expandStateImage.className = (this._isCollapsed ? "expand-image" : "collapse-image");
		// accessibility, provide alt and title text for images
		if (this._isCollapsed) {
			this._expandStateImage.setAttribute("title", this._messages["expandTooltip"]);
			this._expandStateImage.setAttribute("alt", this._messages["expandAlt"]);
		} else {
			this._expandStateImage.setAttribute("title", this._messages["collapseTooltip"]);
			this._expandStateImage.setAttribute("alt", this._messages["collapseAlt"]);
		}
	},
	
	_getExpandStateImageSrc: function(isExpanded) {
		var bundle = "com.ibm.rdm.web";
		if (isExpanded) {
			return dojo.moduleUrl(bundle, "common/images/twistie-open.gif");
		} else { // closed
			return dojo.moduleUrl(bundle, "common/images/twistie-closed.gif");
		}
	},

// TODO Resolve animation overwriting issue.  For now we're not using animations.

    _collapse: function() {
//         var currentHeight = this._contentDiv.clientHeight;
//         var targetHeight = 0;
//         this._animateResizingContentHeight(currentHeight, targetHeight);
    	if (this._domNodes) {
    		dojo.forEach(this._domNodes, function(node){
    			node.style.display = "none";
    		});
    	} else {
    		this._contentDiv.style.display = "none";
    	}
        this._isCollapsed = true;
        this._setExpandedStateIcon();
        dojo.removeClass(this._titleDiv, "expanded");
        dojo.addClass(this._titleDiv, "collapsed");
    },

    _expand: function() {
//         var currentHeight = 0
//         var targetHeight = this._contentDiv.clientHeight;
//         this._animateResizingContentHeight(currentHeight, targetHeight);
    	if (this._domNodes) {
    		dojo.forEach(this._domNodes, function(node){
    			node.style.display = "";
    		});
    	} else {
    		this._contentDiv.style.display = "";
    	}
        this._isCollapsed = false;
        this._setExpandedStateIcon();
        dojo.removeClass(this._titleDiv, "collapsed");
        dojo.addClass(this._titleDiv, "expanded");
    },

//     _animateResizingContentHeight: function(currentHeight, targetHeight) {
//         var self = this;
//         var onResizeSection = function(sectionHeight) {
//             self._contentDiv.style.height = Math.floor(sectionHeight) + "px";
//         };
//         var anim = new dojo._Animation({
//             curve: [currentHeight, targetHeight],
//             duration: 150
//         });
//         this.connect(anim, "onAnimate", onResizeSection);
//         anim.play();
//     },

    _onTitleMouseOver: function(event) {
        dojo.addClass(this._titleDiv, "highlight-title");
    },

    _onTitleMouseOut: function(event) {
        dojo.removeClass(this._titleDiv, "highlight-title");
    }
});
