/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.common.PublicTagsParser");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.server.XhrUtils");

var XhrUtils = com.ibm.rdm.web.server.XhrUtils;
var domutils = com.ibm.rdm.web.util.domutils;
var queryutils = com.ibm.rdm.web.util.queryutils;
var UriHelper = com.ibm.rdm.web.common.UriHelper;

dojo.declare(
	"com.ibm.rdm.web.common.PublicTagsParser",
	null,
{

	updateContext: null,
		
    ///////////////////////////////////////////////////////////////////////////////////
    // Private fields
	
	//	Assume a folder object called imgFoldObj and its resource URI imgFoldURI
	//	then _folders[imgFoldURI] = {0: childFolderObj1, 1: childFolderObj2, self: imgFoldObj}
	_folders: null,
	
	//	Assume a tag object called tagObj and its resource URI tagURI
	//	then _tags[tagURI] = tagObj and
	//	_tags[0] = tagObj
	//	So _tags is an array and a map
	_tags: null,
	
    // _etag: etag for the resource that is being parsed
	_etag: null,
	
    ///////////////////////////////////////////////////////////////////////////////////
    // API
	getETag: function() {
		return this._etag;
	},
		
	getPropertyInfo: function() {
		// summary:
		//    Returns parsed response which is a map of Project URIs to 
		//    project collection URIs
		return {TAG:this._tags, FOLDER: this._folders};
	},
	
	parseResponse: function(response) {
		// summary:
		//    Initializes the parsing of first page of folders/tag feed
		this._folders = [];
		this._tags = [];
		this._etag = XhrUtils.getResponseEtag(response);
		this._parseResponse(response);
	},
	
	parseNextPageResponse: function(response) {
		// summary:
		//    Called when parsing folders/tags feed that is paginated
		this._parseResponse(response);
	},
	
	_parseResponse: function(response) {
		// summary:
		//    Helper Method that extracts the tags and folders and creates
		//    a map for each
		var doc = response.doc;
		var folders = [];
		var entries = domutils.getElementsByTagNameNS(doc, "folder", "http://com.ibm.rdm/navigation#", "ns");
		var folders = [];
		for (var i = 0; i < entries.length; i++) {
			var tagObj = queryutils.getTagObject(entries[i]);
			tagObj.scope = PUBLIC;
			folders.push(tagObj);
			/*if (tagObj.term == FOLDER) {
				folders.push(tagObj);
				//<navigation:parent>/jazz/resources/rdm/projectresources/4/rrc_tag5</navigation:parent>
				var parentURINodes = domutils.getElementsByTagNameNS(
						entries[i], "parent", "http://com.ibm.rdm/navigation", "navigation");
				if (parentURINodes[0]) {
					var parentURI = parentURINodes[0].firstChild.nodeValue;
					tagObj.parentURI = parentURI;//.substring(parentURI.lastIndexOf("/") + 1);
				}
			} else if (tagObj.term == TAG) {
				this._tags.push(tagObj);
				this._tags[tagObj.resourceURI] = tagObj;
			}*/
		}
		this._createFolderTree(folders);
	},
	
	_createFolderTree: function(folders) {
		// summary:
		//    Creates a folder map which looks like this:
		//		Assume a folder object called imgFoldObj and its resource URI imgFoldURI
		//	  	then map[imgFoldURI] = {0: childFolderObj1, 1: childFolderObj2, self: imgFoldObj}  
		
	}

});

var FOLDER = com.ibm.rdm.web.common.PublicTagsParser.FOLDER = "folder";
var TAG = com.ibm.rdm.web.common.PublicTagsParser.TAG = "tag";
var PUBLIC = com.ibm.rdm.web.common.PublicTagsParser.PUBLIC = "public";
var ROOT = com.ibm.rdm.web.common.PublicTagsParser.ROOT = "root";
var SELF = com.ibm.rdm.web.common.PublicTagsParser.SELF = "self";
})();
