/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

(function() {
	
dojo.provide("com.ibm.rdm.web.common.ResourceLinksHandler");

dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.links.linkutils");
dojo.require("com.ibm.rdm.web.pages.ActionViewCache");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("jazz.app.ResourceLink");

var UriHelper = com.ibm.rdm.web.common.UriHelper;

com.ibm.rdm.web.common.ResourceLinksHandler = {
	
	_getResourceURIFromRelativePath: function(pathSpecified) {
		if (this.isLinkInternal(pathSpecified))
			return com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES + "/" + pathSpecified;
		else
			return pathSpecified;
	},
	
	handleAnchorClicked: function(event, isRemoveFromCache) {
		dojo.stopEvent(event);
		var anchor = event.currentTarget;
		var href = anchor.attributes.getNamedItem("href").value;
		if (href) {
			if (this.isLinkInternal(href)) {
				//internal resource
				if (isRemoveFromCache == undefined) {
					isRemoveFromCache = true;
				}
				this.showInternalResource(href, isRemoveFromCache);
			}		        
			else {
				//Mail To Anchor
				if (href.match("mailto:")) {
					this._showMailTo(href);
				}
				else {
					//external resource
					this.showExternalResource(href);
				}
			}
		}
	},
	
	handleAnchorContextMenuClicked: function(event) {
		var anchor = event.currentTarget;
		var href = anchor.attributes.getNamedItem("href").value;
		if (href) {
			if (this.isLinkInternal(href)) {
				anchor.setAttribute(
						"href",
						(
							"#action=com.ibm.rdm.web.pages.showArtifact&artifactURI="
							+ "resources/"
							+ this.normalizeUri(href)
						)
				);
			}
		}
	},
	
	handleCommentArtifactAnchorClicked: function(event, commentURI, isRemoveFromCache) {
		dojo.stopEvent(event);
		var anchor = event.currentTarget;
		var href = anchor.attributes.getNamedItem("href").value;
		if (href) {
			if (this.isLinkInternal(href)) {
				//internal resource
				if (isRemoveFromCache == undefined) { isRemoveFromCache = true; }
				this.showComment(href, commentURI, isRemoveFromCache);
			}		        
			else {
				//external resource
				this.showComment(href, commentURI, false);
			}
		}
	},
	
	handleManagedReqAnchorClicked: function(Uri) {
		 event.preventDefault();
		 event.stopPropagation();
		 window.open( (dojo.isIE ? event.srcElement.href : event.target.href) );
	},
	
	showArtifactInReview: function(href, /*String?*/commentURI) {
		if (commentURI) {
			var cacheKeyCode = href.substring(href.indexOf("&") + 1);
			com.ibm.rdm.web.pages.ActionViewCache.removeCachedActionView("showArtifactInReviewContext", cacheKeyCode);
			href = href + "&commentURI=" + commentURI;
		}
		window.location.href = href;
	},

	isLinkInternal: function(resourceURI) {
		if (resourceURI.indexOf(":") == -1) {
			return true;
		}
		return com.ibm.rdm.web.links.linkutils.sameLocation(resourceURI);
	},
	
	showComment: function(resourceURI, commentURI, isRemoveFromCache) {
		this._showInternalResource(resourceURI, commentURI, isRemoveFromCache);
	},
	
	showInternalResource: function(
			resourceURI, isRemoveFromCache,
			/*Boolean?*/isOpenInHistory, /*String?*/projectSnapshotURI)
	{
		this._showInternalResource(
			resourceURI, null, isRemoveFromCache, isOpenInHistory, projectSnapshotURI);
	},
	
	_showInternalResource: function(
			resourceURI, commentURI, isRemoveFromCache,
			isOpenInHistory, projectSnapshotURI)
	{
		var baseResourceURI = resourceURI;
		
		//check if the artifact uri contains revision, if so, force to open it in history viewer
		var revision = null;
		var splitResourceURI = resourceURI.split("?");
		if (splitResourceURI.length > 0) {
			baseResourceURI = splitResourceURI[0];
			var resourceUriQuery = splitResourceURI[1];
			if (resourceUriQuery) {
				var queryObject = dojo.queryToObject(resourceUriQuery);
				revision = queryObject["revision"];
				projectSnapshotURI = (projectSnapshotURI)?projectSnapshotURI:queryObject["rrc:baseline"];
			}
			isOpenInHistory  = (revision) ? true : isOpenInHistory;
		}
		
		var actionCacheKey = baseResourceURI;

		var hrefPrefix = net.jazz.ajax._contextRoot + net.jazz.ajax._webuiPrefix + "rrc" + (djConfig.isDebug ? "?debug=true" : "");
		if (!isOpenInHistory) {
			var href = hrefPrefix + "#action=com.ibm.rdm.web.pages.showArtifact"
						+ "&artifactURI=" + encodeURIComponent(resourceURI);
			if (isRemoveFromCache) {
				com.ibm.rdm.web.pages.ActionViewCache.removeCachedActionView(
					"showArtifact", actionCacheKey);
			}
		}
		else {			
			var href = hrefPrefix + "#action=com.ibm.rdm.web.pages.showArtifactHistory"
						+ "&artifactURI=" + encodeURIComponent(actionCacheKey)
						+ ((revision) ? "&revision=" + revision : "")
						+ ((projectSnapshotURI) ?
						   "&rrc:baseline=" + encodeURIComponent(projectSnapshotURI) :
						   "");
			if (isRemoveFromCache) {
				com.ibm.rdm.web.pages.ActionViewCache.removeCachedActionView(
					"showArtifactHistory", actionCacheKey);
			}
		}
		if (commentURI != null) {
			href = href + "&commentURI=" + commentURI;
		}
		
		if (window.parent) {
			// we're in an iframe
			try {
				window.parent.location.href = href;
				return;
			}
			catch (e) {
				// permission denied (different origin)
				// fall back to current window
			}
		}
		window.location.href = (href);
	},
	
	showExternalResource: function(resourceURI) {
		window.open(resourceURI);
	},
	
	_showMailTo: function(mailTo) {
		if (mailTo){
			window.location.href = (mailTo);
		}
	},
	
	normalizeUri: function(resourceURI) {
		return com.ibm.rdm.web.util.queryutils.getLastPathSegment(resourceURI);
	},
	
	createResourceLink: function(resourceURI, label, doCreateRichHover, isLazyRichHover) {
		//sanity check
		//if it is a wrapper resource link, change to a resource link
		resourceURI = resourceURI.replace(com.ibm.rdm.web.FRONTING_SERVICE_WRAPPER_RESOURCES, com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES);
		
		var resourceLinkNode = null;
		if (doCreateRichHover /* && this.isLinkInternal(resourceURI)*/) {
			//isLazyRichHover defaults to true by default, assuming that we are creating an internal link
			if (isLazyRichHover == null || isLazyRichHover == undefined) {
				isLazyRichHover = true;
			}
			var resourceLink = new jazz.app.ResourceLink({
			    uri: resourceURI,
			    presentationUri: resourceURI,
			    label: label,
			    lazyFetch: isLazyRichHover
			});
			resourceLinkNode = resourceLink.domNode;
		}
		else {
			resourceLinkNode = document.createElement("A");
            resourceLinkNode.appendChild(document.createTextNode(label));
            resourceLinkNode.setAttribute("href", resourceURI);
		}
		resourceLinkNode.setAttribute("target", "_blank");
		return resourceLinkNode;
	},

	// TODO SPC Suggestion -- I think this name is misleading ... we might
	// want to change the name from "getResourceLink" to "createResourceLink".
	getResourceLink: function(resourceURI, label, doCreateRichHover, doAvoidHookingUpOnClickHandling, isLazyRichHover) {
		//sanity check
		//if it is a wrapper resource link, change to a resource link
		resourceURI = resourceURI.replace(com.ibm.rdm.web.FRONTING_SERVICE_WRAPPER_RESOURCES, com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES);
		
		var resourceLinkNode = null;
		if (doCreateRichHover /* && this.isLinkInternal(resourceURI)*/) {
			//isLazyRichHover defaults to true by default, assuming that we are creating an internal link
			if (isLazyRichHover == null || isLazyRichHover == undefined) {
				isLazyRichHover = true;
			}
			var resourceLink = new jazz.app.ResourceLink({
			    uri: resourceURI,
			    presentationUri: resourceURI,
			    label: label,
			    lazyFetch: isLazyRichHover
			});
			resourceLinkNode = resourceLink.domNode;
		}
		else {
			resourceLinkNode = document.createElement("A");
            resourceLinkNode.appendChild(document.createTextNode(label));
            resourceLinkNode.setAttribute("href", resourceURI);
		}
		if (doAvoidHookingUpOnClickHandling) {
			resourceLinkNode.setAttribute("target", "_blank");
		} else {
			// TODO This 'dojo.connect' leads to leaked connections.  Instead
			// the callers should explicitly connect -- so that when the
			// calling widget's destroy function is eventually called, the
			// _Widget.destroy function will properly remove the connection
			// (by calling disconnect).
			dojo.connect(resourceLinkNode, "onclick", this, this.handleAnchorClicked);
		}
		return resourceLinkNode;
	}

};
	
})();
