/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
dojo.provide("com.ibm.rdm.web.common.UserHover");
dojo.require("jazz.ui.HoverPopup");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.common.ProjectCache");

dojo.requireLocalization("com.ibm.rdm.web.common", "common");

var HoverPopup = jazz.ui.HoverPopup;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;

dojo.declare("com.ibm.rdm.web.common.UserHover", [dijit._Widget, dijit._Templated], {
	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "common/templates/UserHover.html"),
   //Public fields
	projectURI: null,
	//Private fields
	_messages: null,
	_htmlListSelectNode: null,
	_userInfoPopUp: null,

	postCreate: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
	},

	destroy: function() {
		if (this._userInfoPopUp) {
			this._userInfoPopUp.destroy();
			this._userInfoPopUp = null;
		}
        this.inherited(arguments);
	},
	
	createHoverPopup: function(userNode, userInfo) {
		var self = this;
		var createHtmlContent = function createPopupContents(popupWidget) {
			var table = document.createElement("table");
			var tbody = document.createElement("tbody");
			
			var userRow = document.createElement("tr");    			
			var projectTableRow = document.createElement("tr");
			var roleTableRow = document.createElement("tr");

			userRow.appendChild(self._createUserPhotoTD(userInfo));
			userRow.appendChild(self._createUserTd(userInfo));
			tbody.appendChild(userRow);
			
			if(self.projectURI){
				projectTableRow.appendChild(self._createProjectTd(userInfo));
				roleTableRow.appendChild(self._createRoleTd(userInfo));
				tbody.appendChild(projectTableRow);
				tbody.appendChild(roleTableRow);
			}
			
			table.appendChild(tbody);

			return table;
		};
		
		this._userInfoPopUp = new HoverPopup({
			around: [userNode],
			createContents: createHtmlContent
		}, null);  
	},
	
	_createUserPhotoTD: function(userInfo) {
		var img = document.createElement("img");
		img.className = "user-hover-user-image";
		var td = document.createElement("td");
		td.setAttribute("rowSpan", "3");
		var photoURI = userInfo.photoUri;
		img.setAttribute("src", photoURI);
		
		td.appendChild(img);
		return td;
	},
	
	_createUserTd: function(userInfo) {
		var td = document.createElement("td");
		dojo.addClass(td, "user-hover-user-name");
		var username = userInfo.name ? userInfo.name : "";
		td.appendChild(document.createTextNode(username));
		return td;
	},
	
	_createProjectTd: function(userInfo) {
		var td = document.createElement("td");
		dojo.addClass(td, "user-hover-user-project");
		ProjectCache.getProject(this.projectURI).addCallback(this,
				function(project){
					td.appendChild(document.createTextNode(this._messages.project + project.name));
				}
		);
		return td;
	},
	
	_createRoleTd: function(userInfo) {
		var td = document.createElement("td");
		dojo.addClass(td, "user-hover-user-roles");
		var userId = userInfo.userId;
		var onProjectMembersComplete = function(userPropertyInfos) {
			for (var userUri in userPropertyInfos){		
				if(userId == userPropertyInfos[userUri].userId){
					var userRoles = userPropertyInfos[userUri].userRoles;
					userRoles.sort(compareLabel);	
					var roleText = "";
					if(userRoles.length <= 1)
					{
						roleText = this._messages.role;
					}
					else{
						roleText = this._messages.roles;
					}
					for(i=0; i<userRoles.length; i++){
						if(i == userRoles.length - 1){
							roleText = roleText + userRoles[i].label;
						}
						else{
							roleText = roleText + userRoles[i].label + this._messages.seperator;
						}
					}					
				}
			}
			td.appendChild(document.createTextNode(roleText));
		}
		var deferred = ProjectCache.getProjectUsersWithRoles(this.projectURI);
		deferred.addCallback(this, onProjectMembersComplete);
		
		return td;
	}    
}); //End of dojo declare

compareLabel = function(userRole1, userRole2) {
	  var label1 = userRole1.label;
		var label2 = userRole2.label;
		return ((label1 < label2) ? -1 : 1);
};


