/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
dojo.provide("com.ibm.rdm.web.common.UsersListWidget");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.rdm.web.common", "common");

com.ibm.rdm.web.common.UsersSelectWidgetUNSPECIFIED = "UnspecifiedUsersSelectValue";

dojo.declare("com.ibm.rdm.web.common.UsersListWidget", [dijit._Widget, dijit._Templated], {
    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "common/templates/UsersListWidget.html"),
    UNSPECIFIED: com.ibm.rdm.web.common.UsersSelectWidgetUNSPECIFIED,
    projectURI: null,
    _messages: null,
    _htmlListSelectNode: null,

    postCreate: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
		if (this.projectURI) {
			var deferred = com.ibm.rdm.web.common.ProjectCache.getProjectUsers(this.projectURI);
			deferred.addCallback(this, this._populateUsersList);	
			deferred.addErrback(this, this._populateUsersSelectWhenError);	
		} else {
			com.ibm.rdm.web.common.ProjectCache.getAllUsers().addCallback(this, this._populateUsersList);
		}
    },
    
    selectValues: function(selectedValues) {
        for (var i=0; i<this._htmlListSelectNode.options.length; i++) {
        	var option = this._htmlListSelectNode.options[i];
        	var selected = false;
        	for (var j=0; j<selectedValues.length; j++) {
        		selected = selectedValues[j] == option.value;
        		if (selected) break;
        	} 
        	option.selected = selected;
        }
    },
    
    getSelectedValues: function() {
    	var values = new Array();
        for (var i=0; i<this._htmlListSelectNode.options.length; i++) {
        	if (this._htmlListSelectNode.options[i].selected) {
        		var value = this._htmlListSelectNode.options[i].value;
        		values.push(value);
        	}
        }
        return values;
    },
    
    _populateUsersSelectWhenError: function(response) {
    	// TODO (LPK)
        return response;
    },
    
    _populateUsersList: function(usersDoc) {
        var users = new Array();
        function nameComparator(user1, user2) {
           	var a = user1[0].toLowerCase();
           	var b = user2[0].toLowerCase();
           	return (a<b)?-1:((a>b)?1:0);
        };
        for (var userURL in usersDoc) {
         	users.push([usersDoc[userURL].name, usersDoc[userURL].userId]);
        }
        users.sort(nameComparator);  
        for (i=0; i<users.length; i++) {
        	 var user = users[i]; 
        	 var option = this._createListItem(user[0], user[1]);
        	 this._htmlListSelectNode.appendChild(option);
        }    	
    	return usersDoc;
    },
    
    _createListItem: function(label, value) {
          var option = document.createElement("option");
          option.value = value;
          if (dojo.isIE) {
          	option.innerHTML = label;
          } else {
          	option.text = label;
          }
          return option;
    }
    
});
