/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.web.common.controlTypes");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.rdm.web.common", "controlTypes");


com.ibm.rdm.web.common.controlTypes = {

	// summary:
	//    This singleton is used to represent the elements found within
	//    an artifact's content area.
	// description:
	//    As it stands today, this class is only dealing with "control types" 
	//    found in GEF-based artifact types (e.g. User Interface Sketch,
	//    Use-Case Diagram, etc.), but perhaps we should also handle rich text
	//    based types here as well (Document, Actor, etc.).

	// TODO Consider adding code to this class to enable obtaining an icon
	// for each of the control types (the Links sidebar section could make
	// use of this ability if it were provided).

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	CONTROL_TYPES: {
		// A note about the key/value pairs found here ... the values MUST
		// represent the underlying GEF EditPart's model class display name.
		// The reason we're using this convention is that the div "hot region"
		// elements produced by the htmlgen code contains a 'controlType'
		// attribute whose value is the name of the underlying EMF model
		// element's class display name, and this 'controlType' attribute value
		// is the value passed as a parameter to the getControlTypeDisplayName
		// function (found in this file).

		// Control types found in "User Interface Sketch" and "User Interface Part"
		Button: "Button",
		Cell: "Cell",
		CheckBox: "Check Box",
		Column: "Column",
		ContextMenu: "Context Menu",
		Cursor: "Cursor",
		DropDownList: "Drop-Down List",
		GroupBox: "Group Box",
		Hyperlink: "Hyperlink",
		Image: "Image",
		Item: "Item",
		Label: "Label",
		ListBox: "List Box",
		Menu: "Menu",
		MenuBar: "Menu Bar",
		Note: "Note",
		Panel: "Panel",
		Paragraph: "Paragraph",
		RadioButton: "Radio Button",
		Row: "Row",
		Table: "Table",
		TextArea: "Text Area",
		TextField: "Text Field",
		ToolItem: "Tool Item",
		Toolbar: "Toolbar",
		Tree: "Tree",
		TreeTable: "Tree Table",
		Window: "Window",

		// Control types found in "Business Process Diagram" (Simple Process)
		CallActivity: "CallActivity",
		DataAssociation: "DataAssociation",
		DataObject: "DataObject",
		EndEvent: "EndEvent",
		ExclusiveGateway: "ExclusiveGateway",
		Group: "Group",
		InclusiveGateway: "InclusiveGateway",
		IntermediateReceiveEvent: "IntermediateReceiveEvent",
		IntermediateSendEvent: "IntermediateSendEvent",
		Lane: "Lane",
		ParallelGateway: "ParallelGateway",
		SequenceFlow: "SequenceFlow",
		StartEvent: "StartEvent",
		Task: "Task",
		TextAnnotation: "TextAnnotation",
		MessageFlow: "MessageFlow",

		// Control types found in "Business Process Diagram" (Business2Bussiness Process)
		// Contains the same types as above (Simple Process) ... AND Pool.
		Pool: "Pool",

		// Control types found in "Use-Case Diagram"
		ActorRef: "ActorRef",
		Association: "Association",
		SystemBoundary: "SystemBoundary",
		UsecaseRef: "UsecaseRef",

		// Control types found in "Screen Flow"
		Transition: "Transition",
		UserInterfaceActivity: "User Interface Activity"
	},

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_mapControlTypeToDisplayName: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	getControlTypeDisplayName: function(controlType) {
		// summary:
		//    Return a string appropriate for the current locale to represent
		//    the control type.
		// controlType:
		//    One of the values found in CONTROL_TYPES (e.g.
		//    CONTROL_TYPES.Button).
		// example:
		//    A call to:
		//    |    com.ibm.rdm.web.common.controlTypes.getControlTypeDisplayName(
		//    |        com.ibm.rdm.web.common.controlTypes.CONTROL_TYPES.Button);
		//    Would return (ussuming English locale):
		//    |    "Button"
		if (! this._mapControlTypeToDisplayName) {
			this._mapControlTypeToDisplayName = new Object;
			// Fill in the _mapControlTypeToDisplayName field
			for (prop in this.CONTROL_TYPES) {
				var key = this.CONTROL_TYPES[prop];
				this._mapControlTypeToDisplayName[key] = this._messages[key];
			}
		}
		return this._mapControlTypeToDisplayName[controlType];
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_initMessagesIfNeeded: function() {
		if (! this._messages) {
			this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "controlTypes");
		}
	}
};

com.ibm.rdm.web.common.controlTypes._initMessagesIfNeeded();
