/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.history.HistoryViewer");
dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.layout.LayoutContainer");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.commenting.ResourceViewerWidget");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.history.RevisionsPane");
dojo.requireLocalization("com.ibm.rdm.web.history", "HistoryViewer");

var UriHelper = com.ibm.rdm.web.common.UriHelper;
var domutils = com.ibm.rdm.web.util.domutils;

var REVISIONS_PANE_HEIGHT = 70;

dojo.declare(
	"com.ibm.rdm.web.history.HistoryViewer",
	[dijit._Widget, dijit._Templated],
{
    // summary:
	//    Provides a viewer which displays history artifacts in a similar
	//    manner to the history editor found in the native RCP client.

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "history/templates/HistoryViewer.html"),
	widgetsInTemplate: false,

    ///////////////////////////////////////////////////////////////////////////////////
    // Attach points

    _frameViewerDiv: null,
	_thumbnailStackDiv: null,
	_historyViewerDiv: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields

	resourceURI: null,
	resourceViewer: null,
	_revisionsPane: null,
	_layoutContainer: null,
	_resourceViewerWidget: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	
	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited("postCreate", arguments);
		this._messages = dojo.i18n.getLocalization(
			"com.ibm.rdm.web.history", "HistoryViewer");
		var self = this;
		this._layoutContainer = new dijit.layout.LayoutContainer(
			{},
			this._historyViewerDiv
		);
		this._revisionsPane = new com.ibm.rdm.web.history.RevisionsPane(
			{
				resourceURI: this.resourceURI,
				historyViewer: this
			});
		this._layoutContainer.addChild(this._revisionsPane);
		this._layoutContainer.addChild(this._resourceViewerWidget);
	},
	
	startup: function() {
		this.inherited("startup", arguments);
 		this._layoutContainer.startup();
	},
	
	setDimensions: function(width, height){
		this._layoutContainer.resize({w:width, h:height});
		if (this._resourceViewerWidget && this._revisionsPane) {
			this._resourceViewerWidget.setDimensions(width, height - REVISIONS_PANE_HEIGHT);
			this._revisionsPane.setDimensions(width, REVISIONS_PANE_HEIGHT + "px");
		}
    },
    
    showRevision: function(revision, projectSnapshotURI) {
    	this._revisionsPane._revisionsDeferred.addCallback(this, function(parsedResponse){
    		if (!revision) {
    			revision = parsedResponse.initialRevision.revisionId;
    		}

	    	this.resourceViewer.projectSnapshotURI = projectSnapshotURI;

	        var newRevision = revision;
	        if (projectSnapshotURI) {
	    		newRevision = projectSnapshotURI;
	        }
	        
	        //do not reload revision if it is already selected
			if (!this._revisionsPane.isRevisionSelected(newRevision)) {
	        	this._revisionsPane.selectRevisionThumbnail(newRevision);
	        	this.resourceViewer.loadResource(revision);
			}
    	});
    }
});


})();
