/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.history.RevisionsCache");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.history.RevisionsParser");
dojo.require("com.ibm.rdm.web.history.RevisionsRetriever");

var RevisionsRetriever = com.ibm.rdm.web.history.RevisionsRetriever;
var UriHelper = com.ibm.rdm.web.common.UriHelper;


com.ibm.rdm.web.history.RevisionsCache = {
	// summary:
	//    This singleton is used to cache artifact revision information.

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_revisionsRetriever: new RevisionsRetriever(),
	
	///////////////////////////////////////////////////////////////////////////
	// API

	getRevisions: function(resourceURI) {
		// summary:
		//    Get information about all revisions associated with a particular resource.
		// returns:
		//    A deferred which you can to obtain the most up to date revision
		//    information.
		//    Please note that the passed reponse to your callback is of a
		//    similar format to that used by the RDMClient.doGet function:
		//    {
		//        doc: <an instance of a RevisionsParser>
		//        ioArgs: <the original ioArgs that was used in the GET>
		//    }
		//    The only difference is that the "doc" property is an instance of a
		//    RevisionsParser rather than an instance of an XML document (as it
		//    would be with the RDMClient.doGet function).
		// example:
		//    |    var onRevisionsLoaded = function(response) {
		//    |        var revisionsParser = response.doc;
		//    |        // do something with the revisionsParser ...
		//    |    };
		//    |    var deferred = com.ibm.rdm.web.history.RevisionsCache.getRevisions(resourceURI);
		//    |    deferred.addCallback(onRevisionsLoaded);
		// resourceURI:
		//    A string such as "https://localhost:9443/rdm/resources/_lckswDjhEd67TdjgK4pk9g".
		//
		var deferred = this._revisionsRetriever.getRevisions(resourceURI);
		return deferred;
	}
};

})();
