/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
dojo.provide("com.ibm.rdm.web.history.RevisionsPane");
dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dojo.i18n");
dojo.require("dojo.colors");
dojo.require("dijit.Tooltip");

dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.common.DateTimeHelper");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.history.RevisionsCache");
dojo.require("com.ibm.rdm.web.history.RevisionsParser");
dojo.require("com.ibm.rdm.web.common.ProjectCache");

dojo.requireLocalization("com.ibm.rdm.web.history", "RevisionsPane");

var START_COLOR_RANGE = "#CEAA84";
var END_COLOR_RANGE = "#7BA2C6";
var queryutils = com.ibm.rdm.web.util.queryutils;
var DateTimeHelper = com.ibm.rdm.web.common.DateTimeHelper;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var RevisionsCache = com.ibm.rdm.web.history.RevisionsCache;
var RevisionsParser = com.ibm.rdm.web.history.RevisionsParser;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;

dojo.declare(
	"com.ibm.rdm.web.history.RevisionsPane",
	[dijit._Widget, dijit._Templated],
{
    // summary:
	//    Provides a viewer which displays history artifacts in a similar
	//    manner to the history editor found in the native RCP client.

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "history/templates/RevisionsPane.html"),
	widgetsInTemplate: false,

    ///////////////////////////////////////////////////////////////////////////////////
    // Attach points

    _frameViewerDiv: null,
	_thumbnailStackDiv: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields

	resourceURI: null,


	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_previouslySelectedRevisionNode: null,
	_revisionThumbnailsNodesMap: {},
	_revisionsEntryNodes: {},
	_revisionInfosMap: {},
	
	// _revisionInfos: Object
	//    See the RevisionsParser.getRevisionInfos function for documentation.
	_revisionInfos: null,
	
	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this._revisionsDeferred = new dojo.Deferred();
		var revisionsDeferred = RevisionsCache.getRevisions(this.resourceURI);
		revisionsDeferred.addCallback(this, function(revisionsResponse){
			var revisionsParser = dojo.clone(revisionsResponse.doc);
			var descriptionDeferred = ProjectCache.getResourceDescriptionMultiFetch(this.resourceURI, null, true);
			descriptionDeferred.addCallback(this, function(descriptionResponse){
				this.headResourceDescription = descriptionResponse;
				if (descriptionResponse) {
					var snaphotsDeferred = ProjectCache.getProjectSnapshots(descriptionResponse.projectURI);
					snaphotsDeferred.addCallback(this, function(snapshotsResponse){
						revisionsParser.mergeRevisionsWithSnapshots(snapshotsResponse);
						this._revisionsDeferred.callback(revisionsParser);
					});
				}
				//artifact is deleted, but we still need to show it in the history viewer
				else {
					var descriptionDeferred2 = ProjectCache.getResourceDescriptionMultiFetch(this.resourceURI, this.historyViewer.resourceViewer.projectSnapshotURI, true);
					descriptionDeferred2.addCallback(this, function(descriptionResponse2){
						this.headResourceDescription = descriptionResponse2;
							var snaphotsDeferred = ProjectCache.getProjectSnapshots(descriptionResponse2.projectURI);
							snaphotsDeferred.addCallback(this, function(snapshotsResponse2){
								revisionsParser.mergeRevisionsWithSnapshots(snapshotsResponse2);
								revisionsParser.removeRevisionsForDeletedArtifact(snapshotsResponse2);
								this._revisionsDeferred.callback(revisionsParser);
							});
						});
				}
			});
		});
		this._revisionsDeferred.addCallback(this, this.onRevisionsLoaded);
	},

	constructor: function() {
		this._previouslySelectedRevisionNode = null;
		this._revisionThumbnailsNodesMap = {};
		this._revisionsEntryNodes = {};
		this._revisionInfosMap = {};
		this._revisionsDeferred = null;
		this._frameViewerDiv = null;
		this._thumbnailStackDiv = null;
		this.resourceURI = null;
		this._revisionInfos = null;
		this._initialRevision = null;
		this._messages = dojo.i18n.getLocalization(
			"com.ibm.rdm.web.history", "RevisionsPane");		
	},
		
	destroy: function() {
		this.inherited("destroy", arguments);
	},
	
	onRevisionsLoaded: function(revisionsParser) {
		var revisions = revisionsParser.getTimeSlicedRevisionInfos();
		this._revisionInfos = revisionsParser.getRevisionInfos();
		this._revisionInfosMap = revisionsParser._revisionInfosMap;
		this._initialRevision = revisionsParser.initialRevision;
		
		for (revisionsInDateRange in revisions) {
			
			for (var i=0; i<revisions[revisionsInDateRange].length; i++) {

				var revision = revisions[revisionsInDateRange][i];
				
				var div = document.createElement("span");
				
				dojo.addClass(div, "dijitInline");
				
				if (!revision.snapshot) {
					dojo.addClass(div, "revisionThumbnail");
					var color = dojo.blendColors(
								new dojo.Color(START_COLOR_RANGE), 
								new dojo.Color(END_COLOR_RANGE),
								i/(revisions[revisionsInDateRange].length > 1?revisions[revisionsInDateRange].length-1:1));
				
					div.style.backgroundColor = color.toHex();
				}
				else {
					dojo.addClass(div, "revisionProjectSnapshotThumbnail");
				}
				

				dojo.attr(div, "revision", revision.revisionId);
				
				dojo.connect(div, "onclick", this, this.onRevisionThumbnailClick);
				dojo.connect(div, "onmouseenter", this, this.onRevisionThumbnailHover);
				
				dojo.connect(div, "onmouseleave", this, this.onRevisionThumbnailHoverOut);
				dojo.connect(div, "onmouseout", this, this.onRevisionThumbnailHoverOut);
				
				
				
				//TODO
				//skip <link rel="http://jazz.net/xmlns/v0.6#predecessor"
				
				switch (revision.dateRange) {
					case DateTimeHelper.DATE_RANGE_TYPE.Today:
						this._todayRevisionsNode.appendChild(div);
						break;
					case DateTimeHelper.DATE_RANGE_TYPE.Yesterday:
						this._yesterdayRevisionsNode.appendChild(div);			
						break;
					case DateTimeHelper.DATE_RANGE_TYPE.PastWeek:
						this._pastweekRevisionsNode.appendChild(div);
						break;
					case DateTimeHelper.DATE_RANGE_TYPE.PastMonth:
						this._pastmonthRevisionsNode.appendChild(div);
						break;
					case DateTimeHelper.DATE_RANGE_TYPE.Earlier:
					default:
						this._earlierRevisionsNode.appendChild(div);
						break;
				}
				
				this._revisionThumbnailsNodesMap[revision.revisionId] = div;
			}
		}
		
		//update title sections
		//TODO: make it prettier
		var self = this;
		var getTimeRangeLabelToUse = function(dateRange) {
			// dateRange: a value from DateTimeHelper.DATE_RANGE_TYPE
			dateRangeRevisions = revisions[dateRange];
			var numEntries = (dateRangeRevisions ? dateRangeRevisions.length : 0);
			return dojo.string.substitute(
				self._messages["revisionsInDateRange"],
				[ numEntries ]
			);
		};
		this._todayRevisionsTitleNode.appendChild(
			document.createTextNode(
				" " + getTimeRangeLabelToUse(DateTimeHelper.DATE_RANGE_TYPE.Today))
		);
		this._yesterdayRevisionsTitleNode.appendChild(
			document.createTextNode(
				" " + getTimeRangeLabelToUse(DateTimeHelper.DATE_RANGE_TYPE.Yesterday))
		);
		this._pastweekRevisionsTitleNode.appendChild(
			document.createTextNode(
				" " + getTimeRangeLabelToUse(DateTimeHelper.DATE_RANGE_TYPE.PastWeek))
		);
		this._pastmonthRevisionsTitleNode.appendChild(
			document.createTextNode(
				" " + getTimeRangeLabelToUse(DateTimeHelper.DATE_RANGE_TYPE.PastMonth))
		);
		this._earlierRevisionsTitleNode.appendChild(
			document.createTextNode(
				" " + getTimeRangeLabelToUse(DateTimeHelper.DATE_RANGE_TYPE.Earlier))
		);
	},
	
	onRevisionThumbnailClick: function(event) {
		var revisionNode = event.target;
		var revision = dojo.attr(revisionNode, "revision");
		//this.selectRevisionThumbnail(revision);
		this.onRevisionThumbnailHoverOut(event);
		
		var projectSnapshotURI = "";
		if (this._revisionInfosMap[revision] && this._revisionInfosMap[revision].snapshot) {
			projectSnapshotURI = this._revisionInfosMap[revision].snapshot.resource;
		}
		var href = UriHelper.getResourceHistoryJafLink(
								this.resourceURI, 
								(projectSnapshotURI)?this._revisionInfosMap[revision].snapshotRevisionId:revision);
		//changing the URI passes control to the ResourcesPage.showArtifactHistory()
		window.location.href =
			href + (projectSnapshotURI ? ("&rrc:baseline=" + encodeURIComponent(projectSnapshotURI)) : "");
	},
	
	isRevisionSelected: function(revision) {
		if (this._previouslySelectedRevisionNode) {
			var selectedRevision = dojo.attr(this._previouslySelectedRevisionNode, "revision");
			var newRevision = revision;
			if (!newRevision) {
				newRevision = this._initialRevision.revisionId;
			}
			return (newRevision == selectedRevision);
		}
		return false;
	},
	
	selectRevisionThumbnail: function(revision) {
		var revision = revision;
		if (!revision) {
			revision = this._initialRevision.revisionId;
		}
		this._revisionsDeferred.addCallback(this, function(revisionsParser) {
			var revisionNode = this._revisionThumbnailsNodesMap[revision];
			if (this._previouslySelectedRevisionNode)
				this._previouslySelectedRevisionNode.style.borderColor = this._previouslySelectedRevisionNode.parentNode.style.backgroundColor;
			revisionNode.style.borderColor = "#316AC5";
			this._previouslySelectedRevisionNode = revisionNode;
			
			this.historyViewer.resourceViewer._revisionInfoNode.innerHTML = "";
			
			ProjectCache.getProjectUsers(this.headResourceDescription.projectURI).addCallback(this, 
				function(users) {
					var modifiedLabel = dojo.string.substitute(
						 this._messages["modifiedOnAndBy"],
						 [
						 	this._revisionInfosMap[revision].revisionDateLabel,
							users[this._revisionInfosMap[revision].authorURI].name
						 ]
					);
					this.historyViewer.resourceViewer._revisionInfoNode.appendChild(
						document.createTextNode(modifiedLabel));
				}
			);
		});
	},
	
	onRevisionThumbnailHover: function(event) {
		var revisionNode = event.target;
		var revision = dojo.attr(revisionNode, "revision");
		var hoverVersionLabel = dojo.string.substitute(
			 this._messages["hoverVersionLabel"],
			 [ this._revisionInfosMap[revision].revisionDateLabel ]
		);
		ProjectCache.getProjectUsers(this.headResourceDescription.projectURI).addCallback(this, 
			function(users) {
				var hoverAuthorLabel = dojo.string.substitute(
					 this._messages["hoverAuthorLabel"],
					 [users[this._revisionInfosMap[revision].authorURI].name]
				);
				dijit.showTooltip(
					hoverVersionLabel + "<br>" + hoverAuthorLabel + ((this._revisionInfosMap[revision].snapshot)?"<br>" + this._messages.inSnapshot + this._revisionInfosMap[revision].snapshot.title:""),
					revisionNode,
					["before", "below"]
				);
			}
		);
		
		if (! (this._previouslySelectedRevisionNode &&
				(this._previouslySelectedRevisionNode == revisionNode))) {
			revisionNode.style.borderColor = revisionNode.style.backgroundColor;
		}
	},
	
	onRevisionThumbnailHoverOut: function(event) {
		var revisionNode = event.target;
		if (dijit._masterTT) {
			dijit._masterTT._onHide();
		}
		if (this._previouslySelectedRevisionNode &&
				(this._previouslySelectedRevisionNode == revisionNode)) {
			return;
		}
		revisionNode.style.borderColor = revisionNode.parentNode.style.backgroundColor;
		
	},
	
	setDimensions: function(width, height) {
		this._revisionsTable.style.height = height;
	}
});


})();
