/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
dojo.provide("com.ibm.rdm.web.history.RevisionsParser");
dojo.require("dojo.date.locale");
dojo.require("com.ibm.rdm.web.common.DateTimeHelper");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.history.revisionsHelper");

var DateTimeHelper = com.ibm.rdm.web.common.DateTimeHelper;
var domutils = com.ibm.rdm.web.util.domutils;
var revisionsHelper = com.ibm.rdm.web.history.revisionsHelper;

// Here's an example of the artifact version history that this RevisionsParser
// class might be asked to parse ...

// JRS example:
// <?xml version="1.0" encoding="UTF-8"?>
//	<feed xmlns="http://www.w3.org/2005/Atom">
//	  <title>rdm/resources/rrc76 revisions</title>
//	  <updated>2009-03-05T13:10:15-05:00</updated>
//	   <link rel="self" href="rdm/resources/rrc76?revisions" type="application/atom+xml;type=feed" />
//	  <generator>Jazz Resource Services</generator>
//	  <entry>
//	    <title>rdm/resources/rrc76?revision=_0Z7b0wjcEd61NYgj7k09Fg</title>
//	    <author><name>/jazz/users/tim</name></author>
//	    <summary></summary>
//	    <id>_J7dR8gE6Ed66Y8d5_Fow7g</id>
//	    <updated>2009-03-04T11:52:17-05:00</updated>
//	    <link rel="alternate" href="/jazz/resources/rdm/resources/rrc76?revision=_0Z7b0wjcEd61NYgj7k09Fg"/>
//	    <link rel="self" href="/jazz/resources/rdm/resources/rrc76?revision=_0Z7b0wjcEd61NYgj7k09Fg"/>
//	    <link rel="http://jazz.net/xmlns/v0.6#parent" href="rdm/resources"/>
//	    <link rel="http://jazz.net/xmlns/v0.6#predecessor" type="application/x-com.ibm.rdm.richtext+xml" href="/jazz/resources/rdm/resources/rrc76?revision=_71NEMQgkEd6OAsH03qyUBg"/>
//	<content type="application/x-com.ibm.rdm.richtext+xml" src="/jazz/resources/rdm/resources/rrc76?revision=_0Z7b0wjcEd61NYgj7k09Fg"/>
//	  </entry>
//	  <entry>
//	    <title>rdm/resources/rrc76?revision=_71NEMQgkEd6OAsH03qyUBg</title>
//	    <author><name>/jazz/users/tim</name></author>
//	    <summary></summary>
//	    <id>_J7dR8gE6Ed66Y8d5_Fow7g</id>
//	    <updated>2009-03-03T13:55:59-05:00</updated>
//	    <link rel="alternate" href="/jazz/resources/rdm/resources/rrc76?revision=_71NEMQgkEd6OAsH03qyUBg"/>
//	    <link rel="self" href="/jazz/resources/rdm/resources/rrc76?revision=_71NEMQgkEd6OAsH03qyUBg"/>
//	    <link rel="http://jazz.net/xmlns/v0.6#parent" href="rdm/resources"/>
//	    <link rel="http://jazz.net/xmlns/v0.6#predecessor" type="application/x-com.ibm.rdm.richtext+xml" href="/jazz/resources/rdm/resources/rrc76?revision=_1AULoQgGEd6OAsH03qyUBg"/>
//	    <link rel="http://jazz.net/xmlns/v0.6#successor" type="application/x-com.ibm.rdm.richtext+xml" href="/jazz/resources/rdm/resources/rrc76?revision=_0Z7b0wjcEd61NYgj7k09Fg"/>
//	<content type="application/x-com.ibm.rdm.richtext+xml" src="/jazz/resources/rdm/resources/rrc76?revision=_71NEMQgkEd6OAsH03qyUBg"/>
//	  </entry>
//	  <entry>
//	    <title>rdm/resources/rrc76?revision=_1AULoQgGEd6OAsH03qyUBg</title>
//	    <author><name>/jazz/users/tim</name></author>
//	    <summary></summary>
//	    <id>_J7dR8gE6Ed66Y8d5_Fow7g</id>
//	    <updated>2009-03-03T10:20:28-05:00</updated>
//	    <link rel="alternate" href="/jazz/resources/rdm/resources/rrc76?revision=_1AULoQgGEd6OAsH03qyUBg"/>
//	    <link rel="self" href="/jazz/resources/rdm/resources/rrc76?revision=_1AULoQgGEd6OAsH03qyUBg"/>
//	    <link rel="http://jazz.net/xmlns/v0.6#parent" href="rdm/resources"/>
//	    <link rel="http://jazz.net/xmlns/v0.6#predecessor" type="application/x-com.ibm.rdm.richtext+xml" href="/jazz/resources/rdm/resources/rrc76?revision=_rARDAQgGEd6OAsH03qyUBg"/>
//	    <link rel="http://jazz.net/xmlns/v0.6#successor" type="application/x-com.ibm.rdm.richtext+xml" href="/jazz/resources/rdm/resources/rrc76?revision=_71NEMQgkEd6OAsH03qyUBg"/>
//	<content type="application/x-com.ibm.rdm.richtext+xml" src="/jazz/resources/rdm/resources/rrc76?revision=_1AULoQgGEd6OAsH03qyUBg"/>
//	  </entry>

// JFS example:
// <?xml version="1.0" encoding="UTF-8"?>
// <feed xmlns="http://www.w3.org/2005/Atom">
//   <title>https://localhost:9443/jazz/history</title>
//   <updated>2009-05-12T10:58:42-04:00</updated>
//    <link rel="self" href="https://localhost:9443/jazz/history" type="application/atom+xml;type=feed" />
//   <generator>Jazz Resource Services</generator>
//   <entry>
//     <title>storage/com.ibm.rdm.resources/_lckswDjhEd67TdjgK4pk9g?revision=_zObo4To7Ed62cpPlu7nHxQ</title>
//     <author><name>/jazz/users/TestJazzAdmin1</name></author>
//     <summary></summary>
//     <id>_lckswTjhEd67TdjgK4pk9g</id>
//     <updated>2009-05-06T08:45:38-04:00</updated>
//     <link rel="alternate" href="/jazz/storage/com.ibm.rdm.resources/_lckswDjhEd67TdjgK4pk9g?revision=_zObo4To7Ed62cpPlu7nHxQ"/>
//     <link rel="self" href="/jazz/storage/com.ibm.rdm.resources/_lckswDjhEd67TdjgK4pk9g?revision=_zObo4To7Ed62cpPlu7nHxQ"/>
//     <link rel="http://jazz.net/xmlns/v0.6#predecessor" type="application/x-com.ibm.rdm.ba.actor+xml" href="/jazz/storage/com.ibm.rdm.resources/_lckswDjhEd67TdjgK4pk9g?revision=_-1O-ITjhEd67TdjgK4pk9g"/>
// <content type="application/x-com.ibm.rdm.ba.actor+xml" src="/jazz/storage/com.ibm.rdm.resources/_lckswDjhEd67TdjgK4pk9g?revision=_zObo4To7Ed62cpPlu7nHxQ"/>
//   </entry>
//   <entry>
//     <title>storage/com.ibm.rdm.resources/_lckswDjhEd67TdjgK4pk9g?revision=_-1O-ITjhEd67TdjgK4pk9g</title>
//     <author><name>/jazz/users/TestJazzAdmin1</name></author>
//     <summary></summary>
//     <id>_lckswTjhEd67TdjgK4pk9g</id>
//     <updated>2009-05-04T15:30:10-04:00</updated>
//     <link rel="alternate" href="/jazz/storage/com.ibm.rdm.resources/_lckswDjhEd67TdjgK4pk9g?revision=_-1O-ITjhEd67TdjgK4pk9g"/>
//     <link rel="self" href="/jazz/storage/com.ibm.rdm.resources/_lckswDjhEd67TdjgK4pk9g?revision=_-1O-ITjhEd67TdjgK4pk9g"/>
//     <link rel="http://jazz.net/xmlns/v0.6#predecessor" type="application/x-com.ibm.rdm.ba.actor+xml" href="/jazz/storage/com.ibm.rdm.resources/_lckswDjhEd67TdjgK4pk9g?revision=_lcqzYDjhEd67TdjgK4pk9g"/>
//     <link rel="http://jazz.net/xmlns/v0.6#successor" type="application/x-com.ibm.rdm.ba.actor+xml" href="/jazz/storage/com.ibm.rdm.resources/_lckswDjhEd67TdjgK4pk9g?revision=_zObo4To7Ed62cpPlu7nHxQ"/>
// <content type="application/x-com.ibm.rdm.ba.actor+xml" src="/jazz/storage/com.ibm.rdm.resources/_lckswDjhEd67TdjgK4pk9g?revision=_-1O-ITjhEd67TdjgK4pk9g"/>
//   </entry>
//   <entry>
//     <title>storage/com.ibm.rdm.resources/_lckswDjhEd67TdjgK4pk9g?revision=_lcqzYDjhEd67TdjgK4pk9g</title>
//     <author><name>/jazz/users/TestJazzAdmin1</name></author>
//     <summary></summary>
//     <id>_lckswTjhEd67TdjgK4pk9g</id>
//     <updated>2009-05-04T15:27:19-04:00</updated>
//     <link rel="alternate" href="/jazz/storage/com.ibm.rdm.resources/_lckswDjhEd67TdjgK4pk9g?revision=_lcqzYDjhEd67TdjgK4pk9g"/>
//     <link rel="self" href="/jazz/storage/com.ibm.rdm.resources/_lckswDjhEd67TdjgK4pk9g?revision=_lcqzYDjhEd67TdjgK4pk9g"/>
//     <link rel="http://jazz.net/xmlns/v0.6#successor" type="application/x-com.ibm.rdm.ba.actor+xml" href="/jazz/storage/com.ibm.rdm.resources/_lckswDjhEd67TdjgK4pk9g?revision=_-1O-ITjhEd67TdjgK4pk9g"/>
// <content type="application/x-com.ibm.rdm.ba.actor+xml" src="/jazz/storage/com.ibm.rdm.resources/_lckswDjhEd67TdjgK4pk9g?revision=_lcqzYDjhEd67TdjgK4pk9g"/>
//   </entry>
// </feed>


dojo.declare(
	"com.ibm.rdm.web.history.RevisionsParser",
	null,
{
	///////////////////////////////////////////////////////////////////////////
	// Public fields
	updateContext: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_revisionInfos: null,
	_revisionInfosMap: {},
	initialRevision: null,

	///////////////////////////////////////////////////////////////////////////
	// API
	constructor: function(xml) {
		this._revisionInfos = null;
		this._revisionInfosMap = {};
		this.initialRevision = null;
	},
	init: function(response) {
		this._parseContent(response.doc);
	},

	mergeRevisionsWithSnapshots: function(snapshots) {
		function snapshotDateComparator(snapshot1, snapshot2) {
           var a = snapshot1.snapshotDate;
           var b = snapshot2.snapshotDate;
           return (a<b)?1:((a>b)?-1:0);
        }
        var projectSnapshots = [];
        for (var snapshotURI in snapshots) {
        	projectSnapshots.push(snapshots[snapshotURI]);
        }
        projectSnapshots.sort(snapshotDateComparator);
		
		var mergedRevisionInfosWithSnapshots = [];
		for (var i=0; i<this._revisionInfos.length; i++) {
			for (var j=0; j<projectSnapshots.length; j++) {
				if(projectSnapshots[j].snapshotDate > this._revisionInfos[i].revisionDate) {
					if (i==0 || projectSnapshots[j].snapshotDate < this._revisionInfos[i-1].revisionDate) {
						var cloneRevision = dojo.clone(this._revisionInfos[i]);
						cloneRevision.snapshot = projectSnapshots[j];
						cloneRevision.snapshotRevisionId = cloneRevision.revisionId;
						cloneRevision.revisionId = projectSnapshots[j].resource;
						mergedRevisionInfosWithSnapshots.push(cloneRevision);
						this._revisionInfosMap[cloneRevision.revisionId] = cloneRevision;
					}
				}
			}
			mergedRevisionInfosWithSnapshots.push(this._revisionInfos[i]);
		}
		this._revisionInfos = mergedRevisionInfosWithSnapshots;
	},
	
	removeRevisionsForDeletedArtifact: function(snapshots) {
		//remove revision entries only and leave snapshots
        var projectSnapshotRevisions = [];
		var projectSnapshotRevisionsMap = {};
		
		var mergedRevisionInfosWithSnapshots = [];
		for (var i=0; i<this._revisionInfos.length; i++) {
			if (this._revisionInfos[i].snapshot) {
				projectSnapshotRevisions.push(this._revisionInfos[i]);
				projectSnapshotRevisionsMap[this._revisionInfos[i].revisionId] = this._revisionInfos[i];
			}
		}
		this._revisionInfos = projectSnapshotRevisions;
		this._revisionInfosMap = projectSnapshotRevisionsMap;
	},

	getTimeSlicedRevisionInfos: function() {
		// summary:
		//    Get time-slice based information about the revisions.
		// returns: an Object
		//    The properties of the returned Object will include time
		//    range values for all time slices that pertain to the
		//    revisions.  For example there might ONLY be a property
		//    for DateTimeHelper.DATE_RANGE_TYPE.Yesterday (but there
		//    COULD be one for every time range, as it depends on which
		//    ranges pertain to the involved revisions).
		//    The values for these properties are the RevisionInfos
		//    that are associated with a particular time slice.
		// example:
		//    A call to:
		//    |    revisionParser.getTimeSlicedRevisionInfos(
		//    |        com.ibm.rdm.web.common.DateTimeHelper.DATE_RANGE_TYPE.Yesterday)
		//    might return:
		//    |    [ { author: "tim", ... }, { author: "bob", ... } ]
		//    but it might also return null (as it depends on whether any
		//    revisions were made yesterday.
		var timeSlicedRevisionInfos = [];
		for (var i=0; i<this._revisionInfos.length; i++) {	
			var dateRange = this._revisionInfos[i].dateRange;
			if (!timeSlicedRevisionInfos[dateRange]) {
				timeSlicedRevisionInfos[dateRange] = new Array(this._revisionInfos[i]);
			}
			else {
				timeSlicedRevisionInfos[dateRange].push(this._revisionInfos[i]);
			}
		}
		return timeSlicedRevisionInfos;
	},

	getRevisionInfos: function() {
		// summary:
		//    Get information about all revisions.
		// returns: Array that is also a hashmap
		//    An array of time ordered property lists.  The ordering is from
		//    most recent to least recent based on the 'revisionDate' property
		//    found in each property list.  Each property list will contain the
		//    properties of the following form:
		//    {
		//        author: "bob",
		//        dateRange: "Today",
		//        id: "_0zQqUh-wEd6WZPkZ_nI-wQ",
		//        revisionDate: <a javascript Date object>
		//        revisionDateLabel: "April 13, 2009 4:27 p.m.",
		//        revisionId: "_chlHcihpEd68ntlx2Q9dDA",
		//        title: "rdm/resources/rrc137?revision=_chlHcihpEd68ntlx2Q9dDA",
		//        updated: "2009-04-13T16:27:01-04:00"
		//    }
		//    This array also serves as a hashmap, where the key is the
		//    revisionId (e.g. "_chlHcihpEd68ntlx2Q9dDA") and the value is the
		//    appropriate revisionInfo property list entry.
		return this._revisionInfos;
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_parseContent: function(doc) {
		// summary:
		//    Fill this._timeSlicedRevisionInfos and this._revisionInfos with
		//    content parsed from the passed doc.
		var revisions = {};
		var revisionInfos = [];
		var entryNodes = doc.getElementsByTagName("entry");
		for (var i=0; i<entryNodes.length; i++) {
			var revisionNodeChildren = entryNodes[i].childNodes;
			var revisionInfo = {};
			for (var j=0; j<revisionNodeChildren.length; j++) {
				var childNode = revisionNodeChildren[j];
				if (childNode.tagName == "link") {
					var rel = childNode.getAttribute("rel");
					var selfLinkURI = childNode.getAttribute("href");
					if (rel == "self") {
						revisionInfo["revisionId"] = selfLinkURI.split("revision=")[1];
					}
					revisionInfo[rel] = selfLinkURI.split("revision=")[1];
				}
				var childNodeValue = dojo.isIE?(childNode.text):(childNode.textContent);
				revisionInfo[childNode.tagName] = (childNodeValue)?childNodeValue:"";
			}

			var revisionUpdateDate = dojo.date.stamp.fromISOString(revisionInfo.updated);
			var dateStr = dojo.date.locale.format(revisionUpdateDate, {formatLength:'long',selector:'date'});
			var timeStr = dojo.date.locale.format(revisionUpdateDate, {formatLength:'short',selector:'time'});
			
			var dateRange = DateTimeHelper.getDateRange(new Date(revisionUpdateDate));
			revisionInfo.dateRange = dateRange;
			//20612: History viewer is not showing correct UTF8 user name
			revisionInfo.authorURI = com.ibm.rdm.web.FRONTING_SERVICE_JFS_USERS + "/" + com.ibm.rdm.web.util.queryutils.getTrimmedUserName(revisionInfo.author);
			revisionInfo.revisionDate = revisionUpdateDate;
			revisionInfo.revisionDateLabel = dateStr + " " + timeStr;
			if (!revisions[dateRange]) {
				revisions[dateRange] = new Array(revisionInfo);
			} else {
				revisions[dateRange].push(revisionInfo);
			}

			this._revisionInfosMap[revisionInfo.revisionId] = revisionInfo;	
			revisionInfos[revisionInfo.revisionId] = revisionInfo;	
		}
		revisionInfos = this.sortRevisions(revisionInfos);
		this.initialRevision = revisionInfos[0];
		this._revisionInfos = revisionInfos;
	},
	
	sortRevisions: function(revisionInfos) {
		//let make sure that revisions are sorted properly
		var sortedRevisions = [];
		//find the very latest revision
		for(var latestRevisionId in revisionInfos) {
			if(!revisionInfos[latestRevisionId]['http://jazz.net/xmlns/v0.6#successor']) {
				sortedRevisions.push(revisionInfos[latestRevisionId]);
				var predecessorRevisionId = revisionInfos[latestRevisionId]["http://jazz.net/xmlns/v0.6#predecessor"];
				for(var revisionId in revisionInfos) {
					if (predecessorRevisionId) {
						sortedRevisions.push(revisionInfos[predecessorRevisionId]);
					}
					if (revisionInfos[predecessorRevisionId]) {
						predecessorRevisionId = revisionInfos[predecessorRevisionId]["http://jazz.net/xmlns/v0.6#predecessor"];
					}
				}
				return sortedRevisions;
			}
		}
		return sortedRevisions;
	}
});

})();
