/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.history.RevisionsRetriever");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.history.RevisionsParser");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.history", "RevisionsRetriever");
dojo.require("com.ibm.rdm.web.server.XhrUtils");

var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var RevisionsParser = com.ibm.rdm.web.history.RevisionsParser;
var XhrUtils = com.ibm.rdm.web.server.XhrUtils;

dojo.declare(
	"com.ibm.rdm.web.history.RevisionsRetriever",
	null,
{
	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	
	///////////////////////////////////////////////////////////////////////////
	// API

	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization(
			"com.ibm.rdm.web.search.results", "RevisionsRetriever");
	},

	getRevisions: function(resourceURI) {
		var deferred = new dojo.Deferred();
		var uri = com.ibm.rdm.web.FRONTING_SERVICE_REVISIONS + "?url=" + resourceURI;
		var parser = new RevisionsParser();
		var revisionsDeferred = RDMClient.doGet({
				uri:uri,
				parser: parser
			});
			
		//18351: Web UI:  Last history doesn't work, throws multi-Fetch, Status 401, Status500 type  errors
		//for all uploaded artifact, first revision is an invalid placeholder revision
		revisionsDeferred.addCallback(this, function(revisionsParser){
			if (revisionsParser.doc._placeholderRevisionDeleted) {
				deferred.callback(revisionsParser);
				return;
			}
			var placeholderRevisionIndex = revisionsParser.doc._revisionInfos.length-1;
			var placeholderRevisionId = revisionsParser.doc._revisionInfos[placeholderRevisionIndex].revisionId;
			var placeholderRevisionHeadDeferred = RDMClient.doHead({
				uri: resourceURI,
				revision: placeholderRevisionId
			});
			placeholderRevisionHeadDeferred.addCallback(this, function(placeholderRevisionHeadResponse){
				var contentLength = XhrUtils.getHeader(placeholderRevisionHeadResponse, "Content-Length");
				if (contentLength == "42" && 
					!revisionsParser.doc._placeholderRevisionDeleted &&
					revisionsParser.doc._revisionInfos.length > 1) {
						var placeholderRevisionContentDeferred = RDMClient.doGet({
							uri: resourceURI,
							revision: placeholderRevisionId,
							xhrArgs: {
								handleAs:"text"
							}
						});
						placeholderRevisionContentDeferred.addCallback(this, function(placeholderRevisionContentResponse){
							//in FF dojo.trim() and dojo.string.trim() do not truncate null characters in the string so we have to do it manually
							var nullChar = String.fromCharCode(0);
							var isNullString = false;
							if (!placeholderRevisionContentResponse.doc || dojo.string.trim(placeholderRevisionContentResponse.doc) == "") {
								isNullString = true;
							}
							else {
								for (var i=0; i<placeholderRevisionContentResponse.doc.length; i++) {
									if (placeholderRevisionContentResponse.doc.charAt(i) != nullChar) {
										break;
									}
									else if (i == placeholderRevisionContentResponse.doc.length-1) {
										isNullString = true;
									}
								}
							}
							if (isNullString) {
								if (revisionsParser.doc._revisionInfos[placeholderRevisionIndex].revisionId == placeholderRevisionId){
									revisionsParser.doc._revisionInfos.splice(placeholderRevisionIndex, 1);
								}
								delete revisionsParser.doc._revisionInfosMap[placeholderRevisionId];
								revisionsParser.doc._placeholderRevisionDeleted = true;
							}
							deferred.callback(revisionsParser);
						});
						placeholderRevisionContentDeferred.addErrback(this, function(placeholderRevisionContentErrorResponse){
							deferred.errback(placeholderRevisionContentErrorResponse);
						});
				}
				else {
					deferred.callback(revisionsParser);
				}
				
			});
			placeholderRevisionHeadDeferred.addErrback(this, function(placeholderRevisionHeadErrorResponse){
				deferred.errback(placeholderRevisionHeadErrorResponse);
			});
		});
		return deferred;
	}
});

})();
