/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function(){
dojo.provide("com.ibm.rdm.web.links.Link"); 
dojo.require("com.ibm.rdm.web.links.linkutils");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.common.MimeTypes");

dojo.declare("com.ibm.rdm.web.links.Link", null, {
	
	sourceId: null,  // element id of the source of the link
	sourceDisplayName: null,  // display name for the source of the link
	targetUri: null, 
	uri: null,
	displayName: null, // target display name
	targetDisplayName: null, // used by links explorer
	targetIconUri: null, // icon for link target
	contentType: null, // target content type
	lastModifiedDate: null,
	lastModifiedBy: null,
	userHasPermissionToRedirect: null, 
	linkDescription: null,
	etag: null,
	artifactUri: null, // the artifact that contains the link
	isAbsolutePath:false,
	isExternalLink:false, //flag to determine whether this link points to an external
	                      //URL or to an artifact in the same web application
	isReqPlaceHolderLink: false,  //flag used to determine whether this is a placeholder link that should be
                                  //displayed in the Requirements sidebar section
	
    hasrrmRelationship: false,
	isLinkToValidArtifact :  true, //Determines if the artifact pointed to exists
	_outgoing: null, // flag that indicates if it is an outgoing (vs. incoming) link
	isCALMLink: false,
	_locationInfo: null, 
	messages: null,
	linkType: null,
	
	SKETCH_ID: "sketch:Document",
	STORYBOARD_ID: "story:Document",
	PART_ID: "part:Document",
	SCREEN_FLOW_ID: "flow:Document",
	GLOSSARY_ID: "glossary:glossary",
	PROCESS_DGM_ID: "bpmn20:definitions",
	USECASE_DGM_ID: "usecasecontext:usecasecontext",
	USECASE_ID: "usecase:usecase",
	ACTOR_ID: "actor:actor",
	REQUIREMENT_ID: "requirement:requirement",
	DOCUMENT_ID: "rte:text",
	TERM_ID: "term:term",
	COLLECTION_ID: "rrmCollection:Collection",
	
	constructor: function(params) {
	this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.links", "LinksWidget");
		if (params) {
    		this._outgoing = params._outgoing;
    		this.artifactUri = params.artifactUri;
    	}
		this.userHasPermissionToRedirect = true;
		
	},
	
	init: function(linkNode){
	   
		if (linkNode.getAttribute("href") || linkNode.getAttribute("src") ||
			linkNode.getAttribute("master") || linkNode.getAttribute("ref") ||
			linkNode.getAttribute("href") == "") {

			if( linkNode.getAttribute( "href" ) == "" ) {
				// under what circumstances does this happen?
				resourceURI = this.artifactUri;
			} else {
				resourceURI = linkNode.getAttribute( 
					(linkNode.getAttribute("href"))? "href" : 
						(linkNode.getAttribute("src")) ? "src" :
							(linkNode.getAttribute("master")) ? "master" : "ref");
				
				if(linkNode.getAttribute("rrm:relationship"))
					this.hasrrmRelationship = true;
			}
			// Is this link External or internal?(pointing to an artifact in the same web application
			var isExternalLink = this.isExternalLink(resourceURI);
		
			//Internal URLs are relative. Determine if this is an external URL but examining its authority		
			var authority ="//";	
			var hasAuth = resourceURI.indexOf(authority);	
			this.isAbsolutePath = (hasAuth == -1)?false:true;	
			this.isExternalLink = this.isAbsolutePath;
						
			this.linkDescription = linkNode.getAttribute("title");
			
			if (!this.isExternalLink) {
				// Storing information in this outgoingLinksDescription array to be later joined 
				// with the actual outgoing links result that is returned from the xquery 
				
				// Need to remove # and any thing after that. This happens in Glossary Terms 
				resourceURI = resourceURI.substring( 0, ( resourceURI.indexOf( "#" ) == -1 ) ? resourceURI.length : resourceURI.indexOf( "#" ) );			
				var fullQualifiedURI = com.ibm.rdm.web.util.queryutils.getFullyQualifiedUriFromRelativeLink(com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES,resourceURI);
				this.targetUri = fullQualifiedURI;
				this.uri = this.targetUri;
				this.contentType = "";
			} else {				
				if (this.linkDescription == null) { this.linkDescription = linkNode.getAttribute("alt"); }
				this.displayName = this.linkDescription;
				if (this.displayName == null) this.displayName = resourceURI;
				this.targetUri = resourceURI; 
				this.uri = resourceURI; 		
				this.contentType = com.ibm.rdm.web.common.MimeTypes.LINK_TYPE_EXTERNAL; 		
			}
		} else { 
			// Case when there is an empty link that is a place holder and can only be set in rich client
			this.linkDescription = linkNode.getAttribute("title");
			this.displayName = this.linkDescription;
			this.targetUri = "";
			this.uri = this.linkDescription;
			this.contentType = com.ibm.rdm.web.common.MimeTypes.LINK_TYPE_PLACEHOLDER;
			this._outgoing = false;
			// Place holder links that should be displayed in the Requirements section contain a "rel" attribute
			// For example, rel=reqArtifact. Check if this attribute exists and if so, mark this placeholder link
			// as a requirements place holder link.
			if(linkNode.getAttribute("rel")){
				this.isReqPlaceHolderLink=true;
			}
		}
		
		// set a default Date
        var firstDay = new Date();
        firstDay.setFullYear(0);
        firstDay.setMonth(1);
        firstDay.setDate(1);
        firstDay.setHours(0, 0, 0, 0);
        this.lastModifiedDate = firstDay;
        
		this.documentType = this.contentType;
    	this.targetIconUri = (this.contentType) ? com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(this.contentType) : null;
    	
    	
    	this.sourceId = this.getElementId(linkNode);
		
    	
    	//UIActivities
    	var flowUIActivity = com.ibm.rdm.web.util.queryutils.SCREENFLOW_QUALIFIER +  ":" + com.ibm.rdm.web.util.queryutils.UIACTIVITY_QUALIFIER;
    	if(linkNode.nodeName == flowUIActivity){
			this.linkDescription = linkNode.getAttribute("base:name");
			this.targetDisplayName = this.linkDescription ;
			this.sourceId =  linkNode.getAttribute("key");
			return;
    	}
		//Collection links
		if(linkNode.nodeName.toLowerCase() == "rrm:link"){
			resourceURI =linkNode.getAttribute("href");
			resourceURI = resourceURI.substring( 0, ( resourceURI.indexOf( "#" ) == -1 ) ? resourceURI.length : resourceURI.indexOf( "#" ) );			
			var fullQualifiedURI = com.ibm.rdm.web.util.queryutils.getFullyQualifiedUriFromRelativeLink(com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES,resourceURI);
			hasAuth = resourceURI.indexOf(authority);	
		    this.targetUri = (hasAuth != -1)?resourceURI:fullQualifiedURI;				
			this.uri = this.targetUri;
			this.linkDescription = linkNode.getAttribute("title");
			this.targetDisplayName = this.linkDescription ;
			this.sourceId = com.ibm.rdm.web.links.linkutils.ARTIFACT_SOURCE_ID;
		}
	
		if(this.sourceId == null){
			var userImageNode = com.ibm.rdm.web.util.queryutils.USERINTERFACE_QUALIFIER + ":" + "UserImage";
			if(linkNode.nodeName == "h:img" || linkNode.nodeName == userImageNode)
				this.sourceId = com.ibm.rdm.web.links.linkutils.IMAGE_ID;
			else if(linkNode.nodeName == "rte:diagram")			
				this.sourceId = com.ibm.rdm.web.links.linkutils.EMBEDDED_DIAGRAM_SOURCE_ID;			
			else if(linkNode.nodeName == "sketch:Diagram"){
				var inheritedSketch = linkNode.getAttribute("master");
				if(inheritedSketch)
					this.sourceId = com.ibm.rdm.web.links.linkutils.ARTIFACT_SOURCE_ID
					else
						this.sourceId = com.ibm.rdm.web.links.linkutils.EMBEDDED_DIAGRAM_SOURCE_ID;
			}else if(linkNode.nodeName == "part:Reuse")
				this.sourceId = com.ibm.rdm.web.links.linkutils.EMBEDDED_REUSE_SOURCE_ID;
			else if(linkNode.nodeName == "rte:story")
				this.sourceId = com.ibm.rdm.web.links.linkutils.EMBEDDED_STORYBOARD_SOURCE_ID;
		}
		
	},

	_getHierarchicalKeyId: function (node, documentNodeName) {
		// Build the element id key for sketch and part nodes
		var parentNode = node;
		var elementId = null;
		while (parentNode != null && parentNode.nodeName != documentNodeName) {
			var key = parentNode.getAttribute("key");
			if (key != null) {
				if (elementId == null) {
					elementId = key;
				} else {
					elementId = key + "/" + elementId;
				}
			}
			parentNode = parentNode.parentNode;
		}
		return elementId;
	},
	
	_getSketchElementId: function (linkNode) {
		// Get the source element id for a given link in a Sketch
		var elementId = null;
		switch (linkNode.nodeName) {
		case "base:link" : 	
			var parentNode = linkNode.parentNode;
			elementId = "";
			if (parentNode.nodeName == "sketch:Diagram") {
				elementId = ""; // link from artifact
			} else {
				elementId = this._getHierarchicalKeyId(parentNode, this.SKETCH_ID);
			};
			break;
		case "ui:UserImage" : 	
			var parentNode = linkNode.parentNode;
			elementId = "";
			if (parentNode.nodeName == "sketch:Diagram") {
				elementId = ""; // link from artifact
			} else {
				elementId = this._getHierarchicalKeyId(parentNode, this.SKETCH_ID);
			};
			this.linkType = com.ibm.rdm.web.links.linkutils.IMAGE_ID;
			break;
			
		}
		
		if(linkNode.nodeName != "ui:UserImage"){
			//Adjust hirearchy so that it is in the form parentNode/childNodePart
			if(elementId){
				var splitResult = elementId.split("/");
				var elementId = splitResult[0] + "/" + splitResult[splitResult.length -1];
			}	
		}
		
		
		

		return elementId; 
	},

	_getPartElementId: function (linkNode) {
		// Get the source element id for a given link in a Part
		var elementId = null;
		switch (linkNode.nodeName) {
		case "base:link" : 	
			var parentNode = linkNode.parentNode;
			elementId = "";
			if (parentNode.nodeName == "part:Diagram") {
				elementId = ""; // link from artifact
			} else {
				elementId = this._getHierarchicalKeyId(parentNode, this.PART_ID);
			};
			break;
		case "ui:UserImage" : 	
			var parentNode = linkNode.parentNode;
			elementId = "";
			if (parentNode.nodeName == "part:Diagram") {
				elementId = ""; // link from artifact
			} else {
				elementId = this._getHierarchicalKeyId(parentNode, "part:Diagram");
			};
			this.linkType = com.ibm.rdm.web.links.linkutils.IMAGE_ID;
			break;
		}
		
			//Adjust hirearchy so that it is in the form parentNode/childNodePart
			if(elementId){
				var splitResult = elementId.split("/");
				var elementId = splitResult[0] + "/" + splitResult[splitResult.length -1];
			}		
	
		return elementId;
	},

	_getScreenFlowElementId: function (linkNode) {
		// Get the source element id for a given link in a Screen Flow
		var elementId = null;
		var parentNode = linkNode.parentNode;
		if (parentNode != null) {
			switch (parentNode.nodeName) {
			case "flow:UIActivity" : 
				elementId = parentNode.getAttribute("key");
				break;
			case "flow:Diagram" :
				elementId = "";
				break;
			}
		}
		return elementId;
	},
	
	_getStoryboardElementId: function (linkNode) {
		// TODO - need to implement this when storyboard issues resolved
		return null;
	},
	
	_getUseCaseDiagramElementId: function (linkNode) {
		// Get the source element id for a given link in a Use Case Diagram
		var elementId = null;
		switch (linkNode.nodeName) {
		case "base:link" : 	
			var parentNode = linkNode.parentNode;
			if (parentNode != null) {
				switch (parentNode.nodeName) {
				case this.USECASE_DGM_ID :
					elementId = "";
					break;
				default : 
					elementId = parentNode.getAttribute("base:id");
					break;
				}
			}
			if (elementId != null) break;
		case "usecasecontext:usecase" : 
		case "usecasecontext:actor" : 
			var parentNode = linkNode.parentNode;
			elementId = parentNode.getAttribute("base:id");
			break;
		}
		return elementId;
	},
	
	_getProcessDiagramElementId: function (linkNode) {
		// Get the source element id for a given link in a Process Diagram
		var elementId = null;
		if (linkNode.nodeName == "base:link") {
			elementId = linkNode.getAttribute("id");
			if (elementId == null) {
				var parentNode = linkNode.parentNode;
				if (parentNode != null) {
					switch (parentNode.nodeName) {
					case this.PROCESS_DGM_ID :
						elementId = "";
						break;
					default : 
						elementId = parentNode.getAttribute("base:id");
						if (elementId == null) elementId = parentNode.getAttribute("bpmn20:id");
						break;
					}
				}
			}
		}
		return elementId;
	},
	
	_getRichTextElementId: function (linkNode) {
		// Get the source element id for a given link in a Document 
		var elementId = null;
		switch (linkNode.nodeName) {
		case "rte:diagram" :
			topNode = linkNode; 
			while(topNode.parentNode){				
				topNode = topNode.parentNode;
				if(topNode.getAttribute("id") != null)
				   break;
			}
			elementId = topNode.getAttribute("id");
			this.sourceDisplayName=com.ibm.rdm.web.links.linkutils.EMBEDDED_DIAGRAM_SOURCE_ID;
			break;
		case "h:a" :
			topNode = linkNode; 
			while(topNode.parentNode){				
				topNode = topNode.parentNode;
				if(topNode.getAttribute("id") != null)
				   break;
			}
			elementId = topNode.getAttribute("id");
			break;
		case "h:img" :
			//TODO: Return the image id and handle image highlighting (wrapper resource) 
			break;
		case "rte:richtext" :
			elementId = linkNode.getAttribute("id");
			break;
		case "base:link" : 
			elementId = "";
			break;
		}
		return elementId;
	},
	
	_getTermElementId: function(linkNode) {
		return "";
	},
	
	_getGlossaryElementId: function(linkNode) {
		// Get the source element id for a given link in a Glossary, only links from the 
		// artifact should exist.  Links for a term will exist in the term.
		var elementId = null;
		if (linkNode.nodeName == "base:link") {
			elementId = "";		
		
			//Get the base:name from the collection document.
			topNode = linkNode;
			while(topNode.parentNode && topNode.parentNode.nodeName != "#document") {
				topNode = topNode.parentNode;
			};
			var glossaryName = topNode.getAttribute("base:name");
			this.linkDescription = (glossaryName)?glossaryName:"";
		};
		
		return elementId;
	},
	_getCollectionElementId: function(linkNode) {
		// Get the source element id for a given link in a Glossary, only links from the 
		// artifact should exist.  Links for a term will exist in the term.
		var elementId = null;
		
		if (linkNode.nodeName == "base:link") {
			elementId = "";
		};	
		
		//links in a collection always show the collection name as the source
		//Get the base:name from the collection document.
		topNode = linkNode;
		while(topNode.parentNode && topNode.parentNode.nodeName != "#document") {
			topNode = topNode.parentNode;
		};
		var collectionName = topNode.getAttribute("base:name");
		this.linkDescription = (collectionName)?collectionName:"";			
		return elementId;
	},
	
	getElementId: function (linkNode) {
		// given a particular link dom node, determine the source element id for the link
		var elementId = null;
		var topNode = linkNode;
		while(topNode.parentNode && topNode.parentNode.nodeName != "#document") {
			topNode = topNode.parentNode;
		};
		// determine the id based on the document type that contains the link
		switch (topNode.nodeName) {
			case this.SKETCH_ID : 
				elementId = this._getSketchElementId(linkNode);
				break;
			case this.SCREEN_FLOW_ID :
				elementId = this._getScreenFlowElementId(linkNode);
				break;
			case this.USECASE_DGM_ID :
				elementId = this._getUseCaseDiagramElementId(linkNode);
				break;
			case this.PROCESS_DGM_ID :
				elementId = this._getProcessDiagramElementId(linkNode);
				break;
			case this.DOCUMENT_ID :
			case this.USECASE_ID :
			case this.ACTOR_ID :
			case this.REQUIREMENT_ID :
				elementId = this._getRichTextElementId(linkNode);
				break;
			case this.PART_ID :
				elementId = this._getPartElementId(linkNode);
				break;
			case this.TERM_ID :
				elementId = this._getTermElementId(linkNode);
				break;
			case this.GLOSSARY_ID :
				elementId = this._getGlossaryElementId(linkNode);
				break;
			case this.COLLECTION_ID:
				elementId = this._getCollectionElementId(linkNode); 
		};
		if (elementId == "") { elementId = com.ibm.rdm.web.links.linkutils.ARTIFACT_SOURCE_ID; };
		return elementId;
	},
	
	
	getID: function() {
		return this.id;
	},
	
	isExternalLink: function(uri) {
		
		if (!this._locationInfo) {
			this._locationInfo = com.ibm.rdm.web.links.linkutils.getURLInfo(uri);
		}
		return !this._locationInfo.sameLocation;
	},
	
	isPlaceHolderLink: function() {
		return this.contentType == com.ibm.rdm.web.common.MimeTypes.LINK_TYPE_PLACEHOLDER;
	},
	
	isOutgoingLink: function() {
		return this._outgoing;
	},
	
	isIncomingLink: function() {
		return !this._outgoing;
	}
});

})();
