/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
dojo.provide("com.ibm.rdm.web.links.LinksWidget");

dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.layout.ContentPane");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.links.linkutils");
dojo.require("com.ibm.rdm.web.links.LinkCache");
dojo.require("com.ibm.rdm.web.links.LinkRowWidget");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.search.results.SearchHeader");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("dojo.i18n");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.Form");
dojo.require("dijit.Menu");
dojo.require("dijit.Toolbar");
dojo.require("com.ibm.rdm.web.common.CollapseableTitleSection");
//dojo.require("com.ibm.rdm.web.links.GraphicalLinksExplorer");
dojo.require("com.ibm.rdm.web.links.RequirementsFilterWidget");
dojo.require("com.ibm.rdm.web.links.FilterCheckedMenuItem");
dojo.requireLocalization("com.ibm.rdm.web.search", "common");
dojo.requireLocalization("com.ibm.rdm.web.links", "LinksWidget");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.commenting.EventTopics");

var queryutils = com.ibm.rdm.web.util.queryutils;
var domutils = com.ibm.rdm.web.util.domutils;
var MAX_CHAR = 9;

LinkFilterTypes = {
	ARTIFACT_TYPES: 0,
	MIME_TYPES: 1,
	OTHER_TYPES: 2
};

/**
 * Main links widget. 
*/
dojo.declare("com.ibm.rdm.web.links.LinksWidget", [dijit._Widget, dijit._Templated], {
    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "links/templates/LinksWidget.html"),

	widgetsInTemplate: true,

    ///////////////////////////////////////////////////////////////////////////////////
    // Public fields

    viewer: null,
    resourceURI: null,
    ioArgs: null,
    viewType: null,
    doc:null,
	hasStarted: null,

	///////////////////////////////////////////////////////////////////////////////////
	// NLS Fields

	sortByName: null,
	sortByType: null,
	sortByLastModifiedDate: null,
	
    ///////////////////////////////////////////////////////////////////////////////////
    // Private fields

	_messages: null,
    _commonMessages: null,
	
	_filterWidget: null,  // used by the Requirements section
	_filterMenu: null,  // used by the Links section
	_sidebarSection: null,
	_linkRowsWidgets: null,
	
	_incomingLinkRowWidgets: null,
	_outgoingLinkRowWidgets: null,
	_outgoingCALMLinkRowWidgets: null,
	_incomingLinks: null,
	_outgoingLinks: null,
	// this._outgoingLinks.length is not a reliable way of getting the total number of outgoing links
	// since placeholder links can throw off the total.
	_totalOutgoingLinks: 0, 
	_totalIncomingLinks: 0,  
	
	_resourceContentType: null,
	
	_isOutgoingLinksLoading: null,
	_isIncomingLinksLoading: null,
	_isOutgoingCALMLinkLoading: null,
	_isOutgoingCALMValidatedByLinkLoading: null,
	
	_resultsSortBy: null,
	_doc: null,
	_etags: null,
	
	_isLinkCountDisplayed: null,
	_onHeaderCountScope: null,
	_onHeaderCountComplete: null,
	
	_showAllArtifactTypesWidget: null,
	_showAllMimeTypesWidget: null,
	
    postCreate: function() {
    	var self = this;
        this._resultsSortBy = com.ibm.rdm.web.links.LinksWidget.sorts.NAME;
    	this.inherited("postCreate", arguments);
		this._linkRowsWidgets = [];
		this._incomingLinkRowWidgets = [];
		this._outgoingLinkRowWidgets = [];
		this._outoingCALMLinkRowWidgets = [];
		this._outgoingCALMValidatedLinkRowWidget = [];
		if (this.viewType == com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS) {
//			this._showLinksExplorerNode.destroyRecursive();
			this._sortByTypeNode.destroyRecursive();
		} else {
//			this.connect(this._showLinksExplorerNode, "onClick", this._onShowLinksExplorer);
			this.connect(this._sortByTypeNode, "onClick", this.handleSortByType );
			if (this._isDisplayIncomingLinks()) {
				this._incomingLinksCollapseableHeader = new com.ibm.rdm.web.common.CollapseableTitleSection({
					sectionTitle: this._messages.loading
				}, this._showingIncomingLinkCountNode);
			}
			this._outgoingLinksCollapseableHeader = new com.ibm.rdm.web.common.CollapseableTitleSection({
				sectionTitle: this._messages.loading
			}, this._showingOutgoingLinkCountNode);
			
		}
		if (this.viewType == com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS) {
			this._linkToolbarNode.removeChild(this._linkToolbarFilterDropdownNode);
			this.connect(this._filterToggleNode, "onClick", this._onFilterNodeClick);
		} else {
			// TODO - would be better to create the menus on popup, need to figure out how to, 
			// PopupMenuItem doesn't respond to onClick
			this._linkToolbarNode.removeChild(this._filterToggleNode);
	 		this._filterMenu = this._filterTypesMenu;
			this._createArtifactTypesMenu();
			this._createMimeTypesMenu();
			this._createOtherTypesMenu();
		};
		//this.connect(this._locateInDocToggleNode, "onClick", this._onShowLinksNodeClick );
		this.connect(this._sortByNameNode, "onClick", this.handleSortByName );
		this.connect(this._sortByDateNode, "onClick", this.handleSortByDate);
		// Accessibility
		if (this.viewType == com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS) {
			this._linkToolbarSortDropdownNode.setAttribute("label", this._messages["sortReqs"]);
			this._linkToolbarSortDropdownNode.setAttribute("title", this._messages["sortReqsTooltip"]);
		} else {
			this._linkToolbarSortDropdownNode.setAttribute("label", this._messages["sortLinks"]);
			this._linkToolbarSortDropdownNode.setAttribute("title", this._messages["sortLinksTooltip"]);
		}
    },

    startup: function() {
    	//TODO: Hook up to the deferred from the ResourceViewer here in order  make sure the LinksWidget is instantiated
    	//after the main document has been processed.
    	//this._resourceLoadDeferred.addCallback(new function(resource) {
    	//this.doc = resource;
	    //	if (!this.hasStarted) {
	    		this.hasStarted = true;
		    	this.getLinks(this.ioArgs);
		    	this.inherited("startup", arguments);
	    //	}
    	//});
    },
    
    postMixInProperties: function() {
		this._commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
        this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.links", "LinksWidget");
    },
	
    destroy: function() {
    	if (this._filterWidget) {
    		this._filterWidget.destroyRecursive();
    	}
    	if (this._filterMenu) {
    		this._filterMenu.destroyRecursive();
    	}
		dojo.forEach( this._linkRowsWidgets, function( widget ) {
			widget.destroyRecursive();
		});
		this.inherited(arguments);
    },
    
    containerDimensionsHaveChanged: function() {
    	if (this.domNode.parentNode) {
	        var height = this.domNode.parentNode.clientHeight;
	        var width = this.domNode.parentNode.clientWidth;
	        var contentPaneTop = (
	            this._contentPane.domNode.offsetTop
	            - this._contentPane.domNode.parentNode.offsetTop);
	        var scrollHeightToUse = (height - contentPaneTop);
	        this._contentPane.resize({ h: scrollHeightToUse, w: width });
    	}
    },
    
    _createArtifactTypesMenu: function() {
    	var self = this;
    	this._createShowAllCheckbox(this._artifactTypesMenu, LinkFilterTypes.ARTIFACT_TYPES);
    	dojo.forEach(com.ibm.rdm.web.common.MimeTypes.ARTIFACTS_ORDERED, function(resourceType) {
    		self._createFilterCheckboxMenuItem(self._artifactTypesMenu, resourceType, LinkFilterTypes.ARTIFACT_TYPES);
    	});
    },
    
    _createMimeTypesMenu: function() {
    	var self = this;
    	this._createShowAllCheckbox(this._mimeTypesMenu, LinkFilterTypes.MIME_TYPES);
    	var comparator = function(type1, type2) {
    		return ((type1.name.toLowerCase() < type2.name.toLowerCase()) ? -1 : 1);
    	};
    	com.ibm.rdm.web.common.MimeTypes.ABBREVIATED_WRAPPERS.sort(comparator);
    	dojo.forEach(com.ibm.rdm.web.common.MimeTypes.ABBREVIATED_WRAPPERS, function(resourceType) {
    		self._createFilterCheckboxMenuItem(self._mimeTypesMenu, resourceType, LinkFilterTypes.MIME_TYPES);
    	});
    },
    
    _createOtherTypesMenu: function() {
    	var self = this;
    	dojo.forEach(com.ibm.rdm.web.common.MimeTypes.LINK_TYPES, function(resourceType) {
    		self._createFilterCheckboxMenuItem(self._otherTypesMenu, resourceType, LinkFilterTypes.OTHER_TYPES);
    	});
    },
    
    _createShowAllCheckbox: function(subMenu, filterType) {
		// viewers are cached, so create a unique id
    	var resourceURI = this.viewer.resourceURI;
		var index = resourceURI.indexOf("rrc");
		id = resourceURI.substring(index);
		var showAllMenuItem = new com.ibm.rdm.web.links.FilterCheckedMenuItem({type: filterType, label: this._messages.showAll, checked: true, disabled: true});
		switch (filterType) {
		case LinkFilterTypes.ARTIFACT_TYPES :
			this._showAllArtifactTypesWidget = showAllMenuItem;
	    	break;
		case LinkFilterTypes.MIME_TYPES :
			this._showAllMimeTypesWidget = showAllMenuItem;
	    	break;
		};
		
		showAllMenuItem.contentType = null;
		subMenu.addChild(showAllMenuItem);
	    this.connect(showAllMenuItem, "onChange", function(event) {
	    	this._onShowAllChanged(showAllMenuItem);
	    });
		var separatorMenuItem = new dijit.MenuSeparator();
		separatorMenuItem.contentType = null;
		subMenu.addChild(separatorMenuItem);
    },
    
    _onShowAllChanged: function(menuItem) {
    	var menu = null;
		switch (menuItem.type) {
		case LinkFilterTypes.ARTIFACT_TYPES :
			menu = this._artifactTypesMenu;
	    	break;
		case LinkFilterTypes.MIME_TYPES :
			menu = this._mimeTypesMenu;
	    	break;
		};
		this._showAllItems(menu);
		menuItem.attr("disabled", true);
    },
    
    _createFilterCheckboxMenuItem: function(subMenu, resourceType, filterType) {
        var imageURI = com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(resourceType.contentType);
		var menuItem = new com.ibm.rdm.web.links.FilterCheckedMenuItem({label: resourceType.name, checked: true, iconURL: imageURI});
		subMenu.addChild(menuItem);
		menuItem.contentType = resourceType.contentType;
	    this.connect(menuItem, "onChange", function(event) {
	    	this._onFilterCheckBoxChanged(filterType, menuItem);
	    });
    },
	
    _onFilterCheckBoxChanged: function(filterType, menuItem) {
		var widget = null;
		switch (filterType) {
		case LinkFilterTypes.ARTIFACT_TYPES :
			widget = this._showAllArtifactTypesWidget;
	    	break;
		case LinkFilterTypes.MIME_TYPES :
			widget = this._showAllMimeTypesWidget;
	    	break;
		};
    	if (menuItem.checked) {
    		// if all items are now checked, disable the show all button
    		var menu = null;
    		switch (filterType) {
    		case LinkFilterTypes.ARTIFACT_TYPES :
    			menu = this._artifactTypesMenu;
    	    	break;
    		case LinkFilterTypes.MIME_TYPES :
    			menu = this._mimeTypesMenu;
    	    	break;
    		}
    		if (menu != null && this._areAllItemsChecked(menu)) {
        		widget.attr("disabled", true);
        		widget.attr("checked", true);
    		}
    	} else {
    		// enable the show all button
    		if (widget != null) {
    			widget.attr("disabled", false);
    			widget.attr("checked", false);
    		}
    	}
	    this._applyFilter(menuItem);
    },
    
    _showAllItems: function(menu) {
    	var self = this;
	    dojo.forEach(menu.getChildren(), function(menuItem) {
    		// skip show all and separator items
    		if ((menuItem.contentType != null) && !menuItem.checked) {
    			menuItem.attr("checked", true);
    			self._applyFilter(menuItem);
    		}
	    });
    },
	
    _areAllItemsChecked: function(menu) {
	    var allChecked = true;
    	dojo.every(menu.getChildren(), function(menuItem) {
    		// skip show all and separator items
    		if ((menuItem.contentType != null) && !menuItem.checked) {
    			allChecked = false;
    			return false; //terminate dojo.every
    		}
    		return true; //carry on the every loop
    	});
    	return allChecked;
    },
    
    _isImageType: function(contentType) {
	    var isImage = false;
    	dojo.every(com.ibm.rdm.web.common.MimeTypes.IMAGE_TYPES, function(type) {
    		if (type.contentType == contentType) {
    			isImage = true;
    			return false; //terminate dojo.every
    		}
    		return true; //carry on the every loop
    	});
    	return isImage;
    },
    
    _isVideoType: function(contentType) {
	    var isImage = false;
    	dojo.every(com.ibm.rdm.web.common.MimeTypes.VIDEO_TYPES, function(type) {
    		if (type.contentType == contentType) {
    			isImage = true;
    			return false; //terminate dojo.every
    		}
    		return true; //carry on the every loop
    	});
    	return isImage;
    },
    
    _applyFilter: function(checkedItem) {
	    // Update the links based on the value of the given checkbox item.
    	var self = this;
	    var incomingOutgoingRowWidgets = this.getLinkRowWidgets();
	    dojo.forEach( incomingOutgoingRowWidgets, function( rows ) {
		    dojo.forEach(rows, function( row ) {
		    	if(checkedItem.contentType == com.ibm.rdm.web.common.MimeTypes.ALL_IMAGE_TYPES.contentType) {
		    		if (self._isImageType(row.contentType)) {
		    			if(checkedItem.checked) {
		    				row.unhide();
		    			} else {
		    				row.hide();
		    			}
		    		}
		    	} else if(checkedItem.contentType == com.ibm.rdm.web.common.MimeTypes.ALL_VIDEO_TYPES.contentType) {
		    		if (self._isVideoType(row.contentType)) {
		    			if(checkedItem.checked) {
		    				row.unhide();
		    			} else {
		    				row.hide();
		    			}
		    		}
		    	} else {
		    		if(row.contentType == checkedItem.contentType) {
		    			if(checkedItem.checked) {
		    				row.unhide();
		    			} else {
		    				row.hide();
		    			}
		    		}
		    	}
		    });
	   });
	   this.updateLinkCountHeader();
    },

    _applyActiveMenuFilter: function(linkRowWidgets) {
    	var self = this;
    	var contentTypes = new Array();
    	// Gather active types
	    dojo.forEach(this._artifactTypesMenu.getChildren(), function(menuItem) {
	    	// skip show all and separator items
	    	if (menuItem.contentType != null && menuItem.checked) {
	    		contentTypes.push(menuItem.contentType);
	    	}
	    });
	    dojo.forEach(this._mimeTypesMenu.getChildren(), function(menuItem) {
	    	// skip show all and separator items
	    	if (menuItem.contentType != null && menuItem.checked) {
	    		if (menuItem.contentType == com.ibm.rdm.web.common.MimeTypes.ALL_IMAGE_TYPES.contentType) {
	    			dojo.forEach(com.ibm.rdm.web.common.MimeTypes.IMAGE_TYPES, function(type) {
	    	    		contentTypes.push(type.contentType);
	    	    	});
	    		} else if (menuItem.contentType == com.ibm.rdm.web.common.MimeTypes.ALL_VIDEO_TYPES.contentType) {
	    			dojo.forEach(com.ibm.rdm.web.common.MimeTypes.VIDEO_TYPES, function(type) {
	    	    		contentTypes.push(type.contentType);
	    	    	});
	    		} else {
		    		contentTypes.push(menuItem.contentType);
	    		}
	    	}
	    });
	    dojo.forEach(this._otherTypesMenu.getChildren(), function(menuItem) {
	    	if (menuItem.checked) {
	    		contentTypes.push(menuItem.contentType);
	    	}
	    });
	    // apply the active filter
	    dojo.forEach(linkRowWidgets, function(row) {
			var found = false;
		    dojo.every(contentTypes, function(type) {
		    	if (type == row.contentType || row.hasrrmRelationship) {
		    		found = true;
		    		return false; //terminate dojo.every
		    	}
		    	return true; //carry on the every loop
		    });
		    if (found) {
		    	row.unhide();
		    } else {
		    	row.hide();
		    }
	    });
    },

    _isDisplayIncomingLinks: function() {
    	
    	if (this.viewType == com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS) 
    		return false;
    	if(this.viewer.isInReviewMode()){   
    		if(this.viewer.isInInformalReviewMode())
    			return true;
    		else
    			return false;
    	}
    	if(this.viewer.viewType == com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactHistory)
    		return false;	
      	
    	//Do display incoming links
    	return true;
    },
    
    _isDisplayOutoingCALMLinks:function(){
    	return true;
    	
    },
	setSidebarSection: function(sidebar) {
    	this._sidebarSection = sidebar;
    },
    
    expandSidebar: function() {
    	this._sidebarSection.expandSection();
    },
    
    handleSortByName: function() {
    	this._resultsSortBy = com.ibm.rdm.web.links.LinksWidget.sorts.NAME;
    	this._displayOutgoingLinks( this._outgoingLinks );
    	if (this._isDisplayIncomingLinks()) {
    		this._displayIncomingLinks( this._incomingLinks );
    	}
    },
    
    handleSortByType: function() {
    	this._resultsSortBy = com.ibm.rdm.web.links.LinksWidget.sorts.TYPE;
    	this._displayOutgoingLinks( this._outgoingLinks );
    	if (this._isDisplayIncomingLinks()) {
    		this._displayIncomingLinks( this._incomingLinks );
    	}
    },
    
    handleSortByDate: function() {
    	this._resultsSortBy = com.ibm.rdm.web.links.LinksWidget.sorts.DATE;
    	this._displayOutgoingLinks( this._outgoingLinks );
    	if (this._isDisplayIncomingLinks()) {
    		this._displayIncomingLinks( this._incomingLinks );
    	}
    },
    
    dateSort: function(links) {
		links.sort(com.ibm.rdm.web.links.LinksWidget.dateComparator);
		links.reverse();
		dojo.removeClass(this._sortByNameNode.iconNode, "commentingToolbarDropdownBullet");
		dojo.removeClass(this._sortByTypeNode.iconNode, "commentingToolbarDropdownBullet");
		dojo.addClass(this._sortByDateNode.iconNode, "commentingToolbarDropdownBullet");
    },

    nameSort: function( links ) {
		links.sort(com.ibm.rdm.web.links.LinksWidget.nameComparator);
        dojo.removeClass(this._sortByTypeNode.iconNode, "commentingToolbarDropdownBullet");
        dojo.removeClass(this._sortByDateNode.iconNode, "commentingToolbarDropdownBullet");
		dojo.addClass(this._sortByNameNode.iconNode, "commentingToolbarDropdownBullet");
    },
    
    typeSort: function( links ) {
		var commonMessages = this._commonMessages;
		var typeComparator = function(linkObj1, linkObj2) {
			var contentType1 = com.ibm.rdm.web.common.MimeTypeRegistry.getMimeTypeDisplayName(linkObj1.contentType);
			var contentType2 = com.ibm.rdm.web.common.MimeTypeRegistry.getMimeTypeDisplayName(linkObj2.contentType);
			return (contentType1 == contentType2) ?
					com.ibm.rdm.web.links.LinksWidget.nameComparator(linkObj1, linkObj2) 
					: ((contentType1 < contentType2) ? -1 : 1);
		};
		links.sort(typeComparator);
		dojo.removeClass(this._sortByNameNode.iconNode, "commentingToolbarDropdownBullet");
		dojo.removeClass(this._sortByDateNode.iconNode, "commentingToolbarDropdownBullet");
		dojo.addClass(this._sortByTypeNode.iconNode, "commentingToolbarDropdownBullet");
    },
    
    getLinks: function(ioArgs) {
    	this.setLoadingMessage();
    	domutils.createHourGlassCursor();
    	this._isOutgoingLinksLoading = true;
    	if (this._isDisplayIncomingLinks()) {
        	this._isIncomingLinksLoading = true;
    	}
    	this._isOutgoingCALMLinkLoading = true;
    	this._isOutgoingCALMValidatedByLinkLoading = true;

		var isWrapperResource = com.ibm.rdm.web.common.MimeTypeRegistry.isWrapperResource(this.viewer._contentType);
		var revision = null;
		if (isWrapperResource) {
			// TODO (LPK) - not working properly, returning head version
			revision = this.viewer._revision;//this.viewer._wrapperInfo.etag;
		} else {
			revision = this.viewer.revision;
		}
    	var outDeferred = com.ibm.rdm.web.links.LinkCache.fetchOutgoingLinks(this.resourceURI, ioArgs, false, revision, this.doc, this.viewer.getContentType());
    	outDeferred.addCallback(this, this._processFetchedOutgoingLinks);
    	if (this._isDisplayIncomingLinks()) {
	    	var inDeferred = com.ibm.rdm.web.links.LinkCache.fetchIncomingLinks(this.resourceURI, this._resourceContentType);
	    	inDeferred.addCallback(this, this._displayIncomingLinks);
    	}
    	if (this._isDisplayOutoingCALMLinks()) {
	    	var outCALMDeferred = com.ibm.rdm.web.links.LinkCache.fetchOutgoingCALMLinks(this.resourceURI, this._resourceContentType);
	    	outCALMDeferred.addCallback(this, this._displayOutgoingCALMLinks);
    	}
    },
    
    getLinkCount: function(callbackFn, callbackScope) {
    	this._onHeaderCountComplete = callbackFn;
    	this._onHeaderCountScope = callbackScope;
    	if (this._isLinkCountDisplayed) {
    		this._onHeaderCountComplete.call(this._onHeaderCountScope, this._totalOutgoingLinks);
    	}
    },
    
    getLinkRowWidgets: function() {
    	if(!this._isDisplayIncomingLinks()) {
    		return [ this._outgoingLinkRowWidgets ];	
    	} else {
    		return [ this._outgoingLinkRowWidgets, this._incomingLinkRowWidgets ];
    	}
    },
    
    setProjectURI: function(projURI) {
    	this.projectURI = projURI;
    	if (this._filterWidget) {
    		this._filterWidget.createTeamMembersSelectOption();
    	}
    },
    
    _processFetchedOutgoingLinks: function(links) {
    	
    	// Special case handling for storyboard resources
    	var resourceViewer = (this.viewer._resourceViewerWidget._resourceViewerWidget)?
			this.viewer._resourceViewerWidget._resourceViewerWidget:
			this.viewer._resourceViewerWidget;
    	
    	if (resourceViewer && (resourceViewer._documentHasBeenLoaded || this.viewer._resourceViewerWidget._frameInfos)) {
    		// the resource viewer has been loaded, so the source element information is now available
            if (this._updateTimeout) {
            	clearTimeout(this._updateTimeout);
            	this._updateTimeout = null;
            }
            for(var i = 0; i < links.length; i++) {
            	link = links[i];
            	if(link.sourceId != com.ibm.rdm.web.links.linkutils.ARTIFACT_SOURCE_ID){//Set the sourceDisplayName
            		if (link.sourceId == com.ibm.rdm.web.links.linkutils.IMAGE_ID) 
            			link.sourceDisplayName = this._messages.image;
            		else if(link.sourceId == com.ibm.rdm.web.links.linkutils.EMBEDDED_DIAGRAM_SOURCE_ID)
            			link.sourceDisplayName = this._messages.embeddedDiagram;
            		else if(link.sourceId == com.ibm.rdm.web.links.linkutils.EMBEDDED_REUSE_SOURCE_ID)
            			link.sourceDisplayName = this._messages.reuse;
            		else if(link.sourceId == com.ibm.rdm.web.links.linkutils.EMBEDDED_STORYBOARD_SOURCE_ID)
            			link.sourceDisplayName = this._messages.embeddedStoryboard;         
            		else{
            			if (!link.sourceDisplayName)
            				link.sourceDisplayName = this.viewer.getElementDisplayName(link.sourceId)
            			else{
            				if(link.sourceDisplayName == com.ibm.rdm.web.links.linkutils.EMBEDDED_DIAGRAM_SOURCE_ID)
            					link.sourceDisplayName = this._messages.embeddedDiagram;
            			}       				     
            		}
            	}
            	if (link.sourceDisplayName == null || link.sourceDisplayName == "") { link.sourceDisplayName = this._messages.sourceUnknown};
            	
            	if(link.linkType == com.ibm.rdm.web.links.linkutils.IMAGE_ID){
            		link.sourceDisplayName = this._messages.image;
            	}

            }
    		this._displayOutgoingLinks(links);
    	} else {
    		// use a timer to wait for the resource viewer to load
    		if (this._updateTimeout) {
            	clearTimeout(this._updateTimeout);
            	this._updateTimeout = null;
            }
    		this._updateTimeout = setTimeout(
   	             dojo.hitch(this, function() {
   	            	 this._processFetchedOutgoingLinks(links);
   	             }), 500);
    	}
    	
    	domutils.createDefaultCursor();
    },
    
    closeReqsFilter: function() {
	    this._filterWidget.hide();
	    this._filterToggleNode.checked = false;
	    this._filterToggleNode.focus();
	},
    
    /////////////////////////////////////////////////////////////////////////////// 
    // Method that shows the filter widget
    _onFilterNodeClick: function() {
    	if (this._filterToggleNode.checked) {
	    	if (!this._filterWidget) {
	    		this._filterWidget = new com.ibm.rdm.web.links.RequirementsFilterWidget({
	    			_linksWidget: this
	    		}, this._filterWidgetNode);
	    	}
    	    this._filterWidget.unhide();
    	} else {
    		if (this._filterWidget) {
	    		this._filterWidget.hide();
    		}
	    }
    },
    
    _onShowLinksExplorer: function() {
    	var self = this;
    	var node = document.createElement("div"); 
    	node.setAttribute("id", "container");        
    	var onComplete = function(size) {
    	  
    		node.removeChild(loadingNode);
    		size.w = size.w + 120;
    		size.h = size.h + 65;
    		if (size.w >= document.body.clientWidth * 0.8) {
    			size.w = document.body.clientWidth * 0.8;
    		}
    		if (size.h >= document.body.clientHeight * 0.8) {
    			size.h = document.body.clientHeight * 0.8;
    		}
    		
    	   	
	   	    // For some reason, in IE, if the node contains VML, Dialog.setContent() does not work.
            //linksExplorerDialog.containerNode.appendChild(node);
         
            linksExplorerDialog.resize(size);

            // dijit.Dialog puts the scrollbar to the entire Dialog, which also includes the title. Make only Dialog's containerNode scroll
            
            if (!dojo.isIE){
	            
	            linksExplorerDialog.domNode.style.overflow="hidden";
	            linksExplorerDialog.containerNode.style.overflow = "hidden";
	            	         	            
	            node.style.border="solid #AEC9F7 2px";
	            node.style.overflow="auto";
	            node.style.height = size.h - 56 + "px";
	            node.style.width = size.w - 33 + "px";
            }
            linksExplorerDialog.layout();
           
            
    	};

        var loadingNode = document.createElement("div");
        var loadingImgNode = document.createElement("img");
        loadingImgNode.src = "com.ibm.rdm.web/common/images/loading.gif";
        loadingNode.style.marginLeft="80px";       
    	loadingNode.appendChild(loadingImgNode);
        node.appendChild(loadingNode);
    	var linksExplorerDialog = new dijit.Dialog({ title: this._messages.linksExplorerTitle });
    	dojo.body().appendChild(linksExplorerDialog.domNode);
    	linksExplorerDialog.startup();
    	
    
        linksExplorerDialog.setContent(node);
    	linksExplorerDialog.show();
    	var extResourceURI = "/jazz/resources/rdm/extendedResources/"
    		+ this.resourceURI.substring(this.resourceURI.lastIndexOf("/") + 1);
    	/*var deferred = com.ibm.rdm.web.common.ResourceCache.getResource(extResourceURI);
    	deferred.addCallback(function(response){
       		var displayName = response.doc.getElementsByTagName("name")[0].firstChild.nodeValue;
       		if (displayName.length > MAX_CHAR) {
       			displayName = dojo.string.substitute(this._messages.more, [displayName.substring(0, MAX_CHAR)]);
       		}
            var contentType = response.doc.getElementsByTagName("format")[0].firstChild.nodeValue;
           	var iconUri = com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(contentType);
           	com.ibm.rdm.web.links.GraphicalLinksExplorer.createGraph(node, self.resourceURI,
           			{url: self.resourceURI,iconUri: iconUri, label: displayName}, onComplete, this);
    		return response;
    	});*/
    },

    _onShowLinksNodeClick: function() {
    	if (this._locateInDocToggleNode &&
    			this._locateInDocToggleNode.checked) {
    		this._locateLinksInDocument();
    	} else {
    		this._clearLinksInDocument();
    	}
    },
    
    _locateLinksInDocument: function() {
/*        var result = {};
        var self = this;
        dojo.forEach( this._outgoingLinks, function( link ) {
        	if( self._isForRequirements ) {
        		if( link.getFormat() == com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REQUIREMENT ) {
		        	if( !result[ link.getID() ] ) {
		        		result[ link.getID() ] = 1;
		        	} else {
		        		result[ link.getID() ] += 1;
		        	}
        		}
        	} else {
        		if( link.getFormat() != com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REQUIREMENT ) {
		        	if( !result[ link.getID() ] ) {
		        		result[ link.getID() ] = 1;
		        	} else {
		        		result[ link.getID() ] += 1;
		        	}
        		}
        	}
        });
        dojo.publish(com.ibm.rdm.web.commenting.EventTopics.LOCATE_LINKED_LOCATIONS_IN_RESOURCE, [result]);*/
    },
    
    _clearLinksInDocument: function() {
        var result = {};
        dojo.publish(com.ibm.rdm.web.commenting.EventTopics.LOCATE_LINKED_LOCATIONS_IN_RESOURCE, [result]);
    },


    _displayOutgoingLinks: function(links) {
    	
    	this._outgoingLinks = [];
    	this._totalOutgoingLinks =0; //Reset the counter
    	for(var i = 0; i < links.length; i++) {
    		var link = links[i];
    		//Make sure we do not display any links that point to deleted resources.    		
    		if(link.isLinkToValidArtifact){
    			if(this.viewType === com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS) {
    				if( link.contentType == com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REQUIREMENT || 
    						link.contentType == com.ibm.rdm.web.common.MimeTypes.LINK_TYPE_PLACEHOLDER) {
    					this._outgoingLinks.push(link);
    				}
    			} else {
    				if(link.contentType != com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REQUIREMENT) {
    					this._outgoingLinks.push(link);
    				}
    			}
    		}
    	}
    	this._isOutgoingLinksLoading = false;
    	this._displayLinks(this._outgoingLinks, this._outgoingLinkRowWidgets, this._linkedOutgoingResultNode, true, true);
    	return links;
    },
    
    _displayOutgoingCALMLinks: function (links){
    	   	
    	if(this.viewType != com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS){
    		if(links["ImplementedBy"].length > 0){ //Create section for Implemented By links
    			this._outgoingCALMLinksCollapseableHeader = new com.ibm.rdm.web.common.CollapseableTitleSection({
    				sectionTitle: this._messages.loading
    			}, this._showingOutgoingCALMLinkCountNode);
    		}

    		if(links["ValidatedBy"].length > 0){  //Create section for Validated By links
    			this._outgoingCALMValidatedByLinksCollapseableHeader = new com.ibm.rdm.web.common.CollapseableTitleSection({
    				sectionTitle: this._messages.loading
    			}, this._showingOutgoingCALMValidatedLinkCountNode);
    		}

    		domutils.createDefaultCursor();

    		if(links["ImplementedBy"].length > 0){
    			this._isOutgoingCALMLinkLoading = false;
    			this._displayCALMLinks(links["ImplementedBy"], this._outoingCALMLinkRowWidgets, this._linkedCALMResultNode, "ImplementedBy");
    		}
    		if(links["ValidatedBy"].length > 0){
    			this._isOutgoingCALMValidatedByLinkLoading = false;
    			this._displayCALMLinks(links["ValidatedBy"], this._outgoingCALMValidatedLinkRowWidget, this._linkedCALMValidatedResultNode, "ValidatedBy");
    		}
    	}
    },
    
    _displayIncomingLinks: function(links) {
    	this._incomingLinks = (links)? links : this._incomingLinks;
    	this._isIncomingLinksLoading = false;
    	this._totalIncomingLinks = 0; //Reset the counter
    	domutils.createDefaultCursor();
    	this._displayLinks( this._incomingLinks, this._incomingLinkRowWidgets, this._linkedIncomingResultNode, true, false);
    	return links;
    },

    _displayCALMLinks: function(links, linkRowWidgets, resultNode,calmLinkType){
 
		dojo.forEach(linkRowWidgets, function(widget) {
			widget.destroyRecursive();
		});
		linkRowWidgets.length = 0;
		com.ibm.rdm.web.util.domutils.removeChildNodes(resultNode);

    	var table = document.createElement("table");
        var tbody = document.createElement("tbody");
        table.appendChild(tbody);
		
		for( var i = 0; i < links.length; i++ ) {
			
			var row = new com.ibm.rdm.web.links.LinkRowWidget({
				_linkObj: links[i], 
				_artifactURI: this.viewer.resourceURI,
				resourceViewer: this.viewer
			});
			linkRowWidgets.push( row );
			tbody.appendChild(row.domNode);	

    	}
    	resultNode.appendChild(table);
    	
    	if(calmLinkType == "ImplementedBy"){
    		this._outgoingCALMLinksCollapseableHeader.setTitle(this._messages.implementedBy);
         	this._outgoingCALMLinksCollapseableHeader.setContent([this._linkedCALMResultNode]);
    	}else if(calmLinkType == "ValidatedBy"){
    		this._outgoingCALMValidatedByLinksCollapseableHeader.setTitle(this._messages.validatedBy);
         	this._outgoingCALMValidatedByLinksCollapseableHeader.setContent([this._linkedCALMValidatedResultNode]);
    	} 
    },
    /////////////////////////////////////////////////////////////////////////////// 
    // Constructs a table with each row having a artifact icon along with the name
    // and link to the articaft that is linked. If it is outgoing link then it attaches a 
    // description from where it came
    _displayLinks: function(links, linkRowWidgets, resultNode, showReq, isOutgoingLinkType) {
    	
    	if (this._resultsSortBy == com.ibm.rdm.web.links.LinksWidget.sorts.NAME) {
    		this.nameSort(links);
    	} else if (this._resultsSortBy == com.ibm.rdm.web.links.LinksWidget.sorts.TYPE) {
    		this.typeSort(links);
    	} else if (this._resultsSortBy == com.ibm.rdm.web.links.LinksWidget.sorts.DATE) {
    		this.dateSort(links);
    	}
		dojo.forEach(linkRowWidgets, function(widget) {
			widget.destroyRecursive();
		});
		linkRowWidgets.length = 0;
		com.ibm.rdm.web.util.domutils.removeChildNodes(resultNode);

    	var table = document.createElement("table");
        var tbody = document.createElement("tbody");
        table.appendChild(tbody);

		for( var i = 0; i < links.length; i++ ) {

			if(isOutgoingLinkType){
				if (this.viewType === com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS) {
					if( links[i].contentType == com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REQUIREMENT ||
							(links[i].contentType == com.ibm.rdm.web.common.MimeTypes.LINK_TYPE_PLACEHOLDER && links[i].isReqPlaceHolderLink) ) {
						var row = new com.ibm.rdm.web.links.LinkRowWidget({
							_linkObj: links[i], 
							viewType: this.viewType,
							_artifactURI: this.viewer.resourceURI,
							resourceViewer: this.viewer
						});
						linkRowWidgets.push( row );
						tbody.appendChild(row.domNode);
						this._totalOutgoingLinks++;
					}
				} else {
					if( showReq || links[i].contentType != com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REQUIREMENT ) {
						// If link is of placeholder type, check if it is to be be displayed in general Links.
						if(links[i].contentType == com.ibm.rdm.web.common.MimeTypes.LINK_TYPE_PLACEHOLDER && links[i].isReqPlaceHolderLink){ 
							//Do nothing
						}else{
							
							var rrmRelationship=(links[i].hasrrmRelationship)?true:false; 								
							var row = new com.ibm.rdm.web.links.LinkRowWidget({
								_linkObj: links[i], 
								_artifactURI: this.viewer.resourceURI,
								resourceViewer: this.viewer,
								hasrrmRelationship: rrmRelationship
							});
											
							linkRowWidgets.push( row );
							tbody.appendChild(row.domNode);	
							this._totalOutgoingLinks++;
						}
					}
				}
			}else {//Display Incoming links
				var row = new com.ibm.rdm.web.links.LinkRowWidget({
					_linkObj: links[i], 
					_artifactURI: this.viewer.resourceURI,
					resourceViewer: this.viewer
				});
				linkRowWidgets.push( row );
				tbody.appendChild(row.domNode);	
				this._totalIncomingLinks++
			}
		}
    	resultNode.appendChild(table);
    	
    	if (this._filterWidget) {
    		this._filterWidget.applyActiveFilter();
    	}
    	if (this._filterMenu) {
   		 	this._applyActiveMenuFilter(linkRowWidgets);
    	}
    	if (!this._isOutgoingLinksLoading && !this._isIncomingLinksLoading) {
    		this.updateLinkCountHeader();
    	}   	
    },
    
    updateLinkCountHeader: function() {
    	this._isLinkCountDisplayed = true;
    	if (this._onHeaderCountComplete) {
    		this._onHeaderCountComplete.call(this._onHeaderCountScope, this._totalOutgoingLinks);
    	}
    	
    	var totalShowingCount = 0;
    	
    	var showingLinkCount = 0;
    	dojo.forEach( this._outgoingLinkRowWidgets, function(row) {
    		if (row.status != com.ibm.rdm.web.links.LinkRowWidget.HIDDEN){
    			showingLinkCount++; 
    		}
    	});
    	totalShowingCount += showingLinkCount;
    	if (this.viewType != com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS) {
    		var numShownLabel = dojo.string.substitute( this._messages.showingNofM,[showingLinkCount, this._totalOutgoingLinks,this._messages.outgoing]);
       		this._outgoingLinksCollapseableHeader.setTitle(numShownLabel);
        	this._outgoingLinksCollapseableHeader.setContent([this._linkedOutgoingResultNode]);
        	
        	if (this._isDisplayIncomingLinks()) {
        		showingLinkCount = 0;
        		dojo.forEach( this._incomingLinkRowWidgets, function( row ) {
        			if( row.status != com.ibm.rdm.web.links.LinkRowWidget.HIDDEN ){
        				showingLinkCount++; 
        			}
        		});
        		numShownLabel = dojo.string.substitute( this._messages.showingNofM, [showingLinkCount, this._totalIncomingLinks, this._messages.incoming]);
        		this._incomingLinksCollapseableHeader.setTitle(numShownLabel);
        		this._incomingLinksCollapseableHeader.setContent([this._linkedIncomingResultNode]);
        		totalShowingCount += showingLinkCount;
        	}
        	
    	}
    	
    	this._sidebarSection.clearSectionTitleInfo();
    	var message = "";
    	if (this.viewType === com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS) {
			var totalLinkCount = this._totalOutgoingLinks;
	    	if (totalShowingCount == totalLinkCount) {
	    		message = dojo.string.substitute(this._messages.reqsShowing,[totalShowingCount]);
	    	} else {
	    		message = dojo.string.substitute(this._messages.reqsShowingNofM,[totalShowingCount, totalLinkCount]);
	    	}
	    	this.viewer._updateRequirementsLinksCounterInHeader("(" + totalLinkCount + ")");
    	}
    	else {
    		if (this._isDisplayIncomingLinks()) {
    			message = dojo.string.substitute(this._messages.showingNofOutgoingAndMOfIncoming, [this._totalOutgoingLinks, this._incomingLinks.length]);
    		} else {
    			message = dojo.string.substitute(this._messages.showingNofOutgoing,[this._totalOutgoingLinks]);	
    		}
    		this.viewer._updateOutgoingLinksCounterInHeader("(" + this._totalOutgoingLinks + ")");
    	}
		this._sidebarSection.setSectionTitleInfo(message);
		
		
    },
    
    setLoadingMessage: function() {
    	if (this._sidebarSection) {
	    	var message = this._messages.loading;
	    	this._sidebarSection.clearSectionTitleInfo();
	    	this._sidebarSection.setSectionTitleInfo(message);
    	}
    },

//     handleRefresh: function() {
//     	this.getLinks();
//     }

	_setDocAttr: function(doc) {
		// summary:
		//    Reset the doc that this widget is associated with.
		// description:
		//    This class does NOT contain a handleRefresh function, as it
		//    needs to piggyback on the parsed document that the
		//    ResourceViewer produces.  Instead, when the resourceViewer's
		//    handleRefresh is called, it makes sure that this method is
		//    called via a call such as the following:
		//        this._linksWidget.attr("doc", doc);
		//    As a result of a call like this, this _setDocAttr function
		//    will be invoked to update the UI appropriately.
		this.doc = doc;
		if (this.hasStarted) {
			this.getLinks();
		}
	}
});

com.ibm.rdm.web.links.LinksWidget.sorts = {
		NAME: 0,
		TYPE: 1,
		DATE: 2
};

com.ibm.rdm.web.links.LinksWidget.nameComparator = function( linkObj1, linkObj2 ) {
    var resourceName1 = linkObj1.displayName;
    var resourceName2 = linkObj2.displayName;
    return ((resourceName1 == resourceName2) ?
    	((linkObj1.source) ? 
    		com.ibm.rdm.web.links.LinksWidget.dateComparator( linkObj1, linkObj2 )
    		: 0
    	)
        : ((resourceName1.toLowerCase() < resourceName2.toLowerCase() ) ? -1 : 1));
};

com.ibm.rdm.web.links.LinksWidget.dateComparator = function( linkObj1, linkObj2 ) {
	var nameComparator = function( linkObj1, linkObj2 ) {
	    var resourceName1 = linkObj1.displayName;
	    var resourceName2 = linkObj2.displayName;
	    return ((resourceName1 == resourceName2) ? 
	    		0 : ((resourceName1.toLowerCase() > resourceName2.toLowerCase() ) ? -1 : 1));
	};
	var resourceDate1 = linkObj1.lastModifiedDate;
	var resourceDate2 = linkObj2.lastModifiedDate;
	var compare = dojo.date.compare(resourceDate1, resourceDate2)
	if(compare == 0) {
		return nameComparator(linkObj1, linkObj2);
	} else {
		return compare;
	}
};

})();

//////////////////////////////////////////////////////////////////////////////////////
// Inspiration from JAF and Jazz Web UI from Search Widget in left sidebar section
// 
(function() { 
dojo.declare( "com.ibm.rdm.web.links.ToolbarDropdownMenu", [dijit._Widget, dijit._Templated], {

	templateString: "<div class=\"toolbar-dropdown-menu\"><a dojoAttachPoint=\"_itemType\" href=\"#\"><img class=\"link-dropdown-img\" dojoAttachPoint=\"_img\" /></a></div>",
	DROP_DOWN_ICON_CLASS: "option",
	_imgURI: null,
	_options: null,
	_optionsSelectedArray: null,
	_scope: null,
	_callback: null,
	_menu: null,
	_selectedOption: null,
	_unSelectedOption: null,
	_type: null,
	
	postCreate: function() {
		this._optionsSelectedArray = [];
		this._selectedOption = this.DROP_DOWN_ICON_CLASS+"-selected";
		this._unSelectedOption = this.DROP_DOWN_ICON_CLASS;
		this.connect(this._itemType,"onclick", this._openMenu);
		this.reset();
		this._img.setAttribute( "src", this._imgURI );
	},
	destroy: function() {
		try {
			if(this._menu)	this._menu.destroyRecursive();
			this.inherited(arguments);
		} catch (e) {
		}
	},
	reset: function() {
		for (var i = 0; i < this._options.length; i++) {
			if (i == 0 || this._type == "CHECKBOX") {
				this._optionsSelectedArray[i] = true;
			} else {
				this._optionsSelectedArray[i] = false;
			}
		}
	},
	_openMenu: function(event) {
		if( event ) {
			event.preventDefault();
			event.stopPropagation();
		}
		if( this._menu ) {
			this._closeMenu();
			return;
		}
//		if (this.openOnly) {
//			var allIcon = this.DROP_DOWN_ICON_CLASS;
//			var allOpenIcon = this.DROP_DOWN_ICON_CLASS+"-selected";
//		} else {
//			var allIcon = this.DROP_DOWN_ICON_CLASS+"-selected";
//			var allOpenIcon = this.DROP_DOWN_ICON_CLASS;
//		}
		var menu = new dijit.Menu();
		for( var i = 0; i < this._optionsSelectedArray.length; i++ ) {
			if( this._optionsSelectedArray[i] ) {
				menu.addChild(new dijit.MenuItem({label: this._options[i], iconClass: this._selectedOption, onClick: dojo.hitch(this,this._performCallback, i) }));
			} else {
				menu.addChild(new dijit.MenuItem({label: this._options[i], iconClass: this._unSelectedOption, onClick: dojo.hitch(this,this._performCallback, i) }));
			}
		}
		var pos = dojo.coords(this._itemType);
		pos.x += document.documentElement.scrollLeft;
		pos.y += document.documentElement.scrollTop;
		dojo.addClass(this._itemType,"link-dropdown-menu-selected");
		dijit.popup.open({ popup: menu, parent: this, x: pos.x, y: pos.y + pos.h - 2, onExecute: dojo.hitch(this,this._closeMenu),orient: 'BL' });
		this._menuCloseOnBody = this.connect(this,"_onBlur",this._closeMenu);
		this._menu = menu;
		menu.focus();
	},
	_closeMenu: function(event){
		if(event) {
			event.preventDefault();
			event.stopPropagation();
		}
		dojo.removeClass(this._itemType,"link-dropdown-menu-selected");
		this.disconnect(this._menuCloseOnBody);
		dijit.popup.close(this._menu);
		this._menu.destroyRecursive();
		this._menu = null;
	},
	_performCallback: function( indexSelected ) {
		if( this._type == "RADIO" ) {
			for( var i = 0; i < this._optionsSelectedArray.length; i++ ) {
				if( i == indexSelected ) {
					this._optionsSelectedArray[i] = true;
				} else {
					this._optionsSelectedArray[i] = false;
				}
			}
			this._callback.call( this._scope, this._options, indexSelected );
		} else if( this._type == "CHECKBOX" ) {
			this._optionsSelectedArray[indexSelected] = !this._optionsSelectedArray[indexSelected];
			this._callback.call( this._scope, this._options, this._optionsSelectedArray, indexSelected );
		}
	}
});
})();
