/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
dojo.provide("com.ibm.rdm.web.links.RequirementsFilterWidget");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.commenting.utils");
dojo.requireLocalization("com.ibm.rdm.web.links", "RequirementsFilterWidget");

dojo.declare("com.ibm.rdm.web.links.RequirementsFilterWidget", [dijit._Widget, dijit._Templated], {
    
    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "links/templates/RequirementsFilterWidget.html"),
    
    ///////////////////////////////////////////////////////////////////////////////////
    // NLS fields requiring translation
    by: null,
    howRecent: null,
    showAll: null,
    
    _linksWidget: null,
    _messages: null,
    _authorLabelNode: null,
    _authorWidget: null,
    _howRecentSelectWidget: null,
    _mapRecentSelectionToDate: {},
    
    postCreate: function() {
		this.inherited("postCreate", arguments);
		if (this._linksWidget.projectURI) {
	        this._authorWidget = new com.ibm.rdm.web.common.UsersSelectWidget({
	        	widgetmode: "full", forFilter: true,
	        	projectURI: this._linksWidget.projectURI
	        }, this._authorSelectNode);
		}

        // accessibility, match up label with the widget
		this._authorWidget._htmlSelectNode.setAttribute("id", this._authorWidget.id);
		this._authorLabelNode.setAttribute("for", this._authorWidget._htmlSelectNode.id);
        this._howRecentSelectWidget = new com.ibm.rdm.web.common.IconSelectWidget({
            widgetmode: "full", 
            iconSelectOptions: com.ibm.rdm.web.commenting.utils.getReqUpdatedOptions()
        }, this._howRecentSelectNode);
        // accessibility, match up label with the widget
		this._howRecentSelectWidget._htmlSelectNode.setAttribute("id", this._howRecentSelectWidget.id);
		this._howRecentLabelNode.setAttribute("for", this._howRecentSelectWidget._htmlSelectNode.id);
        
        this.connect( this._closeNode, "onclick", this._onCloseClicked );
        document.getElementById("showAllId").checked = true;
        this._disableShowAll();
        this._connectFormEvents();
    },
    
    startup: function() {
    	this.inherited("startup", arguments);
    },
    
    postMixInProperties: function() {
        this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.links", "RequirementsFilterWidget");
        dojo.forEach(["by", "howRecent", "showAll", "close"], function(prop){
            this[prop] = this._messages[prop];
        }, this);
    },

    destroy: function() {
        this.inherited(arguments);
    },

    hide: function() {
    	this.filterWidget.style.display = "none";
    },
    unhide: function() {
    	this.filterWidget.style.display = "";
    },
    
    createTeamMembersSelectOption: function() {
		if (!this._authorWidget) {
	        this._authorWidget = new com.ibm.rdm.web.common.UsersSelectWidget({
	        	widgetmode: "full", forFilter: true,
	        	projectURI: this._linksWidget.projectURI
	        }, this._authorSelectNode);
		}
    },
    
    _connectFormEvents: function() {
    	var formSelectFieleds = this.domNode.getElementsByTagName("SELECT");
    	for (var i = 0; i < formSelectFieleds.length; i++) {
    		dojo.connect(formSelectFieleds[i], "onchange", this, "_onStartFilteringClicked");
    	}
    },
    
    _onStartFilteringClicked: function() {
    	if (this._isFilterShowAll()) {
            this._disableShowAll();
            document.getElementById("showAllId").checked = true;
    	} else {
            this._enableShowAll();
            document.getElementById("showAllId").checked = false;
    	}
        this.applyActiveFilter();
    },
    _isFilterShowAll: function() {
    	if (this._howRecentSelectWidget.selected != com.ibm.rdm.web.commenting.UPDATED_UNSPECIFIED) return false;
    	if (this._authorWidget.selected != this._authorWidget.UNSPECIFIED) return false;
    	return true;
    },
    applyActiveFilter: function() {
    	var self = this;
    	var interval = this._howRecentSelectWidget.selected;
    	var incomingOutgoingRowWidgets = this._linksWidget.getLinkRowWidgets();
    	dojo.forEach( incomingOutgoingRowWidgets, function( rows ) {
    		dojo.forEach( rows, function( row ) {
	    		var show = false;
	    		//FIXME: HACK
	    		//20616: Requirement filtering by last modified user not working with UTF-8 user ids
    			var user = decodeURIComponent(com.ibm.rdm.web.util.queryutils.getTrimmedUserName(row.lastModifiedBy));
    	    	if (user == self._authorWidget.selected || self._authorWidget.selected == self._authorWidget.UNSPECIFIED) {
    	    		show = true;
    	    	}
    	    	// normalize the compare date to be based on the beginning of the current day
    	    	var compareDate = new Date();
    	    	compareDate.setHours(0);
    	    	compareDate.setMinutes(0);
	    		var difference = dojo.date.difference(row.lastModifiedDate, compareDate, "day");
	    		if (interval == com.ibm.rdm.web.commenting.UPDATED_UNSPECIFIED) {
	    			show = show && true;
	    		} else if (interval == com.ibm.rdm.web.commenting.UPDATED_TODAY) { 
	    			show = show && (difference <= 1);
	    		} else if (interval == com.ibm.rdm.web.commenting.UPDATED_YESTERDAY) {
	    			show = show && (difference > 1 && difference <= 2);
	    		} else if (interval == com.ibm.rdm.web.commenting.UPDATED_PAST_WEEK) {
	    			show = show && (difference > 2 && difference <= 7);
    			} else if (interval == com.ibm.rdm.web.commenting.UPDATED_PAST_MONTH) {
	    			show = show && (difference > 7 && difference <= 30);
				} else if (interval == com.ibm.rdm.web.commenting.UPDATED_EARLIER) {
					show = show && (difference > 30);
				}
				if (show) { row.unhide(); }
				else { row.hide(); }
    		});
    	});
	    this._linksWidget.updateLinkCountHeader();
    },
    
    _onCloseClicked: function(event) {
    	if( event ){
	        dojo.stopEvent(event);
    	}
    	this._linksWidget.closeReqsFilter();
    },
    
    _onShowAllClicked: function(event) {
    	this._howRecentSelectWidget.select(com.ibm.rdm.web.commenting.UPDATED_UNSPECIFIED);
    	this._authorWidget.select(this._authorWidget.UNSPECIFIED);
        this.applyActiveFilter();
        this._disableShowAll();
    },
    
    _disableShowAll: function() {
        document.getElementById("showAllId").disabled = true;
		this._showAllLabel.className = "showall-label-disabled";
    },
    
    _enableShowAll: function() {
        document.getElementById("showAllId").disabled = false;
		this._showAllLabel.className = "showall-label-enabled";
    }
    

});