/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){

dojo.provide ("com.ibm.rdm.web.links.Tree");
dojo.require("dojox.collections.Queue");

//modules for drawing
dojo["require"]("dojox.gfx"); //https://jazz.net/jazz/resource/itemName/com.ibm.team.workitem.WorkItem/77632


dojo.provide("com.ibm.rdm.web.common.ResourceLinksHandler");

/* GLOBAL VARIABLES */
defaultNodeWidth = 250;
defaultNodeHeight = 40;
parent_queue     = new dojox.collections.Queue();
children_queue   = new dojox.collections.Queue();
column_to_draw   = new dojox.collections.Queue();

canvasoffsetTop = 49;
canvasoffsetLeft = 8;

com.ibm.rdm.web.links.Tree = function(rootNode){
    this.rootNode = rootNode;
    this.layout = null;
}

com.ibm.rdm.web.links.Tree.prototype.setLayout = function(layout){
    this.layout = layout;
    this.layout.rootNode = this.rootNode;
    this.container = null;
}
com.ibm.rdm.web.links.Tree.prototype.render = function(_container, w, h){
    this.canvasHeight = h;
    this.canvasWidth = w;
    this.container = _container;//dojo.byId(_container);
    this.surface = dojox.gfx.createSurface(this.container,this.canvasWidth, this.canvasHeight);
    this.layout._treePosition();
    parent_queue.enqueue(this.rootNode); //Traverse the nodes from the root
    this._levelOrderTraveseTree();
}

com.ibm.rdm.web.links.Tree.TreeNode = function (id){

    this.id = id;                 /* id assigned to this tree node        */
    this.data = null;             /* data returned by the deferred method */
    this.parent = null;           /* link to the parent node              */
    this.children = [];           /* link to the children nodes           */
    this.leftNeighbor = null;
    this.rightNeighbor = null;

    this.w = defaultNodeWidth;
    this.h = defaultNodeHeight;
    this.XPosition = 0;
    this.YPosition = 0;
    this.prelim = 0;
    this.modifier = 0;
}

com.ibm.rdm.web.links.Tree.prototype._levelOrderTraveseTree = function(){
    var parentToProcess = (parent_queue.count > 0) ? parent_queue.dequeue() : null;
    if (parentToProcess != null) {
        column_to_draw.enqueue(parentToProcess); //add node to list of nodes to draw
        //Get children and add them to the queue
        var children = parentToProcess.children;

        for (var i = 0; i < children.length; i++)
            children_queue.enqueue(children[i]);
        this._levelOrderTraveseTree(); //current node has already been visited. Do not request children again!
   }
   else {
       /* Draw column of children  */
       this._drawLevelNodes(column_to_draw.toArray());
       column_to_draw.clear(); //Reset the column to draw array
       parent_queue.clear(); //clear the parent queue
       parent_queue = children_queue.clone(); //children queue becomes the parent queue for the next iteration
       children_queue.clear(); //clear the children queue

      if (parent_queue.count > 0)
         this._levelOrderTraveseTree(); //Recursively visit the rest of the parents.
 }
}

com.ibm.rdm.web.links.Tree.prototype._drawLevelNodes = function(children){

    for(var i=0; i < children.length; i++){
      console.log(children[i].id);
      this._drawSingleNode(children[i]);
      this._drawPerperndicularLines(children[i]);
    }
}

com.ibm.rdm.web.links.Tree.prototype._drawSingleNode = function(node){
	
    if (!dojo.isIE) {
      /* Node image */
      var nodeImage = this.surface.createImage({
          width: defaultNodeWidth,
          height: defaultNodeHeight,
          // MBA src: "images/lightbluebutton.gif"
          src: dojo.moduleUrl("com.ibm.rdm.web.links", "images/lightbluebutton.gif").uri
      });
      nodeImage.setTransform([dojox.gfx.matrix.translate(node.XPosition + canvasoffsetLeft , node.YPosition + canvasoffsetTop) ]);

      /* Node image action */
      nodeImage.connect("onclick", dojo.hitch(null, this._nodeAction, node));
  
      /* icon image */
      var iconImage = this.surface.createImage({
          width: 16,
          height:16,
          src:  node.data.iconUri
      });
  
      iconImage.setTransform([dojox.gfx.matrix.translate(node.XPosition + canvasoffsetLeft + 5 , node.YPosition + canvasoffsetTop + 7) ]);

      /* icon image action */
      iconImage.connect("onclick", dojo.hitch(null, this._nodeAction, node));

    }
    else{

      /* Draw rectangle instead of image button */
      var nodeBox = this.surface.createRect({
         x: node.XPosition + canvasoffsetLeft,
         y: node.YPosition + canvasoffsetTop,
         width: defaultNodeWidth,
         height: defaultNodeHeight
      }).setStroke("blue").setFill("white");

      /* Node container box action */
         nodeBox.connect("onclick", dojo.hitch(null, this._nodeAction, node));
    }
    horizontal_indent = 25;
    vertical_indent = Math.ceil(node.h/2);


    /* node text */
    var nodeText = this.surface.createText({
        x: (node.XPosition + canvasoffsetLeft + horizontal_indent),
        y: (node.YPosition + canvasoffsetTop + vertical_indent) ,
        text: node.data.targetDisplayName ,
        align: "start"
    });

    nodeText.setFont({family: "Arial",size: "8pt"});
    nodeText.setFill("blue");

    /* text actions */
    nodeText.connect("onclick",  dojo.hitch(null, this._nodeAction, node));
}
com.ibm.rdm.web.links.Tree.prototype._drawPerperndicularLines = function(node){

   if (node.parent != null){    //Draw a line from the parent of this node to the node

     var _x1 = (node.parent.XPosition + canvasoffsetLeft + node.parent.w);
     var _y1 = ((node.parent.YPosition + canvasoffsetTop) + (node.parent.h /2));
     var _x2 = (node.XPosition + canvasoffsetLeft);
     var _y2 = ((node.YPosition + canvasoffsetTop) + (node.h /2));

     var poly = null;

        var p1 = {x: _x1, y: _y1};
        var p2 = {x: ((_x2 - _x1)/2) + _x1, y: _y1};
        var p3 = {x: ((_x2 - _x1)/2) + _x1, y: _y2};
        var p4 = {x: _x2, y: _y2};
        poly = this.surface.createPolyline([p1, p2, p3, p4])
        poly.setStroke({color: "blue", width: 1});

  }
}


com.ibm.rdm.web.links.Tree.prototype._nodeAction = function(node){
           // alert(" Clicked on node " + node.id);
           var uri = node.data.targetUri;
           var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
           if (ResourceLinksHandler.isLinkInternal(uri)) {
		        var lastSlashIndex = uri.lastIndexOf("/");
		        var lastSegment = (lastSlashIndex != -1 ? uri.substring(lastSlashIndex+1) : uri);
		        uri = "resources/" + lastSegment;
               ResourceLinksHandler.showInternalResource(uri, false);
           }
           else {
               ResourceLinksHandler.showExternalResource(uri);
           }
}

})();
