/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
dojo.provide("com.ibm.rdm.web.parsers.DoorsLinksParser");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");


var domutils = com.ibm.rdm.web.util.domutils;
var queryutils = com.ibm.rdm.web.util.queryutils;

dojo.declare(
	"com.ibm.rdm.web.parsers.DoorsLinksParser",
	null,
{
	// summary:
	//    Parses the content related to the Doors integration.

	///////////////////////////////////////////////////////////////////////////
	// Public fields


	///////////////////////////////////////////////////////////////////////////
	// Private fields
	_documentName: null,
	_doorsLinkTarget: null,


	///////////////////////////////////////////////////////////////////////////
	// API

	constructor: function(args) {
		var descriptionNode = args.descNode;
		this._parseContent(descriptionNode);
	},
	
	getDocumentName: function(){
		return this._documentName;
	},
	
	getDoorsLinkTarget: function(){
		return this._doorsLinkTarget;
	},


	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_parseContent: function(descriptionNode) {
		this._parseDocumentName(descriptionNode);
		this._parseDoorsLinkTarget(descriptionNode);
	},

	_parseDocumentName: function(descriptionNode) {
		// summary:
		// Parse Doors Document Name
		this._documentName = this._getTextNodeValue(descriptionNode, "label", queryutils.RDFS_NAMESPACE, queryutils.RDFS_QUALIFIER);
	
	},
	
	_parseDoorsLinkTarget: function(descriptionNode) {
		// summary:
		// Parse Doors Link Target
		var linkTargetNode = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
								descriptionNode, "linkTarget",
								queryutils.RRM_NAMESPACE, 
								queryutils.RRM_QUALIFIER);
		
		var linkTarget = domutils.getAttributeNS(
    					linkTargetNode[0], "resource", 
    					queryutils.RDF_NAMESPACE, 
    					queryutils.RDF_QUALIFIER);
    					
    	var decodedOnce = decodeURIComponent(linkTarget);
    	var finalTarget = decodeURIComponent(decodedOnce);

		this._doorsLinkTarget = finalTarget ? finalTarget: "";
	},
	
	_getTextNodeValue: function(node, unqualifiedTagName, namespace, qualifier) {
		var nodes = domutils.getElementsByTagNameNS(node, unqualifiedTagName, namespace, qualifier);
		return (
			nodes.length > 0 && nodes[0].firstChild ?
			nodes[0].firstChild.nodeValue :
			""
		);
	}
});


})();
