/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/


(function() {

	// declare dependencies on other Dojo modules
	dojo.provide("com.ibm.rdm.web.picker.CollectionPicker");
	dojo.require("dijit._Templated");
	dojo.require("dijit._Widget");
	dojo.require("dijit.form.Button");
	dojo.require("dijit.form.TextBox");
	dojo.require("dojo.i18n");
	dojo.require("jazz.ui.RemoteChooserResponder");
	dojo.require("com.ibm.rdm.web.pages.WebUIinitializer");
	dojo.require("com.ibm.rdm.web.common.ProjectCache");
	dojo.require("com.ibm.rdm.web.server.client.RDMClient");
	dojo.require("com.ibm.rdm.web.common.MimeTypes");
	dojo.require("com.ibm.rdm.web.util.queryutils");


	var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
	var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
	var QueryUtils = com.ibm.rdm.web.util.queryutils;
	
	// delay between user input and running a search (in milliseconds)
	var DELAY					= 750;
	
	//declare and implement the widget "class"
	dojo.declare("com.ibm.rdm.web.picker.CollectionPicker", [ dijit._Widget, dijit._Templated ], {


		templatePath: dojo.moduleUrl("com.ibm.rdm.web", "picker/templates/collectionSelector.html"),
		widgetsInTemplate: true, 
		_dialogArguments:null,
		_projectURL:null,
		 projectUser: null,
		_adjustedWindowWidth: null,
		_selectedFolderResources:[],
		_artifactTypeSelectionNode:null,
		_messages:null,
				
	constructor: function(params){		
		//Load nls messages
		dojo.requireLocalization("com.ibm.rdm.web.picker", "RRCPicker");
	    this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.picker", "RRCPicker");
	    
		var searchString = location.search.replace(/^\?/,''); // remove leading '?'
		this._dialogArguments = dojo.queryToObject(searchString);
		
		if(!this._dialogArguments.projectURL) throw new Error(this._messages.missingProMsg);
		
		//Set the project URI. We expect a full encoded URL . For example:
		//"https%3A//IBM-ADDFD.lexma.ibm.com%3A9443/jazz/process/project-areas/_e_EOIDW9Ed6U09wKTb_h5A" is the url for "project 1"
		
		this._projectURL = unescape(this._dialogArguments.projectURL); 
					
		this._remoteFrame= new jazz.ui.RemoteChooserResponder();
		this.htmlEnterSearchString = this._messages.searchInputCollectionLbl;
		this.htmlMatchingString = this._messages.matchingCollections;
		this.htmlResultNoString = "0" + " " +  this._messages.resultNoMessage;
		this.buttonOK = this._messages.buttonOK;
		this.buttonCancel = this._messages.buttonCancel;
	
	},
	postCreate: function() {
		
		// override the styles from the UI module (minWith big, blue background)
		var body= dojo.body();
		dojo.style(body, "minWidth", "200px");
		dojo.style(body, "background", "none");
		dojo.style(body, "overflow", "hidden");
		
		//Initialize project properties
		var def = com.ibm.rdm.web.pages.WebUIinitializer.initializeJFSForConfigProperties();
		def.addCallback(this, this.start);		
		
		this._okButtonNode.disabled= true;
	},
	
	start: function(){	
		this.createCollectionDisplaySection();
		this._connectEvents();
	},

	createCollectionDisplaySection : function(){
		this._populateContents(null);
	},
	_connectEvents: function() {
		this.connect(this.queryInput.domNode, "onkeyup", this._doCancellableSearch);
		this.connect(this._okButtonNode, "onclick",  this._onOk);
		this.connect(this._cancelButtonNode, "onclick", this._onCancel);
		this.connect(this._requirementsSelectionArea, "onchange", this._handleSelectionChange);

	},
	_doCancellableSearch: function(e) {
		if(e.keyCode === 40){ // arrow down 
			//this.queryResults.focus();
			return;
		} 
		if(e.keyCode === 38){ // arrow up 
			//this._projectSelect.focus();
			return;
		} 
		if (this._currentId) {
			clearTimeout(this._currentId);
		}
		var self= this;
		var doSearchFunc= function() {			
			//Populate the folder contents
			self._populateContents(self.queryInput.getValue());	
		};
		this._currentId= setTimeout(doSearchFunc, DELAY);
	},
	_composeMultiFetchPostData: function(searchString) {

		if(!this._projectURL)
			return null;

		var selectedProject = this._projectURL;
		selectedProject = selectedProject.substring(selectedProject.lastIndexOf("/") + 1);
		var queryInfo = null;
		
		var query = "contentType=\"" + com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_COLLECTION + "\" and namespace in"; 
		query +=  "[<" + com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES + ">]";
		var namespaces = null;
		
		var searchFor = (searchString)?dojo.trim(searchString):"";
		var searchValue = "";
		
		if(searchFor != ""){
			   searchFor = QueryUtils.removeSpecialLuceneChars(searchFor);
			   searchValue = "((weighted:value*))";
			   searchValue = searchValue.replace(/value/g, searchFor);
			}
		
		queryInfo = {
				query: query,
				search: searchValue,
				namespaces: namespaces,
				projectURI: this._projectURL,
				extendedQueryMode: true,
				cancelDeferreds: false,
				disablePagination: true
		};
	
		return queryInfo;
	},

	_populateContents: function(searchString) {
		
		if(searchString != null){
			//Check if this is a search by id
			var searchIdStr = com.ibm.rdm.web.util.queryutils.asSearchId(searchString);

			if(searchIdStr != null){
				this._removeSelectionNode();

				//Clear the results counter
				if(this.resultNumber.childNodes.length > 0)
					this.resultNumber.removeChild(this.resultNumber.firstChild);

				var resourceURI = [];
				resourceURI.push(com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES + "/" + searchIdStr);
				var descriptionsDeferred = ProjectCache.getBatchResourceDescriptionsMultiFetch(resourceURI);
				descriptionsDeferred.addCallback(this, function(response){
					//Total results counter
					var totalResults = 0;
					for(artifact in response){	
						if(response[artifact] != null){
							var displayName = response[artifact].displayName;	
							var projectURI = response[artifact].projectURI;
							if(projectURI == this._projectURL){	
								if(response[artifact].contentType == com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_COLLECTION){
									var newOption = new Option(displayName,artifact);
									var optionArr= this._requirementsSelectionArea.options;
									optionArr[optionArr.length]= newOption;
									totalResults++;
								}						
							}	
						}
					}									
					this.resultNumber.appendChild(document.createTextNode( totalResults  + " " +  this._messages.resultNoMessage));
				});

			}else{
				var postData = this._composeMultiFetchPostData(searchString);
				// Note: if postData is null, this means that no filters have been applied.
				if (postData) {			
					this._displayResultMsg(this._messages.loading, "loadingMsg");
					var deferred = ProjectCache.executeMultiFetch(postData);
					deferred.addCallback(this, this._handleQuerySuccessResponse);
					deferred.addErrback(this, this._handleQueryErrorResponse);
				}
			}

		}else{
			var postData = this._composeMultiFetchPostData(searchString);
			// Note: if postData is null, this means that no filters have been applied.
			if (postData) {			
				this._displayResultMsg(this._messages.loading, "loadingMsg");
				var deferred = ProjectCache.executeMultiFetch(postData);
				deferred.addCallback(this, this._handleQuerySuccessResponse);
				deferred.addErrback(this, this._handleQueryErrorResponse);
			}
		}
	},		
	_handleQuerySuccessResponse: function(collections) {

		this._removeSelectionNode();
		
		//Clear the results counter
		if(this.resultNumber.childNodes.length > 0)
			this.resultNumber.removeChild(this.resultNumber.firstChild);
	
		//Total results counter
		var totalResults = collections.length;

		for (var i in collections){		
			//Populate the selection area
			var displayName = collections[i].displayName;
			var resourceURI = collections[i].resourceURI;
			var newOption = new Option(displayName,resourceURI);
			var optionArr= this._requirementsSelectionArea.options;
			optionArr[optionArr.length]= newOption;
		}
		this.resultNumber.appendChild(document.createTextNode( totalResults  + " " +  this._messages.resultNoMessage));
	},
	
	_handleQueryErrorResponse: function(response) {	
		console.log("Error!" + response);
		return response;
	},

	_close: function(resultArray) {
		var items = [];
		var correctedItems = [];
		if (resultArray) { // expected [ { id, summary } ]
			for (var i = 0; i < resultArray.length; i++) {
				var item = resultArray[i];	
				var itemUrl = item.id;
				var label = item.summary; 
				var newItem = { "rdf:resource": itemUrl };
				newItem["oslc_rm:label"] = label;			        
				items.push(newItem);
				// This is a workaround for 33132: RRC 2.0.0.2: not following OSCL_RM V1.0 spec regarding picker responses
				var newItem2 = {
					"http://www.w3.org/1999/02/22-rdf-syntax-ns#resource": itemUrl,
					"http://www.w3.org/2000/01/rdf-schema#label": label
				};
				correctedItems.push(newItem2);
			}
		}
		var response = {};
		response["oslc_rm:message"] = "oslc_rm.select";
		response["oslc_rm:results"] = items;
		// This is a workaround for 33132: RRC 2.0.0.2: not following OSCL_RM V1.0 spec regarding picker responses
		response["http://open-services.net/xmlns/rm/1.0/web/message"] = "http://open-services.net/xmlns/rm/1.0/web/select";
		response["http://open-services.net/xmlns/rm/1.0/web/results"] = correctedItems;
		this._remoteFrame.sendResponse(dojo.toJson(response));
	},

	_onOk: function(workItems, key) {
		var resultArray = [];
		//Get the current selections and return the  as [ { id, summary } ]
		for(i=0; i < this._requirementsSelectionArea.options.length; i++){
			var selection = this._requirementsSelectionArea.options[i];
			if(selection.selected == true){
				var item_id = selection.value;
				var item_summary = selection.firstChild.nodeValue;          
				var newItem = {"id": item_id, "summary":item_summary};		
				resultArray.push(newItem);
			}     				
		}
		this._close(resultArray);
	},

	_onCancel: function() {
		var emptyResponse = [];		
		this._close(emptyResponse);
	},

	_handleSelectionChange: function() {
		// disable the 'OK' button if nothing is selected
		this._okButtonNode.disabled= this._requirementsSelectionArea.selectedIndex === -1;
	},
	_removeSelectionNode: function(){	
		this._requirementsSelectionArea.length=0;
		this._okButtonNode.disabled=true;
	},
	_displayResultMsg: function(msg, value){
		this._removeSelectionNode();
		var newOption = new Option(msg,value);
		var optionArr= this._requirementsSelectionArea.options;
		optionArr[optionArr.length]= newOption;	
	}

	});

})();
