/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/


(function() {

	// declare dependencies on other Dojo modules
	dojo.provide("com.ibm.rdm.web.picker.RequirementCreatorPicker");
	dojo.require("dijit._Templated");
	dojo.require("dijit._Widget");
	dojo.require("dijit.form.Button");
	dojo.require("dijit.form.TextBox");
	dojo.require("dojo.i18n");
	dojo.require("jazz.ui.RemoteChooserResponder");
	dojo.require("com.ibm.rdm.web.pages.WebUIinitializer");
	dojo.require("com.ibm.rdm.web.project.FoldersSection");
	dojo.require("com.ibm.rdm.web.server.client.RDMClient");
	dojo.require("com.ibm.rdm.web.common.MimeTypes");


	var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
	var LOCATION_HEADER = "Location";
	var CONTAINED_RESOURCE_ARGS = "containedResources";
	var RDF_CONTENT_TYPE =  "application/rdf+xml";
	var ReqTitlePlaceholder="[REQUIREMENT_TITLE]";
	var ReqDescPlaceholder ="[REQUIREMENT_DESCRIPTION]";
	
	var REQ_POST_MSG = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
	   "<requirement:requirement xmlns:attribute=\"http://schema.ibm.com/rdm/2008/Attribute\"\n" + 
	     "xmlns:base=\"http://jazz.net/xmlns/alm/rm/Base/v0.1\" xmlns:h=\"http://www.w3.org/TR/REC-html40\"\n" + 
	     "xmlns:dc=\"http://purl.org/dc/terms/\" \n" +
	     "xmlns:requirement=\"http://jazz.net/xmlns/alm/rm/Requirement/v0.1\"\n" + 
	     "base:id=\"Requirement__11tG0fxmEd2jR7BV-MRUZA\">\n" +
	   "<dc:title>" + ReqTitlePlaceholder + "</dc:title>" + "\n" +
	   "<dc:description>" + ReqDescPlaceholder + "</dc:description>" + "\n" +
	   "<requirement:richTextBody id=\"_12TjwPxmEd2jR7BV-MRUZA\">\n" +
	    "</requirement:richTextBody>\n" +
	   "</requirement:requirement>\n";
	
	var ResourceURIPlaceholder="[RESOURCE_URI]";
	
	var MOVE_FOLDER_POST_MSG = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
    "<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">" +
    "<ns:containedResources xmlns:ns=\"http://com.ibm.rdm/navigation#\" rdf:about=\"\">" +
    "<ns:containedResource rdf:resource=\"" + ResourceURIPlaceholder + "\" xmlns:ns=\"http://com.ibm.rdm/navigation#\"/>" + 
    "</ns:containedResources></rdf:RDF>";
	
	//declare and implement the widget "class"
	dojo.declare("com.ibm.rdm.web.picker.RequirementCreatorPicker", [ dijit._Widget, dijit._Templated ], {


		templatePath: dojo.moduleUrl("com.ibm.rdm.web", "picker/templates/requirementCreator.html"),
		widgetsInTemplate: true, 
		_foldersSection: null,
		_dialogArguments:null,
		_projectURL:null,
		_selectedFolderResources:[],
		_currentSelectedFolder:null,
		_doSelectRootFolder:false ,
		_messages:null,

				
	constructor: function(params){		
		//Load nls messages
		dojo.requireLocalization("com.ibm.rdm.web.picker", "RRCPicker");
	    this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.picker", "RRCPicker");

		var searchString = location.search.replace(/^\?/,''); // remove leading '?'
		this._dialogArguments = dojo.queryToObject(searchString);
		
		if(!this._dialogArguments.projectURL) throw new Error(this._messages.missingProMsg);
		
		//Set the project URI. We expect a full encoded URL . For example:
		//"https%3A//<server_name>%3A<port>/jazz/process/project-areas/<project ID>" is the url for the project
		
		this._projectURL = unescape(this._dialogArguments.projectURL);
					
		this._remoteFrame= new jazz.ui.RemoteChooserResponder();
		this.htmlEnterDescString = this._messages.descriptionLbl;
		this.buttonOK = this._messages.buttonOK;
		this.buttonCancel = this._messages.buttonCancel;
	
	},
	postCreate: function() {
		
		// override the styles from the UI module (minWidth big, blue background)
		var body= dojo.body();
		dojo.style(body, "minWidth", "200px");
		dojo.style(body, "background", "none");
		dojo.style(body, "overflow", "hidden");
				
		//Initialize project properties
		var def = com.ibm.rdm.web.pages.WebUIinitializer.initializeJFSForConfigProperties();
		def.addCallback(this, this.start);	
		
		this._okButtonNode.disabled= true;
		
	},
	
	start: function(){	
		
		var nameLabel = document.createTextNode(this._messages.nameLbl);
		this._nameLabel.appendChild(nameLabel);		
		var folderLabel = document.createTextNode(this._messages.selectFolder);
		this._folderLabel.appendChild(folderLabel);	
		//By default select the root folder
		var projectID = this._projectURL.substring(this._projectURL.lastIndexOf("/"));
		this._currentSelectedFolder= com.ibm.rdm.web.FRONTING_SERVICE_FOLDERS + projectID;
	
		this.createFolderSection();		
		this._connectEvents();
	},

	createFolderSection: function(){
		
		this._foldersSection = new com.ibm.rdm.web.project.FoldersSection(
			{
				projectURI: this._projectURL,
				dashboardTabPage: this  /* FoldersSection calls refresh method on this object */
			},
			null
		);

		this._folderBrowser.appendChild(this._foldersSection.domNode);

		//Start the folder widget
		this._foldersSection.startup();
		this._doSelectRootFolder = true;
	},
	
	refresh: function(){
		
		this._selectedFolderResources = (
				this._foldersSection ?
						this._foldersSection.getSelectedFolders() :
							[]
		);	
		//Store selected folder URI
		this._currentSelectedFolder =this._selectedFolderResources[0];
	}
	,
	_connectEvents: function() {
		this.connect(this._nameInput.domNode, "onkeyup", this._doEnableOkAction);
		this.connect(this._okButtonNode, "onclick",  this._onOk);
		this.connect(this._cancelButtonNode, "onclick", this._onCancel);

	},
	
	_close: function(resultArray) {
		var items = [];
		var correctedItems = [];
		if (resultArray) { // expected [ { id, summary } ]
			for (var i = 0; i < resultArray.length; i++) {
				var item = resultArray[i];	
				var itemUrl = item.id;
				var label = item.summary; 
				var newItem = { "rdf:resource": itemUrl };
				newItem["oslc_rm:label"] = label;			        
				items.push(newItem);
				// This is a workaround for 33132: RRC 2.0.0.2: not following OSCL_RM V1.0 spec regarding picker responses
				var newItem2 = {
					"http://www.w3.org/1999/02/22-rdf-syntax-ns#resource": itemUrl,
					"http://www.w3.org/2000/01/rdf-schema#label": label
				};
				correctedItems.push(newItem2);
			}
		}
		var response = {};
		response["oslc_rm:message"] = "oslc_rm.create";
		response["oslc_rm:results"] = items;
		// This is a workaround for 33132: RRC 2.0.0.2: not following OSCL_RM V1.0 spec regarding picker responses
		response["http://open-services.net/xmlns/rm/1.0/web/message"] = "http://open-services.net/xmlns/rm/1.0/web/create";
		response["http://open-services.net/xmlns/rm/1.0/web/results"] = correctedItems;
		this._remoteFrame.sendResponse(dojo.toJson(response));
	},

	_onOk: function() {
		
		//Get new requirement title and create the resource
		var reqTitle =  this._nameInput.attr("value");
				
		var reqDeferred = this._createNewReqResource(this._currentSelectedFolder, this._projectURL, reqTitle);
	
		reqDeferred.addCallback(this, function(newReq){
			var resultArray = [];
			
			//return the requirement as [ { id, summary } ]
			var item_id = newReq.resourceURI;
			var item_summary =  newReq.label;      
			var newItem = {"id": item_id, "summary":item_summary};		
		    resultArray.push(newItem);

			this._close(resultArray);
		    
		});	
		reqDeferred.addErrback(this, function(response){
		    
		});			
	},

	_onCancel: function() {
		var emptyResponse = [];		
		this._close(emptyResponse);
	},
	
	_doEnableOkAction: function(){

		if(this._nameInput.attr("value"))
			this._okButtonNode.disabled=false;
		else
			this._okButtonNode.disabled=true;
	},
	
	_createNewReqResource: function(folderLocation, projectArea, reqTitle){
		
		var fakeDeferred = new dojo.Deferred();	 
		
	    var postURL = com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES;    
	    REQ_POST_MSG = REQ_POST_MSG.replace(ReqTitlePlaceholder, reqTitle);
	    //add a Description to the requirement if there is one
	    var reqDesc = (this._descriptionInput.attr("value"))? this._descriptionInput.attr("value"):"";	    
	    REQ_POST_MSG = REQ_POST_MSG.replace(ReqDescPlaceholder, reqDesc); 
	    	    
    	var requestHeaders = {
		       	"Content-Type": com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REQUIREMENT,
			 	"Content-Length": REQ_POST_MSG.length			 	
		    };
    	
		var deferred = RDMClient.doPost({
				uri: postURL,
				projectURI: projectArea,
		        postData: REQ_POST_MSG,
		        xhrArgs: {
		        	headers: requestHeaders
		        }
			});
		deferred.addCallback(this, function(response){
			var newReqLocation = response.ioArgs.xhr.getResponseHeader(LOCATION_HEADER);
			newReqInfo ={"resourceURI":newReqLocation, "label": reqTitle};
			this._moveResourceToSelectedFolder(fakeDeferred, folderLocation, projectArea, newReqInfo);
		    
		});	
		
		deferred.addErrback(this, function(response){
			//TODO: Handle Error
	
		});
		
		return fakeDeferred;
		
	},
	_moveResourceToSelectedFolder:function(newReqDeferred, folderLocation, projectArea, reqInfo){
		
	    var postURL=folderLocation + "?" + CONTAINED_RESOURCE_ARGS;
	    
	    MOVE_FOLDER_POST_MSG = MOVE_FOLDER_POST_MSG.replace(ResourceURIPlaceholder, reqInfo.resourceURI);
		
		var requestHeaders = {
		   "Content-Type": RDF_CONTENT_TYPE,
		   "Content-Length": MOVE_FOLDER_POST_MSG.length			 	
		};		
		var deferred = RDMClient.doPost({
			uri: postURL,
			projectURI: projectArea,
	        postData: MOVE_FOLDER_POST_MSG,
	        xhrArgs: {
	        	headers: requestHeaders
	        }
		});
		
		deferred.addCallback(this, function(response){
			newReqDeferred.callback(reqInfo);
		    
		});	
		
		deferred.addErrback(this, function(response){
			newReqDeferred.callback(response);
		    
		});
	
	}

	});

})();
