/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.project.AttributesSection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.form.TextBox");
dojo.require("com.ibm.rdm.web.project.DashboardSection");
dojo.require("com.ibm.rdm.web.project.FilterGroupSection");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.search.results.ResourceInfo");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.project", "AttributesSection");


dojo.declare(
	"com.ibm.rdm.web.project.AttributesSection",
	com.ibm.rdm.web.project.DashboardSection,
{
	// summary:
	//    A widget used to provide content for the "Filter Display by
	//    Attributes" section.
	// description:
	//	  Presents a section view on the left in the project dashboard to enable
	//    users to filter by different attributes (artifact name, last modified
	//    date, last modified user, content type or attribute group).

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields

	// viewType: object (representing an enum)
	//	The value of this field must be one of the values found in the
	//	com.ibm.rdm.web.project.DashboardTabPage.viewType property list.
	viewType: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,

	_filterByNameSection: null,
	_filterByUserSection: null,
	_filterByDateSection: null,
	_filterByArtifactTypeSection: null,
	_filterByAttributeValueSection: null,
	_allFilterBySections: null,

	_table: null,

	// This attribute will ONLY hold a value when a refresh operation is in
	// progress.  (Note: This attribute is null'd out in the constructor.)
	_refreshInfo: {
		// The callback to invoke when the refresh operation has completed (i.e.
		// after new values have been added to the DOM that reflect a refreshed
		// tag list).
		refreshCompletedCallback: null
	},

	///////////////////////////////////////////////////////////////////////////////////
	// API

	constructor: function() {
		this._refreshInfo = null;
	},

	postCreate: function() {
		this.inherited(arguments);
		this._isVerticalScrollbarPresent = false;
		this._messages = dojo.i18n.getLocalization(
			"com.ibm.rdm.web.project", "AttributesSection");
		// Create all filter by sections.
 		var filterByNameSection = new com.ibm.rdm.web.project.FilterByNameSection({
			attributesSection: this,
			viewType: this.viewType}, null);
		var filterByUserSection = new com.ibm.rdm.web.project.FilterByUserSection({
			attributesSection: this,
			viewType: this.viewType});
		var filterByDateSection = new com.ibm.rdm.web.project.FilterByDateSection({
			attributesSection: this,
			viewType: this.viewType});
		// Note: if this attributes section is on the Requirements page, we take
		// care to NOT show "Filter by content-type".
		var filterByArtifactTypeSection = (
			(this.viewType == com.ibm.rdm.web.project.DashboardTabPage.viewType.ARTIFACTS) ?
			new com.ibm.rdm.web.project.FilterByArtifactTypeSection({
				attributesSection: this,
				viewType: this.viewType}) :
			null);
		var filterByAttributeValueSection = new com.ibm.rdm.web.project.FilterByAttributeValueSection({
			attributesSection: this,
			viewType: this.viewType});
		// Add these sections to the DOM.
		var table = document.createElement("table");
		var tbody = document.createElement("tbody");
		table.className = "filter-by-attribute-section-table";
		this._contentAreaDiv.appendChild(table);
		table.appendChild(tbody);
		this._addFilterGroupSectionRow(filterByNameSection, tbody,
			"com.ibm.rdm.web/project/images/icons/obj16/filter-string.gif");
		this._addFilterGroupSectionRow(filterByUserSection, tbody,
			"com.ibm.rdm.web/project/images/icons/obj16/filter-user.gif");
		this._addFilterGroupSectionRow(filterByDateSection, tbody,
			"com.ibm.rdm.web/project/images/icons/obj16/filter-calendar.gif");
		if (filterByArtifactTypeSection) {
			this._addFilterGroupSectionRow(filterByArtifactTypeSection, tbody,
				"com.ibm.rdm.web/project/images/icons/obj16/filter-attrb-types.gif");
		}
		this._addFilterGroupSectionRow(filterByAttributeValueSection, tbody,
			"com.ibm.rdm.web/project/images/icons/obj16/filter-attrb-value.gif");
		// Store these sections in an array.
		this._allFilterBySections = [
			filterByNameSection,
			filterByUserSection,
			filterByDateSection,
			filterByAttributeValueSection
		];
		if (filterByArtifactTypeSection) {
			this._allFilterBySections.push(filterByArtifactTypeSection);
		}
		// Store these sections individually.
		this._filterByNameSection = filterByNameSection;
		this._filterByUserSection = filterByUserSection;
		this._filterByDateSection = filterByDateSection;
		this._filterByArtifactTypeSection = filterByArtifactTypeSection;
		this._filterByAttributeValueSection = filterByAttributeValueSection;
		// Store the table.
		this._table = table;
	},

	startup: function() {
		this.inherited(arguments);
	},

	destroy: function() {
		this.inherited(arguments);
		for (var i = 0; i < this._allFilterBySections.length; i++) {
			this._allFilterBySections[i].destroy;
		}
	},

	refresh: function(refreshCompletedCallback) {
		this._startRefresh(refreshCompletedCallback);
	},

	isAtLeastOneAttributeApplied: function() {
		for (var i = 0; i < this._allFilterBySections.length; i++) {
			if (this._allFilterBySections[i].isFilterApplied()) {
				return true;
			}
		}
		return false;
	},

	getAppliedArtifactName: function() {
		return this._filterByNameSection.getAppliedTextString();
	},

	getAppliedUsers: function() {
		return this._filterByUserSection.getAppliedUsers();
	},

	getAppliedStartDate: function() {
		return this._filterByDateSection.getAppliedStartDate();
	},

	getAppliedEndDate: function() {
		return this._filterByDateSection.getAppliedEndDate();
	},

	getAppliedArtifactTypes: function() {
		// Note: The "Requirements" tab page doesn't contain a
		// "filter by artifact type" section.
		return (
			this._filterByArtifactTypeSection ?
			this._filterByArtifactTypeSection.getAppliedArtifactTypes() :
			null
		);
	},

	getAppliedAttributeValuesInfo: function() {
		return this._filterByAttributeValueSection.getAppliedAttributeValuesInfo();
	},

	isArtifactTypesFilterApplied: function() {
		// Note: The "Requirements" tab page doesn't contain a
		// "filter by artifact type" section.
		return (
			this._filterByArtifactTypeSection ?
			this._filterByArtifactTypeSection.isFilterApplied() :
			false
		);
	},

	isAttributeValueFilterApplied: function() {
		return this._filterByAttributeValueSection.isFilterApplied();
	},

	handleSectionSelectionHasChanged: function() {
		// summary:
		//	This function is called whenever selection changes occur
		//	within the sections (like "filter by user").
		this._filterSelectionHasChanged();
		this.dashboardTabPage.refresh();
	},

	handleContentAreaHeightHasChanged: function() {
		this._handleHeightHasChanged();
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	// Superclass override functions ...

	_populateContents: function() {
		// summary:
		//    This is an override of a superclass function.
		this._clearLastResultsIfNeeded();
		this._deferredArray = [];
		var self = this;
		var numOutstandingRefreshes = this._allFilterBySections.length;
		var onFilterGroupRefreshCompleted = function() {
			numOutstandingRefreshes--;
			if (numOutstandingRefreshes === 0) {
				self._endRefresh();
			}
		};
		for (var i = 0; i < this._allFilterBySections.length; i++) {
			this._allFilterBySections[i].refresh(onFilterGroupRefreshCompleted);
		}
	},

	_handleClearFilters: function() {
		// summary:
		//    This is an override of a superclass function.
		for (var i = 0; i < this._allFilterBySections.length; i++) {
			this._allFilterBySections[i].clearFilter();
		}
		// TODO Perhaps we shouldn't call the following function if it's the
		// case that there are no filters currently in effect?
		this.handleSectionSelectionHasChanged();
	},

	_isAtLeastOneFilterSpecified: function() {
		// summary:
		//    This is an override of a superclass function.
		return this.isAtLeastOneAttributeApplied();
	},

	// Normal (non-override) functions ...

	_addFilterGroupSectionRow: function(filterGroupSection, tbody, iconImgSrc) {
		var tr = document.createElement("tr");
		var td1 = document.createElement("td");
		var td2 = document.createElement("td");
		var img = document.createElement("img");
		tbody.appendChild(tr);
		tr.appendChild(td1);
		tr.appendChild(td2);
		td1.className = "filter-by-attribute-section-group-icon-td";
		img.setAttribute("src", iconImgSrc);
		td1.appendChild(img);
		td2.appendChild(filterGroupSection.domNode);
		this.connect(img, "onclick", function() {
			filterGroupSection.toggleSectionExpansionState();
		});
	},

	_startRefresh: function(refreshCompletedCallback) {
		// To handle the case of a refresh being asked for before the last one
		// has completed, we'll ignore subsequent requests if one is currently
		// under way.
		if (!this._refreshInfo) {
			// FIXME Implement
			this._refreshInfo = {
				refreshCompletedCallback: refreshCompletedCallback
			};
			this._populateContents();
		}
	},

	_endRefresh: function() {
		if (this._refreshInfo) {
			var callback = this._refreshInfo.refreshCompletedCallback;
			this._refreshInfo = null;
			callback();
		}
	},

	_clearLastResultsIfNeeded: function() {
	}
});

})();
