/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.project.DashboardSection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.form.Button");
dojo.require("dijit.layout.ContentPane");
dojo.require("dojo.date.locale");
dojo.require("dojox.collections.Dictionary");
dojo.require("com.ibm.rdm.web.search.results.ResourceInfo");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.common", "common");
dojo.requireLocalization("com.ibm.rdm.web.project", "common");

//TODO Read these in from CSS ...
var CURVED_BORDER_HEIGHT = 3;
var SHADOW_WIDTH = 2;
var CONTENT_AREA_PADDING = 5;
var SECTION_WIDTH = 250;

var IE_VERTICAL_SCROLLBAR_WIDTH = 18;

dojo.declare("com.ibm.rdm.web.project.DashboardSection", [dijit._Widget, dijit._Templated], {
    // summary:
    //    This class is meant to be subclassed by all section views presented on
    //    the left hand side in the project dashboard.

    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "project/templates/DashboardSection.html"),

    ///////////////////////////////////////////////////////////////////////////////////
    // Attach points

	_sectionHeader: null,
	_clearFiltersAnchor: null,
	_toggleExpansionStateAnchor: null,
	_topBorderDiv: null,
	_topDiv: null,
    _contentPaneDiv: null,
	_bottomBorderDiv: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // Public fields

    projectURI: null,
    sectionTitle: null,
    dashboardTabPage: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // Private fields

	_messages: null,
	_commonMessages: null,
    _contentPane: null,
    _contentAreaDiv: null,
    _deferredArray: null,
	_isSectionExpanded: true,
	// _isVerticalScrollbarPresent: Boolean
	//    This field is only used to work around an IE issue.
	_isVerticalScrollbarPresent: null,

	_CLEAR_FILTERS_ENABLED_CLASS: "clear-filters-enabled",
	_CLEAR_FILTERS_DISABLED_CLASS: "clear-filters-disabled",

    ///////////////////////////////////////////////////////////////////////////////////
    // API

    postCreate: function() {
		var self = this;
		this.connect(this._sectionHeader, "onclick", this._toggleExpansionState);
		this.connect(this._clearFiltersAnchor, "onclick", function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			if (dojo.hasClass(self._clearFiltersAnchor, self._CLEAR_FILTERS_ENABLED_CLASS)) {
				self._handleClearFilters();
			}
		});
		this._clearFiltersAnchor.title = this._messages["clearFilters"];
		dojo.addClass(this._clearFiltersAnchor, this._CLEAR_FILTERS_DISABLED_CLASS);
		this._isVerticalScrollbarPresent = false;
        this._contentPane = new dijit.layout.ContentPane({}, this._contentPaneDiv);
        this._contentAreaDiv = document.createElement("div");
        dojo.addClass(this._contentAreaDiv, "content-area");
        this._contentPane.startup();
        this._contentPane.attr('content', this._contentAreaDiv);
        this._deferredArray = null;
		this._isSectionExpanded = true;
		this._updateExpansionState();
    },

    postMixInProperties: function() {
		// TODO rename nls/common.js to be nls/DashboardSection.js(?) as it's no
		// longer the case that >1 class in this package accesses it ...
        this._messages = dojo.i18n.getLocalization(
            "com.ibm.rdm.web.project", "common");
		this._commonMessages = dojo.i18n.getLocalization(
            "com.ibm.rdm.web.common", "common");
    },

    startup: function() {
        var sectionHeight = this.domNode.clientHeight - CURVED_BORDER_HEIGHT;
        var sectionWidth = this.domNode.clientWidth - SHADOW_WIDTH;
        var sectionPos = dojo._abs(this.domNode);
        var contentPanePos = dojo._abs(this._contentPane.domNode);
        var heightToUse = sectionHeight - (contentPanePos.y - sectionPos.y);
        var widthToUse = sectionWidth;
        this._contentPane.resize({h: heightToUse, w: widthToUse});
        this._populateContents();
    },

    destroy: function() {
        this.inherited(arguments);
        if (this._deferredArray) {
            for (var i = 0; i < this._deferredArray.length; i++) {
                var deferred = this._deferredArray[i];
                if (deferred.fired == -1) {
                    deferred.cancel();
                }
            }
        }
    },

	setHeight: function(height) {
		this.domNode.style.height = height + "px";
		var contentPaneSize = {
			h: (height
				- this._topBorderDiv.scrollHeight
				- this._topDiv.scrollHeight
				- this._bottomBorderDiv.scrollHeight),
			// FIXME Why is -2 needed in the following?
			// (on IE w/o this -2, the vert scrollbar would appear 2 pixels too far to right).
			w: SECTION_WIDTH - 2
		};
		this._contentPane.resize(contentPaneSize);
		this._handleHeightHasChanged();
	},

    refresh: function() {
        // Subclasses may wish to override this ...
    },

    showAsBusy: function(isBusy) {
        // Subclasses may wish to override this ...
    },

	showLoadingMessage: function(/*Optional*/messageToDisplay) {
		if (!messageToDisplay) {
			messageToDisplay = this._commonMessages["loading"];
		}
		this._contentAreaDiv.innerHTML = (
			"<span class=\"loading-message\">"
			+ messageToDisplay
			+ "</span>"
		);
	},

	isSectionExpanded: function() {
		return this._isSectionExpanded;
	},

	getCollapsedHeight: function() {
		return (
			this._topBorderDiv.scrollHeight
			+ this._topDiv.scrollHeight
			+ this._bottomBorderDiv.scrollHeight
		);
	},

    ///////////////////////////////////////////////////////////////////////////////////
    // Helpers

	_toggleExpansionState: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
        this._isSectionExpanded = (!this._isSectionExpanded);
		this._updateExpansionState();
		this.dashboardTabPage.filterSectionExpansionStateHasChanged(this);
	},

	_updateExpansionState: function() {
		if (this._isSectionExpanded) {
			this._toggleExpansionStateAnchor.title = this._commonMessages["collapse"];
			dojo.removeClass(this._toggleExpansionStateAnchor, "expand");
			dojo.addClass(this._toggleExpansionStateAnchor, "collapse");
		} else {
			this._toggleExpansionStateAnchor.title = this._commonMessages["expand"];
			dojo.removeClass(this._toggleExpansionStateAnchor, "collapse");
			dojo.addClass(this._toggleExpansionStateAnchor, "expand");
		}
	},

	_updateClearFilterState: function() {
		if (this._isAtLeastOneFilterSpecified()) {
			dojo.removeClass(this._clearFiltersAnchor, this._CLEAR_FILTERS_DISABLED_CLASS);
			dojo.addClass(this._clearFiltersAnchor, this._CLEAR_FILTERS_ENABLED_CLASS);
		} else {
			dojo.removeClass(this._clearFiltersAnchor, this._CLEAR_FILTERS_ENABLED_CLASS);
			dojo.addClass(this._clearFiltersAnchor, this._CLEAR_FILTERS_DISABLED_CLASS);
		}
	},

	_handleHeightHasChanged: function() {
		// summary:
		//    This function is called whenever the section's height has been
		//    changed.  In addition, subclasses should call this when they
		//    dynamically change their content height.  We work around an IE
		//    issue here -- without this work around IE would add a horizontal
		//    scroll bar as soon as the vertical scroll bar appears (probably
		//    because the content normally includes a table whose width is
		//    100%).
		if (dojo.isIE && this._contentAreaDiv.scrollWidth > 0) {
			var self = this;
			var resetWidthForContentAreaChildren = function(width) {
				for (var child = self._contentAreaDiv.firstChild;
					 child != null;
					 child = child.nextSibling) {
					self._contentAreaDiv.style.width = width;
				}
			};
			if (this._contentAreaDiv.scrollHeight > this._contentPane.domNode.clientHeight) {
				// A vertical scroll bar is needed.
				if (! this._isVerticalScrollbarPresent) {
					var width = this._contentAreaDiv.scrollWidth;
					resetWidthForContentAreaChildren((width - (CONTENT_AREA_PADDING*2) - IE_VERTICAL_SCROLLBAR_WIDTH) + "px");
					this._isVerticalScrollbarPresent = true;
				}
			} else {
				// A vertical scroll bar is NOT needed.
				if (this._isVerticalScrollbarPresent) {
					resetWidthForContentAreaChildren("");
					this._isVerticalScrollbarPresent = false;
				}
			}
		}
	},

    _populateContents: function() {
        // Subclasses need to override this ...
		throw new Error('Unimplemented API: com.ibm.rdm.web.project.DashboardSection._populateContents');
    },

    _handleClearFilters: function() {
        // Subclasses need to override this ...
		throw new Error('Unimplemented API: com.ibm.rdm.web.project.DashboardSection._handleClearFilters');
    },

	_filterSelectionHasChanged: function() {
        // Subclasses need to call this function whenever a new filter is added or removed.
		this._updateClearFilterState();
	},

	_isAtLeastOneFilterSpecified: function() {
        // Subclasses need to override this ...
		throw new Error('Unimplemented API: com.ibm.rdm.web.project.DashboardSection._isAtLeastOneFilterSpecified');
	}
});



})();
