/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.project.DashboardTabPage");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.form.Button");
dojo.require("dojo.date.locale");
dojo.require("dojo.date.stamp");
dojo.require("com.ibm.rdm.web.project.AttributesSection");
dojo.require("com.ibm.rdm.web.project.FoldersSection");
dojo.require("com.ibm.rdm.web.project.TagsSection");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.search.results.ResourceInfo");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.project.ResultSetView");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.provide("com.ibm.rdm.web.search.QueryBuilder");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.project", "DashboardTabPage");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");

var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var domutils = com.ibm.rdm.web.util.domutils;
var MimeTypes = com.ibm.rdm.web.common.MimeTypes;
var QueryUtils = com.ibm.rdm.web.util.queryutils;

//TODO Read these in from CSS ...
var FILTER_SECTION_MARGIN_TOP = 7;
var FILTER_SECTION_MARGIN_BOTTOM = 7;

dojo.declare(
	"com.ibm.rdm.web.project.DashboardTabPage",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    This class is used to present the content within either the
	//    "Artifacts" tab page or the "Requirements" tab page within the
	//    ProjectDashboard.

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "project/templates/DashboardTabPage.html"),
	widgetsInTemplate: false,

	///////////////////////////////////////////////////////////////////////////////////
	// Attach points

	_refreshButton: null,
	_resourcesHeaderTable: null,
	_tagsSectionDiv: null,
	_attributesSectionDiv: null,
	_foldersSectionDiv: null,
	_resultSetDiv: null,
	_totalPageCount: null,
	_pagesDisplayed: null,
	_totalResultCount: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields

	projectURI: null,
	footerHeight: null,
	// viewType: object (representing an enum)
	//    The value of this field must be one of the values found in the
	//    com.ibm.rdm.web.project.DashboardTabPage.viewType property list.
	viewType: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_tagsSection: null,
	_attributesSection: null,
	_foldersSection: null,
	_allDashboardSections: null,
	_refreshInProgress: null,
	_resourcesDiv: null,
	_resultSetView: null,
	_resultSetPaneSize: null,
	_started: false,
	_beingDestroyed: false,
	_lastTabPageY: null,
	_sectionsAndResultsSetViewAlreadyStarted: null,
	_deferredArray: null,

	_busyCursorTally: 0,

	_mapAttributeTypeToExtendedFieldType: {
		"Boolean": "booleanAttribute",
		"Date":    "dateAttribute",
		"Decimal": "decimalAttribute",
		"Enum":    "enumerationAttribute",
		"Float":   "floatAttribute",
		"Integer": "integerAttribute",
		"String":  "stringAttribute",
		"URI":     "URIAttribute"
	},

	///////////////////////////////////////////////////////////////////////////////////
	// NLS fields requiring translation

	name: null,
	lastModifiedBy: null,
	lastModifiedDate: null,

	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this._sectionsAndResultsSetViewAlreadyStarted = false;
		this._refreshInProgress = false;
		this._busyCursorTally = 0;
		// Add the "Filter Display by Folder" section (but only if we're in
		// the Artifacts tab page).
		if (this.viewType === com.ibm.rdm.web.project.DashboardTabPage.viewType.ARTIFACTS) {
			this._foldersSection = new com.ibm.rdm.web.project.FoldersSection(
				{
					projectURI: this.projectURI,
					sectionTitle: this._messages.filterDisplayByFolder,
					dashboardTabPage: this
				},
				null
			);
			this._foldersSectionDiv.appendChild(this._foldersSection.domNode);
		}

		// Add the "Filter Display by Tag" section.
		this._tagsSection = new com.ibm.rdm.web.project.TagsSection(
			{
				projectURI: this.projectURI,
				sectionTitle: this._messages.filterDisplayByTag,
				dashboardTabPage: this
			},
			null
		);
		this._tagsSectionDiv.appendChild(this._tagsSection.domNode);
		// Add the "Filter Display by Attribute" section.
		this._attributesSection = new com.ibm.rdm.web.project.AttributesSection(
			{
				projectURI: this.projectURI,
				sectionTitle: this._messages.filterDisplayByAttribute,
				dashboardTabPage: this,
				viewType: this.viewType
			},
			null
		);
		this._attributesSectionDiv.appendChild(this._attributesSection.domNode);

		this._resultSetView = new com.ibm.rdm.web.project.ResultSetView(
			{
				projectURI: this.projectURI,
				_viewType: this.viewType,
				projectToolbar: this.dashboardToolbar,
				numArtifacts: this.numArtifacts
			},
			this._resultSetDiv
			
			);
		if (this.dashboardToolbar)
			this.dashboardToolbar.attr("projectURI", this.projectURI);
		this._started = false;
		this._beingDestroyed = false;
		this._deferredArray = [];
		this._lastTabPageY = null;
		// Initialize _allDashboardSections
		this._allDashboardSections =
			[this._tagsSection, this._attributesSection];
		if (this._foldersSection) {
			this._allDashboardSections.push(this._foldersSection);
		}
	},

	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.project", "DashboardTabPage");
		this._messages2 = dojo.i18n.getLocalization("com.ibm.rdm.web.viewer", "ResourceViewer");
		dojo.forEach(
			[
				"name",
				"lastModifiedBy",
				"lastModifiedDate"
			],
			function(prop){ this[prop] = this._messages[prop]; }, this);
	},

	initIfNeeded: function() {
		// summary:
		//    Initialize the dashboard sections and results view when the
		//    tab container containing this page is first selected.
		// description:
		//    We use this function, rather than a 'startup' function, as startup
		//    is automatically called by the tab container when content is
		//    added to one of its content panes.  By using a separate function
		//    like this we can avoid needlessly initializing the content until
		//    it's actually needed.
		if (this._sectionsAndResultsSetViewAlreadyStarted) {
			return;
		}
		if (this._tagsSection) {
			this._tagsSection.startup();
		}
		if (this._attributesSection) {
			this._attributesSection.startup();
		}
		this._resultSetView.startup();
		if (this._foldersSection) {
			// we need to wait until folders section is populated 
			// so that we could get our hands on the ROOT element 
			// and make it a default selection
			this._foldersSection.startup();
			this._foldersSection._folderStore._foldersDeferred.addCallback(this, this._populateContents, null);
		}
		else {
			this._populateContents();
		}
		this._sectionsAndResultsSetViewAlreadyStarted = true;
	},

	destroy: function() {
		this._beingDestroyed = true;
		RDMClient.cancelDeferredsInCurrentUpdateContext();
		if (this._tagsSection) {
			this._tagsSection.destroyRecursive();
		}
		if (this._attributesSection) {
			this._attributesSection.destroyRecursive();
		}
		if (this._foldersSection) {
			this._foldersSection.destroyRecursive();
		}
		this._resultSetView.destroyRecursive();
		delete this._allDashboardSections;
		this.inherited(arguments);
	},

	resize: function(args) {
		// summary:
		//    This function is here to receive resize calls that are made when
		//    this widget is used as the sole content of a ContentPane.  Please
		//    see ConentPane's 'resize' function for more information.
		if (args && args.w > 0 && args.h > 0) {
			var width = args.w;
			var height = args.h;
			var self = this;
			var resizeContentPane = function() {
				var tabPagePos = dojo._abs(self.domNode);
				var contentPanePos = dojo._abs(self._resultSetView.domNode);
				self._adjustHeightsOfFilterSections(height);
				if (contentPanePos.x >= 0 && contentPanePos.y >= 0) {
					var contentPaneLeft = (
						contentPanePos.x - tabPagePos.x);
					var contentPaneSize = {
						h: height,
						w: (width - contentPaneLeft)
					};
					self._resultSetView.resize(contentPaneSize);
					self._resultSetPaneSize = contentPaneSize;
				}
			};
			resizeContentPane();
		}
	},

	handleRefresh: function() {
		// summary:
		//    This function is called whenever the user clicks on the "global
		//    refresh" button.
		if (!this._refreshInProgress) {
			this._showAsBusy(true);
			if (this._tagsSection.getSelectedTags().length > 0
				|| this._attributesSection.isAtLeastOneAttributeApplied()
				|| (this._foldersSection
					&& this._foldersSection.getSelectedFolders().length > 0)) {
				// First ask all filter sections to update themselves.  Only
				// AFTER they have all updated do we then ask to populate the
				// results area.  The reason we defer updating the results area
				// until after all filters have been updated is to account for
				// the fact that one of the selected items from these filters
				// MAY have been deleted on the server (e.g. a selected folder
				// may have been deleted, in which case the query we issue to
				// populate the results should NOT include an "and" clause for
				// this folder).
				var numOutstandingRefreshOperations = (this._foldersSection ? 3 : 2);
				var self = this;
				var onResponseHandled = function(response) {
					self._showAsBusy(false);
					return response;
				};
				var handleFilterSectionRefreshHasCompleted = function() {
					numOutstandingRefreshOperations--;
					if (numOutstandingRefreshOperations === 0) {
						if (!this._beingDestroyed) {
							self._populateContents(onResponseHandled);
						}
					}
				}
				this._tagsSection.refresh(handleFilterSectionRefreshHasCompleted);
				this._attributesSection.refresh(handleFilterSectionRefreshHasCompleted);
				if (this._foldersSection) {
					this._foldersSection.refresh(handleFilterSectionRefreshHasCompleted);
				}
			} else {
				// No tags or filters have been specified ... issue the result
				// set refresh as well as all filter section refresh requests
				// concurrently.
				var numOutstandingRefreshOperations = (this._foldersSection ? 4 : 3);
				var self = this;
				var onResponseHandled = function() {
					self._showAsBusy(false);
				};
				var handleFilterSectionOrResultSetRefreshHasCompleted = function(response) {
					numOutstandingRefreshOperations--;
					if (numOutstandingRefreshOperations === 0) {
						onResponseHandled();
					}
					return response;
				}
				self._populateContents(handleFilterSectionOrResultSetRefreshHasCompleted);
				this._tagsSection.refresh(handleFilterSectionOrResultSetRefreshHasCompleted);
				this._attributesSection.refresh(handleFilterSectionOrResultSetRefreshHasCompleted);
				if (this._foldersSection) {
					this._foldersSection.refresh(handleFilterSectionOrResultSetRefreshHasCompleted);
				}
			}
		}
	},

	refresh: function() {
		// summary:
		//    This function is called by the filter sections whenever selection
		//    change events occur.
		var self = this;
		var onResponseHandled = function(response) {
			self._showAsBusy(false);
			return response;
		};
		this._showAsBusy(true);
		this._populateContents(onResponseHandled);
	},

	hasStarted: function() {
		return this._started;
	},
	
	getResultSetView: function() {
		return this._resultSetView;
	},

	filterSectionExpansionStateHasChanged: function(dashboardSection) {
		this._adjustHeightsOfFilterSections(this._resultSetPaneSize.h, dashboardSection);
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_adjustHeightsOfFilterSections: function(height, /*Optional*/newlyExpandedOrCollapsedDashboardSection) {
		// FIXME Why do we need this fudgeFactor(?) -- there must be something else besides
		// margins that we need to account for ... but not sure what it is.
		var fudgeFactor = 4;
		var collapsedSectionHeight = this._tagsSection.getCollapsedHeight();
		var numSections = this._allDashboardSections.length;
		var totalHeightOfAllMargins =
			numSections * (FILTER_SECTION_MARGIN_TOP + FILTER_SECTION_MARGIN_BOTTOM - fudgeFactor);
		var numExpandedSections = 0;
		var totalHeightOfAllCollapsedSections = 0;
		for (var i = 0; i < numSections; i++) {
			if (! this._allDashboardSections[i].isSectionExpanded()) {
				totalHeightOfAllCollapsedSections += collapsedSectionHeight;
			} else {
				numExpandedSections++;
			}
		}
		var heightToUseForExpandedSections = Math.max(
			Math.floor(
			   (height - totalHeightOfAllMargins - totalHeightOfAllCollapsedSections)
				/ numExpandedSections
			),
			collapsedSectionHeight
		);
		if (newlyExpandedOrCollapsedDashboardSection) {
			// The user has requested that a section be collapsed or expanded.
			// We take care here to make sure that we "shrink" appropriate
			// sections BEFORE expanding other sections to account for the new
			// vertical real-estate.  We do this to avoid temporarily
			// overlapping the footer.
			var isNowExpanded = newlyExpandedOrCollapsedDashboardSection.isSectionExpanded();
			if (isNowExpanded) {
				// First shrink the other expanded sections appropriately.
				for (var i = 0; i < numSections; i++) {
					var dashboardSection = this._allDashboardSections[i];
					if (dashboardSection.isSectionExpanded()
						&& dashboardSection !== newlyExpandedOrCollapsedDashboardSection) {
						dashboardSection.setHeight(heightToUseForExpandedSections);
					}
				}
				// Now expand the newly expanded section.
				newlyExpandedOrCollapsedDashboardSection.setHeight(heightToUseForExpandedSections);
			} else {
				// First shink the newly collapse section
				newlyExpandedOrCollapsedDashboardSection.setHeight(collapsedSectionHeight);
				// Now grow the other expanded sections appropriately.
				for (var i = 0; i < numSections; i++) {
					var dashboardSection = this._allDashboardSections[i];
					if (dashboardSection.isSectionExpanded()
						&& dashboardSection !== newlyExpandedOrCollapsedDashboardSection) {
						dashboardSection.setHeight(heightToUseForExpandedSections);
					}
				}
			}
		} else {
			for (var i = 0; i < numSections; i++) {
				var dashboardSection = this._allDashboardSections[i];
				dashboardSection.setHeight(
					dashboardSection.isSectionExpanded() ?
					heightToUseForExpandedSections :
					collapsedSectionHeight
				);
			}
		}
	},

	_cancelOutstandingDeferreds: function() {
		if (this._deferredArray) {
			for (var i = 0; i < this._deferredArray.length; i++) {
				var deferred = this._deferredArray[i];
				if (deferred.fired == -1) {
					ErrorHandler.warn("Project Dashboard deferred is cancelled");
					deferred.cancel();
				}
			}
		}
		this._deferredArray = [];
	},

	_populateContents: function(onResponseHandled) {
		this._resultSetView.showLoadingArtifactsMessage();
		var queryInfo = this._composeMultiFetchPostData();
		// Note: if queryInfo is null, this means that no filters have been applied.
		if (this.viewType == com.ibm.rdm.web.project.DashboardTabPage.viewType.COLLECTION) {
			domutils.createHourGlassCursor();
			var collectionDeferred = ProjectCache.getCollection(this.resourceURI, this.revision, this.projectSnapshotURI, queryInfo);
			this._deferredArray[0] = collectionDeferred;
			collectionDeferred.addCallback(this, function(collection){
				this._handleMultiFetchSuccessResponse(collection.collectionArtifacts);
				
				//process associated collection project snapshot
				this.resourceViewer._collectionSnapshotInfoNode.innerHTML = "";
				this.resourceViewer._collectionSnapshotInfoNode.style.display = "none";
				if (collection.associatedSnapshotURI) {
					var projectSnapshotsDeferred = ProjectCache.getProjectSnapshot(collection.associatedSnapshotURI);
					projectSnapshotsDeferred.addCallback(this, function (projectSnapshot){
						if (projectSnapshot) {
							this.resourceViewer._collectionSnapshotInfoNode.style.display = "";
							this.resourceViewer._collectionSnapshotInfoNode.appendChild(document.createTextNode(this._messages2.snapshotInfoLabel + projectSnapshot.name));
						}
					});
				}
			});
			if (onResponseHandled) {
				collectionDeferred.addBoth(onResponseHandled);
			}
		}
		else if (queryInfo && (queryInfo.query || queryInfo.search)) {
			domutils.createHourGlassCursor();
			var queryDeferred = ProjectCache.executeMultiFetch(queryInfo);
			this._deferredArray[0] = queryDeferred;
			queryDeferred.addBoth(this, this._handleMultiFetchSuccessResponse);
			if (onResponseHandled) {
				queryDeferred.addBoth(onResponseHandled);
			}
		} else {
			this._resultSetView.showFilterNeededMessage();
			this._resultSetView.updateNumArtifacts(0);
			if (onResponseHandled) {
				onResponseHandled();
			}
		}
	},

	_showAsBusy: function(isBusy) {
		// summary:
		//    Push or pop the cursor's busy state.  When pushed (i.e. 'isBusy'
		//    is true) for the first time we show the busy cursor.  When popped
		//    (i.e. 'isBusy' is false) for the last time we reset the cursor to
		//    its original appearance.
		if (isBusy) {
			if (this._busyCursorTally == 0) {
				this._toggleRefreshInProgress();
			}
			this._busyCursorTally++;
		} else {
			this._busyCursorTally--;
			if (this._busyCursorTally == 0) {
				this._toggleRefreshInProgress();
			}
		}
	},

	_toggleRefreshInProgress: function() {
		if (this._refreshInProgress) {
			this._refreshInProgress = false;
			this.domNode.style.cursor = "";
			if (this._foldersSection) {
				this._foldersSection.showAsBusy(false);
			}
		} else {
			this._refreshInProgress = true;
			this.domNode.style.cursor = "progress";
			if (this._foldersSection) {
				this._foldersSection.showAsBusy(true);
			}
		}
	},

	_composeMultiFetchPostData: function() {
		var queryInfo = null;
		var selectedTagResources = (
			this._tagsSection ? 
			this._tagsSection.getSelectedTags():
			[]
		);
		var selectedFolderResources = (
			this._foldersSection ?
			this._foldersSection.getSelectedFolders() :
			[]
		);
		var isAtLeastOneAttributeApplied = (
			this._attributesSection ?
			this._attributesSection.isAtLeastOneAttributeApplied():
			false
		);
		if (selectedTagResources.length > 0
			|| isAtLeastOneAttributeApplied
			|| selectedFolderResources.length > 0
			|| this.viewType == com.ibm.rdm.web.project.DashboardTabPage.viewType.COLLECTION) {
			// Use a complicated query involving the filter sections.
			var attributeValuesInfo = ( 
				this._attributesSection ?		
					(this._attributesSection.isAttributeValueFilterApplied() ?
					this._attributesSection.getAppliedAttributeValuesInfo() : null):
					null 
			);
			var clauseQueryParameters = [];
			var attributesFilter = this._composeAttributesClauseAndNamespace(attributeValuesInfo);
			var foldersClause = this._composeFoldersClause(selectedFolderResources);
			var userNameClause = this._composeUsersAttributeClause();
			var dateRangeClause = this._composeDateRangeAttributeClause();
			var viewTypeClause = this._composeTypeClause();
			var artifactTypeClause = this._composeTypeAttributeClause();
			var tagsClause = this._composeTagsClause();
			var nameSearchParams = this._composeArtifactNameSearchValue();
			var nameSearchClause = "";
			if (nameSearchParams.searchQuery) {
				if (nameSearchParams.isLucene) {
					nameSearchClause = nameSearchParams.searchQuery;
				}
				else {
					clauseQueryParameters.push(nameSearchParams.searchQuery);
				}
			}
			clauseQueryParameters.push(foldersClause);
			clauseQueryParameters.push(attributesFilter.clause);
			clauseQueryParameters.push(userNameClause);
			clauseQueryParameters.push(dateRangeClause);
			clauseQueryParameters.push(viewTypeClause);
			clauseQueryParameters.push(artifactTypeClause);
			clauseQueryParameters.push(tagsClause);
				
			for (var i=0; i<clauseQueryParameters.length; i++) {
				// If at least one query parameter is valid, append namespaces clause.
				if (clauseQueryParameters[i] || nameSearchParams.searchQuery) {
					clauseQueryParameters.push(this._composeNamespaceClause());
					
					//20138: Optimize Dashboard queries where contentType!="x-com.ibm.rdm.saved-filter/rdf+xml" is used
					//if any of the filters below are set, append a special "filter" clause
					if (!attributesFilter.clause && !foldersClause && !viewTypeClause && !artifactTypeClause && !tagsClause && (userNameClause || dateRangeClause || nameSearchParams.searchQuery)) {
						clauseQueryParameters.push(this._composeSavedFilterClause());
					}
					break;
				}
			}
			
			var query = "";
			var numClausesAdded = 0;
			for (var i = 0; i < clauseQueryParameters.length; i++) {
				if (clauseQueryParameters[i]) {
					if (numClausesAdded > 0)
						query += ' and ';
					query += clauseQueryParameters[i];
					numClausesAdded++;
				}
			}
			
			var namespaces = 'nav=http://com.ibm.rdm/navigation#, dc=http://purl.org/dc/terms/, jazz=http://jazz.net/xmlns/foundation/1.0/, att=http://schema.ibm.com/rdm/2008/Attribute#';
			if (attributesFilter.namespace) {
				namespaces += ", " + attributesFilter.namespace;
			}

			queryInfo = {
				query: query,
				search: nameSearchClause,
				namespaces: namespaces,
				projectURI: this.projectURI,
				cancelDeferreds: true,
				disablePagination: (this.viewType == com.ibm.rdm.web.project.DashboardTabPage.viewType.COLLECTION)?false:true
			};
		}
		return queryInfo;
	},
	
	_composeNamespaceClause: function() {
		return (
			'namespace=<' + com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES + '>'
		);
	},
	
	_composeSavedFilterClause: function() {
		return (
			'contentType!="' + MimeTypes.CONTENT_TYPE_FILTER + '"'
		);
	},
	
	_composeTypeClause: function() {
		var artifactTypesClause = "";
		var mimeType = null;
		switch(this.viewType) {
			case com.ibm.rdm.web.project.DashboardTabPage.viewType.ARTIFACTS:
				// Do nothing ... we used to set the typeClause to a condition
				// that excluded Terms, but not anymore.
				break;
			case com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS:
				mimeType = MimeTypes.CONTENT_TYPE_REQUIREMENT;
				break;
			case com.ibm.rdm.web.project.DashboardTabPage.viewType.COLLECTIONS:
				mimeType = MimeTypes.CONTENT_TYPE_COLLECTION;
				break;
		}
		if (mimeType) {
			artifactTypesClause = 'contentType="' + mimeType + '"';
		}
		return artifactTypesClause;
	},

	_composeTagsClause: function() {
		var tags = this._tagsSection.getSelectedTags();
		var tagsClause = "";
		if (tags.length === 1) {
			tagsClause = 'tag=<' + tags[0] + '>';
		}
		else if (tags.length > 1) {
			tagsClause = 'tag in [';
			for (var i = 0; i < tags.length; i++) {
				tagsClause += (i === 0 ? '' : ',') + '<' + tags[i] + '>';
			}
			tagsClause += ']';
		}
		return tagsClause;
	},
	
	_composeArtifactNameSearchValue: function() {
		var artifactName = this._attributesSection.getAppliedArtifactName();
		return com.ibm.rdm.web.search.QueryBuilder.getSearchQueryParams(artifactName, false);
	},
	
	_composeUsersAttributeClause: function() {
		var usersClause = "";
		var userNames = this._attributesSection.getAppliedUsers();
		if (userNames.length === 1) {
			usersClause = 'lastModifiedBy=<' + userNames[0] + '>';
		}
		else if (userNames.length > 1) {
			usersClause = 'lastModifiedBy in [';
			for (var i = 0; i < userNames.length; i++) {
				usersClause += (i === 0 ? '' : ',') + '<' + userNames[i] + '>';
			}
			usersClause += ']';
		}
		return usersClause;
	},
	
	_composeDateRangeAttributeClause: function() {
		var dateClause = "";
		var startDate = this._attributesSection.getAppliedStartDate();
		var endDate = this._attributesSection.getAppliedEndDate();
		if (startDate) {
			var startIsoDate = dojo.date.stamp.toISOString(startDate);
			dateClause = 'dc:modified>="' + startIsoDate + '"^^dateTime';
			if (endDate) {
				// Note: we add 1 day to endDate to account for the 24 hours that
				// are found (but not represented) in this endDate.
				endDate = dojo.date.add(endDate, "day", 1);
				var endIsoDate = dojo.date.stamp.toISOString(endDate);
				dateClause += ' and dc:modified<="' + endIsoDate + '"^^dateTime';
			}
		}
		return dateClause;
	},
	
	_composeTypeAttributeClause: function() {
		var artifactTypes = this._attributesSection.getAppliedArtifactTypes();
		var artifactTypesClause = "";
		var doAllContentTypes = false;
		
		if (this.viewType != com.ibm.rdm.web.project.DashboardTabPage.viewType.ARTIFACTS) {
			//this is only applicable to the artifacts section
			return "";
		}
		
		if (artifactTypes && artifactTypes.length > 0) {
			artifactTypesClause = 'contentType in [';
			for (var i=0; i<artifactTypes.length; i++) {
				artifactTypesClause += (i==0?'':',') + '"' + artifactTypes[i] + '"';
			}
			artifactTypesClause += ']';
		}
		
		return artifactTypesClause;
	},

	_composeAttributesClauseAndNamespace: function(attributeValuesInfo) {
		var attributeValuesClause = "";
		var attributeGroupNamespace = "";
		if (attributeValuesInfo && attributeValuesInfo.attributeValues.length > 0) {
			var attributeValues = attributeValuesInfo.attributeValues;
			if (attributeValues.length === 1) {
				var quotesAroundValueStart = '"'; //integer and boolean values do not use quotes
				var quotesAroundValueEnd = '"'; //integer and boolean values do not use quotes
				if (attributeValuesInfo.attributeType == 'Integer' || attributeValuesInfo.attributeType == 'Boolean') {
					quotesAroundValueStart = '';
					quotesAroundValueEnd = '';
				}
				else if (attributeValuesInfo.attributeType == 'URI') {
					quotesAroundValueStart = '<';
					quotesAroundValueEnd = '>';
				}
				attributeValuesClause = (
					attributeValuesInfo.attributeGroupId + ':' + attributeValuesInfo.attributeNameId
					+ '=' + quotesAroundValueStart + QueryUtils.escapeQueryStringParameter(attributeValues[0]) + quotesAroundValueEnd
				);
			}
			else {
				// Handle the case of >1 attribute value (e.g. priority High or Low) ...
				var buf = '["' + QueryUtils.escapeQueryStringParameter(attributeValues[0]) + '"';
				for (var i = 1; i < attributeValues.length; i++) {
					buf += ',"' + QueryUtils.escapeQueryStringParameter(attributeValues[i]) + '"';
				}
				buf += ']';
				attributeValuesClause = (
					attributeValuesInfo.attributeGroupId + ':' + attributeValuesInfo.attributeNameId
					+ ' in ' + buf
				);
			}
			attributeGroupNamespace = attributeValuesInfo.attributeGroupId + '=' + attributeValuesInfo.attributeGroupNamespace + '#';
		}		
		return {clause: attributeValuesClause, namespace: attributeGroupNamespace};
	},

	_composeFoldersClause: function(folderResources) {
		var foldersClause = "";
		if (folderResources.length > 0) {
			foldersClause = 'parentFolder=<' + folderResources[0] + '>';
		}
		return foldersClause;
	},

	//query is limited to already known resource URIs
	_composeURIsClause: function() {
		var URIsClause = "";
		var URIs = this._URIs;
		if (URIs && URIs.length > 0) {
			URIsClause = 'resource in [';
			for (var i=0; i<URIs.length; i++) {
				URIsClause += (i==0?'':',') + '<' + URIs[i] + '>';
			}
			URIsClause += ']';
		}
		return URIsClause;
	},

	_handleMultiFetchSuccessResponse: function(response) {
		domutils.createDefaultCursor();
		this._resultSetView.setResults(response);
		this._resultSetView.showArtifacts();
 		return response;
	},

	_handleMultiFetchErrorResponse: function(error) {
		domutils.createDefaultCursor();
		if (! (error.dojoType && error.dojoType == "cancel")) {
			// TODO Implement
		}
		return arguments;
	},

	_isMoreThanOneFilterSpecified: function() {
		var numFiltersSpecified = 0;
		// Account for the "Filter Display by Tags" section.
		if (this._tagsSection.getSelectedTags().length > 0) {
			numFiltersSpecified++;
		}
		// Account for the "Filter Display by Folders" section.
		if (this._foldersSection && this._foldersSection.getSelectedFolders().length > 0) {
			numFiltersSpecified++;
		}
		if (numFiltersSpecified === 2) { return true; }
		// Account for the "Filter Display by Attributes" section.
		if (this._attributesSection.getAppliedArtifactName()) {
			numFiltersSpecified++;
			if (numFiltersSpecified === 2) { return true; }
		}
		if (this._attributesSection.getAppliedUsers().length > 0) {
			numFiltersSpecified++;
			if (numFiltersSpecified === 2) { return true; }
		}
		if (this._attributesSection.getAppliedStartDate()
			|| this._attributesSection.getAppliedEndDate()
			) {
			numFiltersSpecified++;
			if (numFiltersSpecified === 2) { return true; }
		}
		var artifactTypes = this._attributesSection.getAppliedArtifactTypes();
		if (artifactTypes && artifactTypes.length > 0) {
			numFiltersSpecified++;
			if (numFiltersSpecified === 2) { return true; }
		}
		if (this._attributesSection.isAttributeValueFilterApplied()) {
			numFiltersSpecified++;
			if (numFiltersSpecified === 2) { return true; }
		}
		return false;
	}
});



})();

com.ibm.rdm.web.project.DashboardTabPage.viewType = {
	ARTIFACTS: 0,
	REQUIREMENTS: 1,
	RESOURCES: 2,
	COLLECTIONS: 3,
	COLLECTION: 4
};
