/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.web.project.DashboardToolbarWidget");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.form.Button");
dojo.require("dijit.Toolbar");
dojo.require("dojo.date.locale");
dojo.require("com.ibm.rdm.web.project.UploadFileWidget");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.project", "DashboardToolbarWidget");
dojo.requireLocalization("com.ibm.rdm.web.common", "common");


dojo.declare(
    "com.ibm.rdm.web.project.DashboardToolbarWidget",
    [dijit._Widget, dijit._Templated],
{
    // summary:
    //    Provides a title section that highlights when moused over and
    //    expands/collpses its content when clicked.

    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "project/templates/DashboardToolbarWidget.html"),
    widgetsInTemplate: true,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

    viewType: null,
	projectURI: null,
	resourcesPage: null,
	doDisplayWithRefreshButton: null,
    
	///////////////////////////////////////////////////////////////////////////
	// Attach points

    _highlightRecentModifiedNode: null,
    _highlightRecentCommentedNode: null,


	///////////////////////////////////////////////////////////////////////////
	// Private fields

    _artifactsResultSetView: null,
    _requirementsResultSetView: null,
    _currentResultSet: null,
    _connections: null,
    _menuItems: null,
    _uploadFileWidget: null,

	///////////////////////////////////////////////////////////////////////////
	// API

    postCreate: function() {
    	this._connections = [];
    	this._menuItems = [this._sortByNameNode,
  				    	 this._sortByUserNode,
  				    	 this._sortByDateNode,
  				    	 this._sortByTypeNode,
  				    	 this._groupByNoneNode,
  				    	 this._groupByUserNode,
  				    	 this._groupByTypeNode,
  				    	 this._groupByDateNode];
    	
    	this.inherited("postCreate", arguments);
    	this._setUploadAuthorization();       
    	//this._uploadFileWidget = new com.ibm.rdm.web.project.UploadFileWidget({projectURI: this.projectURI}, null);
		if (this.doDisplayWithRefreshButton) {
			this._refreshNode.domNode.style.display = "";
		}
    },
    
    postMixInProperties: function() {
    	this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.project", "DashboardToolbarWidget");
		this._commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
    },

    startup: function() {
    	this.inherited("startup", arguments);
    },
    
    destroy: function() {
    	this._removeConnections();
    	this._toolbarSortGroupNode.destroyRecursive();
    	this._highlightRecentModifiedNode.destroyRecursive();
    	this._highlightRecentCommentedNode.destroyRecursive();
    	this._toolbar.destroyRecursive();
    	this.inherited("destroy", arguments);
    },
    
    _removeConnections: function() {
    	dojo.forEach(this._connections, function(link) {
    		dojo.disconnect(link);
    	});
    },
    
    _createConnections: function() {
    	this._connections.push(dojo.connect(this._sortByNameNode, "onClick", this._currentResultSet, this._currentResultSet.sortByName));
    	this._connections.push(dojo.connect(this._sortByUserNode, "onClick", this._currentResultSet, this._currentResultSet.sortByUser));
    	this._connections.push(dojo.connect(this._sortByDateNode, "onClick", this._currentResultSet, this._currentResultSet.sortByDate));
    	this._connections.push(dojo.connect(this._groupByNoneNode, "onClick", this._currentResultSet, this._currentResultSet.noGrouping));
    	this._connections.push(dojo.connect(this._groupByUserNode, "onClick", this._currentResultSet, this._currentResultSet.groupByUser));
    	this._connections.push(dojo.connect(this._groupByDateNode, "onClick", this._currentResultSet, this._currentResultSet.groupByDate));
        if (this.viewType === com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS) {
        	this._groupByTypeNode.attr("disabled", true);
        	this._sortByTypeNode.attr("disabled", true);
        } else {
        	this._groupByTypeNode.attr("disabled", false);
        	this._sortByTypeNode.attr("disabled", false);
        	this._connections.push(dojo.connect(this._sortByTypeNode, "onClick", this._currentResultSet, this._currentResultSet.sortByType));
        	this._connections.push(dojo.connect(this._groupByTypeNode, "onClick", this._currentResultSet, this._currentResultSet.groupByType));
        }
    	this._connections.push(
			dojo.connect(
				this._highlightRecentModifiedNode,
				"onClick",
				this._currentResultSet,
				this._currentResultSet.highlightRecentArtifacts
		));
    	this._connections.push(
			dojo.connect(
				this._highlightRecentCommentedNode,
				"onClick",
				this._currentResultSet,
				this._currentResultSet.highlightRecentCommentedArtifacts
		));
		if (this.doDisplayWithRefreshButton) {
	    	this._connections.push(
				dojo.connect(
					this._refreshNode,
					"onClick",
					this.resourcesPage,
					this.resourcesPage.handleRefresh
			));
		}
    	// TODO: Disabling until File Upload is re-enabled
    	//this._connections.push(dojo.connect(this._fileUploadNode, "onClick", this, this.uploadFile));
		var self = this;
    	dojo.forEach(
			this._menuItems,
			function(item) {
				self.connect(item, "onClick", self._checkMenuItem);
			}
		);
    },
    
    _setUploadAuthorization: function(){
  
	    //Only enable the upload button if the current user has Writer or administrative rights.
	    /*var self = this;		 
	    var onUserRolesComplete = function(userRoleInfos) {	
    		var isAuthorized=false;
			var thisuser = com.ibm.rdm.web.AUTHENTICATED_USER.userId;
			for(var userURI in userRoleInfos){
				if(userRoleInfos[userURI].userId == thisuser){
					for(var j=0; j < userRoleInfos[userURI].userRoles.length; j++){				
						var role = userRoleInfos[userURI].userRoles[j];
						if (role == "Administrator"){
							isAuthorized = true;							
						}
						if(role == "Writer"){
							isAuthorized = true;
						}
					}
				}
			}						
			if(!isAuthorized){
		    	// TODO: Disabling until File Upload is re-enabled
				// self._fileUploadNode.disabled=true;		        
			}
			return userRoleInfos;
		};
		
    	var deferred = com.ibm.rdm.web.common.ProjectCache.getProjectUserRoles(this.projectURI);
    	deferred.addCallback(this, onUserRolesComplete);*/
    },
    uploadFile: function(){   
    	
    	/*var dialogWidget = dijit.byId("com.ibm.rdm.web.project.UploadFileDialog");    	
       
    	if (!dialogWidget) {
    		form = this._uploadFileWidget;
    		dialogWidget = new dijit.Dialog(
    		 {
    			id: "com.ibm.rdm.web.project.UploadFileDialog",
    			title: "Upload Artifact",
    			duration: 100
    		 },
   				form.domNode
    		);
    		form._dialogWidget = dialogWidget;
    	}

    	form._dialogWidget = dialogWidget;
    	form.currentProject = this.projectURI;
    	form.show();*/
    },
    
    setViewType: function(viewType, resultSetView) {
    	this.viewType = viewType;
    	this._currentResultSet = resultSetView;
    	
    	this._removeConnections();
    	this._createConnections();
    	this._checkMenuItem();
    	this._currentResultSet.updateNumArtifacts(this._currentResultSet._arrayListOfResources.length);
    },    
    
    _checkMenuItem: function() {
    	var groupOrder = this._currentResultSet.getGroupOrder();
    	var sortOrder = this._currentResultSet.getSortOrder();
    	var isEnabled = this._currentResultSet.getToggleButtonState();
    	
    	dojo.forEach(this._menuItems, function(item){
    		dojo.removeClass(item.iconNode, "commentingToolbarDropdownBullet");
    	});

    	if(groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.USER) {
    		dojo.addClass(this._groupByUserNode.iconNode, "commentingToolbarDropdownBullet");
    		if (this.viewType !== com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS) {
    			this._sortByTypeNode.attr("disabled", false);
    		}
        	this._sortByUserNode.attr("disabled", true);
    	} else if (groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.TYPE) {
    		dojo.addClass(this._groupByTypeNode.iconNode, "commentingToolbarDropdownBullet");
        	this._sortByUserNode.attr("disabled", false);
        	this._sortByTypeNode.attr("disabled", true);
    	} else if(groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.DATE) {
    		dojo.addClass(this._groupByDateNode.iconNode, "commentingToolbarDropdownBullet");
    		this._sortByUserNode.attr("disabled", false);
    		if (this.viewType !== com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS) {
    			this._sortByTypeNode.attr("disabled", false);
    		}
    	} else {
    		dojo.addClass(this._groupByNoneNode.iconNode, "commentingToolbarDropdownBullet");
    		this._sortByUserNode.attr("disabled", false);
    		if (this.viewType !== com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS) {
    			this._sortByTypeNode.attr("disabled", false);
    		}
    	}
    	
    	if(sortOrder == com.ibm.rdm.web.project.ResultSetView.sortBy.NAME) {
    		dojo.addClass(this._sortByNameNode.iconNode, "commentingToolbarDropdownBullet");	
    	} else if(sortOrder == com.ibm.rdm.web.project.ResultSetView.sortBy.TYPE) {
    		dojo.addClass(this._sortByTypeNode.iconNode, "commentingToolbarDropdownBullet");	
    	} else if(sortOrder == com.ibm.rdm.web.project.ResultSetView.sortBy.USER) {
    		dojo.addClass(this._sortByUserNode.iconNode, "commentingToolbarDropdownBullet");	
    	} else {
    		dojo.addClass(this._sortByDateNode.iconNode, "commentingToolbarDropdownBullet");	
    	}
    	
    	this._highlightRecentModifiedNode.attr('checked', isEnabled);
    }
});
