/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.project.ProjectDashboard");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.form.Button");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.Menu");
dojo.require("com.ibm.rdm.web.project.DashboardSection");
dojo.require("com.ibm.rdm.web.project.DashboardTabPage");
dojo.require("com.ibm.rdm.web.project.projectutils");
dojo.require("com.ibm.rdm.web.search.results.ResourceInfo");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.project.DashboardToolbarWidget");
dojo.require("com.ibm.rdm.web.viewer.viewerutils");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.rdm.web.viewer", "ResourceViewer");
dojo.requireLocalization("com.ibm.rdm.web.project", "ProjectDashboard");


var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var projectutils = com.ibm.rdm.web.project.projectutils;
var viewerutils = com.ibm.rdm.web.viewer.viewerutils;


dojo.declare(
    "com.ibm.rdm.web.project.ProjectDashboard",
    [dijit._Widget, dijit._Templated],
{
    // summary:
    //    Shows a project dashboard view similar to that displayed by the
    //    native RCP client.

    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "project/templates/ProjectDashboard.html"),
    widgetsInTemplate: true,

    ///////////////////////////////////////////////////////////////////////////////////
    // Attach points

    _headerImage: null,
	_projectTitleDropDown: null,
	_openInRichClientMenuItem: null,
	_numArtifactsSpan: null,
    _tabContainer: null,
    _artifactsContentPane: null,
    _requirementsContentPane: null,
    _collectionsContentPane: null,
    _toolbar: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // Public fields

    projectURI: null,
	footerHeight: null,
	resourcesPage: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // Private fields

	_messages: null,
	_resourceViewerMessages: null,
	_tabContainerOffsetTop: null,
	_initialSetDimensionsHasBeenInvoked: null,
	_windowResizeEventHandle: null,
    _artifactsTabPage: null,
    _requirementsTabPage: null,
    // _currentTabPage: object
    //    The value of this field will either be the '_artifactsTabPage' or the
    //    '_requirementsTabPage'.
    _currentTabPage: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // API

    postCreate: function() {
        var img = this._headerImage;
        // TODO Set img via template html file instead?
        img.setAttribute("src", "com.ibm.rdm.web/project/images/icons/header/project-view.gif");
        this._numArtifactsSpan.appendChild(document.createTextNode(""));
        ProjectCache.getProject(this.projectURI).addCallback(
			this,
    		function(project) {
				this._projectTitleDropDown.attr("label", project.name);
    			return project;
    		}
    	);
        this._toolbar = new com.ibm.rdm.web.project.DashboardToolbarWidget(
			{
            	projectURI: this.projectURI,
				resourcesPage: this.resourcesPage,
				doDisplayWithRefreshButton: true
        	},
			this._toolbarNode
		);
        this._artifactsTabPage = new com.ibm.rdm.web.project.DashboardTabPage(
            {
                projectURI: this.projectURI,
                viewType: com.ibm.rdm.web.project.DashboardTabPage.viewType.ARTIFACTS,
				footerHeight: this.footerHeight,
				dashboardToolbar: this._toolbar,
				numArtifacts: this._numArtifactsSpan
            },
            null
        );
        this._requirementsTabPage = new com.ibm.rdm.web.project.DashboardTabPage(
            {
                projectURI: this.projectURI,
                viewType: com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS,
				footerHeight: this.footerHeight,
				dashboardToolbar: this._toolbar,
				numArtifacts: this._numArtifactsSpan
            },
            null
        );
        this._collectionsTabPage = new com.ibm.rdm.web.project.DashboardTabPage(
            {
                projectURI: this.projectURI,
                viewType: com.ibm.rdm.web.project.DashboardTabPage.viewType.COLLECTIONS,
				footerHeight: this.footerHeight,
				dashboardToolbar: this._toolbar,
				numArtifacts: this._numArtifactsSpan
            },
            null
        );
        this._currentTabPage = this._artifactsTabPage;
		this._tabContainerOffsetTop = null;
        // TODO Is there a cleaner approach to knowing when a tab page is
        // selected?  Use this for now ...
        this.connect(this._tabContainer, "selectChild", this._tabPageSelected);
        this._tabPageSelected(this._currentTabPage);
        
        this._cshNode.appendChild(new jazz.app.ua.PageHoverHelp({uaKey: com.ibm.rdm.web.help.context.project}).domNode);
		this.connect(this._openInRichClientMenuItem, "onClick", this._handleOpenInRichClientLinkClick);
    },

    postMixInProperties: function() {
        this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.project", "ProjectDashboard");
        this._resourceViewerMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.viewer", "ResourceViewer");
    },

    startup: function() {
		this._tabContainer.startup();
        this.inherited(arguments);
    },

    destroy: function() {
    	this._toolbar.destroyRecursive();
        this._tabContainer.destroyRecursive();
        this._artifactsTabPage.destroyRecursive();
        this._requirementsTabPage.destroyRecursive();
        this._collectionsTabPage.destroyRecursive();
        this.inherited(arguments);
    },

    setDimensions: function(width, height) {
		var isFirstTimeThrough = (! this._initialSetDimensionsHasBeenInvoked);
		this._initialSetDimensionsHasBeenInvoked = true;
		var self = this;
		var resizeTabContainer = function() {
			self._tabContainer.resize({
				w: width,
				h: (height - self._getTabContainerOffsetTop())
			});
			if (isFirstTimeThrough) {
				self._artifactsContentPane.attr('content', self._artifactsTabPage.domNode);
				self._requirementsContentPane.attr('content', self._requirementsTabPage.domNode);
				self._collectionsContentPane.attr('content', self._collectionsTabPage.domNode);
			}
		}
		if (width > 0 && height > 0) {
			if (isFirstTimeThrough) {
				resizeTabContainer();
			} else {
				if (this._windowResizeEventHandle) {
					window.clearTimeout(this._windowResizeEventHandle);
				}
		    	this._windowResizeEventHandle = window.setTimeout(
		        	function () {
		        		resizeTabContainer();
		    		},
		    		250 //interval between dispatches
		    	);
			}
		}
    },

    handleRefresh: function() {
        this._currentTabPage.handleRefresh();
    },

    ///////////////////////////////////////////////////////////////////////////////////
    // Helpers

	_handleOpenInRichClientLinkClick: function() {
		window.location = viewerutils.getNativeClientUrlForOpenProject(this.projectURI);
	},

	_getTabContainerOffsetTop: function() {
		// Note: For some reason, when you similtaneously both shorten and
		// narrow the Browser window, the following:
		//    this._tabContainer.domNode.offsetTop
		// is set to a really high number (somewhere in the hundreds) when
		// it's normally set to only 28 (at least in Firefox).  To avoid this
		// situation we're caching the value it's set at originally and then
		// we make use of this initial offset value whenever resize events
		// occur.
		if (!this._tabContainerOffsetTop) {
			this._tabContainerOffsetTop = dojo.coords(this._titleDiv).h;
		}
		return this._tabContainerOffsetTop;
	},

    _tabPageSelected: function(tabPage) {
        // summary:
		//    A tab page has been selected.  By default we only call 'startup'
		//    on the Artifacts tab page and defer calling 'startup' for the
		//    Requirements tab page until the first time the user selects this
		//    page.
        if (tabPage.id.indexOf("requirementsTab") != -1) {
            // The "Requirements" tab has been selected.
            if (! this._requirementsTabPage.hasStarted()) {
                // Startup the "Requirements" tab page.
                this._requirementsTabPage.startup();
            }
            this._currentTabPage = this._requirementsTabPage;
            this._toolbar.setViewType(com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS,
            		this._requirementsTabPage.getResultSetView());
        }
        else if (tabPage.id.indexOf("collectionsTab") != -1) {
            // The "Requirements" tab has been selected.
            if (! this._collectionsTabPage.hasStarted()) {
                // Startup the "Requirements" tab page.
                this._collectionsTabPage.startup();
            }
            this._currentTabPage = this._collectionsTabPage;
            this._toolbar.setViewType(com.ibm.rdm.web.project.DashboardTabPage.viewType.COLLECTIONS,
            		this._collectionsTabPage.getResultSetView());
        }
        else {
            // The "Artifacts" tab has been selected.
            this._currentTabPage = this._artifactsTabPage;
            this._toolbar.setViewType(com.ibm.rdm.web.project.DashboardTabPage.viewType.ARTIFACTS,
            		this._artifactsTabPage.getResultSetView()); 
        }
		this._currentTabPage.initIfNeeded();
    }
});

})();
