/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.project.ProjectSection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.pages.ActionViewCache");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");

dojo.requireLocalization("com.ibm.rdm.web.common", "common");

var UriHelper = com.ibm.rdm.web.common.UriHelper;
var domutils = com.ibm.rdm.web.util.domutils;
var ActionViewCache = com.ibm.rdm.web.pages.ActionViewCache;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var PAGE_SIZE = 7;

dojo.declare("com.ibm.rdm.web.project.ProjectSection", [dijit._Widget, dijit._Templated], {

    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "project/templates/ProjectSection.html"),

    ///////////////////////////////////////////////////////////////////////////////////
    // Public fields

    ///////////////////////////////////////////////////////////////////////////////////
    // Private fields

    _messages: null,
    _isStarted: null,
    _currProjName: null,
    _isDoneLoading: null,
    _currentPage: null,
    _totalPages: null,
    _mouseOverHandle: null,
    _mouseOutHandle: null,
    _projects: null,
    _tbodyNode: null,
    

    ///////////////////////////////////////////////////////////////////////////////////
    // API

    postCreate: function() {
        this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
        this._currentPage = 1;
        this._previousLink.innerHTML = this._messages.previous;
        this._nextLink.innerHTML = this._messages.next;
		this._firstLinkImage.src = dojo.moduleUrl("com.ibm.rdm.web", "common/images/icons/etool16/beginning.gif");
		this._lastLinkImage.src = dojo.moduleUrl("com.ibm.rdm.web", "common/images/icons/etool16/end.gif");
    },
    
    startup: function() {
    	if (!this._isStarted) {
        	var tableNode = document.createElement("table");
        	dojo.addClass(tableNode, "projectsTable");
        	this._tbodyNode = document.createElement("tbody");
        	tableNode.appendChild(this._tbodyNode);
        	this._projSection.appendChild(tableNode);
    		this._populateContents();
    		this._isStarted = true;
    	}
    },

    ///////////////////////////////////////////////////////////////////////////////////
    // Helpers

    _populateContents: function() {
        this._isDoneLoading = false;
        var resourceQueryParamsString = "resource-uri=*";
        var deferred = com.ibm.rdm.web.common.ProjectCache.getUserMemberProjects();
        deferred.addCallback(this, this._handleLoadProjectRoles);        
    },
    
    handleRefresh: function(){
		this._populateContents();
    },
    
    highlightProject: function(projURI) {
    	var currProjName = com.ibm.rdm.web.util.queryutils.getLastPathSegment(projURI);
    	this._currProjName = com.ibm.rdm.web.util.queryutils.decode(currProjName);
    	if (this._isDoneLoading) {
    		this._highlightProject();
    	}
    	return projURI;
    },
    
    removeHighlight: function() {
    	var projects = this.domNode.getElementsByTagName("a");
    	this._currProjName = null;
    	for (var i = 0; i < projects.length; i++) {
    		dojo.removeClass(projects[i], "highlight-project");
    	}
    },
    
    _highlightProject: function() {
    	var projects = this.domNode.getElementsByTagName("a");
    	for (var i = 0; i < projects.length; i++) {
    		var projectName = (projects[i].firstChild)?
    				projects[i].firstChild.nodeValue : "";
    		if (projectName == this._currProjName) {
    			dojo.addClass(projects[i], "highlight-project");
    		} else {
    			dojo.removeClass(projects[i], "highlight-project");
    		}
    	}
    },
    
    _handleLoadProjectRoles: function(projects) {
		for(var i = this._tbodyNode.rows.length; i > 0;i--) {
			this._tbodyNode.deleteRow(i -1);
		}
    	var projArray = new Array();
    	var i = 0;
    	for(proj in projects){
    		projArray[i] = projects[proj];
    		i++;
    	}
    	this._projects = projects;
    	this._updatePagination(projArray.length);
    	projArray.sort(compareProjectName);
    	
        if(projArray){
        	var projCount = projArray.length;
        }
        else{
        	var projCount = 0;
        }  
        for (var i = (this._currentPage-1)*PAGE_SIZE; i < projArray.length && i < ((this._currentPage-1)*PAGE_SIZE+PAGE_SIZE); i++){
            var projectURI = projArray[i].url;
            var projectName = projArray[i].name;
            var anchor = ResourceLinksHandler.getResourceLink(projectURI, projectName, false, true);
            var projectTr = document.createElement("tr");
            var projectTd = document.createElement("td");
            dojo.addClass(projectTd, "projectNameTd");
            projectTd.appendChild(anchor);
            dojo.connect(anchor, "onclick", dojo.hitch(this, this._handleProjectLinkClick, encodeURIComponent(projectURI)));
            var imgNode = document.createElement("img");
            var imgTD = document.createElement("td");
            dojo.addClass(imgTD, "projectImageTd");
			imgNode.setAttribute("src", dojo.moduleUrl("com.ibm.rdm.web", "project/images/icons/obj16/project.gif"));
			imgTD.appendChild(imgNode);
			projectTr.appendChild(imgTD);
			projectTr.appendChild(projectTd);
			this._tbodyNode.appendChild(projectTr);
        }

        this._isDoneLoading = true;
        this._updateUserDashHeader(projCount);
        return projects;
    },
    
    _updateUserDashHeader: function(projCount) {
    	this.site.setTitle(dojo.string.substitute(this._messages.myProjects, [projCount]));	
    },

    _handleProjectLinkClick: function(projectURI, event) {
    	dojo.stopEvent(event);
        var href = net.jazz.ajax._contextRoot + net.jazz.ajax._webuiPrefix + "rrc" + (djConfig.isDebug ? "?debug=true" : "") + "#action=com.ibm.rdm.web.pages.showProjectDashboard&projectURI=" + projectURI;
		ActionViewCache.removeCachedActionView("showProjectDashboard", projectURI);
		if (window.parent) {
			// we're in an iframe
			try {
				window.parent.location.href = href;
				return;
			}
			catch (e) {
				// permission denied (different origin)
				// fall back to current window
			}
		}
		window.location.href = href;
    },
    
    _handleFirstPage: function() {
		this._currentPage = 1;
        dojo.addClass(this._first, "invisible");
        dojo.addClass(this._prev, "invisible");
        dojo.removeClass(this._last, "invisible");
        dojo.removeClass(this._next, "invisible");
        this._clearLastAndShowCurrentPageResults();
	},
	
	_handlePreviousPage: function(){
		if (this._currentPage > 1) {
        	this._currentPage--;
        	if (this._currentPage == 1) {
        		dojo.addClass(this._first, "invisible");
                dojo.addClass(this._prev, "invisible");
        	}
        	dojo.removeClass(this._next, "invisible");
            dojo.removeClass(this._last, "invisible");
        	this._clearLastAndShowCurrentPageResults();
        }
	},
	
	_handleNextPage: function(){
		if (this._currentPage < this._totalPages) {
        	this._currentPage++;
        	if (this._currentPage == this._totalPages) {
        		dojo.addClass(this._next, "invisible");
                dojo.addClass(this._last, "invisible");		
        	}
        	dojo.removeClass(this._first, "invisible");
            dojo.removeClass(this._prev, "invisible");
        	this._clearLastAndShowCurrentPageResults();
        }	
	},
	
	_handleLastPage: function(){
		this._currentPage = this._totalPages;
    	dojo.addClass(this._next, "invisible");
        dojo.addClass(this._last, "invisible");
        dojo.removeClass(this._first, "invisible");
        dojo.removeClass(this._prev, "invisible");
        this._clearLastAndShowCurrentPageResults();
		
	},
	
	_showNavOptions: function(event) {
		if (event) {
        	event.preventDefault();
    		event.stopPropagation();
        }
        if (this._totalPages > 1) {
        	if (this._currentPage != this._totalPages) {
        		dojo.removeClass(this._next, "invisible");
        		dojo.removeClass(this._last, "invisible");
        	}
        	if (this._currentPage != 1) {
        		dojo.removeClass(this._first, "invisible");
                dojo.removeClass(this._prev, "invisible");
        	}
        }
	},
        
	_hideNavOptions: function(event) {
		if (event) {
        	event.preventDefault();
    		event.stopPropagation();
        }
        dojo.addClass(this._first, "invisible");
        dojo.addClass(this._last, "invisible");
        dojo.addClass(this._prev, "invisible");
        dojo.addClass(this._next, "invisible");        	
	},
	
	_clearLastAndShowCurrentPageResults: function() {
		//this._pageText.innerHTML = dojo.string.substitute(this._messages.pageNOfM, [this._currentPage, this._totalPages]);
		this._handleLoadProjectRoles(this._projects);
	},
	
	_updatePagination: function(projCount){

        this._totalPages = Math.ceil(projCount/PAGE_SIZE);
        this._pageText.innerHTML = 
            dojo.string.substitute(this._messages.pageNOfM, [this._currentPage,this._totalPages]);
        if (this._totalPages == 0) {
        	this._footer.style.visibility = "hidden";
        } else {
        	this._footer.style.visibility = "";
        	if (!this._mouseOverHandle) {
        		this._mouseOverHandle = this.connect(this.domNode, "onmouseover", this._showNavOptions);
        	}
        	if (!this._mouseOutHandle) {
        		this._mouseOutHandle = this.connect(this.domNode, "onmouseout", this._hideNavOptions);
        	}
        }
    }

});


})(); // End of Dojo Declare

compareProjectName = function(projectArray1, projectArray2) {
	  var name1 = projectArray1.name.toLowerCase();
	    var name2 = projectArray2.name.toLowerCase();
	    return ((name1 < name2) ? -1 : 1);
};
