/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.ApprovalEditor");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.form.Button");
dojo.require("dojo.date.locale");

dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.review.ParticipantResultsParser");
dojo.require("com.ibm.rdm.web.review.ReviewCache");
dojo.require("com.ibm.rdm.web.review.ReviewContainer");
dojo.require("com.ibm.rdm.web.review.ReviewParser");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.review.reviewUpdateMediator");
dojo.require("com.ibm.rdm.web.server.Headers");
dojo.require("com.ibm.rdm.web.server.XhrUtils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.util.queryutils");


var Headers = com.ibm.rdm.web.server.Headers;
var ParticipantResultsParser = com.ibm.rdm.web.review.ParticipantResultsParser;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var ReviewCache = com.ibm.rdm.web.review.ReviewCache;
var ReviewContainer = com.ibm.rdm.web.review.ReviewContainer;
var ReviewParser = com.ibm.rdm.web.review.ReviewParser;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var XhrUtils = com.ibm.rdm.web.server.XhrUtils;
var queryutils = com.ibm.rdm.web.util.queryutils;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;
var reviewUpdateMediator = com.ibm.rdm.web.review.reviewUpdateMediator;


dojo.declare(
	"com.ibm.rdm.web.review.ApprovalEditor",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    An editor capable of showing Review artifacts.

	// NOTE: This class is called ApprovalEditor instead of ReviewEditor due to
	// the fact that we may wish to enable creating reviews via the web in the
	// future (and therefore we'd probably call that editor ReviewEditor).

	templateString: "<div class=\"rdmApprovalEditor\"><div dojoAttachPoint=\"_contentDiv\"></div></div>",

	///////////////////////////////////////////////////////////////////////////
	// Attach points

	_contentDiv: null,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	resourceURI: null,
	resourceViewer: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_reviewContainer: null,
	_width: null,
	_height: null,

	///////////////////////////////////////////////////////////////////////////
	// NLS fields requiring translation

	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		// Note: The Comments section of the sidebar assumes that ResourceViewer
		// has a reviewId field set.  This is normally only set when
		// ResourcesPage's showArtifactInReviewContext function is involved.
		// However, when displaying the Review itself, we should ALSO make sure
		// to set this field appropriately, so we do so here ...
		this.resourceViewer.attr("reviewId", queryutils.getLastPathSegment(this.resourceURI));
	},

	setDimensions: function(width, height) {
		this._width = width;
		this._height = Math.round(height); // FIXME Why is height passed as float?
		this.domNode.style.width = this._width + "px";
		this.domNode.style.height = this._height + "px";
		if (this._reviewContainer) {
			this._reviewContainer.setDimensions(this._width, this._height);
		}
	},

	loadResource: function(contentType, revision) {
		var deferred = RDMClient.doGet({
			uri: this.resourceURI,
			revision: revision,
		    projectSnapshotURI: this.resourceViewer.projectSnapshotURI
		});
		deferred.addCallback(this, this._handleLoadResponse);
		return deferred;
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_handleLoadResponse: function(response) {
		var reviewParser = new ReviewParser({
			response: response
		});
		var revision = XhrUtils.getResponseEtag(response);
		var self = this;
		var onDescriptionComplete = function(indexedProperties) {
			var projectURI = indexedProperties.projectURI;
			reviewParser.setProjectUri(projectURI);
			if (reviewParser.getReviewState() === reviewTypes.REVIEW_STATE_TYPES.Draft) {
				// Note: If the review is in the Draft state we don't need to
				// ask for vote casting results.  Populate the contents ...
				self._updateReviewCache(reviewParser);
				self._populateContents(reviewParser);
			} else {
				// Ask for vote casting results and THEN populate the contents.
				self._loadParticipantResultsAndPopulateContents(
					reviewParser, self.resourceURI, revision, projectURI);
			}
			return indexedProperties;
		};
		var deferred = ProjectCache.getResourceDescriptionMultiFetch(this.resourceURI);
		deferred.addCallback(onDescriptionComplete);
		return response;
	},

	_loadParticipantResultsAndPopulateContents: function(reviewParser, reviewURI, revision, projectURI) {
		var headers = new Object;
		headers[Headers.OWNING_CONTEXT] = projectURI;
		var self = this;
		var updateReviewCacheAndPopulateContents = function() {
			self._updateReviewCache(reviewParser);
			self._populateContents(reviewParser);
		};
		var onParticipantResultsComplete = function(response) {
			var participantResultsParser = new ParticipantResultsParser({
				doc: response.doc
			});
			reviewParser.updateParticipantResults(participantResultsParser);
			var participantResultResourceURI = reviewParser.getParticipantResultUri();
			if (participantResultResourceURI) {
				var headDeferred =
					ReviewCache.getParticipantResultHeadDeferred(participantResultResourceURI, projectURI);
				headDeferred.addCallback(this, function(response){
					reviewParser.setParticipantResultRevision(XhrUtils.getResponseEtag(response));
					updateReviewCacheAndPopulateContents();
					return response;
				});
			} else {
				updateReviewCacheAndPopulateContents();
			}
			return response;
		}
		var deferred = RDMClient.doGet({
			uri: UriHelper.getReviewParticipantResultsUri(reviewURI),
			xhrArgs: {
				headers: headers
			}
		});
		deferred.addCallback(onParticipantResultsComplete);
	},

	_updateReviewCache: function(reviewParser) {
		// summary:
		//    Update the ReviewCache with the newly constructed ReviewParser
		//    that this class needs to independently construct.
		// description:
		//    Due to the fact that this class needs its own independent
		//    loadResource function (in order to live well with ResourceViewer)
		//    it's not possible to just make use of the ReviewCache to obtain
		//    our sought after reviewParser.  Instead this class constructs one
		//    on its own and then "shares it" with the ReviewCache by calling
		//    this function.
		ReviewCache.setReview(
			reviewParser.getSelfUri(),
			reviewParser.getRevision(),
			reviewParser
		);
	},

	_populateContents: function(reviewParser) {
		if (this._reviewContainer != null) {
			reviewUpdateMediator.handleRefresh(reviewParser);
		} else {
			var oldHeight = this._height;
			var oldWidth = this._width;
			var reviewContainer = new ReviewContainer({
				resourceViewer: this.resourceViewer,
				reviewParser: reviewParser,
				width: this._width,
				height: this._height
			});
			this._contentDiv.appendChild(reviewContainer.domNode);
			reviewContainer.startup();
			// Note: It's possible that setDimensions has been invoked since
			// setting the oldHeight/oldWidth vars ... in which case we need
			// to inform the reviewContainer of the new dimensions as we just
			// added content that needs to be displayed.
			// More specifically, this 'if' clause is needed to avoid a
			// situation in which Draft reviews will frequently appear without
			// content [see Defect 22503] due to the fact that a different
			// deferred may be involved in calling setDimensions "behind this
			// deferred's back" (so to speak).
			if (oldHeight != this._height || oldWidth != this._width) {
				reviewContainer.setDimensions(this._width, this._height);
			}
			this._reviewContainer = reviewContainer;
			reviewUpdateMediator.reviewHasBeenOpened(reviewParser);
		}
	}
});


})();
