/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.ArtifactBanner");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.form.Button");
dojo.require("dijit.layout.StackContainer");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.review.ReviewCache");
dojo.require("com.ibm.rdm.web.review.ReviewProgressBar");
dojo.require("com.ibm.rdm.web.review.VoteCastingButtons");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.review.reviewUpdateMediator");
dojo.require("com.ibm.rdm.web.review.reviewutils");
dojo.require("com.ibm.rdm.web.util.domutils");

dojo.requireLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var ReviewCache = com.ibm.rdm.web.review.ReviewCache;
var ReviewProgressBar = com.ibm.rdm.web.review.ReviewProgressBar;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var VoteCastingButtons = com.ibm.rdm.web.review.VoteCastingButtons;
var domutils = com.ibm.rdm.web.util.domutils;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;
var reviewutils = com.ibm.rdm.web.review.reviewutils;


dojo.declare(
	"com.ibm.rdm.web.review.ArtifactBanner",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    Provides a banner used for vote casting when viewing an artifact that
	//    is under review (i.e. an artifact displayed by the ArtifactInReviewViewer).

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "review/templates/ArtifactBanner.html"),
	widgetsInTemplate: false,

	///////////////////////////////////////////////////////////////////////////
	// Attach points

	_myStatusTD: null,
	_myStatusValueSpan: null,
	_dividerBeforeVoteCastingButtonsTD: null,
	_voteCastingButtonsTD: null,
	_dividerBeforeOverallStatusTD: null,
	_voteCastingButtonsSpan: null,
	_overallStatusTD: null,
	_overallStatusValueTD: null,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	resourceViewer: null,
	artifactInReviewViewer: null,
	resourceURI: null,
	reviewParser: null,
	disabled: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_artifactInfo: null,
	_reviewAnchor: null,
	_voteCastingButtons: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		var self = this;
		// Create the vote casting button area.
		var onVoteCastHandler = function(artifactStatusType, doPromptForComments) {
			self._castVote(artifactStatusType, doPromptForComments);
		};
		var voteCastingButtons = new VoteCastingButtons(
			{
				viewType: VoteCastingButtons.VIEW_TYPE.ArtifactInReview,
				onVoteCastHandler: onVoteCastHandler,
				disabled: this.disabled
			},
 			this._voteCastingButtonsSpan
		);
		this._voteCastingButtons = voteCastingButtons;
	},

	postMixInProperties: function() {
		this._messages =
			dojo.i18n.getLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");
	},

	startup: function() {
		if (! this._started) {
			this._populateContents();
			this._overallStatusTD.style.display = "";
			var self = this;
			// Initialize the link that enables clicking through to the containing
			// review.
			var reviewLabel = this.reviewParser.getReviewName();
			var href = this.reviewParser.getSelfUri();
			this._reviewAnchor.setAttribute("href", href);
			this._reviewAnchor.appendChild(document.createTextNode(reviewLabel));
			this.connect(this._reviewAnchor, "onclick", function(event) {
				ResourceLinksHandler.handleAnchorClicked(event);
			});
		}
		this.inherited(arguments);
	},

	isStarted: function() {
		// Note: Unfortunately dojo's _Widget.js doesn't provide an 'isStarted'
		// function to access the '_started' field that it declares ... so we
		// do so here instead.
		return this._started;
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_setDisabledAttr: function(/*Boolean*/value){
		var originalValue = this.disabled;
		if (value != originalValue) {
			this.disabled = value;
			this._updateVoteCastingButtons();
 		}
	},

	_setReviewParserAttr: function(reviewParser) {
		// summary:
		//    Reset the current value for the reviewParser field and make
		//    appropriate UI updates.
		// description:
		//    When consumers of this class invoke this class's 'attr' function
		//    to set the "reviewParser" field, this function picks up the
		//    request in order to make appropriate UI element updates.
		//
		//    NOTE: This function customizes the default implementation that
		//    would normally put in place by:
		//          _Widget.attr
		//
		var oldRole = (this.reviewParser && this.reviewParser.getRole());
		var newRole = reviewParser.getRole();
		var isRoleDifferent = (oldRole != newRole);
		this.reviewParser = reviewParser;
		if (this._started) {
			this._populateContents();
		}
		if (newRole && isRoleDifferent) {
			// The role is different and this means we need to recreate the
			// vote casting buttons.  The reason we recreate these vote casting
			// buttons is due to the fact that the votes you can cast may
			// be role specific (e.g. an Approver's buttons are different
			// than the buttons used by a Reviewer or Optional Reviewer).
			this._voteCastingButtons.updateVotingButtons();
		}
	},

	_populateContents: function() {
		var artifactInfo = this.reviewParser.getArtifactInfo(this.resourceURI);
		this._artifactInfo = artifactInfo;
		this._updateMyStatusSection();
		this._updateVoteCastingButtonsSection();
		this._updateOverallStatusSection(this._overallStatusValueTD);
	},

	_updateMyStatusSection: function() {
		if (this._myStatusTD.firstChild) {
			domutils.removeChildNodes(this._myStatusTD);
		}
		var reviewState = this.reviewParser.getReviewState();
		if (this.reviewParser.getRole()
			&& (! (
					reviewState === reviewTypes.REVIEW_STATE_TYPES.Draft
					|| reviewState === reviewTypes.REVIEW_STATE_TYPES.Finalized
				))
			) {
			// The current user IS a participant, and the Review state is one
			// for which it makes sense to add this cell ... so we'll show the
			// user's vote icon and vote text.
			this._myStatusTD.style.display = "";
			this._dividerBeforeVoteCastingButtonsTD.style.display = "";
			var artifactInfo = this.reviewParser.getArtifactInfo(this.resourceURI);
			var myArtifactStatusType = artifactInfo.myArtifactStatusType;
			var img = reviewutils.createArtifactStatusTypeImage(
				myArtifactStatusType, "bannerImage");
			var statusLabel = reviewTypes.getArtifactStatusTypeDisplayName(
				myArtifactStatusType);
			var statusDisplayNameSpan =
				domutils.createSpannedTextNode("bannerLabel", statusLabel);
			this._myStatusTD.appendChild(img);
			this._myStatusTD.appendChild(statusDisplayNameSpan);
		} else {
			// The current user is NOT a participant, so we'll hide the span.
			this._myStatusTD.style.display = "none";
			this._dividerBeforeVoteCastingButtonsTD.style.display = "none";
		}
	},

	_updateVoteCastingButtonsSection: function() {
		// Note: We only show the vote casting buttons if the current user is a
		// participant AND the Review is not in the Finalized state.
		if (this.reviewParser.getReviewState() != reviewTypes.REVIEW_STATE_TYPES.Finalized
			&& this.reviewParser.getRole() // Note: this would return null if not a participant
			) {
			// We _do_ want to show the vote casting buttons.
			this._voteCastingButtonsTD.style.display = "";
			this._dividerBeforeOverallStatusTD.style.display = "";
			this._voteCastingButtons.attr(
				"reviewParser", this.reviewParser);
			this._updateVoteCastingButtons();
		} else {
			// Hide the vote casting buttons section.
			this._voteCastingButtonsTD.style.display = "none";
			this._dividerBeforeOverallStatusTD.style.display = "none";
		}
	},

	_updateVoteCastingButtons: function() {
		var buttonsShouldShowAsDisabled = (
			this.disabled
			|| (! reviewutils.canVote(this.reviewParser))
		);
		this._voteCastingButtons.attr("disabled", buttonsShouldShowAsDisabled);
		this._voteCastingButtons.updateVotingButtons(this._artifactInfo);
	},

	_updateOverallStatusSection: function(parentNode) {
		var clearCurrentStatusValueIfNeeded = function(parentNode) {
			if (parentNode.firstChild) {
				var overallStatusValueNode = parentNode.firstChild;
				if (overallStatusValueNode
					&& (overallStatusValueNode.nodeType == 1) // Test if element node
					) {
					var widget = dijit.byNode(overallStatusValueNode);
					if (widget) {
						widget.destroyRecursive();
					}
				}
				domutils.removeChildNodes(parentNode);
			}
		}
		clearCurrentStatusValueIfNeeded(parentNode);
		var artifactInfo = this.reviewParser.getArtifactInfo(this.resourceURI);
		var percentCompleted = artifactInfo.overallArtifactStatusInfo.percentageVotesCast;
		if (percentCompleted > 0 || percentCompleted < 100) {
			var reviewProgressBar = new ReviewProgressBar(
				{
			   		progress: percentCompleted,
					viewType: ReviewProgressBar.VIEW_TYPE.Reviewed
				}
			);
			parentNode.appendChild(reviewProgressBar.domNode);
		} else {
			var statusDisplayName =
				reviewTypes.getReviewStateDisplayName(reviewStateType);
			var img = reviewutils.createReviewStateImage(reviewStateType);
			var statusDisplayNameSpan =
				domutils.createSpannedTextNode("titleFieldValueSpan", statusDisplayName);
			parentNode.appendChild(img);
			parentNode.appendChild(statusDisplayNameSpan);
		}
	},

	_castVote: function(artifactStatusType, doPromptForComments) {
		if (! this.artifactInReviewViewer.isBusy()) {
			this._artifactInfo.myArtifactStatusType = artifactStatusType;
			var self = this;
			var onSaveComplete = function() {
				self.artifactInReviewViewer.showAsBusy(false);
				// Note: We're already hooked up to the reviewChangeNotifier
				// (indirectly through the _setReviewParserAttr function)
				// so there's no need to separately update the UI elements
				// here ...
			};
			var onSaveCancelled = function() {
				self.artifactInReviewViewer.showAsBusy(false);
				// Note: We're already hooked up to the reviewChangeNotifier
				// (indirectly through the _setReviewParserAttr function)
				// so there's no need to separately update the UI elements
				// here ...
			};
			var onSaveError = function() {
				self.artifactInReviewViewer.showAsBusy(false);
			};
			this.artifactInReviewViewer.showAsBusy(true);
			com.ibm.rdm.web.review.reviewUpdateMediator.castArtifactVotes(
				this.reviewParser, [this._artifactInfo], doPromptForComments,
				onSaveComplete, onSaveCancelled, onSaveError);
		}
	}
});


})();
