/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
dojo.provide("com.ibm.rdm.web.review.DashboardReviewParser");
dojo.require("dojo.date");
dojo.require("dojo.date.locale");

dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");

var domutils = com.ibm.rdm.web.util.domutils;
var queryutils = com.ibm.rdm.web.util.queryutils;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;


// Here's an example of a Review artifact that this DashboardReviewParser class might be
// asked to parse. Most of this code was taken from RevParser.
//
// [From JFS]
//<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
 //  xmlns:xsd="http://www.w3.org/2001/XMLSchema#">
//<rdf:Description rdf:about="https://localhost:9443/rdm/reviews/_XKehYGG4Ed6S3t05lc3Raw">
//<contributor rdf:resource="https://localhost:9443/jazz/users/TestJazzAdmin1" xmlns="http://purl.org/dc/terms/"/>
//<creator rdf:resource="https://localhost:9443/jazz/users/TestJazzAdmin1" xmlns="http://purl.org/dc/terms/"/>
//<resource-context rdf:resource="https://localhost:9443/jazz//jazz/process/project-areas/_9R9K0FTrEd6FJ7qAKt8xmA" xmlns="http://jazz.net/xmlns/foundation/1.0/"/>
//<type rdf:resource="http://www.ibm.com/xmlns/rrm/reviews/1.0/Review" xmlns="http://www.w3.org/1999/02/22-rdf-syntax-ns#"/>
//<isPartOf rdf:resource="https://localhost:9443/rdm/reviews" xmlns="http://purl.org/dc/terms/"/>
//<index-for xmlns="http://jazz.net/xmlns/foundation/1.0/">storage/com.ibm.rdm.reviews/_XKehYGG4Ed6S3t05lc3Raw</index-for>
//<format xmlns="http://purl.org/dc/terms/">x-com.ibm.rdm.review/rdf+xml</format>
//<resource-context-id xmlns="http://jazz.net/xmlns/foundation/1.0/">_9R9K0FTrEd6FJ7qAKt8xmA</resource-context-id>
//<etag xmlns="http://jazz.net/xmlns/foundation/1.0/">&amp;quot;_XLXSM2G4Ed6S3t05lc3Raw&amp;quot;</etag>
//
//<title xmlns="http://purl.org/dc/elements/1.1/">Reviewing</title>
//<description xmlns="http://purl.org/dc/elements/1.1/"></description>
//<reviewDueDate xmlns="http://www.ibm.com/xmlns/rrm/reviews/1.0/">Jul 2, 2009</reviewDueDate>
//<reviewStatus xmlns="http://www.ibm.com/xmlns/rrm/reviews/1.0/">Draft</reviewStatus>
//<modified xsd:datatype="http://www.w3.org/2001/XMLSchema#dateTime" xmlns="http://purl.org/dc/terms/">2009-06-25T18:45:32.499Z</modified>
//<created xsd:datatype="http://www.w3.org/2001/XMLSchema#dateTime" xmlns="http://purl.org/dc/terms/">2009-06-25T18:45:32.499Z</created>
//</rdf:Description>
//<rdf:Description rdf:about="https://localhost:9443/rdm/reviews/_I2zp0GHsEd6S3t05lc3Raw">
//<contributor rdf:resource="https://localhost:9443/jazz/users/TestJazzAdmin1" xmlns="http://purl.org/dc/terms/"/>
//<creator rdf:resource="https://localhost:9443/jazz/users/TestJazzAdmin1" xmlns="http://purl.org/dc/terms/"/>
//<resource-context rdf:resource="https://localhost:9443/jazz//jazz/process/project-areas/_9R9K0FTrEd6FJ7qAKt8xmA" xmlns="http://jazz.net/xmlns/foundation/1.0/"/>
//<type rdf:resource="http://www.ibm.com/xmlns/rrm/reviews/1.0/Review" xmlns="http://www.w3.org/1999/02/22-rdf-syntax-ns#"/>
//
//<isPartOf rdf:resource="https://localhost:9443/rdm/reviews" xmlns="http://purl.org/dc/terms/"/>
//<hasParticipant rdf:resource="http:nb450dfff-5b20-4dbf-9358-94910487d97c#ua6b87ef2-edec-4128-bf15-0eb812bd8142" xmlns="http://www.ibm.com/xmlns/rrm/reviews/1.0/"/>
//<index-for xmlns="http://jazz.net/xmlns/foundation/1.0/">storage/com.ibm.rdm.reviews/_I2zp0GHsEd6S3t05lc3Raw</index-for>
//<format xmlns="http://purl.org/dc/terms/">x-com.ibm.rdm.review/rdf+xml</format>
//<resource-context-id xmlns="http://jazz.net/xmlns/foundation/1.0/">_8lNyYNwSEd2pIJ5QVwgQGg</resource-context-id>
//<etag xmlns="http://jazz.net/xmlns/foundation/1.0/">&amp;quot;_SztX0WHsEd6S3t05lc3Raw&amp;quot;</etag>
//<title xmlns="http://purl.org/dc/elements/1.1/">Reviewing 2</title>
//<reviewDueDate xmlns="http://www.ibm.com/xmlns/rrm/reviews/1.0/">Jul 2, 2009</reviewDueDate>
//<reviewStatus xmlns="http://www.ibm.com/xmlns/rrm/reviews/1.0/">Draft</reviewStatus>
//
//<reviewInstructions xmlns="http://www.ibm.com/xmlns/rrm/reviews/1.0/">It's all about the details.</reviewInstructions>
//<reviewRole xmlns="http://www.ibm.com/xmlns/rrm/reviews/1.0/">Reviewer</reviewRole>
//<modified xsd:datatype="http://www.w3.org/2001/XMLSchema#dateTime" xmlns="http://purl.org/dc/terms/">2009-06-26T00:57:17.005Z</modified>
//<created xsd:datatype="http://www.w3.org/2001/XMLSchema#dateTime" xmlns="http://purl.org/dc/terms/">2009-06-26T00:56:10.284Z</created>
//</rdf:Description>
//</rdf:RDF>




dojo.declare(
	"com.ibm.rdm.web.review.DashboardReviewParser",
	null,
{
	// summary:
	//    Parses the content found in multi-fetch for Reviews.

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_resourceURI: null,
	_revision: null,
	_lastModifiedDate: null,
	_role: null,
	_reviewName: null,
	//_creatorUserId: null,
	_creatorUserURI: null,
	_lastModifiedByUserURI: null,
	_dueDate: null,
	_reviewState: null,
	_instructions: null,
	// _descriptionInfo: property list
	//    See the getDescriptionInfo function for details about this field.
	_descriptionInfo: null,
	// _artifactInfos: Array (of property lists)
	//    See the getArtifactInfos function for details about this field.
	_artifactInfos: null,
	// _participantInfos: Array (of property lists)
	//    See the getParticipantInfos function for details about this field.
	_participantInfos: null,
	// _participantResultInfos: Array (of property lists)
	//    See the getParticipantResultInfos function for details about this field.
	_participantResultInfos: null,
	// _artifactInfos: Array (of property lists)
	//    See the getArtifactResultInfos function for details about this field.
	_artifactResultInfos: null,
	// _percentReviewCompleted: integer
	//    See the getPercentReviewCompleted function for details about this field.
	_percentReviewCompleted: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	constructor: function(args) {

		var reviewNode = args.review;
		this._parseContent(reviewNode);
	},

	getSelfUri: function() {
		// summary:
		//    Get the resource URI for this review (e.g.
		//    "https://localhost:9443/rdm/resources/_h8ZFADsaEd6IT-BLBFfrNQ").
		return this._resourceURI;
	},

	getReviewId: function() {
		// summary:
		//    Get the shortname resource id for this review (e.g.
		//    "_h8ZFADsaEd6IT-BLBFfrNQ").
		return queryutils.getLastPathSegment(this._resourceURI);
	},

	getRevision: function() {
		// summary:
		//    Get the revision of this review (e.g. "_Zzb6sTs3Ed6IT-BLBFfrNQ").
		return this._revision;
	},

	getLastModifiedDate: function() {
		// summary:
		//    Get the date at which this review was last modified.
		// returns: Date object
		return this._lastModifiedDate;
	},

	getReviewName: function() {
		// summary:
		//    Get the review name for this review (e.g. "My Test Review").
		return this._reviewName;
	},

	getReviewState: function() {
		// summary:
		//    Get the review's state (a value found in reviewTypes.REVIEW_STATE_TYPES).
		return this._reviewState;
	},

	//getCreatorUserId: function() {
	//	return this._creatorUserId;
	//},
	
	getCreatorUserURI: function() {
		return this._creatorUserURI;
	},

	getLastModifiedByUserURI: function() {
		return this._lastModifiedByUserURI;
	},

	getRole: function() {
		// summary:
		//    Gets the participant role type for the current user.
		// returns:
		//    If the current user is not a participant, null will be returned.
		//    Otherwise (if current user is a participant) the returned value
		//    will be one of the values found in the enum represented by
		//    com.ibm.rdm.web.review.reviewTypes.PARTICIPANT_ROLE_TYPES.
		return this._role;
	},

	getDueDate: function() {
		// summary:
		//    Gets the due date for this review.
		// returns: Date object
		return this._dueDate;
	},

	getInstructions: function() {
		// summary:
		//    Get the review instructions.
		// returns: String
		return this._instructions;
	},

	setRole: function(role){
		this._role = role;
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_parseContent: function(descriptionNode) {
		this._parseCreator(descriptionNode);
		this._parseLastModifiedBy(descriptionNode);
		this._parseReviewName(descriptionNode);
		this._parseDueDate(descriptionNode);
		this._parseReviewState(descriptionNode);
		this._parseLastModifiedDate(descriptionNode);
	},

	_parseCreator: function(descriptionNode) {
		// summary:
		//    Fill this._creatorUserURI with content parsed from the passed doc.
		var creatorNode = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
			descriptionNode, "creator",
			queryutils.DCTERMS_NAMESPACE, queryutils.DCTERMS_QUALIFIER);
		var resourceNode = creatorNode[0].attributes.getNamedItem("rdf:resource");
		this._creatorUserURI = resourceNode.firstChild.nodeValue;
	},

	_parseLastModifiedBy: function(descriptionNode) {
		// summary:
		//    Fill this._lastModifiedByUserURI with content parsed from the passed doc.
		var contributorNode = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
			descriptionNode, "contributor",
			queryutils.DCTERMS_NAMESPACE, queryutils.DCTERMS_QUALIFIER);
		var resourceNode = contributorNode[0].attributes.getNamedItem("rdf:resource");
		this._lastModifiedByUserURI = resourceNode.firstChild.nodeValue;
	},
	
	_parseLastModifiedDate: function(review) {
		// summary:
		// last modified date        
        this._lastModifiedDate = this._getTextNodeValue(
			review, "modified", queryutils.DCTERMS_NAMESPACE, queryutils.DCTERMS_QUALIFIER);
	},

	_parseReviewName: function(review) {
		// summary:
		//    Fill this._reviewName with content parsed from the passed doc.
		this._reviewName = this._getTextNodeValue(
			review, "title",
			queryutils.DCTERMS_NAMESPACE, queryutils.DCTERMS_QUALIFIER
		);
	},

	_parseDueDate: function(review) {
		// summary:
		//    Fill this._dueDate with content parsed from the passed doc.
		var dueDateAsString = this._getTextNodeValue(
			review, "reviewDueDate",
			queryutils.REVIEW_NAMESPACE, queryutils.RRM_QUALIFIER
		);		
		if (dueDateAsString) {
			var date = dojo.date.stamp.fromISOString(dueDateAsString);
			var dateStr = dojo.date.locale.format(
				date, { formatLength:"medium", selector:"date" }
			);
			dueDateAsString = (dateStr);
		}
		this._dueDate = dueDateAsString;
	},

	_parseReviewState: function(doc) {
		// summary:
		//    Fill this._reviewState with content parsed from the passed doc.
		var getReviewStateType = function(reviewState) {
			switch (reviewState) {
				case "Draft":
					return reviewTypes.REVIEW_STATE_TYPES.Draft;
				case "Started":
					return reviewTypes.REVIEW_STATE_TYPES.Started;
				case "Stopped":
					return reviewTypes.REVIEW_STATE_TYPES.Stopped;
				case "Finalized":
					return reviewTypes.REVIEW_STATE_TYPES.Finalized;
				case "Paused":
					return reviewTypes.REVIEW_STATE_TYPES.Paused;
				case "Complete":
					return reviewTypes.REVIEW_STATE_TYPES.Reviewed;
			};
		};
  		var reviewStateAttribute = this._getTextNodeValue(
			doc, "reviewStatus",
			queryutils.REVIEW_NAMESPACE, queryutils.RRM_QUALIFIER
		);
		this._reviewState = (
			reviewStateAttribute ?
			getReviewStateType(reviewStateAttribute) :
			reviewTypes.REVIEW_STATE_TYPES.Draft
		);
	},

	_getLastModifiedDateLabel: function(lastModifiedIsoString) {
		var lastModifiedDateLabel = "";
		if (lastModifiedIsoString) {
			var date = dojo.date.stamp.fromISOString(lastModifiedIsoString);
			var dateStr = dojo.date.locale.format(
				date, { formatLength:"medium", selector:"date" }
			);
			var timeStr = dojo.date.locale.format(
				date, { formatLength:"short", selector:"time" }
			);
			lastModifiedDateLabel = (dateStr + " " + timeStr);
		}
		return lastModifiedDateLabel;
	},
	
	_getTextNodeValue: function(node, unqualifiedTagName, namespace, qualifier) {
		var nodes = domutils.getElementsByTagNameNS(node, unqualifiedTagName, namespace, qualifier);
		return (
			nodes.length > 0 && nodes[0].firstChild ?
			nodes[0].firstChild.nodeValue :
			""
		);
	}
});


})();
