/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.OverallReview");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.form.Button");
dojo.require("dojo.date.locale");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.review.ArtifactTableDescriptionSection");
dojo.require("com.ibm.rdm.web.review.ReviewArtifactsViewerSection");
dojo.require("com.ibm.rdm.web.review.ReviewParser");
dojo.require("com.ibm.rdm.web.review.ReviewParticipantsViewerSection");

dojo.requireLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


var ArtifactTableDescriptionSection = com.ibm.rdm.web.review.ArtifactTableDescriptionSection;
var ReviewArtifactsViewerSection = com.ibm.rdm.web.review.ReviewArtifactsViewerSection;
var ReviewParser = com.ibm.rdm.web.review.ReviewParser;
var ReviewParticipantsViewerSection = com.ibm.rdm.web.review.ReviewParticipantsViewerSection;

var BORDER_AFFORDANCE = 51;


dojo.declare(
	"com.ibm.rdm.web.review.OverallReview",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    Shows details on the overall status of a Review artifact.

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "review/templates/OverallReview.html"),
	widgetsInTemplate: true,

	///////////////////////////////////////////////////////////////////////////
	// Attach points

	_dueDateSpan: null,
	_reviewerInstructionsTextArea: null,
	_artifactTableDescriptionSectionDiv: null,
	_reviewArtifactsViewerSectionDiv: null,
	_participantsGroupSection: null,
	_reviewParticipantsViewerSectionDiv: null,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	resourceViewer: null,
	reviewParser: null,
	reviewContainer: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_artifactTableDescriptionSection: null,
	_reviewArtifactsViewerSection: null,
	_reviewParticipantsViewerSection: null,
	_lastWidth: null,
	_timerHandle: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		var self = this;
		onArtifactRowHighlighted = function(artifactIndex) {
			self._reviewParticipantsViewerSection.highlightArtifactIndex(artifactIndex);
		};
		onArtifactRowUnhighlighted = function(artifactIndex) {
			self._reviewParticipantsViewerSection.unhighlightArtifactIndex(artifactIndex);
		};
		onParticipantRowArtifactResultCellHighlighted = function(artifactIndex) {
			self._reviewParticipantsViewerSection.highlightArtifactIndex(artifactIndex);
			self._reviewArtifactsViewerSection.highlightArtifactIndex(artifactIndex);
		}
		onParticipantRowArtifactResultCellUnhighlighted = function(artifactIndex) {
			self._reviewParticipantsViewerSection.unhighlightArtifactIndex(artifactIndex);
			self._reviewArtifactsViewerSection.unhighlightArtifactIndex(artifactIndex);
		}
		this._reviewParticipantsViewerSection = new ReviewParticipantsViewerSection(
			{
				onParticipantRowArtifactResultCellHighlighted: onParticipantRowArtifactResultCellHighlighted,
				onParticipantRowArtifactResultCellUnhighlighted: onParticipantRowArtifactResultCellUnhighlighted
			},
			this._reviewParticipantsViewerSectionDiv);
		this._artifactTableDescriptionSection = new ArtifactTableDescriptionSection(
			{},
			this._artifactTableDescriptionSectionDiv);
		this._reviewArtifactsViewerSection = new ReviewArtifactsViewerSection(
			{
				onArtifactRowHighlighted: onArtifactRowHighlighted,
				onArtifactRowUnhighlighted: onArtifactRowUnhighlighted
			},
			this._reviewArtifactsViewerSectionDiv);
		this._populateContents();
	},

	postMixInProperties: function() {
		this._messages =
			dojo.i18n.getLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");
	},

	setWidth: function(width) {
		this._lastWidth = width;
		this._resizeTextareaControl(width);
	},

	paneHasBeenSelected: function() {
		var self = this;
		if (this._timerHandle) {
			window.clearTimeout(this._timerHandle);
		}
    	this._timerHandle = window.setTimeout(
        	function () {
				// Note by SPC: This is kind of a hack, but without doing this
				// twice the instructions textarea won't properly account for
				// the addition or removal of the pane's vertical scrollbar
				// (i.e. when it's added or removed, the textarea's width would
				// be improperly set to too little or too much by an amount
				// equals to the scrollbar's width [around 18 pixels]).
				self.setWidth(self._lastWidth);
				window.setTimeout(
					function () {
						self.setWidth(self._lastWidth);
					},
					250 //interval between dispatches
				);
    		},
    		250 //interval between dispatches
    	);
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_setReviewParserAttr: function(reviewParser) {
		// summary:
		//    Reset the current value for the reviewParser field and make
		//    appropriate UI updates.
		// description:
		//    When consumers of this class invoke this class's 'attr' function
		//    to set the "reviewParser" field, this function picks up the
		//    request in order to make appropriate UI element updates.
		//
		//    NOTE: This function customizes the default implementation that
		//    would normally put in place by:
		//          _Widget.attr
		//
		this.reviewParser = reviewParser;
		if (this._reviewArtifactsViewerSection) {
			this._reviewArtifactsViewerSection.attr("reviewParser", this.reviewParser);
		}
		if (this._reviewParticipantsViewerSection) {
			this._reviewParticipantsViewerSection.attr("reviewParser", this.reviewParser);
		}
		this._updateDueDateAndInstructions();
	},

	_resizeTextareaControl: function(accordionPaneWidth) {
		var labelSpanWidth = dojo.coords(this._reviewerInstructionsLabelSpan).w;
		if (labelSpanWidth > 0) {
			var groupSectionWidth = dojo.coords(this._participantsGroupSection).w;
			var borderAffordance = accordionPaneWidth - groupSectionWidth;
			var textareaWidthToUse = groupSectionWidth - labelSpanWidth - BORDER_AFFORDANCE;
			this._reviewerInstructionsTextArea.style.width = textareaWidthToUse + "px";
		}
	},

	_populateContents: function() {
		this._updateDueDateAndInstructions();
		this._artifactTableDescriptionSection.attr("reviewParser", this.reviewParser);
		this._artifactTableDescriptionSection.startup();
		this._reviewArtifactsViewerSection.attr("reviewParser", this.reviewParser);
		this._reviewArtifactsViewerSection.startup();
		this._reviewParticipantsViewerSection.attr("reviewParser", this.reviewParser);
		this._reviewParticipantsViewerSection.startup();
	},

	_updateDueDateAndInstructions: function() {
		var dueDate = this.reviewParser.getDueDate();
		var dueDateAsString = dojo.date.locale.format(
			dueDate,
			{ selector: "date", formatLength: "full" }
		);
		this._dueDateSpan.innerHTML = dueDateAsString;
		var instructions = this.reviewParser.getInstructions();
		if (dojo.isIE) {
			// Note: On IE attempting to use innerHTML instead will result
			// in removal of any newlines that are found in the text.
			this._reviewerInstructionsTextArea.innerText = instructions;
		} else {
			this._reviewerInstructionsTextArea.innerHTML = instructions;
		}
	}
});


})();
