/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
dojo.provide("com.ibm.rdm.web.review.ParticipantResultSaver");
dojo.require("dojo.date");
dojo.require("dojo.date.locale");

dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.server.Headers");
dojo.require("com.ibm.rdm.web.server.XhrUtils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");

dojo.requireLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


var Headers = com.ibm.rdm.web.server.Headers;
var XhrUtils = com.ibm.rdm.web.server.XhrUtils;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var MimeTypes = com.ibm.rdm.web.common.MimeTypes;
var domutils = com.ibm.rdm.web.util.domutils;
var messages = dojo.i18n.getLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");
var queryutils = com.ibm.rdm.web.util.queryutils;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;

var MAX_DELTA_TIME_FOR_SAVING_IN_MILLISECONDS = 120000; // 2 mins
var INTERVAL_BETWEEN_SAVE_ATTEMPTS_IN_MILLISECONDS = 1000;


// This class is responsible for saving away the ParticipantResult artifact for
// the current user.  Here's an example of the artifact that gets saved.
//
// <?xml version="1.0" encoding="UTF-8"?>
// <rdf:RDF
//    xmlns:rrmReview="http://www.ibm.com/xmlns/rrm/reviews/1.0/"
//    xmlns:dc="http://purl.org/dc/elements/1.1/"
//    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
//    <rrmReview:ParticipantResult rdf:about="https://localhost:9443/rdm/reviews/results/_VQ5n8WgLEd6NlcEl9xCf0g">
//      <rrmReview:associatedReview rdf:resource="https://localhost:9443/rdm/reviews/_LvKL8GgLEd6NlcEl9xCf0g" />
//      <rrmReview:hasParticipant rdf:resource="https://localhost:9443/jazz/users/Anne" />
//      <rrmReview:reviewRole>Approver</rrmReview:reviewRole>
//      <rrmReview:isDone>false</rrmReview:isDone>
//      <rrmReview:hasArtifactResult>
//        <rrmReview:ArtifactResult>
//          <rrmReview:artifactId>https://localhost:9443/rdm/resources/_vFKIYFxTEd6WMqY891WRNA?revision=_vG8RElxTEd6WMqY891WRNA</rrmReview:artifactId>
//          <rrmReview:participantResultStatus>Abstained</rrmReview:participantResultStatus>
//        </rrmReview:ArtifactResult>
//      </rrmReview:hasArtifactResult>
//      <rrmReview:hasArtifactResult>
//        <rrmReview:ArtifactResult>
//          <rrmReview:artifactId>https://localhost:9443/rdm/resources/_vT5j0WGJEd6napvdB5J99g?revision=_CJf9gWTPEd6FApb3gAdFRg</rrmReview:artifactId>
//          <rrmReview:participantResultStatus>Approved</rrmReview:participantResultStatus>
//        </rrmReview:ArtifactResult>
//      </rrmReview:hasArtifactResult>
//    </rrmReview:ParticipantResult>
// </rdf:RDF>

// Representation used on 9/24/09 for a started formal review:
// <rdf:RDF
//    xmlns:rrmReview="http://www.ibm.com/xmlns/rrm/reviews/1.0/"
//    xmlns:dc="http://purl.org/dc/elements/1.1/"
//    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
//    <rrmReview:ParticipantResult rdf:about="https://localhost:9443/rdm/reviews/results/_x2NIoKk_Ed6MXPez46OKWA">
//      <rrmReview:associatedReview rdf:resource="_f8ow8Kk9Ed6MXPez46OKWA" />
//      <rrmReview:hasParticipant rdf:resource="../../jazz/users/TestJazzAdmin1" />
//      <rrmReview:reviewRole>Required Reviewer</rrmReview:reviewRole>
//      <rrmReview:isDone>false</rrmReview:isDone>
//      <rrmReview:hasArtifactResult>
//        <rrmReview:ArtifactResult>
//          <rrmReview:artifactId>../resources/rrc89?revision=_U4XyQak9Ed6MXPez46OKWA</rrmReview:artifactId>
//          <rrmReview:participantResultStatus>Reviewed</rrmReview:participantResultStatus>
//        </rrmReview:ArtifactResult>
//      </rrmReview:hasArtifactResult>
//    </rrmReview:ParticipantResult>
// </rdf:RDF>


dojo.declare(
	"com.ibm.rdm.web.review.ParticipantResultSaver",
	null,
{
	// summary:
	//    Saves the participant result associated with a particular review for
	//    the current user.

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_reviewParser: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	constructor: function(keywordArgs) {
		// summary:
		//    Parses the content found within the RDF that represents a
		//    ParticipantResult artifact.
		// keywordArgs.doc:
		//    The response returned by a doGet for the participant results
		//    associated with a review.
		this._reviewParser = keywordArgs.reviewParser;
	},

	castArtifactVotes: function(myChangedArtifactInfos, onSaveComplete, /*Callback?*/onSaveError) {
		// summary:
		//    Save away new artifact vote casts for the the current user.
		// myChangedArtifactInfos: Array of artifactInfos
		//    Please note that the artifactInfos found in this array represent
		//    ONLY those artifactInfo's whose vote setting has changed (i.e.
		//    it is NOT intended to represent the complete list of new values
		//    to record [changed vote values as well as original vote values]).
		var self = this;
		var participantResultContent = this._composeParticipantResult(null, myChangedArtifactInfos);
		this._usingPreviouslyStoredRevisionSaveParticipantResult(
			 participantResultContent,
			 onSaveComplete, onSaveError);
	},

	markMyReviewStatusAsDone: function(onSaveComplete, /*Callback?*/onSaveError) {
		// summary:
		//    Mark the current user's reviewing state as "Done" for the
		//    entire review.
		// description:
		//    Create artifactResults elements using Abstained state for all
		//    artifacts that the user has not yet started.  Also for all
		//    artifactResults associated with the user that are InProgress,
		//    move these to Abstained.  Finally add an isDone="true" attribute
		//    to the current user's participant results element and save this
		//    new version of the Review.
		//    Calls onSaveComplete when save has completed.
		var self = this;
		var artifactStatusType = reviewTypes.ARTIFACT_STATUS_TYPES.Reviewed; // FIXME Use this for Done? Confirm w/ Eric ...
		var participantResultContent = this._composeParticipantResult(artifactStatusType, null);
		this._usingPreviouslyStoredRevisionSaveParticipantResult(
			 participantResultContent,
			 onSaveComplete, onSaveError);
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_usingPreviouslyStoredRevisionSaveParticipantResult: function(
			participantResultContent, onSaveComplete, /*Callback?*/onSaveError) {
		var participantResultResourceURI = this._reviewParser.getParticipantResultUri();
		var self = this;
		var performSave = function(headerValueForIfMatch) {
			var deferred = self._saveParticipantResult(participantResultContent, headerValueForIfMatch);
			deferred.addCallback(function(response) {
				onSaveComplete();
				return response;
			});
			deferred.addErrback(function(response) {
				if (onSaveError) {
					onSaveError(response);
				}
				return response;
			});
		};
		if (participantResultResourceURI) {
			// In the context of this review session, the user has already
			// made at least one update to his/her participant results.
			// Make use of the revision returned by this last successfull
			// save when attempting this new save (to enable save conflict
			// detection for PUTs that may have occurred since we originally
			// loaded this review).
			var revision = this._reviewParser.getParticipantResultRevision();
			if (revision) {
				performSave('"' + revision + '"');
			} else {
				// This should never happen ...
				console.error("Unable to find user's participant result revision in the reviewParser");
			}
		}
		else {
			// It must be the case that this user has never previously voted
			// within the context of this review.  So we'll end up doing a POST.
			performSave("*");
		}
	},

	_saveParticipantResult: function(participantResultContent, headerValueForIfMatch) {
		// summary:
		//    Attempt to save a new version of the review with the new review content
		//    found in the passed participantResultContent parameter.
		// returns:
		//    A deferred.
		var myUserURI = com.ibm.rdm.web.AUTHENTICATED_USER.uri;
		var participantResultResourceURI = this._reviewParser.getParticipantResultUri();
		var headers = [];
		headers[Headers.OWNING_CONTEXT] = this._reviewParser.getProjectUri();
		headers[Headers.CONTENT_TYPE] = MimeTypes.CONTENT_TYPE_REVIEW_RESULT;
		headers[Headers.IF_MATCH] = headerValueForIfMatch;
		var deferred = null;
		// Do either a POST (in the case of never having voted before) or a PUT
		// (if the user has previously cast votes in the context of this
		// review).
		com.ibm.rdm.web.review.reviewUpdateMediator.reviewSaveInProgress(
			this._reviewParser);
		if (participantResultResourceURI) {
			deferred = RDMClient.doPut({
				uri: participantResultResourceURI,
				putData: participantResultContent,
				xhrArgs: {
					headers: headers,
					skipErrors: true
				}
			});
		} else {
			deferred = RDMClient.doPost({
				uri: com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL + "/results",
				postData: participantResultContent,
				xhrArgs: {
					headers: headers,
					skipErrors: true
				}
			});
		}
		var self = this;
		deferred.addCallback(this, function(response){
			var revision = XhrUtils.getResponseEtag(response);
			if (! participantResultResourceURI) {
				// Note: When performing a POST the returned "Location" response
				// header contains the new resource URI.
				participantResultResourceURI = XhrUtils.getHeader(response, "Location");
			}
			com.ibm.rdm.web.review.reviewUpdateMediator.reviewSaveHasCompleted(
				self._reviewParser, participantResultResourceURI, revision);
			return response.doc;
		});
		deferred.addErrback(this, function(response){
			var putOrPostStatus = response.ioArgs.xhr.status;
			com.ibm.rdm.web.review.reviewUpdateMediator.reviewSaveHasFailed(
				self._reviewParser, participantResultResourceURI, putOrPostStatus);
			ErrorHandler.warn(
				"Received abnormal response as a result of attempting to save ParticipantResult: "
				+ putOrPostStatus
			);
			// Check if this abnormal response was returned due to the fact
			// that the underlying Review has been deleted.
			var headers = [];
			headers[Headers.OWNING_CONTEXT] = this._reviewParser.getProjectUri();
			var headDeferred = RDMClient.doHead({
				uri: self._reviewParser.getSelfUri(),
				xhrArgs: { headers: headers },
				skipCache: true,
				skipErrors: true
			});
			headDeferred.addCallback(this, function(response){
				self._showCouldNotSaveVotesError(putOrPostStatus);
				return response;
			});
			headDeferred.addErrback(this, function(response){
				var status = response.ioArgs.xhr.status;
				if (status === 410) { // Note: 410 is "Gone"
					ErrorHandler.warn(
						"The underlying review '"
						+ self._reviewParser.getReviewName()
						+ "' has been deleted"
					);
 					self._showReviewHasBeenDeletedError();
				} else {
					self._showCouldNotSaveVotesError(putOrPostStatus);
				}
				return response;
			});
			return response;
		});
		return deferred;
	},

	_showCouldNotSaveVotesError: function(statusCode) {
		var msg = messages["unableToSaveParticipantResult"];
		var details = dojo.string.substitute(
			messages["unableToSaveParticipantResultDetails"],
			[
				"ParticipantResultSaver._saveParticipantResult",
				this._reviewParser.getReviewName(),
				statusCode
			]
		);
		ErrorHandler.getErrorSectionWidget().showErrorMessage(
		   msg, details);
	},

	_showReviewHasBeenDeletedError: function() {
		var msg = messages["saveParticipantResultsFailedDueToDeletedReview"];
		var details = dojo.string.substitute(
			messages["saveParticipantResultsFailedDueToDeletedReviewDetails"],
			[
				"ParticipantResultSaver._saveParticipantResult",
				this._reviewParser.getReviewName()
			]
		);
		ErrorHandler.getErrorSectionWidget().showErrorMessage(
		   msg, details);
	},

	_composeParticipantResult: function(artifactStatusType, myChangedArtifactInfos) {
		// summary:
		//    Create the XML that reflects this.reviewParser content overlayed
		//    with either artifactStatusType or myChangedArtifactInfos (whichever is
		//    passed).
		// description:
		//    Note: It's expected that only one of the two params (artifactStatusType or
		//    myChangedArtifactInfos) will hold a non-null value.
		var myUserURI = com.ibm.rdm.web.AUTHENTICATED_USER.uri;
		var userPathToUse = this._reviewParser.getUserPathToUseWhenSaving(myUserURI);
		var myRole = this._reviewParser.getRole();
		var isDone = (artifactStatusType != null);
		var participantResultResourceURI = this._reviewParser.getParticipantResultUri();
		var about = "";
		var reviewId = this._reviewParser.getReviewId();
		var hasArtifactResultElementContent =
			this._composeHasArtifactResultElementContent(artifactStatusType, myChangedArtifactInfos);
		return (
			"<" + queryutils.RDF_QUALIFIER + ":RDF\r\n"
			+ "    xmlns:" + queryutils.REVIEW_QUALIFIER + "=\"" + queryutils.REVIEW_NAMESPACE + "\"\r\n"
			+ "    xmlns:" + queryutils.DC_QUALIFIER + "=\"" + queryutils.DC_NAMESPACE + "\"\r\n"
			+ "    xmlns:" + queryutils.RDF_QUALIFIER + "=\"" + queryutils.RDF_NAMESPACE + "\">\r\n"
			+ "  <" + queryutils.REVIEW_QUALIFIER + ":ParticipantResult " + queryutils.RDF_QUALIFIER + ":about=\"" + about + "\">\r\n"
			+ "    <" + queryutils.REVIEW_QUALIFIER + ":associatedReview rdf:resource=\"" + reviewId + "\"/>\r\n"
			+ "    <" + queryutils.REVIEW_QUALIFIER + ":hasParticipant " + queryutils.RDF_QUALIFIER + ":resource=\"" + userPathToUse + "\"/>\r\n"
			+ "    <" + queryutils.REVIEW_QUALIFIER + ":reviewRole>" + myRole + "</" + queryutils.REVIEW_QUALIFIER + ":reviewRole>\r\n"
			+ "    <" + queryutils.REVIEW_QUALIFIER + ":isDone>" + isDone + "</" + queryutils.REVIEW_QUALIFIER + ":isDone>\r\n"
			+ hasArtifactResultElementContent
			+ "  </" + queryutils.REVIEW_QUALIFIER + ":ParticipantResult>\r\n"
			+ "</" + queryutils.RDF_QUALIFIER + ":RDF>\r\n"
		);
	},

	_composeHasArtifactResultElementContent: function(artifactStatusType, myChangedArtifactInfos) {
		// summary:
		//    Compose the "participantResults" element for the current user.
		// description:
		//    Note: It's expected that only one of the two params (artifactStatusType or
		//    myChangedArtifactInfos) will hold a non-null value.
		// artifactStatusType: enum value from reviewTypes.ARTIFACT_STATUS_TYPES
		//    The "vote" that the current user wishes to cast for the Review.
		// myChangedArtifactInfos:
		//    An array of artifactInfo property lists containing the intended artifact
		//    votes that the user wishes to cast for the Review.
		var artifactResultInfos = this._reviewParser.getArtifactResultInfos();
		var buf = "";
		// Capture the new values that should be recorded for the current user.
 		var myArtifactResultInfos = (
			artifactStatusType ?
			this._composeArtifactResultInfosByAbstainingRemainingArtifacts() :
			this._composeArtifactResultInfosUsingChangedArtifactInfos(
				myChangedArtifactInfos)
		);
		dojo.forEach(
			myArtifactResultInfos,
			function(artifactResultInfo) {
				buf += this._composeArtifactResultElementContent(
					artifactResultInfo);
			},
			this
		);
		return buf;
	},

	_composeArtifactResultElementContent: function(artifactResultInfo) {
		var artifactId = artifactResultInfo.artifact;
		var artifactStatusType = artifactResultInfo.artifactStatusType;
		return (
			"    <" + queryutils.REVIEW_QUALIFIER + ":hasArtifactResult>\r\n"
			+ "      <" + queryutils.REVIEW_QUALIFIER + ":ArtifactResult>\r\n"
			+ "        <" + queryutils.REVIEW_QUALIFIER + ":artifactId>" + artifactId
			+ "</" + queryutils.REVIEW_QUALIFIER + ":artifactId>\r\n"
			+ "        <" + queryutils.REVIEW_QUALIFIER + ":participantResultStatus>" + artifactStatusType
			+ "</" + queryutils.REVIEW_QUALIFIER + ":participantResultStatus>\r\n"
			+ "      </" + queryutils.REVIEW_QUALIFIER + ":ArtifactResult>\r\n"
			+ "    </" + queryutils.REVIEW_QUALIFIER + ":hasArtifactResult>\r\n"
		);
	},

	_composeArtifactResultInfosUsingChangedArtifactInfos: function(myChangedArtifactInfos) {
		// summary:
		//    Get the artifactInfos that should be recorded for the current
		//    user after he/she has made the vote casting changes found in
		//    myChangedArtifactInfos.
		var myUserURI = com.ibm.rdm.web.AUTHENTICATED_USER.uri;
		var myArtifactResultInfos = [];
		var myArtifactInfos = this._reviewParser.getArtifactInfos();
		var mapArtifactIdToMyArtifactInfo = [];
		if (myChangedArtifactInfos) {
			dojo.forEach(myChangedArtifactInfos, function(myChangedArtifactInfo) {
				mapArtifactIdToMyArtifactInfo[myChangedArtifactInfo.id] =
					myChangedArtifactInfo;
			});
		}
		dojo.forEach(
			myArtifactInfos,
			function(myArtifactInfo) {
				// Set artifactInfoToUse to the passed artifactInfo found in
				// the passed myChangedArtifactInfo, or if not found in this
				// array, set it instead to the last recorded value (i.e. the
				// value found in the reviewParser).
				var artifactInfoToUse = (
					mapArtifactIdToMyArtifactInfo[myArtifactInfo.id]
					|| myArtifactInfo
				);
				if (artifactInfoToUse.myArtifactStatusType != reviewTypes.ARTIFACT_STATUS_TYPES.NotStarted) {
					myArtifactResultInfos.push({
						userId: myUserURI,
						artifact: artifactInfoToUse.id,
						artifactStatusType: artifactInfoToUse.myArtifactStatusType
					});
				}
			}
		);
		return myArtifactResultInfos;
	},

	_composeArtifactResultInfosByAbstainingRemainingArtifacts: function() {
		var myUserURI = com.ibm.rdm.web.AUTHENTICATED_USER.uri;
		var myNewArtifactResultInfos = [];
		var myArtifactInfos = this._reviewParser.getArtifactInfos();
		dojo.forEach(
			myArtifactInfos,
			function(myArtifactInfo) {
				var artifactStatusToUse = (
					(myArtifactInfo.myArtifactStatusType === reviewTypes.ARTIFACT_STATUS_TYPES.NotStarted
						|| myArtifactInfo.myArtifactStatusType === reviewTypes.ARTIFACT_STATUS_TYPES.InProgress) ?
					reviewTypes.ARTIFACT_STATUS_TYPES.Abstained :
					myArtifactInfo.myArtifactStatusType
				);
				myNewArtifactResultInfos.push({
					userId: myUserURI,
					artifact: myArtifactInfo.id,
					artifactStatusType: artifactStatusToUse
				});
			}
		);
		return myNewArtifactResultInfos;
	}
});


})();
