/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
dojo.provide("com.ibm.rdm.web.review.ParticipantResultsParser");
dojo.require("dojo.date");
dojo.require("dojo.date.locale");

dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");


var domutils = com.ibm.rdm.web.util.domutils;
var queryutils = com.ibm.rdm.web.util.queryutils;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;


// This class parses the feed response returned when a query such as the following is issued:
//     https://localhost:9443/rdm/reviews/__HvWcWDNEd6napvdB5J99g/results
// [Note: This query requiress that a header such as the following:
//     net.jazz.jfs.owning-context: https://localhost:9443/jazz/process/project-areas/_9R9K0FTrEd6FJ7qAKt8xmA
//  be passed as well.]
//
// Here's an example of the feed results that this class might be asked to parse:
// <?xml version="1.0" encoding="UTF-8"?>
// <rdf:RDF
//    xmlns:rrmReview="http://www.ibm.com/xmlns/rrm/reviews/1.0/"
//    xmlns:dc="http://purl.org/dc/elements/1.1/"
//    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
//   <rrmReview:ReviewResults rdf:about="https://localhost:9443/rdm/reviews/_LvKL8GgLEd6NlcEl9xCf0g/results?dojo.preventCache=1246650961681">
//     <rrmReview:hasParticipantResult>
//       <rrmReview:ParticipantResult rdf:about="https://localhost:9443/rdm/reviews/results/_VQ5n8WgLEd6NlcEl9xCf0g">
//         <rrmReview:associatedReview rdf:resource="https://localhost:9443/rdm/reviews/_LvKL8GgLEd6NlcEl9xCf0g" />
//         <rrmReview:hasParticipant rdf:resource="https://localhost:9443/jazz/users/Anne" />
//         <rrmReview:reviewRole>Approver</rrmReview:reviewRole>
//         <rrmReview:isDone>false</rrmReview:isDone>
//         <rrmReview:hasArtifactResult>
//           <rrmReview:ArtifactResult>
//             <rrmReview:artifactId>https://localhost:9443/rdm/resources/_vFKIYFxTEd6WMqY891WRNA?revision=_vG8RElxTEd6WMqY891WRNA</rrmReview:artifactId>
//             <rrmReview:participantResultStatus>Dissaproved</rrmReview:participantResultStatus>
//           </rrmReview:ArtifactResult>
//         </rrmReview:hasArtifactResult>
//         <rrmReview:hasArtifactResult>
//           <rrmReview:ArtifactResult>
//             <rrmReview:artifactId>https://localhost:9443/rdm/resources/_vT5j0WGJEd6napvdB5J99g?revision=_CJf9gWTPEd6FApb3gAdFRg</rrmReview:artifactId>
//             <rrmReview:participantResultStatus>Approved</rrmReview:participantResultStatus>
//           </rrmReview:ArtifactResult>
//         </rrmReview:hasArtifactResult>
//       </rrmReview:ParticipantResult>
//     </rrmReview:hasParticipantResult>
//   </rrmReview:ReviewResults>
// </rdf:RDF>

// Representation used on 07/15 [A "Started" Review from JFS ... the review URI is "https://localhost:9443/rdm/reviews/_r9CvUXE4Ed6UbrmHoG4dwQ"]
// in which no vote casting has yet been performed:
//
// <?xml version="1.0" encoding="UTF-8"?>
// <rdf:RDF
//    xmlns:rrmReview="http://www.ibm.com/xmlns/rrm/reviews/1.0/"
//    xmlns:dc="http://purl.org/dc/elements/1.1/"
//    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
//   <rrmReview:ReviewResults rdf:about="https://localhost:9443/rdm/reviews/_r9CvUXE4Ed6UbrmHoG4dwQ/results?dojo.preventCache=1247660546343">
//   </rrmReview:ReviewResults>
// </rdf:RDF>
//
// after a vote has been cast it looks like this:
//
// <?xml version="1.0" encoding="UTF-8"?>
// <rdf:RDF
//    xmlns:rrmReview="http://www.ibm.com/xmlns/rrm/reviews/1.0/"
//    xmlns:dc="http://purl.org/dc/elements/1.1/"
//    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
//   <rrmReview:ReviewResults rdf:about="https://localhost:9443/rdm/reviews/_r9CvUXE4Ed6UbrmHoG4dwQ/results?dojo.preventCache=1247666080026">
//     <rrmReview:hasParticipantResult>
//       <rrmReview:ParticipantResult rdf:about="https://localhost:9443/rdm/reviews/results/_zNHDoHE4Ed6UbrmHoG4dwQ">
//         <rrmReview:associatedReview rdf:resource="https://localhost:9443/rdm/reviews/_r9CvUXE4Ed6UbrmHoG4dwQ" />
//         <rrmReview:hasParticipant rdf:resource="https://localhost:9443/jazz/users/TestJazzAdmin1" />
//         <rrmReview:reviewRole>Approver</rrmReview:reviewRole>
//         <rrmReview:isDone>false</rrmReview:isDone>
//         <rrmReview:hasArtifactResult>
//           <rrmReview:ArtifactResult>
//             <rrmReview:artifactId>https://localhost:9443/rdm/resources/rrc1?revision=_BnHo4m_2Ed6X1fZezkbfPg</rrmReview:artifactId>
//             <rrmReview:participantResultStatus>Dissaproved</rrmReview:participantResultStatus>
//           </rrmReview:ArtifactResult>
//         </rrmReview:hasArtifactResult>
//       </rrmReview:ParticipantResult>
//     </rrmReview:hasParticipantResult>
//   </rrmReview:ReviewResults>
// </rdf:RDF>

// Representation used on 9/24/09 for a started formal review:
// <rdf:RDF
//    xmlns:rrmReview="http://www.ibm.com/xmlns/rrm/reviews/1.0/"
//    xmlns:dc="http://purl.org/dc/elements/1.1/"
//    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
//   <rrmReview:ReviewResults rdf:about="https://localhost:9443/rdm/reviews/_f8ow8Kk9Ed6MXPez46OKWA/results">
//     <rrmReview:hasParticipantResult>
//       <rrmReview:ParticipantResult rdf:about="https://localhost:9443/rdm/reviews/results/_x2NIoKk_Ed6MXPez46OKWA">
//         <rrmReview:associatedReview rdf:resource="_f8ow8Kk9Ed6MXPez46OKWA" />
//         <rrmReview:hasParticipant rdf:resource="../../jazz/users/TestJazzAdmin1" />
//         <rrmReview:reviewRole>Required Reviewer</rrmReview:reviewRole>
//         <rrmReview:isDone>false</rrmReview:isDone>
//         <rrmReview:hasArtifactResult>
//           <rrmReview:ArtifactResult>
//             <rrmReview:artifactId>../resources/rrc89?revision=_U4XyQak9Ed6MXPez46OKWA</rrmReview:artifactId>
//             <rrmReview:participantResultStatus>Reviewed</rrmReview:participantResultStatus>
//           </rrmReview:ArtifactResult>
//         </rrmReview:hasArtifactResult>
//       </rrmReview:ParticipantResult>
//     </rrmReview:hasParticipantResult>
//   </rrmReview:ReviewResults>
// </rdf:RDF>

// Representation used on 10/01/09 for a started formal review (note that absolute paths ARE used in this case):
// <?xml version="1.0" encoding="UTF-8"?>
// <rdf:RDF
//    xmlns:rrmReview="http://www.ibm.com/xmlns/rrm/reviews/1.0/"
//    xmlns:dc="http://purl.org/dc/elements/1.1/"
//    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
//   <rrmReview:ReviewResults rdf:about="https://vm2003oracle10.x.x.x.x:9443/rdm/reviews/__CCkoKxcEd6BHIsY82esNg/results">
//     <rrmReview:hasParticipantResult>
//       <rrmReview:ParticipantResult rdf:about="https://vm2003oracle10.x.x.x.x:9443/rdm/reviews/results/_lqvOEKxwEd6BHIsY82esNg">
//         <rrmReview:associatedReview rdf:resource="__CCkoKxcEd6BHIsY82esNg" />
//         <rrmReview:hasParticipant rdf:resource="https://VM2003Oracle10.x.x.x.x:9443/jazz/users/Oscar" />
//         <rrmReview:reviewRole>Required Reviewer</rrmReview:reviewRole>
//         <rrmReview:isDone>false</rrmReview:isDone>
//       </rrmReview:ParticipantResult>
//     </rrmReview:hasParticipantResult>
//   </rrmReview:ReviewResults>
// </rdf:RDF>


dojo.declare(
	"com.ibm.rdm.web.review.ParticipantResultsParser",
	null,
{
	// summary:
	//    Parses the feed that represents all of the ParticipantResults for a
	//    particular Review.

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	// _participantResults: Array
	//    See the getParticipantResults function for details about this field.
	_participantResults: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	constructor: function(keywordArgs) {
		// summary:
		//    Parses the content found within the RDF that represents a
		//    ParticipantResult artifact.
		// keywordArgs.doc:
		//    The response returned by a doGet for the participant results
		//    associated with a review.
		this._parseParticipantResults(keywordArgs.doc);
	},

	getParticipantResults: function() {
		// summary:
		//    Gets the ArtifactResult information found within a
		//    ParticipantResults feed.
		// returns: Array (that also serves as a hash map)
		//    Returns an array in which each element has the following format:
		//      [
		//          resourceURI: "https://localhost:9443/rdm/reviews/results/_VQ5n8WgLEd6NlcEl9xCf0g",
		//          userURI: "https://localhost:9443/jazz/users/Anne",
		//          isDone: false,
		//          artifactResults:
		//              [
		//                  {
		//                      artifactId: https://localhost:9443/rdm/resources/_vFKIYFxTEd6WMqY891WRNA?revision=_vG8RElxTEd6WMqY891WRNA
		//                      participantResultStatus: Approved
		//                  }
		//                  ...
		//              ]
		//      ]
		//    In addition, this array also serves as a hash map, in which the
		//    key is an absolute path-based userURI and the value is the
		//    user's participant results.
		return this._participantResults;
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_parseParticipantResults: function(doc) {
		// summary:
		//    Fill this._participantResults with content parsed from the passed
		//    doc.
		var nodes = domutils.getElementsByTagNameNS(
			doc, "ParticipantResult",
			queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
		);
		var mapUserURIToParticipantResult = new Object;
		dojo.forEach(
			nodes,
			function(participantResult) {
				var resourceURI = this._parseResourceUri(participantResult);
				var participant = this._parseParticipant(participantResult);
				var isDone = this._parseIsDone(participantResult);
				var artifactResults = this._parseArtifactResults(participantResult);
				var participantResult = ({
					resourceURI: resourceURI,
					userURI: participant,
					isDone: isDone,
					artifactResults: artifactResults
				});
				mapUserURIToParticipantResult[participant] = participantResult;
			},
			this
		);
		var participantResults = [];
		for (userURI in mapUserURIToParticipantResult) {
			var participantResult = mapUserURIToParticipantResult[userURI];
			participantResults.push(participantResult);
			participantResults[userURI] = participantResult;
		}
		this._participantResults = participantResults;
	},

	_parseResourceUri: function(participantResult) {
		return domutils.getAttributeNS(
			participantResult, "about",
			queryutils.RDF_NAMESPACE, queryutils.RDF_QUALIFIER
		);
	},

	_parseParticipant: function(participantResult) {
		var nodes = domutils.getElementsByTagNameNS(
			participantResult, "hasParticipant",
			queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
		);
		var userPath = domutils.getAttributeNS(
			nodes[0], "resource",
			queryutils.RDF_NAMESPACE, queryutils.RDF_QUALIFIER
		);
		var fullyQualifiedUserURI = null;
		if (userPath.substring(0, 3) === "../") {
			fullyQualifiedUserURI =
				queryutils.getFullyQualifiedUriFromRelativeLink(
					com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL,
					userPath);
		} else {
			fullyQualifiedUserURI = userPath;
		}
		return fullyQualifiedUserURI;
	},

	_parseIsDone: function(participantResult) {
		var isDoneValue = this._getTextNodeValue(
			participantResult, "isDone", queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
		);
		return (isDoneValue === "true");
	},

	_parseArtifactResults: function(participantResult) {
		// summary:
		//    Parses the ArtifactResult information found within a ParticipantResult.
		// returns: Array
		//    Returns an array in which each element has the following format:
		//      {
		//          artifactId: https://localhost:9443/rdm/resources/_vFKIYFxTEd6WMqY891WRNA?revision=_vG8RElxTEd6WMqY891WRNA
		//          participantResultStatus: Approved
		//      }
		var artifactResults = [];
		var nodes = domutils.getElementsByTagNameNS(
			participantResult, "ArtifactResult",
			queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
		);
		dojo.forEach(
			nodes,
			function(artifactResult) {
				var artifactId = this._getTextNodeValue(
					 artifactResult, "artifactId", queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
				);
				var participantResultStatus = this._getTextNodeValue(
					 artifactResult, "participantResultStatus", queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
				);
				artifactResults.push({
					artifactId: artifactId,
					participantResultStatus: participantResultStatus
				});
			},
			this
		);
		return artifactResults;
	},

	_getTextNodeValue: function(node, unqualifiedTagName, namespace, qualifier) {
		var nodes = domutils.getElementsByTagNameNS(node, unqualifiedTagName, namespace, qualifier);
		return (
			nodes.length > 0 && nodes[0].firstChild ?
			nodes[0].firstChild.nodeValue :
			""
		);
	}
});


})();
