/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.ReviewEntry");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.form.Button");
dojo.require("dojo.date.locale");
dojo.require("dojo.date.stamp");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.review.ReviewParser");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.common.UserHover");
dojo.require("com.ibm.rdm.web.common.ProjectCache");

dojo.requireLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var queryutils = com.ibm.rdm.web.util.queryutils;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var messages = dojo.i18n.getLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


var MILLISECONDS_IN_A_DAY = (24 * 60 * 60 * 1000);


dojo.declare(
	"com.ibm.rdm.web.review.ReviewEntry",
	[dijit._Widget],
{
	// summary:
	//    A review entry in ReviewsViever. Review title, author, date, etc.

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields
	projectURI: null,
	reviewParser: null,
	reviewResourceUri: null,
	reviewModifiedDate: null,
	projectUsersWithRoles: null,
	currentDate: null,
	
	///////////////////////////////////////////////////////////////////////////////////
	// Attach points
	_reviewTitleNode: null,
	_authorNode: null,
	_timestampNode: null,
	_reviewStateNode: null,
	_dueDateNode: null,
	_authorHover: null,
	_resourceLinkWidget: null,
	
	// TODO: Should reviewUri come from Reviewparser?
	postCreate: function() {
		this.inherited("postCreate", arguments);
		this.domNode.appendChild(this._createReviewEntryTable(this.reviewParser));
	},

	destroy: function() {
		if (this._authorHover) {
			this._authorHover.destroy();
			this._authorHover = null;
		}
		if (this._resourceLinkWidget) {
			// Note by SPC on 8/22/2010 ...
			// Avoid JFS memory leak here due to the fact that destroying the
			// ResourceLink does NOT result in destroying the associated
			// HoverPopup (but after looking at the code, I see that calling
			// JFS's internal _disableHover function WILL result in destroying
			// it, so we make this call here).
			// TODO Report this issue to the JFS team.
			if (this._resourceLinkWidget._disableHover) {
				this._resourceLinkWidget._disableHover();
			}
			this._resourceLinkWidget.destroyRecursive();
			this._resourceLinkWidget = null;
		}
		this.reviewParser = null;
		this.domNode.innerHTML = "";
		this.inherited(arguments);
	},

	_createAuthorTd: function(revParser) {
		var authorTd = document.createElement("td");
		var lastModifiedDate = dojo.date.stamp.fromISOString(revParser.getLastModifiedDate());
		
		var userURI = revParser.getLastModifiedByUserURI();
		var userInfo = this.projectUsersWithRoles[userURI];
		var userName = "";
		if(userInfo){
			userName = userInfo.name;
		}
		else {
			// Handle the case of the "rrs-daemon" user.  This user isn't
			// actually found in the project user listing.  However, when
			// displaying a review you might find that the user that last
			// modified the review is "rrs-daemon" (as the R/A service
			// sometimes moves the review from one state to another).
			var usersPrefix = com.ibm.rdm.web.FRONTING_SERVICE_JFS_USERS;
			userName = userURI.substring(usersPrefix.length + 1);
		}
		authorTd.appendChild(
			document.createTextNode(
				dojo.string.substitute(
					messages.lastModifiedTimestamp,
					[userName, dojo.date.locale.format(lastModifiedDate, { formatLength: "short" })]
				)
			)
		);
		
		dojo.addClass(authorTd, "reviewlastModifiedDate");
		
		if(userInfo){
			this._authorHover = new com.ibm.rdm.web.common.UserHover({projectURI : this.projectURI}, null);
			this._authorHover.createHoverPopup(authorTd, userInfo);					
		}
			
		return authorTd;
	},
	
	_createRoleTd: function(revParser) {
		var roleTd = document.createElement("td");
		var participantRole = revParser.getRole();
		switch (participantRole) {
			case reviewTypes.PARTICIPANT_ROLE_TYPES.Approver:
				dojo.addClass(roleTd, "roleApproverImage");
				break;
			case reviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer:
				dojo.addClass(roleTd, "roleReviewerImage");
				break;
			case reviewTypes.PARTICIPANT_ROLE_TYPES.OptionalReviewer:
				dojo.addClass(roleTd, "roleOptionalReviewerImage");
				break;
			default:
				var revCreatorURI = revParser.getCreatorUserURI();
				if(revCreatorURI == com.ibm.rdm.web.AUTHENTICATED_USER.uri){
					dojo.addClass(roleTd, "roleCreatorReviewerImage");
				}
		}
		dojo.addClass(roleTd, "reviewsRolesTableTd");
		return roleTd;
	},
	
	_createReviewImageDiv: function(revParser) {
		// Add the review image.
		var reviewImageDiv = document.createElement("div");
		var reviewImg = document.createElement("img");
		var reviewState = revParser.getReviewState();
	 
		switch (reviewState) {
			case reviewTypes.REVIEW_STATE_TYPES.Draft:
				reviewImg.setAttribute("src", dojo.moduleUrl("com.ibm.rdm.web", "review/images/icons/obj16/review-draft.gif"));
				break;
			case reviewTypes.REVIEW_STATE_TYPES.Started:
				reviewImg.setAttribute("src", dojo.moduleUrl("com.ibm.rdm.web", "review/images/icons/obj16/review-started.gif"));
				break;
			case reviewTypes.REVIEW_STATE_TYPES.Paused:
				reviewImg.setAttribute("src", dojo.moduleUrl("com.ibm.rdm.web", "review/images/icons/obj16/review-paused.gif"));
				break;	
			case reviewTypes.REVIEW_STATE_TYPES.Reviewed:
				reviewImg.setAttribute("src", dojo.moduleUrl("com.ibm.rdm.web", "review/images/icons/obj16/review-reviewed.gif"));
				break;
			case reviewTypes.REVIEW_STATE_TYPES.Finalized:
				reviewImg.setAttribute("src", dojo.moduleUrl("com.ibm.rdm.web", "review/images/icons/obj16/review-finalized.gif"));
				break;		
		}
		
		reviewImg.setAttribute("title", messages["clickOnReviewNameToViewResults"]);
		reviewImageDiv.appendChild(reviewImg);
		
		return reviewImageDiv;
	},
	
	_createReviewNameTd: function(revParser) {
		var nameTd = document.createElement("td");
		dojo.addClass(nameTd, "reviewNameLink");
		// Add the review name link.
		var link = ResourceLinksHandler.getResourceLink(this.reviewResourceUri, revParser.getReviewName(), true, true);
		link.setAttribute("title", messages["clickOnReviewNameToViewResults"]);
		link.setAttribute("label", messages["clickOnReviewNameToViewResults"]);
		var handle = this.connect(link, "onclick", this._handleReviewClicked);
		this._resourceLinkWidget = dijit.byNode(link);
		nameTd.appendChild(link);
		
		return nameTd;
	},
	
	_createReviewDueDateTd: function(revParser) {
		var dateTd = document.createElement("td");
		
		var dueDate = revParser.getDueDate();
		var now = new Date();
		
		var dueDateImg = document.createElement("img");
		var dueDateTextNode = document.createTextNode(dueDate);
		var dueDate = new Date(dueDate);
		var reviewState = revParser.getReviewState();

		if (reviewState !== reviewTypes.REVIEW_STATE_TYPES.Draft
			&& reviewState !== reviewTypes.REVIEW_STATE_TYPES.Finalized
			&& (dueDate.getTime() + MILLISECONDS_IN_A_DAY) < this.currentDate.getTime()
			) {
			//review is past due
			dueDateImg.setAttribute("src", dojo.moduleUrl("com.ibm.rdm.web", "review/images/icons/obj16/overdue.gif"));
			dojo.addClass(dateTd, "reviewDueDatePastDue");
			dojo.addClass(dueDateImg, "reviewNameLink");
		}
		else {
			dueDateImg.setAttribute("src", dojo.moduleUrl("com.ibm.rdm.web", "review/images/icons/obj16/due-date.gif"));
			dojo.addClass(dateTd, "reviewDueDate");
			dojo.addClass(dueDateImg, "reviewNameLink");
		}
		dateTd.appendChild(dueDateImg);
		dateTd.appendChild(dueDateTextNode);
		dateTd.setAttribute('noWrap','true');
		
		return dateTd;
	},
	
	_createReviewEntryTable: function(revParser) {
		var reviewEntryTable = document.createElement("table");
		dojo.addClass(reviewEntryTable, "reviewEntryTable");
		var tableRow = document.createElement("tr");
		
		var th = document.createElement("th");
		th.rowSpan=2;
		dojo.addClass(th, "reviewRoleImageTableTh");
		th.appendChild(this._createRoleTd(revParser));
		
		var th2 = document.createElement("th");
		th2.rowSpan=2;
		dojo.addClass(th2, "reviewOwnerImageTableTh");
		th2.appendChild(this._createReviewImageDiv(revParser));

		tableRow.appendChild(th);
		tableRow.appendChild(th2);
		tableRow.appendChild(this._createReviewNameTd(revParser));
		tableRow.appendChild(this._createReviewDueDateTd(revParser));
			
		var tBody = document.createElement("tbody");
		tBody.appendChild(tableRow);
		
		var tableRow2 = document.createElement("tr");
		tableRow2.appendChild(this._createLastModifiedTd(revParser));
		
		tBody.appendChild(tableRow2);
		
		reviewEntryTable.appendChild(tBody);
			
		return reviewEntryTable;
	},

	_createLastModifiedTd: function(revParser) {
		var modifiedTd = document.createElement("td");
		
		modifiedTd.appendChild(this._createAuthorTd(revParser));
			
		//modifiedTd.setAttribute('noWrap','true');
		dojo.addClass(modifiedTd, "reviewLastModified");
		return modifiedTd;
	},
	
	_handleReviewClicked: function(event) {
		dojo.stopEvent(event);
		ResourceLinksHandler.handleAnchorClicked(event);
	}
});

})();


