/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.VoteButton");

dojo.require("dijit._Widget");
dojo.require("dojo.i18n");
dojo.require("dijit.form.Button");
dojo.require("dijit.Menu");


dojo.requireLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


dojo.declare(
	"com.ibm.rdm.web.review.VoteButton",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    Provides a vote casting button with a drop down menu on the right.
	//    If the button is pressed voting without comments occurs, but if
	//    the user selects the drop down they may select either the "vote
	//    without comments" or "vote with comments" option for whatever vote
	//    is being cast.

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "review/templates/VoteButton.html"),
	widgetsInTemplate: true,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	label: null,
	showLabel: null,
	disabled: null,
	iconClass: null,
	doAlignVertically: null,
	onVoteWithoutCommentsClick: null,
	onVoteWithCommentsClick: null,
	voteWithoutCommentsLabel: null,
	voteWithCommentsLabel: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_voteButton: null,
	_voteWithoutCommentsMenuItem: null,
	_voteWithCommentsMenuItem: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");
		this.inherited("postMixInProperties", arguments);
	},

	postCreate: function() {
		if (!this.doAlignVertically) {
			this.domNode.className = "";
		}
		this.connect(this._voteWithoutCommentsMenuItem, "onClick", this._voteWithoutComments);
		this.connect(this._voteWithCommentsMenuItem, "onClick", this._voteWithComments);
		this._voteButton.attr("label", this.label);
		this._voteButton.attr("showLabel", this.showLabel);
		this._voteButton.attr("disabled", this.disabled);
		this._voteButton.attr("iconClass", this.iconClass);
		this._voteButton.attr("onClick", this.onVoteWithoutCommentsClick);
	},
	
	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_setDisabledAttr: function(/*Boolean*/ value) {
		this._voteButton.attr("disabled", value);
	},

	_voteWithoutComments: function() {
		this.onVoteWithoutCommentsClick();
	},

	_voteWithComments: function() {
		this.onVoteWithCommentsClick();
	}
});


})();
