/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.AdvancedQuerySection");

dojo.require("dijit._base");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.ComboBox");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.search.DocumentTypes");
dojo.require("com.ibm.rdm.web.search.PropertiesSection");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.search.searchutils");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.common", "common");
dojo.requireLocalization("com.ibm.rdm.web.search", "AdvancedQuerySection");


var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var constants = com.ibm.rdm.web.common.MimeTypes;

//Note: Using concatenation here to avoid Jazz bug 51653 ...
var CONTENT_TYPE_IMAGES = "image/" + "*";

dojo.declare(
    "com.ibm.rdm.web.search.AdvancedQuerySection",
    [dijit._Widget, dijit._Templated],
    {
        templatePath: dojo.moduleUrl("com.ibm.rdm.web", "search/templates/AdvancedQuerySection.html"),
        widgetsInTemplate: true,

        ///////////////////////////////////////////////////////////////////////////////////
        // Attach points

        _contentTypeSelect: null,
        _lastModifiedBySelect: null,
        _docTypesTable: null,
        _propertiesSectionDiv: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Public fields

        searchQuery: null,
        documentTypes: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        _messages: null,
        _docTypeDisplayNames: [],
        _docTypeCheckBoxes: [],
        /* Every doc type check box may have an AttributeValueGroup associated with it
           (the mapping is only made the first time the user clicks on the check box). */
        _docTypeAttributeValueGroups: [],
        /* This prop list maps content-types to AttributeValueGroup instances.
           They're demand loaded when the user selects a group [from Type select] for
           the first time (but only if the content-type supports an indexing rule). */
        _resourceAttributeValueGroups: {},
        _currentlyDisplayedResourceAttributeValueGroup: null,
        _attributeValueData: [],
                    
        UNSPECIFIED: "UnspecifiedComboBoxValue",

        ///////////////////////////////////////////////////////////////////////////////////
        // NLS fields requiring translation

        type: null,
        lastModifiedBy: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // API

        postCreate: function() {
            this._docTypeDisplayNames = this.documentTypes.getDocTypeNames().sort();
            this._attributeValueData = this.documentTypes.getAttributeValueProperties();
            this._populateContentTypeSelect();
            this._queryForUsersAndPopulateLastModifiedBySelect();
			// Commenting this out for now ... as we're no longer supporting secondary resource searching
			// TODO Remove all secondary resource searching code
//             this._contentTypes = new com.ibm.rdm.web.search.ContentTypes({});
//             this._populateDocTypesTable(this._docTypeDisplayNames);
//             this.connect(this._contentTypeSelect, "onchange", this._handleContentTypeValueChange);
//             for (var i = 0; i < this._docTypeCheckBoxes.length; i++) {
//                 var checkBox = this._docTypeCheckBoxes[i];
//                 // Note: I would have preferred to use "onchange" here ... but for some
//                 // reason that appears not to work on IE [but does on Firefox].
//                 // So we'll use "onclick" instead ...
//                 this.connect(checkBox.domNode, "onclick", this._handleDocTypeValueChange);
//             }
        },

        postMixInProperties: function() {
            this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search", "AdvancedQuerySection");
            dojo.forEach(["type", "lastModifiedBy"], function(prop){
                this[prop] = this._messages[prop];
            }, this);
        },

        getContentType: function() {
            // summary:
            //     Return the selected Type (e.g. "application/x-com.ibm.sid.sketch+xml").
            //     Or, if no Type has been selected, return null.
            var contentType = null;
            var selectedOption = this._contentTypeSelect.options[this._contentTypeSelect.selectedIndex];
            var value = selectedOption.value;
            if (value != null && value != "*") {
                contentType = value;
            }
            return contentType;
        },

        getUser: function() {
            // summary:
            //     Return the selected Last Modified By username (e.g. "/jazz/users/Bob").
            //     Or, if no user has been selected, return null.
            var user = null;
            var selectedOption = this._lastModifiedBySelect.options[this._lastModifiedBySelect.selectedIndex];
            var value = selectedOption.value;
            if (value != this.UNSPECIFIED) {
                user = value;
            }
            return user;
        },

        // TODO Remove ... this is how things were done before JRS supported XQuery ...
        getSimpleQueryParams: function() {
            var queryParams = [];

            // Gather appropriate info ...
            var contentTypeQueryParam = (
                this._composeContentTypeQueryParam()
            );
            var lastModifiedByQueryParam = (
                this._lastModifiedBySelect.disabled ?
                null :
                this._composeLastModifiedByQueryParam()
            );

            // Now set queryParams accordingly ...
            if (contentTypeQueryParam) {
                queryParams.push(contentTypeQueryParam);
            }
            if (lastModifiedByQueryParam) {
                queryParams.push(lastModifiedByQueryParam);
            }

            return queryParams;
        },

        // TODO Remove ... this is how things were done before JRS supported XQuery ...
        getResourceTypeSpecificQueryParams: function() {
            var queryParams = [];
            var resourceQueryParams = this._composeResourceQueryParams();
            if (resourceQueryParams) {
                queryParams = queryParams.concat(resourceQueryParams);
            }
            return queryParams;
        },

        // TODO Remove ... this is how things were done before JRS supported XQuery ...
        getDocTypeQueryParams: function() {
            var queryParams = [];
            var docTypeQueryParams = this._composeDocTypeQueryParams();
            if (docTypeQueryParams) {
                queryParams = queryParams.concat(docTypeQueryParams);
            }
            return queryParams;
        },

        setFullTextEnablement: function(doEnableFulltext) {
            this._lastModifiedBySelect.disabled = doEnableFulltext;
            for (var prop in this._resourceAttributeValueGroups) {
                var attrValueGroup = this._resourceAttributeValueGroups[prop];
                attrValueGroup.setFullTextEnablement(doEnableFulltext);
            }
            var i;
            for (i = 0; i < this._docTypeCheckBoxes.length; i++) {
                this._docTypeCheckBoxes[i].attr("disabled", doEnableFulltext);
            }
            for (i = 0; i < this._docTypeAttributeValueGroups.length; i++) {
                var attrValueGroup = this._docTypeAttributeValueGroups[i];
                if (attrValueGroup) {
                    attrValueGroup.setFullTextEnablement(doEnableFulltext);
                }
            }
        },

        doctypeInfoHasBeenUpdated: function() {
            this._attributeValueData = this.documentTypes.getAttributeValueProperties();
            this._docTypeDisplayNames = this.documentTypes.getDocTypeNames().sort();
            this._updateDocTypesTable(this._docTypeDisplayNames);
            for (var i = 0; i < this._docTypeCheckBoxes.length; i++) {
                var checkBox = this._docTypeCheckBoxes[i];
                this.connect(checkBox.domNode, "onclick", this._handleDocTypeValueChange);
            }
        },

        resetAllOptions: function() {
            this._contentTypeSelect.selectedIndex = 0;
            this._lastModifiedBySelect.selectedIndex = 0;
            for (var i = 0; i < this._docTypeCheckBoxes.length; i++) {
                this._docTypeCheckBoxes[i].attr('checked', false);
            }
        },

		isImagesContentType: function(contentType) {
			return (contentType === CONTENT_TYPE_IMAGES);
		},

        ///////////////////////////////////////////////////////////////////////////////////
        // Helpers

// **** Note: Commenting out the following until we know for sure that we won't
// **** support specifying attribute group criteria in our searches ...
//         _handleContentTypeValueChange: function(e) {
//             var selectedOption = this._contentTypeSelect.options[this._contentTypeSelect.selectedIndex];
//             var contentType = selectedOption.value;
//             this._replaceDisplayedDocTypeCheckBoxesIfNeeded(contentType);
//             this._showHideOrUpdatePropertiesSectionIfNeeded();
//             if (this.searchQuery.isFullTextSearchingEnabled()) {
//                 for (var prop in this._resourceAttributeValueGroups) {
//                     var attrValueGroup = this._resourceAttributeValueGroups[prop];
//                     attrValueGroup.setFullTextEnablement(true);
//                 }
//             }
//         },
//         _replaceDisplayedDocTypeCheckBoxesIfNeeded: function(contentType) {
//             var tbody = this._docTypesTable.firstChild;
//             if (tbody && tbody.firstChild) {
//                 var tr = tbody.firstChild;
//                 var isImagesContentType = (contentType == CONTENT_TYPE_IMAGES);
//                 for (var i = 0; tr != null; tr = tr.nextSibling, i++) {
//                     var td = tr.childNodes[1];
//                     var doctypeShortname = td.firstChild.nodeValue;
//                     // TODO This needs to change as M5 no longer distinguishes
//                     // req doc types from rich text doc types (instead there's
//                     // now attr groups in which each attr group mentions which
//                     // resource content types it can be applied to ...).
//                     var isRequirementDocType = this.documentTypes.isRequirementDocType(doctypeShortname);
//                     var shouldBeShown = (
//                         (!isImagesContentType)
//                         && (contentType == constants.CONTENT_TYPE_REQUIREMENT ?
//                             isRequirementDocType :
//                             (!isRequirementDocType)
//                             )
//                     );
//                     var isCurrentlyShown = (tr.style.display != "none");
//                     tr.style.display = (shouldBeShown ? "" : "none");
//                     if (isCurrentlyShown != shouldBeShown) {
//                         this._docTypeCheckBoxes[i].setChecked(false);
//                     }
//                 }
//             }
//         },
//         _uncheckAllOtherVisibleDocTypeCheckBoxes: function(selectedCheckBox) {
//             var tbody = this._docTypesTable.firstChild;
//             if (tbody && tbody.firstChild) {
//                 var tr = tbody.firstChild;
//                 for (; tr != null; tr = tr.nextSibling) {
//                     if (tr.style.display != "none") {
//                         var td = tr.firstChild;
//                         var checkBoxNode = td.firstChild;
//                         var checkBox = dijit.byNode(checkBoxNode);
//                         if (checkBox != selectedCheckBox && checkBox.checked) {
//                             checkBox.setChecked(false);
//                         }
//                     }
//                 }
//             }
//         },
//         _handleDocTypeValueChange: function(e) {
//             var selectedOption = this._contentTypeSelect.options[this._contentTypeSelect.selectedIndex];
//             var contentType = selectedOption.value;
//             // If the Type is set to Requirement, we need to uncheck all other
//             // attribute/value groups that are currently selected.
//             if (contentType == constants.CONTENT_TYPE_REQUIREMENT) {
//                 var checkBox = dijit.byNode(e.target.parentNode);
//                 this._uncheckAllOtherVisibleDocTypeCheckBoxes(checkBox);
//             }
//             this._showHideOrUpdatePropertiesSectionIfNeeded();
//         },
        /**
         * If the content type field is changed, or a particular doctype checkbox value is
         * changed, this function is called to make appropriate updates to the attr/value
         * section area.
         */
//         _showHideOrUpdatePropertiesSectionIfNeeded: function() {
//             var heightHasChanged = false;
//             if (this._atLeastOneAttributeGroupShouldBeShown()) {
//                 // Show the properties section.
//                 if (!this._propertiesSection) {
//                     this._propertiesSection = new com.ibm.rdm.web.search.PropertiesSection({}, null);
//                 }
//                 if (this._propertiesSection.domNode.parentNode != this._propertiesSectionDiv) {
//                     this._propertiesSectionDiv.appendChild(this._propertiesSection.domNode);
//                     heightHasChanged = true;
//                 }
//                 this._updateAttributeValueGroups();
//             } else {
//                 // Hide the properties section.
//                 if (this._propertiesSection && this._propertiesSection.domNode.parentNode) {
//                     node = this._propertiesSection.domNode;
//                     node.parentNode.removeChild(node);                    
//                     heightHasChanged = true;
//                 }
//             }
//             if (heightHasChanged) {
//                 this.searchQuery.advancedQueryHeightHasChanged();
//             }
//         },
        /**
         * This doesn't update the contents of the displayed AttributeValueGroups -- instead it
         * updates which AttributeValueGroups are shown.
         */
//         _updateAttributeValueGroups: function() {
//             var previousGroup = null;
//             var selectedOption = this._contentTypeSelect.options[this._contentTypeSelect.selectedIndex];
//             var contentType = selectedOption.value;
//             var resourceAttributeValues = this._contentTypes.getQueryGroupAttributeValues(contentType);
//             // The first attr/val group to show is the type-specific one, if there is one.
//             if (resourceAttributeValues) {
//                 // Add an AttributeValueGroup associated with this content type.
//                 var startupIsNeeeded = false;
//                 var attrValueGroup = this._resourceAttributeValueGroups[contentType];
//                 if (!attrValueGroup) {
//                     startupIsNeeded = true;
//                     attrValueGroup = this._createResourceAttributeValueGroup(
//                         selectedOption.text, resourceAttributeValues);
//                     this._resourceAttributeValueGroups[contentType] = attrValueGroup;
//                 }
//                 if (this._currentlyDisplayedResourceAttributeValueGroup != attrValueGroup) {
//                     if (this._currentlyDisplayedResourceAttributeValueGroup) {
//                         this._propertiesSection.removeAttributeValueGroup(
//                             this._currentlyDisplayedResourceAttributeValueGroup);
//                     }
//                     this._propertiesSection.addAttributeValueGroup(attrValueGroup, null);
//                     if (startupIsNeeded) {
//                         attrValueGroup.startup();
//                     }
//                 }
//                 previousGroup = attrValueGroup;
//                 this._currentlyDisplayedResourceAttributeValueGroup = attrValueGroup;
//             } else {
//                 // Make sure the resource AttributeValueGroup is not showing.
//                 if (this._currentlyDisplayedResourceAttributeValueGroup) {
//                     this._propertiesSection.removeAttributeValueGroup(
//                         this._currentlyDisplayedResourceAttributeValueGroup);
//                     this._currentlyDisplayedResourceAttributeValueGroup = null;
//                 }
//             }
//             // Add, as appropriate, an attr/val group for each of the checked off
//             // checkbox doctypes.
//             for (var i = 0; i < this._docTypeCheckBoxes.length; i++) {
//                 var checkBox = this._docTypeCheckBoxes[i];
//                 var docTypeDisplayName = this._docTypeDisplayNames[i];
//                 var attrValueGroup = this._docTypeAttributeValueGroups[i];
//                 if (checkBox.checked) {
//                     // Add an AttributeValueGroup associated with this checkBox.
//                     if (!attrValueGroup) {
//                         var attrValueGroup = this._createDocTypeAttributeValueGroup(docTypeDisplayName);
//                         this._docTypeAttributeValueGroups[i] = attrValueGroup;
//                     }
//                     this._propertiesSection.addAttributeValueGroup(attrValueGroup, previousGroup);
//                     previousGroup = attrValueGroup;
//                 } else {
//                     // Remove an AttributeValueGroup associated with this checkBox.
//                     if (attrValueGroup) {
//                         this._propertiesSection.removeAttributeValueGroup(attrValueGroup);
//                     }
//                 }
//             }
//         },
//         _createResourceAttributeValueGroup: function(label, attributeValues) {
//             var attrValues = dojo.map(
//                 attributeValues,
//                 function(item) {
//                     return new com.ibm.rdm.web.search.AttributeValue(item);
//                 }
//             );
//             var attrValueGroup = new com.ibm.rdm.web.search.AttributeValueGroup(
//                 {
//                     label: label,
//                     attributeValues: attrValues,
//                     doRestrictToSingleSelection: true
//                 }, null);
//             return attrValueGroup;
//         },
//         _createDocTypeAttributeValueGroup: function(docTypeDisplayName) {
//             var attrValues = dojo.map(
//                 this._attributeValueData[docTypeDisplayName],
//                 function(item) {
//                     return new com.ibm.rdm.web.search.AttributeValue(item);
//                 }
//             );
//             var attrValueGroup = new com.ibm.rdm.web.search.AttributeValueGroup(
//                 {
//                     label: docTypeDisplayName,
//                     attributeValues: attrValues,
//                     doRestrictToSingleSelection: false
//                 }, null);
//             return attrValueGroup;
//         },
//         _atLeastOneAttributeGroupShouldBeShown: function() {
//             var atLeastOneAttributeGroupShouldBeShown = false;
//             var contentType = this._contentTypeSelect.options[this._contentTypeSelect.selectedIndex].value;
//             if (contentType != "*" && this._contentTypes.getQueryGroupAttributeValues(contentType)) {
//                 atLeastOneAttributeGroupShouldBeShown = true;
//             } else {
//                 for (var i = 0; i < this._docTypeCheckBoxes.length; i++) {
//                     var checkBox = this._docTypeCheckBoxes[i];
//                     if (checkBox.checked) {
//                         atLeastOneAttributeGroupShouldBeShown = true;
//                         break;
//                     }
//                 }
//             }
//             return atLeastOneAttributeGroupShouldBeShown;
//         },
//         _composeResourceQueryParams: function() {
//             var queryParams = [];
//             var selectedOption = this._contentTypeSelect.options[this._contentTypeSelect.selectedIndex];
//             var contentType = selectedOption.value;
//             var attrValueGroup = this._resourceAttributeValueGroups[contentType];
//             if (attrValueGroup) {
//                 var groupQueryParams = attrValueGroup.getQueryParams();
//                 if (groupQueryParams) {
//                     queryParams = queryParams.concat(groupQueryParams);
//                 }
//             }
//             return queryParams;
//         },
//         _composeDocTypeQueryParams: function() {
//             var queryParams = [];
//             for (var i = 0; i < this._docTypeAttributeValueGroups.length; i++) {
//                 var checkBox = this._docTypeCheckBoxes[i];
//                 if (checkBox.checked) {
//                     var attrValueGroup = this._docTypeAttributeValueGroups[i];
//                     if (attrValueGroup) {
//                         var groupQueryParams = attrValueGroup.getQueryParams();
//                         if (groupQueryParams) {
//                             queryParams = queryParams.concat(groupQueryParams);
//                         }
//                     }
//                 }
//             }
//             return (queryParams.length == 0 ? null : queryParams);
//         },
//         _populateDocTypesTable: function(docTypeDisplayNames) {
//             var table = this._docTypesTable;
//             var elems = table.getElementsByTagName("tbody");
//             var tbody;
//             if (elems.length == 0) {
//                 tbody = document.createElement("tbody");
//                 table.appendChild(tbody);
//             } else {
//                 tbody = elems[0];
//             }
//             this._addDocTypeTableRows(docTypeDisplayNames, tbody);
//         },
//         _updateDocTypesTable: function(docTypeDisplayNames) {
//             var table = this._docTypesTable;
//             var tbody = table.getElementsByTagName("tbody")[0];
//             // Clear out the old table rows (by removing tbody and adding a new one).
//             if (tbody) {
//                 this._removeDocTypeCheckBoxes();
//                 table.removeChild(tbody);
//             }
//             tbody = document.createElement("tbody");
//             table.appendChild(tbody);
//             // Add new table rows.
//             this._addDocTypeTableRows(docTypeDisplayNames, tbody);
//         },
//         _removeDocTypeCheckBoxes: function() {
//             for (var i = 0; i < this._docTypeCheckBoxes.length; i++) {
//                 this._docTypeCheckBoxes[i].destroyRecursive();
//             }
//             this._docTypeCheckBoxes = [];
//         },
//         _addDocTypeTableRows: function(docTypeDisplayNames, tbody) {
//             for (var i = 0; i < docTypeDisplayNames.length; i++) {
//                 var docTypeDisplayName = docTypeDisplayNames[i];
//                 var checkBox = new dijit.form.CheckBox();
//                 var textNode = document.createTextNode(docTypeDisplayName);
//                 var tr = document.createElement("tr");
//                 // TODO Is it really appropriate to, by default, hide req
//                 // specific doctypes from display?  As coded here, it's only
//                 // AFTER selecting "Requirement" that the user would see
//                 // the req-specific doctypes.  Perhaps we should show them
//                 // all and enable/disable appropriate doctypes when user
//                 // makes the first selection in a check box?  I'm not using
//                 // this enable/disable approach as I'm guessing that this
//                 // would be too confusing for users ...
//                 if (this.documentTypes.isRequirementDocType(docTypeDisplayName)) {
//                     tr.style.display = "none";
//                 }
//                 var td;
//                 tbody.appendChild(tr);
//                 td = document.createElement("td");
//                 td.appendChild(checkBox.domNode);
//                 tr.appendChild(td);
//                 td = document.createElement("td");
//                 td.appendChild(textNode);
//                 tr.appendChild(td);
//                 this._docTypeCheckBoxes.push(checkBox);
//             }
//         }

        _queryForUsersAndPopulateLastModifiedBySelect: function() {
			var onComplete = function(userPropertyInfos) {
				var sortedUserPropertyInfos =
					ProjectCache.sortUserPropertiesArrayByTitle(userPropertyInfos);
				this._populateLastModifiedBySelect(sortedUserPropertyInfos);
				
				return userPropertyInfos;
			}
			ProjectCache.getAllUsers().addCallback(this, onComplete);
        },

        _composeContentTypeQueryParam: function() {
            var queryParam = null;
            var selectedOption = this._contentTypeSelect.options[this._contentTypeSelect.selectedIndex];
            var contentType = selectedOption.value;
            if (contentType != null && contentType != "*") {
                queryParam = "resource-content-type="
                    + com.ibm.rdm.web.search.searchutils.encodeContentType(contentType);
            }
            return queryParam;
        },

        _composeLastModifiedByQueryParam: function() {
            var queryParam = null;
            var selectedOption = this._lastModifiedBySelect.options[this._lastModifiedBySelect.selectedIndex];
            var username = selectedOption.value;
            if (username != this.UNSPECIFIED) {
                queryParam = "resource-last-modified-by=" + username;
            }
            return queryParam;
        },

        /**
         * Populate entries for the select combobox that enables selecting an
         * appropriate content type.
         */              
        _populateContentTypeSelect: function() {
			var commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
			var contentTypeOptions = [
	            {   contentType: "*",
	                displayName: ""
	            },
	            {   contentType: constants.CONTENT_TYPE_ACTOR,
	                displayName: commonMessages["actor"]
	            },
	            {   contentType: constants.CONTENT_TYPE_PROCESS,
	                displayName: commonMessages["process"]
	            },
	            {   contentType: constants.CONTENT_TYPE_ZIP,
	                displayName: commonMessages["zipFile"]
	            },
	            {   contentType: constants.CONTENT_TYPE_RICH_TEXT,
	                displayName: commonMessages["richTextDocument"]
	            },
	            {   contentType: constants.CONTENT_TYPE_XML,
	                displayName: commonMessages["xmlFile"]
	            },
	            {   contentType: constants.CONTENT_TYPE_GLOSSARY,
	                displayName: commonMessages["glossary"]
	            },
	            {   contentType: constants.CONTENT_TYPE_HTML,
	                displayName: commonMessages["htmlFile"]
	            },
	            {   contentType: constants.CONTENT_TYPE_IMAGES,
	                displayName: commonMessages["image"]
	            },
	            {   contentType: constants.CONTENT_TYPE_IRIS,
	                displayName: commonMessages["iRiseSimulation"]
	            },
	            {   contentType: constants.CONTENT_TYPE_JS,
	                displayName: commonMessages["javascriptFile"]
	            },
	            {   contentType: constants.CONTENT_TYPE_XLS,
	                displayName: commonMessages["microsoftExcelDocument"]
	            },
	            {   contentType: constants.CONTENT_TYPE_PPT,
	                displayName: commonMessages["microsoftPowerPointDocument"]
	            },
	            {   contentType: constants.CONTENT_TYPE_PRJ,
	                displayName: commonMessages["microsoftProjectDocument"]
	            },
	            {   contentType: constants.CONTENT_TYPE_DOC,
	                displayName: commonMessages["microsoftWordDocument"]
	            },
	            {   contentType: constants.CONTENT_TYPE_ODP,
	                displayName: commonMessages["openDocumentPresentation"]
	            },
	            {   contentType: constants.CONTENT_TYPE_ODS,
	                displayName: commonMessages["openDocumentSpreadsheet"]
	            },
	            {   contentType: constants.CONTENT_TYPE_ODT,
	                displayName: commonMessages["openDocumentText"]
	            },
	            {   contentType: constants.CONTENT_TYPE_OTHER,
	                displayName: commonMessages["otherTypes"]
	            },
	            {   contentType: constants.CONTENT_TYPE_PDF,
	                displayName: commonMessages["pdfDocument"]
	            },
	            {   contentType: constants.CONTENT_TYPE_TXT,
	                displayName: commonMessages["plainText"]
	            },
	            {   contentType: constants.CONTENT_TYPE_RAVENFLOW,
	                displayName: commonMessages["ravenFlow"]
	            },
	            {   contentType: constants.CONTENT_TYPE_RTP,
	                displayName: commonMessages["realTimeTransportProtocol"]
	            },
	            {   contentType: constants.CONTENT_TYPE_REQUIREMENT,
	                displayName: commonMessages["requirement"]
	            },	            
	            {   contentType: constants.CONTENT_TYPE_SCREEN_FLOW,
	                displayName: commonMessages["screenFlow"]
	            },
	            {   contentType: constants.CONTENT_TYPE_STORY_BOARD,
	                displayName: commonMessages["storyBoard"]
	            },
	            {   contentType: constants.CONTENT_TYPE_TAR,
	                displayName: commonMessages["tapeArchive"]
	            },
	            {   contentType: constants.CONTENT_TYPE_TERM,
	                displayName: commonMessages["term"]
	            },
	            {   contentType: constants.CONTENT_TYPE_TEX,
	                displayName: commonMessages["texFormat"]
	            },
	            {   contentType: constants.CONTENT_TYPE_USE_CASE,
	                displayName: commonMessages["useCase"]
	            },
	            {   contentType: constants.CONTENT_TYPE_USE_CASE_DIAGRAM,
	                displayName: commonMessages["useCaseDiagram"]
	            },
	            {   contentType: constants.CONTENT_TYPE_PART,
	                displayName: commonMessages["part"]
	            },
	            {   contentType: constants.CONTENT_TYPE_SKETCH,
	                displayName: commonMessages["sketch"]
	            },
	            {   contentType: constants.CONTENT_TYPE_VIDEOS,
	                displayName: commonMessages["video"]
	            }
	        ];
            var selectNode = this._contentTypeSelect;
            var size = contentTypeOptions.length;
            for (var i = 0; i < size; i++) {
                var ctEntry = contentTypeOptions[i];
                var option = document.createElement("option");
                option.value = ctEntry.contentType;
                option.appendChild(document.createTextNode(ctEntry.displayName));
                selectNode.appendChild(option);
            }
        },

        /**
         * Populate entries for the select combobox that enables selecting a
         * user to filter by a specific last-modified-by user name.
         */
        _populateLastModifiedBySelect: function(sortedUserPropertyInfos) {
            var selectNode = this._lastModifiedBySelect;
            // Add a default Unspecified entry
            selectNode.appendChild(this._createOption("", this.UNSPECIFIED));
            // Add the users
            for (var i = 0; i < sortedUserPropertyInfos.length; i++) {
				var username = sortedUserPropertyInfos[i].name;
				var userURI = sortedUserPropertyInfos[i].url;
                var option = this._createOption(username, userURI);
                selectNode.appendChild(option);
            }
            if (dojo.isIE) {
                // Don't know why this IE specific code is necessary, but it is.
                // Without this code the select would contain no values until you
                // use the More button to collapse and re-expand the
                // AdvancedQuerySection (see bug 5070).
                var parent = this._lastModifiedBySelect.parentNode;
                parent.removeChild(this._lastModifiedBySelect);
                parent.appendChild(this._lastModifiedBySelect);
            }
        },

        _createOption: function(label, value) {
            var option = document.createElement("option");
            option.value = value;
            option.appendChild(document.createTextNode(label));
            return option;
        }
	}
);



})();
