/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.AttributeValue");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Templated");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.ComboBox");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.NumberTextBox");
dojo.require("dijit.Tooltip");

dojo.requireLocalization("com.ibm.rdm.web.search", "common");

dojo.declare(
    "com.ibm.rdm.web.search.AttributeValue",
    [dijit._Widget, dijit._Templated],
    {
        templatePath: dojo.moduleUrl("com.ibm.rdm.web", "search/templates/AttributeValue.html"),
        widgetsInTemplate: true,

        ///////////////////////////////////////////////////////////////////////////////////
        // Attach points

        _checkBoxDiv: null,
        _attributeNameSpan: null,
        _attributeValueDiv: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Public fields

        checked: false,
            
        /**
         * The type of field to be show.  This setting influences which kind of control
         * is shown (for specifying the appropriate field value to match against).
         * Must be set to either "string", "enum", "boolean" or "integer".
         */
        attributeType: "string",
        /**
         * The display name for the field.
         */
        attributeName: null,
        /**
         * The database field name to be be specified in a query param when composing
         * an appropriate query URL.
         */
        attributeField: null,
        /**
         * The text to appear in a tool tip if the user hovers over the attribute
         * name for this attr/value pair.
         */
        attributeToolTip: null,
        /**
         * The database field value to be be specified in a query param when composing
         * an appropriate query URL.
         */
        attributeValue: null,
        /**
         * This property should only be passed if the attributeType is "enum".
         * It contains the appropriate enum values to display.
         */
        attributeEnumValues: null,
            

        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        _checkBox: null,
        _tableRow: null,
        _valueControl: null,
        _fieldNameSpan: null,
        _defaultTextInputSize: "13",
        _attributeValueGroup: null,
        _messages: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // API

        postCreate: function() {
            // TODO Why do the td & tr elements get removed if they're mentioned in the template?
//             if (!this.attributeName) {
//                 this.attributeName = attributeField;
//             }
//             this._checkBox = new dijit.form.CheckBox({}, this._checkBoxDiv);
//             this._attributeNameSpan.appendChild(document.createTextNode(this.attributeName));
            // Instead we'll do this the inline here ...
            var tr = document.createElement("tr");
            this.domNode.appendChild(tr);
            var td;
            //    add checkbox
            td = document.createElement("td");
            var checkBox = new dijit.form.CheckBox({});
            td.appendChild(checkBox.domNode);
            tr.appendChild(td);
            //    add label
            td = document.createElement("td");
            var span = document.createElement("span");
            span.innerHTML = '<span class="search-attribute-name">' + this.attributeName + "</span>";
            td.appendChild(span);
            tr.appendChild(td);

            this._tableRow = tr;
            this._checkBox = checkBox;
            this._fieldNameSpan = span;
            this.connect(this._checkBox, "onChange", this._handleCheckBoxValueChange);
            this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search", "common");
        },

        startup: function() {
            // Add a tooltip, if needed.
            if (this.attributeTooltip) {
                var parentNode = this._fieldNameSpan.parentNode;
                var id = (this.id + ":help-tooltip");
                this._fieldNameSpan.id = id;
                var div = document.createElement("div");
                div.appendChild(document.createTextNode(this.attributeTooltip));
                parentNode.appendChild(div);
                var tooltip = new dijit.Tooltip({connectId:[id]}, div);
                tooltip.startup();
            }
        },

        getQueryParam: function() {
            var queryParam = null;
            if (this._checkBox.checked && this._valueControl && !this._valueControl.disabled) {
                queryParam = this.attributeField + "=";
                switch(this.attributeType) {
                    case "string" :
                        if (this.attributeIsExternalLinkField) {
                            queryParam += encodeURIComponent(
                                this._addLeadingHttpSchemeIfNeeded(this._valueControl.value));
                        } else if (this.attributeIsInternalLinkField) {
                            queryParam += encodeURIComponent(
                                this._addLeadingJRSResourcePrefixIfNeeded(this._valueControl.value));
                        } else {
                            queryParam += encodeURIComponent(
                                this._valueControl.value);
                        }
                        break;
                    case "enum" :
                        queryParam += this._valueControl.value;
                        break;
                    case "boolean" :
                        queryParam += this._valueControl.value;
                        break;
                    case "integer" :
                        queryParam += this._valueControl.value;
                        break;
                }
            }
            return queryParam;
        },

        setFullTextEnablement: function(doEnableFulltext) {
            this._checkBox.attr("disabled", doEnableFulltext);
            if (this._valueControl) {
                this._valueControl.disabled = doEnableFulltext;
            }
        },

        addCheckBoxValueChangedListener: function(attributeValueGroup) {
            this._attributeValueGroup = attributeValueGroup;
            this.connect(this._checkBox.domNode, "onclick", function(e) {
                var checkBox = dijit.byNode(e.target.parentNode);
                this._attributeValueGroup.checkBoxValueHasChanged(this);
            });
        },

        setChecked: function(checked) {
            if (this._checkBox.checked != checked) {
                this._checkBox.attr('checked', checked);
            }
        },

        ///////////////////////////////////////////////////////////////////////////////////
        // Helpers

        _addLeadingHttpSchemeIfNeeded: function(queryValue) {
            var schemePrefix = "http://";
            if (queryValue.substring(0, schemePrefix.length) != schemePrefix) {
                return schemePrefix + queryValue;
            } else {
                return queryValue;
            }
        },

        _addLeadingJRSResourcePrefixIfNeeded: function(queryValue) {
        	return queryValue;
        },

        _handleCheckBoxValueChange: function() {
            if (this._checkBox.checked) {
                // Add a value control.
                var td = document.createElement("td");
                switch(this.attributeType) {
                    case "string" :
                        this._valueControl = this._createTextValueControl(td);
                        break;
                    case "enum" :
                        this._valueControl = this._createEnumValueControl(td);
                        break;
                    case "boolean" :
                        this._valueControl = this._createBooleanValueControl(td);
                        break;
                    case "integer" :
                        this._valueControl = this._createIntegerValueControl(td);
                        break;
                }
                this._tableRow.appendChild(td);
            } else {
                // Remove the value control.
                var td = this._valueControl.parentNode;
                var list = dojo.query('[widgetId]', td);
                dojo.forEach(list.map(dijit.byNode), function(widget){ widget.destroyRecursive(); });
                this._tableRow.removeChild(td);
            }
        },

        _createTextValueControl: function(td) {
            var textBox = new dijit.form.TextBox({size:this._defaultTextInputSize}, null);
            td.appendChild(textBox.domNode);
            return textBox.domNode;
        },

        _createEnumValueControl: function(td) {
            var select = document.createElement("select");
            select.className = "comboBox";
            this._populateSelectionOptions(select, this.attributeEnumValues);
            td.appendChild(select);
            return select;
        },

        _createBooleanValueControl: function(td) {
            var select = document.createElement("select");
            this._populateSelectionOptions(select, [ "true", "false" ], [ this._messages.trueValue, this._messages.falseValue ]);
            td.appendChild(select);
            return select;
        },

        _createIntegerValueControl: function(td) {
            // Note: It appears that you cannot pass null to the NumberTextBox constuctor,
            // as the constuctor defref's the parentNode ... so I'll make sure there's a
            // parent there ...
            var div = document.createElement("div");
            td.appendChild(div);
            var numberTextBox = new dijit.form.NumberTextBox({size:this._defaultTextInputSize}, div);
            return numberTextBox.domNode;
        },

        _populateSelectionOptions: function(selectNode, enumValues, enumLabels) {
            for (var i = 0; i < enumValues.length; i++) {
                var value = enumValues[i];
                var label = (enumLabels ? enumLabels[i] : value);
                var option = this._createOption(label, value);
                selectNode.appendChild(option);
            }
        },

        _createOption: function(label, value) {
            var option = document.createElement("option");
            option.value = value;
            option.appendChild(document.createTextNode(label));
            return option;
        }
    }
);

})();
