/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.DocumentTypeParser");


dojo.declare(
    "com.ibm.rdm.web.search.DocumentTypeParser",
    null,
    {
        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        _doctypeName: null,     // E.g. "FEAT"
        _doctypeNamespace: null, // E.g. "http://com.ibm.rdm/FEAT"
        _fieldProperties: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // API

        /**
         * keywordParameters.doc ... a DocType XSD document
         */
	    constructor: function(/* Object */ keywordParameters) {
            this._doctypeName = "";
            this._fieldProperties = {};
            this._parseXSD(keywordParameters.doc);
        },

        getDocTypeName: function() {
            return this._doctypeName;
        },

        getDocTypeNamespace: function() {
            return this._doctypeNamespace;
        },

        getDocTypeFieldProperties: function() {
            return this._fieldProperties;
        },

        ///////////////////////////////////////////////////////////////////////////////////
        // Helpers

        _parseXSD: function(doc) {
            var elementNodes = doc.getElementsByTagName("element");
            var doctypeName = elementNodes[0].getAttribute("name");
            this._doctypeName = doctypeName;
            var namespace = this._getDocTypeNamespace(doc, doctypeName);
            this._doctypeNamespace = namespace;
            for (var i = 1; i < elementNodes.length; i++) {
                var fieldNode = elementNodes[i];
                var attrName = fieldNode.getAttribute("name");
                var attrField = namespace + "%23" + attrName;
                var type = fieldNode.getAttribute("type");
                if (type) {
                    switch (type) {
                    case "string":
                    case "boolean":
                    case "integer":
                        this._fieldProperties[attrName] =
                            this._createSingleFieldProperties(
                                attrName, attrField, type);
                        break;
                    default:
                        console.error(
                            "Unrecognized type '" + type
                            + "' found in doctype resource '" + doctypeName + "'");
                        break;
                    }
                } else {
                    var enumNodes = fieldNode.getElementsByTagName("enumeration");
                    if (enumNodes.length == 0) {
                        console.error(
                            "An unrecognized field type is associated with field '"
                            + attrName + "' in doctype resource '" + doctypeName + "'");
                    } else {
                        var enumValues = [];
                        for (var j = 0; j < enumNodes.length; j++) {
                            var enumValue = enumNodes[j].getAttribute("value");
                            enumValues.push(enumValue);
                        }
                        this._fieldProperties[attrName] =
                            this._createSingleFieldProperties(
                                attrName, attrField, "enum", enumValues);
                    }
                }
            }
        },

        _createSingleFieldProperties: function(attributeName, attributeField, attributeType, attributeEnumValues) {
            var fieldProps = {
                attributeName: attributeName,
                attributeField: attributeField,
                attributeType: attributeType
            };
            if (attributeEnumValues) {
                fieldProps["attributeEnumValues"] = attributeEnumValues;
            }
            return fieldProps;
        },

        _getDocTypeNamespace: function(doc, doctypeName) {
            var schemaNode = doc.getElementsByTagName("schema")[0];
            return schemaNode.getAttribute("targetNamespace");
        }
    }
);

})();
