/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.QueryHandler");

dojo.require("com.ibm.rdm.web.search.AdvancedQuerySection");
dojo.require("com.ibm.rdm.web.search.searchutils");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.requireLocalization("com.ibm.rdm.web.search", "QueryHandler");

/**
 * This class handles issuing the requests associated with queries (i.e. searching
 * based on resource name, type, last modified by, doc type, or resource type-
 * specific index field).
 */
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var uriHelper = com.ibm.rdm.web.common.UriHelper; 
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
 
dojo.declare(
    "com.ibm.rdm.web.search.QueryHandler",
    null,
    {
        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        _messages: null,
        _searchQuery: null,
        _outstandingRequestCount: null,
        _primaryResourceQueryDoc: null,
        _secondaryResourceQueryDoc: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // API

	    constructor: function(/* Object */ keywordParameters){
            this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search", "QueryHandler");
            this._searchQuery = keywordParameters.searchQuery;
        },

        // TODO Remove ... will probably need to reference this, however, to get attr group
        // styles working so leaving in for now ...
//         /**
//          * Request that a "query" be performed on the server.
//          * See the following:
//          *     http://jazz.net/jazz/web/projects/Jazz%20Project#action=com.ibm.team.workitem.viewWorkItem&id=44461
//          * for why we're issuing two separate queries in those cases in which
//          * the user has specified both doc type(s) as well as a secondary resource
//          * query.
//          */
//         performQuery: function(userQueryString, advancedQuerySection) {
//             com.ibm.rdm.web.search.searchutils.showAsBusy(true);
//             // Note: JSR doesn't know about our home-brewed approach to representing
//             // resource folders ... which means that we've got no recourse but to ask
//             // for all resources and THEN prune them down based on our home brewed
//             // matching scheme.  E.g. if user enters "Doc*" we want hits for such
//             // resources as "/jazz/resources/rdm/Project/CustomerA/Doc1" or
//             // "/jazz/resources/rdm/Project/CustomerA/misc/Doc1" (note: these
//             // resources DO NOT share a common path leading to "Doc*" ...
//             var primaryResourceQueryParamsString = "resource-uri=*";
//             var secondaryResourceQueryParamsString = "";
//             var docTypeQueryParams = [];
//             var resourceTypeSpecificQueryParams = [];
//             if (advancedQuerySection != null) {
//                 var simpleQueryParams = advancedQuerySection.getSimpleQueryParams();
//                 var docTypeQueryParams = advancedQuerySection.getDocTypeQueryParams();
//                 resourceTypeSpecificQueryParams =
//                     advancedQuerySection.getResourceTypeSpecificQueryParams();
//                 for (var i = 0; i < simpleQueryParams.length; i++) {
//                     primaryResourceQueryParamsString += ("&" + simpleQueryParams[i]);
//                 }
//                 for (var i = 0; i < docTypeQueryParams.length; i++) {
//                     primaryResourceQueryParamsString += ("&" + docTypeQueryParams[i]);
//                 }
//                 if (resourceTypeSpecificQueryParams.length != 0) {
//                     // First add query strings for things like resource-content-type
//                     // and resource-last-modified-by to restrict the search space.
//                     for (var i = 0; i < simpleQueryParams.length; i++) {
//                         secondaryResourceQueryParamsString += ("&" + simpleQueryParams[i]);
//                     }
//                     // Now add the secondary resource query strings.
//                     for (var i = 0; i < resourceTypeSpecificQueryParams.length; i++) {
//                         secondaryResourceQueryParamsString += ("&" + resourceTypeSpecificQueryParams[i]);
//                     }
//                     // Avoid the first "&" in the string.
//                     secondaryResourceQueryParamsString = secondaryResourceQueryParamsString.substring(1);
//                 }
//             }
//             // We need to issue 2 queries iff both docTypeQueryParams and
//             // resourceTypeSpecificQueryParams have positive lengths.
//             this._outstandingRequestCount = (
//                 (docTypeQueryParams.length > 0 && resourceTypeSpecificQueryParams.length > 0) ?
//                 2 : 1);
//             this._primaryResourceQueryDoc = null;
//             this._secondaryResourceQueryDoc = null;
//             if (docTypeQueryParams.length > 0 || resourceTypeSpecificQueryParams.length == 0) {
//                 com.ibm.rdm.web.search.SearchController.performQuery(
//                      primaryResourceQueryParamsString,
//                      this,
//                      "_handlePrimaryResourceQuerySuccessResponse",
//                      "_handlePrimaryResourceQueryErrorResponse"
//                 );
//             }
//             if (resourceTypeSpecificQueryParams.length > 0) {
//                 com.ibm.rdm.web.search.SearchController.performQuery(
//                     secondaryResourceQueryParamsString,
//                     this,
//                     "_handleSecondaryResourceQuerySuccessResponse",
//                     "_handleSecondaryResourceQueryErrorResponse"
//                 );
//             }
//         },

        performQuery: function(userQueryString, advancedQuerySection, orderClause) {
            com.ibm.rdm.web.search.searchutils.showAsBusy(true);
            var jrsUser = null;
            var contentType = null;
			var isImagesContentType = false;
            if (advancedQuerySection != null) {
                contentType = advancedQuerySection.getContentType();
                jrsUser = advancedQuerySection.getUser();
				isImagesContentType = advancedQuerySection.isImagesContentType(contentType);
            }
			// Set 'namePattern' to an appropriate search pattern.  Here are
			// some examples of userQueryString to namePattern assignments:
			//     "*"         --> null
			//     "x"         --> "^x.*"
			//     "x*"        --> "^x.*"
			//     "x*y"       --> "^x.*y.*"
			//     "x*y*"      --> "^x.*y.*"
			//     "*details"  --> ".*details.*"
			// [Note: if namePattern is set to 'null', this indicates that all
			//  artifacts should be included in the result set.]
			var namePattern = null;
			if (userQueryString) {
				namePattern = (userQueryString.replace(/\*/g, ".*"));
				if (namePattern.length < 2
					|| (namePattern.substring(0, 2) !== ".*")) {
					// Only if the userQueryString didn't start with "*" do we prefix "^"
					// onto the namePattern.
					namePattern = "^" + namePattern;
				}
				if ((namePattern.length < 2)
					|| (namePattern.substring(namePattern.length - 2) !== ".*")) {
					// Make sure namePattern ends with ".*"
					namePattern += ".*";
				}
				if (namePattern === ".*") {
					namePattern = null;
				}
			}
            this._outstandingRequestCount = 1;
            var postData = (
                "xquery \n"
            	+ "declare namespace ref = \"http://jazz.net/xmlns/alm/rm/Reference/v0.1#\";\n"
                + "declare namespace dc = \"http://purl.org/dc/terms/\";\n"
                + "declare namespace rdf = \"http://www.w3.org/1999/02/22-rdf-syntax-ns#\";\n"
                + "declare namespace atom = \"http://www.w3.org/2005/Atom#\";\n"
                + "declare namespace prop = \"http://jazz.net/xmlns/properties/v0.6#\";\n"
                + "declare namespace resExt = \"http://com.ibm.rdm/resource/extension#\";\n"
                + "for $index in collection('JRS_RDF_INDEX')/rdf:Description\n"
                + "where $index/prop:resource-collection/@rdf:resource=\"/jazz/resources/rdm/extendedResources\"\n"
	            + "    and ($index/resExt:format < 'application/x-com.ibm.rdm.attributegroupstyle+xml'\n" 
	            + "         or $index/resExt:format > 'application/x-com.ibm.rdm.attributegroupstyle+xml')\n" 
	            + "    and $index//prop:archived=\"false\"\n"
                + (contentType ?
					(isImagesContentType ?
						("    and ($index/resExt:format='image/bmp'\n"
							+ "        or $index/resExt:format='image/gif'\n"
							+ "        or $index/resExt:format='image/jpeg'\n"
							+ "        or $index/resExt:format='image/png'\n"
							+ "        or $index/resExt:format='image/x-icon')\n") :
						("    and $index/resExt:format='" + contentType + "'\n")) :
				  	("    and ($index/resExt:format < 'application/atom+xml;type=entry'\n"
					 + "        or $index/resExt:format > 'application/atom+xml;type=entry')\n"))
                + (jrsUser ? ("    and $index/resExt:contributor/@rdf:resource=\"" + jrsUser + "\"\n") : "")
                + (namePattern ?
					("    and $index[fn:matches(fn:lower-case(*:name), fn:lower-case('" + namePattern + "'))]\n"
				   		+ "    and $index[fn:not(fn:empty(*:name))]\n") :
					"")
                + (orderClause ?
					("order by $index/" + orderClause + "\n") :
					"order by $index/fn:lower-case(*:name)\n")
                + "return $index"
            );
        },
        
        _constructQueryNameClause: function (artifactName) {
        	
        	var artifactNameClause = "";
        	if(artifactName){
            	artifactNameClause = 'name="' + artifactName + '"';
        	}
        	return artifactNameClause; 
        },
        
       _constructContentTypeClause: function (contentType) {
        	
        	var contentTypeClause = "";
        	if(contentType){
            	contentTypeClause = 'contentType="' + contentType + '"';
        	}
      		return contentTypeClause;  
        },
        
       _constructLastModifiedByClause: function (lastModifiedBy) {
        	
        	var lastModifiedByClause = "";
        	if(lastModifiedBy){
            	lastModifiedByClause = 'lastModifiedBy=<' + lastModifiedBy + '>';
        	}
      		return lastModifiedByClause;  
        },

        ///////////////////////////////////////////////////////////////////////////////////
        // Helpers

        _decrementOutstandingCountAndShowResultsIfAppropriate: function() {
            this._outstandingRequestCount--;
            if (this._outstandingRequestCount == 0) {
                com.ibm.rdm.web.search.searchutils.showAsBusy(false);
                if (this._primaryResourceQueryDoc && this._secondaryResourceQueryDoc) {
                    var filteredPrimaryResourcesObject = this._createFilteredPrimaryResourcesObject(this._primaryResourceQueryDoc);
                    this._searchQuery.displayQueryResults(
                        this._secondaryResourceQueryDoc, filteredPrimaryResourcesObject);
                } else {
                    var doc = (this._primaryResourceQueryDoc || this._secondaryResourceQueryDoc);
                    this._searchQuery.displayQueryResults(doc);
                }
            }
        },

        _decrementOutstandingCountAndShowResults: function(doc) {
            this._outstandingRequestCount--;
            if (this._outstandingRequestCount == 0) {
                com.ibm.rdm.web.search.searchutils.showAsBusy(false);
                this._searchQuery.displayXQueryResults(doc);
            }
        },

        /**
         * Create an object whose properties will represent the resource URIs of
         * the primary resources that match the primary resource-based query.
         */
        _createFilteredPrimaryResourcesObject: function(primaryResourceQueryDoc) {
            var filteredPrimaryResourcesObject = new Object;
            var entries = primaryResourceQueryDoc.getElementsByTagName("entry");
            for (var i = 0; i < entries.length; i++) {
                var entry = entries[i];
                var contentNode = entry.getElementsByTagName("content")[0];
                var resourceURI = contentNode.getAttribute("src");
                filteredPrimaryResourcesObject[resourceURI] = true;
            }
            return filteredPrimaryResourcesObject;
        },

        /*
         * Callback for a successful resource query.
         */
        _handleResourceQuerySuccessResponse: function(response) {
            this._decrementOutstandingCountAndShowResults(response);
        },

        /*
         * Callback for an unsuccessful resource query.
         */
        _handleResourceQueryErrorResponse: function(error) {
            com.ibm.rdm.web.search.searchutils.reportError(
                dojo.string.substitute(this._messages.errorAttemptingQuery,[(error.description || error)])
            );
            this._decrementOutstandingCountAndShowResultsIfAppropriate();
        },

        /*
         * Callback for a successful primary resources query.
         */
        _handlePrimaryResourceQuerySuccessResponse: function(response) {
            this._primaryResourceQueryDoc = response;
            this._decrementOutstandingCountAndShowResultsIfAppropriate();
        },

        /*
         * Callback for an unsuccessful primary resources query.
         */
        _handlePrimaryResourceQueryErrorResponse: function(error) {
            com.ibm.rdm.web.search.searchutils.reportError(
                dojo.string.substitute(this._messages.errorAttemptingQuery,[(error.description || error)])
            );
            this._decrementOutstandingCountAndShowResultsIfAppropriate();
        },

        /*
         * Callback for a successful secondary resources query.
         */
        _handleSecondaryResourceQuerySuccessResponse: function(response) {
            this._secondaryResourceQueryDoc = response;
            this._decrementOutstandingCountAndShowResultsIfAppropriate();
        },

        /*
         * Callback for an unsuccessful secondary resources query.
         */
        _handleSecondaryResourceQueryErrorResponse: function(error) {
            com.ibm.rdm.web.search.searchutils.reportError(
                dojo.string.substitute(this._messages.errorAttemptingQuery,[(error.description || error)])
            );
            this._decrementOutstandingCountAndShowResultsIfAppropriate();
        },

        _composeResourceURIQueryParam: function(userQueryString) {
            // Default to finding everything if the user doesn't specify a
            // resource path string ...
            if (userQueryString == "") {
                userQueryString = "*";
            }
            return ("resource-uri=" + userQueryString);
        }
    }
);

})();
