/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.QuickSearch");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Menu");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");

dojo.requireLocalization("com.ibm.rdm.web.search", "common");

/*
   Copy from Jazz Web UI (pre 1.0 release). Purpose: To provide Search control
   similar to the one shown in the Jazz Web UI, but more simplified (in that it
   doesn't show a QueryType control [for selecting "All Work Items" or "All
   Unresolved Work Items"] and also doesn't perform auto-completion in the
   text area [i.e. we don't use a SearchResultSelector]).
*/

/**
 * Note by SPC: This class is based on the following class from Jazz's Web UI
 * code:
 *     com.ibm.team.workitem.web.ui.internal.view.query.Search
 * Rather than remove code, I've commented pieces that don't apply to us and
 * I've added comments to those areas where I've added code ...
 * I took this approach as we MAY wish to eventually make use of the
 * SearchResultSelector class that Jazz uses to enable selecting from drop
 * down list.  Leaving this wiring in place [but commented out] should enable
 * us to more easily adopt SearchResultSelector at a later date if we decide
 * we also want to make use of a drop down list.
 * I've also replaced 'templateString' with a 'templatePath' to suite our
 * needs.
 */

dojo.declare("com.ibm.rdm.web.search.QuickSearch", [dijit._Widget, dijit._Templated], {
    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "search/templates/QuickSearch.html"),
    widgetsInTemplate: false,
//     templateString: "<div class=\"com-ibm-team-repository-web-ui-widget-Search\">	<form dojoAttachPoint=\"_searchForm\" class=\"SearchForm\">		<a class=\"QueryType\" dojoAttachPoint=\"_queryType\" href=\"#\"></a>		<div class=\"SearchWrapper\">			<div class=\"SearchInputWrapper\">				<div class=\"SearchInnerInputWrapper\">					<input type=\"text\" class=\"SearchInputText\" dojoAttachEvent=\"onkeyup: _handleSearchInputKeystroke, onblur:_handleSearchInputBlur\"dojoAttachPoint=\"_searchText\" title=\"Search\" onfocus=\"this.select()\"></input>				</div>			</div>			<button class=\"SearchInputSubmit\" dojoAttachPoint=\"_searchButton\" title=\"Search\"></button>		</div>	</form>	<div class=\"bottomRightCorner\"></div></div>",
    actionHandler: null,
    _messages: null,
//    ALL_OPEN_WORK_ITEMS_LABEL: "All Open Work Items",
//    ALL_WORK_ITEMS_LABEL: "All Work Items",
//    DROP_DOWN_ICON_CLASS: "workItemsIcon",
    ACTIVE_SEARCH_ICON_CLASS: "search-button-active",
    postCreate: function() {
//         checkNotNull(this.actionHandler, "this.actionHandler");
        this.connect(this._searchText, "onkeyup", this.checkEmpty);
        this.connect(this._searchButton, "onclick", this._handleSearch);
        this.connect(this._searchForm, "onsubmit", this._handleSearch);
//         this.connect(this._queryType,"onclick", this._openMenu);
        this.connect(this._searchText,"onkeypress",this._sniffKeySubmit);
//         this.connect(this._queryType,"onkeypress",this._sniffKey);
        // SPC Adding these callbacks (used to be inline in template) ...
        this.connect(this._searchText, "onkeyup", this._handleSearchInputKeystroke);
        this.connect(this._searchText, "onblur", this._handleSearchInputBlur);
        this.connect(this._searchText, "onfocus", this._handleFocus);
//        this.connect(this._advancedAnchor, "onclick", this._handleAdvancedClick);
        // ... end of SPC added callbacks.
        this._prevSearchValue = "";
        var self = this;
//         this._searchResultSelector=  new com.ibm.team.workitem.web.ui.internal.view.query.SearchResultSelector( {    
//                 _tagNode: this._searchText,
//                 parentWidget: this,
//                 _parentNode: this.domNode,
//                 _callback: function(str) {
//                     self._updateSearchText(str);
//                 }
//             });
        this.openOnly = false;
//         this._queryType.title = "Search " + this.ALL_WORK_ITEMS_LABEL;
//         this._searchText.title = "Search " + this.ALL_WORK_ITEMS_LABEL;
        this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search", "common");
        this._searchText.title = this._messages.searchArtifacts;
        this._searchButton.title = this._messages.search;
        dojo.addClass(this._searchButton, this.ACTIVE_SEARCH_ICON_CLASS);
    },
    checkEmpty: function() {
//        if (this._searchText.value.length < 1) {
//            dojo.removeClass(this._searchButton, this.ACTIVE_SEARCH_ICON_CLASS);
//        } else {
            dojo.addClass(this._searchButton, this.ACTIVE_SEARCH_ICON_CLASS);
//        }
    },
    clear: function() {
        this._searchText.value= "";
    },
    focus: function() {
        this._searchText.focus();
    },
    _updateSearchText: function(str) {
        if(str.id && str.id  > 0) {
            this._searchText.value = str.id;
        }
    },
    _handleSearch: function(e) {
        var trimmedStr= dojo.trim(this._searchText.value);
        var searchIdStr = com.ibm.rdm.web.util.queryutils.asSearchId(trimmedStr);
        if (searchIdStr != null) {
            if (e) {
                e.preventDefault();    
            }
			var resourceURI = com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES + '/' + searchIdStr;			
        	// see if the artifact exists
        	var headDeferred =
				com.ibm.rdm.web.server.client.RDMClient.doHead({
					uri: resourceURI,
					skipErrors: true
				});
			headDeferred.addCallback(this, function(response){
        		var href = com.ibm.rdm.web.common.UriHelper.getResourceJafLink(resourceURI);
				window.location.href = href;
				return response;
			});
			headDeferred.addErrback(this, function(response){
            	this.actionHandler(trimmedStr, this.openOnly);
				return response;
			});
        } else if (trimmedStr.length === 0) {
            this._handleAdvancedClick(e);
        } else {
            if (e) {
                e.preventDefault();    
            }
            this.actionHandler(trimmedStr, this.openOnly);
        }
    },
//     _sniffKey: function(e){
//         if (e){
//             dojo.stopEvent(e);
//         }
//         if (e.keyCode == dojo.keys.DOWN_ARROW || e.keyCode == dojo.keys.ENTER) this._openMenu();
//     },
    _sniffKeySubmit: function(e){
        if (e.keyCode == dojo.keys.ENTER) {
			this._handleSearch(e);
		}
//         this._checkSearchSelector(e);
    },
//     _checkSearchSelector: function(e) {
//         if(!e) return;
//         if(!this._searchResultSelector.isShowingNow && e.ctrlKey && e.charCode == 32) {
//             this._searchResultSelector.showMatchingResults(this._searchText.value);
//             return;
//         }
//         this._searchResultSelector.enableDisplay(e.keyCode != dojo.keys.ENTER);    
//         if (!this._searchResultSelector.isShowingNow) { return; }
//         switch(e.keyCode) {
//         case 9:
//         case 13:
//             this._searchResultSelector.selectCurrentValue();
//             e.stopPropagation();
//             break;
//         case 40:
//             this._searchResultSelector.highlightNextTag("down");
//             break;
//         case 38:
//             this._searchResultSelector.highlightNextTag("up");
//             break;
//         case 27: // esc
//             this._searchResultSelector.close();
//         }
//         this._searchText.value = this._searchText.value.replace(/\n/g, " ");
//         this._searchText.value = this._searchText.value.replace(/\r/g, " ");
//         this._searchText.value = this._searchText.value.replace(/  /g, " ");
//     },
    _cleanUpSearchText: function(str) {
        return str;
    },
    _handleSearchInputKeystroke: function(e)  {
        var orig= this._prevSearchValue;
        var newVal= this._cleanUpSearchText(this._searchText.value);
        var orig = this._cleanUpSearchText(this._prevSearchValue);    
        var maxLen= Math.max(orig.length, newVal.length);
        for (var i=0; i<maxLen; i++) {
            if (orig.charAt(i) !== newVal.charAt(i)) {
                var searchTerm= dojo.trim(newVal);
//                 if (searchTerm) {
//                     this._searchResultSelector.showMatchingResults(searchTerm);
//                 } else {
//                     this._searchResultSelector.close();
//                 }
                break;
            }
        }
//         if (dojo.trim(this._searchText.value) === "") {
//             this._searchResultSelector.close();
//         }
        this._prevSearchValue= this._searchText.value;
    },
    _handleSearchInputBlur: function(e) {
//         if(!this._searchResultSelector.isMouseOver) {
//             this._searchResultSelector.close();                
//         }
        this._searchText.value = this._cleanUpSearchText(this._searchText.value);
    },
//     _openMenu: function(e) {
//         this._searchResultSelector.close();
//         if (e){
//             dojo.stopEvent(e);
//         }
//         if (this._menu) {
//             this._closeMenu();
//             return;
//         }
//         if (this.openOnly) {
//             var allIcon = this.DROP_DOWN_ICON_CLASS;
//             var allOpenIcon = this.DROP_DOWN_ICON_CLASS+"-selected";
//             this._queryType.title = "Search " + this.ALL_WORK_ITEMS_LABEL;
//             this._searchText.title = "Search " + this.ALL_WORK_ITEMS_LABEL;
//         } else {
//             var allIcon = this.DROP_DOWN_ICON_CLASS+"-selected";
//             var allOpenIcon = this.DROP_DOWN_ICON_CLASS;
//             this._queryType.title = "Search " + this.ALL_OPEN_WORK_ITEMS_LABEL;
//             this._searchText.title = "Search " + this.ALL_OPEN_WORK_ITEMS_LABEL;
//         }
//         var menu = new dijit.Menu();         
//         menu.addChild(new dijit.MenuItem({label: this.ALL_WORK_ITEMS_LABEL, iconClass: allIcon, onClick: dojo.hitch(this,this._searchAllWorkItems)}));
//         menu.addChild(new dijit.MenuItem({label: this.ALL_OPEN_WORK_ITEMS_LABEL, iconClass: allOpenIcon, onClick: dojo.hitch(this,this._searchAllOpenWorkItems)}));
//         var pos = dojo.coords(this._searchForm);
//         pos.x += document.documentElement.scrollLeft;
//         pos.y += document.documentElement.scrollTop;
//         dojo.addClass(this._queryType,"QueryType-selected");
//         dijit.popup.open({ popup: menu, parent: this, x: pos.x + 5, y: pos.y + pos.h - 2, onExecute: dojo.hitch(this,this._closeMenu),orient: 'BL' });
//         this._menuCloseOnBody = this.connect(this,"_onBlur",this._closeMenu);
//         this._menu = menu;
//         menu.focus();
//     },
    _closeMenu: function(e){
        if (e) {
            dojo.stopEvent(e);
        }
        dojo.removeClass(this._queryType,"QueryType-selected");
        this.disconnect(this._menuCloseOnBody);
        dijit.popup.close(this._menu);
        this._menu = null;
        this._searchText.focus();
    },
    _searchAllWorkItems: function() {
        this.openOnly = false;
    },
    _searchAllOpenWorkItems: function() {
        this.openOnly = true;
    },
    /* SPC Added ... */
    _handleAdvancedClick: function(event) {
        event.preventDefault();
        event.stopPropagation();
        var trimmedStr = dojo.trim(this._searchText.value);
        window.location.href =
            "#action=com.ibm.rdm.web.pages.showSearch"
            + "&showAdvancedSearch=true"
            + "&searchText=" + trimmedStr;
    },
    /* SPC Added ... */
    _handleFocus: function(event) {
        event.preventDefault();
        event.stopPropagation();
        this._searchText.select();
    }
});

})();
