/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007,2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.results.CommentEntryTable");

dojo.require("dojo.date.stamp");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rdm.web.search.domutils");
dojo.require("com.ibm.rdm.web.search.results.ResourceInfo");
dojo.require("com.ibm.rdm.web.search.searchutils");
dojo.require("com.ibm.rdm.web.search.SearchCommentsEventTopics");
dojo.require("com.ibm.rdm.web.search.SearchEventConstants");
dojo.requireLocalization("com.ibm.rdm.web.search.results", "CommentEntryTable");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.commenting.utils");
dojo.require("dojo.i18n");
dojo.require("dijit.Tooltip");
dojo.require("dijit.layout.ContentPane");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.util.domutils");

var UriHelper = com.ibm.rdm.web.common.UriHelper;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;

var MAX_TITLE_LENGTH = 30;

dojo.declare(
    "com.ibm.rdm.web.search.results.CommentEntryTable",
    [dijit._Widget, dijit._Templated],
    {
        templatePath: dojo.moduleUrl("com.ibm.rdm.web", "search/results/templates/CommentEntryTable.html"),
        widgetsInTemplate: true,

        ///////////////////////////////////////////////////////////////////////////////////

        _projectDBCommentDiv: null,
        _commentBylineDiv: null,
        _commentSubjectDiv: null,
        _commentTextDiv: null,
        _commentButtonDiv: null,
        _commentMoreLink: null,
        _commentArtifactInfoDiv: null,
        _replyToAnchor: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Public fields

        comment: null,
        recentCommentResults: null,
        artifactName: null,
        artifactContentType: null,
        artifactReviewRevision: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        _locationId: null,
        _isRCP: false,
        _isTextTruncated: null,
        _messages: null,
        _moreHover: null,
        _userHovers: [],

        ///////////////////////////////////////////////////////////////////////////////////
        // API

        postCreate: function() {
    		this._createEntry();
        },
        
   		postMixInProperties: function() {
            this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search.results", "CommentEntryTable");
   		},
   		
    	destroy: function() {
    		if (this._moreHover) {
    			this._moreHover.destroy();
    			this._moreHover = null;
    		}
    		for (var i=0; i<this._userHovers.length; i++) {
    			this._userHovers[i].destroy();
    		}
    		this._userHovers = [];
        	this.inherited(arguments);
    	},
       
        _createEntry: function() {
            this._userHovers = com.ibm.rdm.web.commenting.utils.createCommentByline(this, this._commentBylineDiv, this.comment, true);
            dojo.addClass(this._commentBylineDiv, "comment-indent-div");
            com.ibm.rdm.web.commenting.utils.createTitleNode(this, this._commentSubjectDiv, this.comment);
            dojo.addClass(this._commentSubjectDiv, "comment-indent-div");
        	this._isTextTruncated = com.ibm.rdm.web.commenting.utils.setCommentText(this, this._commentTextDiv, this.comment);
            dojo.addClass(this._commentTextDiv, "comment-indent-div");
            dojo.addClass(this._commentMoreDiv, "comment-indent-div");
            dojo.addClass(this._commentButtonDiv, "comment-indent-div");
            if (this.artifactReviewRevision) {
				com.ibm.rdm.web.commenting.utils.createArtifactInReviewNode(this, this._commentArtifactInfoDiv, this.artifactName, this.artifactContentType, this.artifactReviewRevision, this.comment, true);
            } else {
				com.ibm.rdm.web.commenting.utils.createArtifactNode(this, this._commentArtifactInfoDiv, this.artifactName, this.artifactContentType, this.comment, true);
			}
            dojo.addClass(this._commentArtifactInfoDiv, "comment-indent-div");
            dojo.addClass(this._replyToAnchor, "hideElement");
        },
        
        addToolTip: function(){
       		if (this._isTextTruncated){
				this._moreHover = com.ibm.rdm.web.commenting.utils.hookupRecentCommentMoreLink(this, this._commentMoreLink, this.comment, this.artifactName, this.artifactContentType);
			} else {
       			this._commentMoreLink.innerHTML = "";
       			dojo.addClass(this._commentMoreDiv, "comment-no-more-div");
			};
        },
        
        _replyToClicked: function(event) {
			if (dojo.hasClass(this._replyToAnchor, "enabled")) {
    			event.preventDefault();
				event.stopPropagation();
				com.ibm.rdm.web.commenting.utils.replyToComment(this.comment);
			}
        },

        ///////////////////////////////////////////////////////////////////////////////////
        // Helpers
        
        _handleResourceContextClicked: function (event) {
        	ResourceLinksHandler.handleAnchorContextMenuClicked(event);
        },
        
        _getParentPathIdForHref: function(href) {
            return (href.substring(0, href.lastIndexOf("/")));
        },

        /**
         * Send the status message to the embedded web browser inside the rich client. 
         * It always returns false to avoid undesired action from the web browser.
         */
        _sendStatus: function(event) {
            /*event.preventDefault();
            event.stopPropagation();
            var href = (dojo.isIE ? event.srcElement.href : event.target.href);
            var params = {};
            params[com.ibm.rdm.web.search.SearchEventConstants.JSEVENT_RESOURCE_ID] = this._getParentPathIdForHref(href);
            params[com.ibm.rdm.web.search.SearchEventConstants.JSEVENT_LOCATION_ID] = this._locationId;
            com.ibm.rdm.web.search.searchutils.triggerEvent(
                com.ibm.rdm.web.search.SearchCommentsEventTopics.COMMENT_SELECTED_TOPIC,
                params
            );
            return false;*/
        }
    }
);

})();
