/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.results.DetailsTable");

dojo.require("dojo.date.stamp");
dojo.require("dojo.date.locale");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Tooltip");
dojo.require("com.ibm.rdm.web.search.searchutils");
dojo.require("com.ibm.rdm.web.search.domutils");
dojo.require("com.ibm.rdm.web.search.DocumentTypes");
dojo.require("com.ibm.rdm.web.search.results.ResourceInfo");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.requireLocalization("com.ibm.rdm.web.search.results", "DetailsTable");

var ProjectCache = com.ibm.rdm.web.common.ProjectCache;


/**
 * The DetailsTable is used to present the table data cell value found on the
 * right hand side when viewing verbose search results.
 */
dojo.declare(
    "com.ibm.rdm.web.search.results.DetailsTable",
    [dijit._Widget, dijit._Templated],
    {
        templatePath: dojo.moduleUrl("com.ibm.rdm.web", "search/results/templates/DetailsTable.html"),
        widgetsInTemplate: true,

        ///////////////////////////////////////////////////////////////////////////////////
        // Attach points

        _lastUpdatedSpan: null,
        _lastModifiedBySpan: null,
        _docTypesLabelSpan: null,
        _table: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Public fields

        resourceInfo: null,
        documentTypes: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        _messages: null,
        // E.g. this._docTypeProperties["/jazz/resources/rdm/S1:http://com.ibm.rdm/COMMENT"] = {"Priority":"High", "Comment":"*"}
        _docTypeProperties: null,
            
        ///////////////////////////////////////////////////////////////////////////////////
        // NLS fields requiring translation

        lastUpdated: null,
        modifiedBy: null,
        docTypes: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // API

        postCreate: function() {
            this._docTypeProperties = new Object;
            if(this.fullTextChecked){
            	var lastModifiedDateDisplayString =  this.resourceInfo.lastModifiedDate ? 
														com.ibm.rdm.web.search.searchutils.createDateLabel(
                										this.resourceInfo.lastModifiedDate) : "";
                
            	ProjectCache.getProjectUsers(this.resourceInfo.projectURI).addCallback(this, 
					function(users) {
						for (userUri in users) {
							if(this.resourceInfo.lastModifiedBy == users[userUri].url){
								this._lastModifiedBySpan.appendChild(document.createTextNode(users[userUri].name));
							}
						}
					}
				);																	
            }
            else{
            	var lastModifiedDateDisplayString = com.ibm.rdm.web.search.searchutils.createDateLabel(
                	this.resourceInfo.lastModifiedDate);
            	
            	ProjectCache.getProjectUsers(this.resourceInfo.projectURI).addCallback(this, 
					function(users) {
						for (userUri in users) {
							if(this.resourceInfo.lastModifiedBy == users[userUri].url){
								this._lastModifiedBySpan.appendChild(document.createTextNode(users[userUri].name));
							}
						}
					}
				);	
            }

            this._lastUpdatedSpan.appendChild(document.createTextNode(lastModifiedDateDisplayString));
            this._addDocName(this.resourceInfo);
        },

        postMixInProperties: function() {
            this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search.results", "DetailsTable");
            dojo.forEach(["lastUpdated", "modifiedBy", "docTypes"], function(prop){
                this[prop] = this._messages[prop];
            }, this);
        },

        startup: function() {
            // Note: When using dynamically created tooltips, the connectId that they
            // "attach to" MUST exist within the document's DOM heirarchy.  This is why
            // we defer adding these tooltips until AFTER this table widget has been
            // created.  After constructing this widget, the caller must do an
            // appropriate appendChild using this widget's domNode and only THEN
            // should it call this startup function in order to have the tooltips get
            // correctly installed (as at this point the connectId's WILL exists within
            // the document's DOM).
           // We are using resource links...
           //this._addTooltipsIfNeeded();
        },

        /**
         * For example:
         *   getDocTypeProperty("http://com.ibm.rdm/FEAT", "Type")
         * might return:
         *   "Feature"
         */
        getDocTypeProperty: function(docType, propertyName) {
            var docTypeProps = this._docTypeProperties[this.resourceInfo.resourceURI + ":" + docType];
            return docTypeProps[propertyName];
        },

        ///////////////////////////////////////////////////////////////////////////////////
        // Helpers

        _addSecondaryResourceMatchesRowIfNeeded: function() {
            var secondaryResourceMatchesObject = this.resourceInfo.getSecondaryResourceMatchesObject();
            if (secondaryResourceMatchesObject) {
                var tbody = this._table.lastChild;
                for (prop in secondaryResourceMatchesObject) {
                    var tr = document.createElement("tr");
                    var td = document.createElement("td");
                    tr.appendChild(td);
                    tbody.appendChild(tr);
                    var label = dojo.string.substitute(this._messages.propertyLabel, [prop]);
                    td.appendChild(com.ibm.rdm.web.search.domutils.createSpannedTextNode(
                        "search-result-entry-detail", label));
                    var values = secondaryResourceMatchesObject[prop];
                    com.ibm.rdm.web.search.domutils.appendSecondaryResourceMatches(
                        td, values);
                }
            }
        },

        _addTooltipsIfNeeded: function() {
            for (var prop in this._docTypeProperties) {
                var docTypeSpanId = prop;
                var td = this._docTypesLabelSpan.parentNode;
                var div = document.createElement("div");
                td.appendChild(div);
                var docTypePropsObject = this._docTypeProperties[docTypeSpanId];
                div.appendChild(this._createPropertyTable(docTypePropsObject))
                var tooltip = new dijit.Tooltip({connectId:[docTypeSpanId]}, div);
                tooltip.startup();
            }
        },

        _createPropertyTable: function(propertiesObject) {
            var table = document.createElement("table");
            var propArray = [];
            for (var prop in propertiesObject) {
                propArray.push(prop);
            }
            propArray.sort();
            for (var i = 0; i < propArray.length; i++) {
                var name = propArray[i];
                var value = propertiesObject[name];
                var nameNode = com.ibm.rdm.web.search.domutils.createSpannedTextNode(
                    "search-result-tooltip-doc-type-attribute-name", name + ":");
                var valueNode = com.ibm.rdm.web.search.domutils.createSpannedTextNode(
                    "search-result-tooltip-doc-type-attribute-value", value);
                table.appendChild(com.ibm.rdm.web.search.domutils.createTR([
                    com.ibm.rdm.web.search.domutils.createTD(nameNode),
                    com.ibm.rdm.web.search.domutils.createTD(valueNode)
                ]));
            }
            return table;
        },
        
		_addDocName: function(resourceInfo) {
            var td = this._docTypesLabelSpan.parentNode;

            // Add a space char first (to separate the doc types).
            var space = document.createTextNode(" ");
            td.appendChild(space);

            // Add a span for the the doc type.
            var docTypeSpan = com.ibm.rdm.web.search.domutils.createSpannedTextNode(
                "search-result-doc-type-link", resourceInfo.displayName);
            // Note: The "id" for the span has GOT to be unique ... as a tooltip will be
            // hooked up (during startup) using this id as it's connectId.
            docTypeSpan.id = resourceInfo.resourceURI + ":" + resourceInfo.contentType;
  
            td.appendChild(docTypeSpan);
        },

        _addDocType: function(docType, propertiesObject) {
            var td = this._docTypesLabelSpan.parentNode;
            var docTypeShortNameIndex = docType.lastIndexOf("/")+1;
            var docTypeShortName = docType.substring(docTypeShortNameIndex);

            // Add a space char first (to separate the doc types).
            var space = document.createTextNode(" ");
            td.appendChild(space);

            // Add a span for the the doc type.
            var docTypeSpan = com.ibm.rdm.web.search.domutils.createSpannedTextNode(
                "search-result-doc-type-link", docTypeShortName);
            // Note: The "id" for the span has GOT to be unique ... as a tooltip will be
            // hooked up (during startup) using this id as it's connectId.
            docTypeSpan.id = this.resourceInfo.resourceURI + ":" + docType;
            var tooltipProps = new Object;
            this._docTypeProperties[docTypeSpan.id] = tooltipProps;
            td.appendChild(docTypeSpan);
            for (var prop in propertiesObject) {
                var name = prop;
                var value = propertiesObject[name];
                tooltipProps[name] = value;
            }
        }
    }
);

})();
