/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.results.RecentCommentResults");
  
dojo.require("dijit._base");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.Toolbar");
dojo.require("dojo.date");
dojo.require("dojo.date.stamp");
dojo.require("com.ibm.rdm.web.commenting.utils");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.search.results.CommentEntryTable");
dojo.require("com.ibm.rdm.web.search.results.UserDashboardCommentEntry");
dojo.require("com.ibm.rdm.web.search.results.RecentCommentsOptions");
dojo.require("com.ibm.rdm.web.search.results.RecentResourceResults");
dojo.require("com.ibm.rdm.web.search.results.RecentResourcesHeader");
dojo.require("com.ibm.rdm.web.search.results.TimeWindowSection");
dojo.require("com.ibm.rdm.web.search.searchutils");
dojo.require("com.ibm.rdm.web.commenting.Comment");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.UsersSelectWidget");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.server.client.RecentFeedsClient");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.server.XhrUtils");

dojo.requireLocalization("com.ibm.rdm.web.search.results", "RecentResourceResults");

var PAGE_SIZE = 7;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var RecentFeedsClient = com.ibm.rdm.web.server.client.RecentFeedsClient;
var domutils = com.ibm.rdm.web.util.domutils;
var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var XhrUtils = com.ibm.rdm.web.server.XhrUtils;

/**
 * Shows a presentation appropriate for "recent comment activity".  This
 * presentation is time ordered, most to least recent, and uses "time window
 * sections" to appropriately group resources that are older than today.
 */
dojo.declare(
    "com.ibm.rdm.web.search.results.RecentCommentResults",
    com.ibm.rdm.web.search.results.RecentResourceResults,
    {
        widgetsInTemplate: true,
        ///////////////////////////////////////////////////////////////////////////////////
        // Public fields

        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        _messages: null,
        _maxDayDeltaThreshold: 7,
        _encodedCommentContentType: com.ibm.rdm.web.search.searchutils.encodeContentType(com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_COMMENT),
        _mouseOverHandle: null,
        _mouseOutHandle: null,

        /* This attr is used to specify an appropriate resource-modified-since value.
           If set to 1, it means look back as far as yesterday.
           If set to 7, it means look back as far as last week.
           If set to -1, it means DO NOT use a resource-modified-since query param.
           */
        _showResolved: null,
        _byUser: null,
        _forUser: null,
        _priority: null,
        _numHidden: 0,

        _headerToolbar: null,
    	_activeFilter: null,
    	_comments: null,
    	_filteredComments: null,
    	_artifactInfos: null,
        _deferreds: null,

        ///////////////////////////////////////////////////////////////////////////////////

    	constructor: function(params) {
        	this.viewType = params.viewType;
        	this.displayContext = params.displayContext;
        	this._viewlet = params._viewlet;
        	this._deferreds = [];
            this._activeFilter = com.ibm.rdm.web.search.results.RecentCommentResults.filterBy.MOST_RECENT_COMMENTS;
    	},
  	
        startup: function() {
            this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search.results", "RecentResourceResults");
            if (this.displayContext == com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.USER_DASHBOARD) {
            	//this._headerImageNode.src = "com.ibm.rdm.web/common/images/recent-comments.gif"
            }
            this.inherited("startup", arguments);
	        this._headerTitle = dojo.string.substitute(this._messages.showingComments, [""]);
            if (this._viewlet) {
				this._viewlet.getSite().setTitle(this._headerTitle);
			}
            dojo.addClass(this._searchResultsContent.parentNode, "rdm-recent-comments");
            //dojo.addClass(dijit.byId(this._activeFilter).iconNode, "commentingToolbarDropdownBullet");
            this._setDefaultFilterValues();
        	if (this.displayContext == com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.USER_DASHBOARD) {
            	this._showMostRecentComments.attr('checked', true);
            }
        },

        destroy: function() {
            this._beingDestroyed = true;
            this.inherited("destroy", arguments);
            for (var i = 0; i < this._deferreds.length; i++) {
                var deferred = this._deferreds[i];
                if (deferred.fired == -1) {
                    deferred.cancel();
                }
            }
        },
        
        createResourceTableRow: function(comment,tbody) {
            // Add a new table row.
        	var resourceURI = comment.resourceURI;
            var tr = document.createElement("tr");
            tr.id = comment.resourceURI;
            if (null!=tbody){
            	tbody.appendChild(tr);
            }
            if (this.displayContext == com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.USER_DASHBOARD) {
            	var resDesc = this._artifactInfos.item(comment.resourceUri);
             	var userDBComment = null;
            	if (resDesc) {
             		userDBComment = new com.ibm.rdm.web.search.results.UserDashboardCommentEntry({
                		comment:comment,
                		artifactName: resDesc.displayName,
                		artifactContentType: resDesc.contentType
                	});
                } else {
             		userDBComment = new com.ibm.rdm.web.search.results.UserDashboardCommentEntry({
                		comment:comment,
                		artifactName: "",
                		artifactContentType: null
                	});
                }
                var tableColumn = com.ibm.rdm.web.search.domutils.createTD(userDBComment.domNode);
            	tr.appendChild(tableColumn);
				this.connect(tableColumn, "onmouseover", function(event) {
					this._onCommentMouseOver(event, tableColumn, userDBComment);
				});
                this.connect(tableColumn, "onmouseout", function(event) {
					this._onCommentMouseOut(event, tableColumn, userDBComment);
				});
    			this.connect(tableColumn, "onclick",  function(event) {
              		this._onUserDBCommentClicked(event, userDBComment)});
            	userDBComment.startup();
            	userDBComment.addToolTip();
            	this._entryTables.push(userDBComment);
            } else {
            	var resDesc = this._artifactInfos.item(comment.resourceUri);
            	var entryTable = null;
            	if (resDesc) {
             		entryTable = new com.ibm.rdm.web.search.results.CommentEntryTable(
                		{
                			comment:comment, 
                			artifactName: resDesc.displayName,
                			artifactContentType: resDesc.contentType
                		}
                	, null);
                } else {
             		entryTable = new com.ibm.rdm.web.search.results.CommentEntryTable(
                		{
                			comment:comment, 
                			artifactName: "",
                			artifactContentType: null
                		}
                	, null);
                }
                var tableColumn = com.ibm.rdm.web.search.domutils.createTD(entryTable.domNode);
            	tr.appendChild(tableColumn);
				this.connect(tableColumn, "onmouseover", function(event) {
					this._onCommentMouseOver(event, tableColumn, entryTable);
				});
                this.connect(tableColumn, "onmouseout", function(event) {
					this._onCommentMouseOut(event, tableColumn, entryTable);
				});
            	entryTable.startup();
            	entryTable.addToolTip();
            	this._entryTables.push(entryTable);
            }            
            
            return tr;
        },
        
        applyOptions: function(showResolved, forUser, byUser, priority) {
            this._showResolved = showResolved;
            this._forUser = forUser;
            this._byUser = byUser;
            this._priority = priority;
            var entryTables = this._entryTables;
            for (var i = 0; i < entryTables.length; i++) {
                var entryTable = entryTables[i];
                var isVisible = this.shouldResourceBeVisible(entryTable.comment);
                this._setWidgetGrandparentVisibility(entryTable, isVisible);
            }
            this._updateFilteredHeader();
        },
        
        _onCommentMouseOver: function(event, tableColumn, entry) {
        	if (event) {
        		event.preventDefault();
    			event.stopPropagation();
        	}
            dojo.addClass(tableColumn, "highlight");
            dojo.removeClass(entry._replyToAnchor, "hideElement");
            dojo.addClass(entry._replyToAnchor, "showElement"); 
            if (!entry.comment.isElementDeleted && this._canComment(entry.comment)) {
    			dojo.addClass(entry._replyToAnchor, "enabled");
    			dojo.removeClass(entry._replyToAnchor, "disabled");
            } else {
    			dojo.removeClass(entry._replyToAnchor, "enabled");
    			dojo.addClass(entry._replyToAnchor, "disabled");
            }  
        },

        _onCommentMouseOut: function(event, tableColumn, entry) {
        	if (event) {
        		event.preventDefault();
    			event.stopPropagation();
        	}
            dojo.removeClass(tableColumn, "highlight");
            dojo.removeClass(entry._replyToAnchor, "showElement");
            dojo.addClass(entry._replyToAnchor, "hideElement");   
        },
        
        _onUserDBCommentClicked: function(event, comment) {
            comment.expandCollapse();
        },
        
        _setDefaultFilterValues: function() {
            this._showResolved = true;
            this._byUser = com.ibm.rdm.web.common.UsersSelectWidgetUNSPECIFIED;
            this._forUser = com.ibm.rdm.web.common.UsersSelectWidgetUNSPECIFIED;
            this._priority = com.ibm.rdm.web.commenting.PRIORITY_UNSPECIFIED;
        },
        
        _updateFilteredHeader: function() {
            this._numHidden = 0;
            for (var i = 0; i < this._timeWindowSections.length; i++) {
                var section = this._timeWindowSections[i];
                var numTotal = section.getNumberOfResources();
                var numHidden = section.getTallyOfResourcesThatWouldBeHiddenViaFilters();
                var numShown = numTotal - numHidden;
                this._numHidden += numHidden;
                var title = null;
                if (numShown == numTotal) {
                	if (numShown == 0) {
                        title = dojo.string.substitute(this._messages.noneShowing, [section.sectionTitle]);
                	} else {	
                		title = dojo.string.substitute(this._messages.nShowing, [section.sectionTitle, numTotal]);
                	}
                } else {
                    title = dojo.string.substitute(this._messages.nOfMShowing, [section.sectionTitle, numShown, numTotal]);
                }
                section.setTitle(title);
            }
            this._updateHeaderNumberShownInfo();
        },
        
        shouldResourceBeVisible: function(comment) {
            var resolvedMatched = true;
            if (!this._showResolved) {
            	resolvedMatched = !comment.isResolved;
            }
            var priorityMatched = this._priority == comment.priority || this._priority==com.ibm.rdm.web.commenting.PRIORITY_UNSPECIFIED;
         	var forMatched = this._forUser == com.ibm.rdm.web.common.UsersSelectWidgetUNSPECIFIED || comment.isDirectedTo(this._forUser);
         	var byMatched = this._byUser == comment.author || this._byUser == com.ibm.rdm.web.common.UsersSelectWidgetUNSPECIFIED;
            return resolvedMatched && priorityMatched && forMatched && byMatched;
        },
        
        handleRefresh: function() {
            // summary:
            //    This function is called whenever the user clicks on the "global
            //    refresh" button.
        	if (!this._isLoading) {
        		this._isLoading = true;
	        	if (this._header) {
	        		this._header.informUserThatSearchIsInProgress();
	        	}
	            this._clearLastResultsIfNeeded();
	        	//this._showLoadingMessage();
	        	domutils.createHourGlassCursor();
	        	var recentCommentsDeferred;
	        	if (this.scopingProjectURIs && this.scopingProjectURIs.length > 0) {
	        		recentCommentsDeferred = 
	        			RecentFeedsClient.getRecentComments(this.scopingProjectURIs[0], com.ibm.rdm.web.search.results.RecentCommentResults.dateComparator);
	        		//remove scoping ProjectURIs
	        	} else {
	        		recentCommentsDeferred = RecentFeedsClient.getRecentComments(null, com.ibm.rdm.web.search.results.RecentCommentResults.dateComparator);
	        	}
	        	recentCommentsDeferred.addCallback(this, this._handlePerformQuerySuccessResponse);
	        	if (this.scopingProjectURIs && this.scopingProjectURIs.length > 0) {
	        		recentCommentsDeferred.addErrback(this, function(response) {
 						if (XhrUtils.getResponseCode(response) == 404) {
 							// TODO (LPK) ignore bogus 404 error from server
 							this._handlePerformQuerySuccessResponse([]);
 						} else {
							var errorSectionWidget = ErrorHandler.getErrorSectionWidget();
							if (errorSectionWidget) {
								errorSectionWidget.showErrorMessage(response);
							}
 						}
 						return response;
 					});
	        	} 
        	}
        },

        ///////////////////////////////////////////////////////////////////////////////////
        // Helpers

		_canComment: function(comment) {
			var projectURI = null;
	        if (this.scopingProjectURIs && this.scopingProjectURIs.length > 0) {
	        	var projectURI = this.scopingProjectURIs[0];
	        } else {
	        	var projectURI = comment.projectURI;
	        }
	        if (projectURI == null) return false;
			var projectOperations = com.ibm.rdm.web.AUTHENTICATED_USER.permissions.item(projectURI);	
			return isAllowed = projectOperations.item(com.ibm.rdm.web.common.ProjectCache.MODIFY_COMMENT);	
		},
		
        _clearLastResultsIfNeeded: function() {
        	this._destroyWidgets();
            this.inherited("_clearLastResultsIfNeeded", arguments);
            this._numHidden = 0;
        },

        _updateHeaderNumberShownInfo: function() {
            // Note: The superclass calls this in displayQueryResults but that's
            // too early for us as we have yet to have the chance to (if
            // appropriate) omit the resolved comments.
            var numShown = (this._comments.length - this._numHidden);
            if (this._header) {
            	this._header.updateNumberShownInfo(numShown, this._comments.length);
            }
        },

        _decrementOutstandingRequestCount: function() {
            this.inherited("_decrementOutstandingRequestCount", arguments);
            if (this._numOutstandingPropertyRequests == 0) {
                this._updateHeaderNumberShownInfo();
            }
        },

        _doExcludeEntry: function(atomEntryNode) {
            return false;
        },

        _getTextValue: function(node, attr) {
            try {
                var value = node.getElementsByTagName(attr).item(0).firstChild.nodeValue;
                return value;
            } catch (e) {
                return "";
            }
        },

        _createHeader: function() {
            var optionsSection = new com.ibm.rdm.web.search.results.RecentCommentsOptions(
                {
                    recentCommentResults: this
                },
                null
            );
            return (new com.ibm.rdm.web.search.results.RecentResourcesHeader(
                {
                    recentResourceResults:this,
                    optionsSection:optionsSection
                },
                null
            ));
        },

        /*
         * Callback for a successful query.
         */
        _handlePerformQuerySuccessResponse: function(comments) {
        	var self = this;
        	this._comments = comments;
        	this._filterComments();
        	// now perform another query to get the artifact info for the results
        	// gather the artifact URIs referenced by the comments
        	this._artifactInfos = new dojox.collections.Dictionary();
            for (i = 0; i < this._comments.length; i++) {
            	this._artifactInfos.add(this._comments[i].resourceUri, null);
            }            	
        	var onArtifactsGetInfo = function(resourceDescriptions) {
				for (resourceURI in resourceDescriptions) {
					self._artifactInfos.add(resourceURI, resourceDescriptions[resourceURI]);
				}
            	com.ibm.rdm.web.search.searchutils.showAsBusy(false);
            	domutils.createDefaultCursor();
        		self._hideLoadingMessage();
            	self._showRecentCommentResults();
            	self._isLoading = false;
			};
        	var onArtifactsGetInfoError = (this, function(error) {
            	com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorNumber(
            		7212, [], error
    			);
        	});
        	if (this._artifactInfos.getKeyList().length != 0) {
        		var deferred = com.ibm.rdm.web.common.ProjectCache.getBatchResourceDescriptionsMultiFetch(this._artifactInfos.getKeyList());
            	this._deferreds.push(deferred);
        		deferred.addCallback(onArtifactsGetInfo);
        		deferred.addErrback(onArtifactsGetInfoError);
        	} else {
            	com.ibm.rdm.web.search.searchutils.showAsBusy(false);
            	domutils.createDefaultCursor();
        		self._hideLoadingMessage();
            	self._showRecentCommentResults();
            	self._isLoading = false;
        	}
            //this.displayQueryResults(response);
        },
        
        /*
         * Callback for an unsuccessful query.
         */
        _handlePerformQueryErrorResponse: function(error) {
            com.ibm.rdm.web.search.searchutils.showAsBusy(false);
            domutils.createDefaultCursor();
            if (! (error.dojoType && error.dojoType == "cancel")) {
                com.ibm.rdm.web.search.searchutils.reportError(
                      dojo.string.substitute(this._messages.errorAttemptingToPerformQuery,
                          [(error.description || error)])
                );
            }
        },

        _addResourceEntriesToUserDashboard: function() {
			for(var i = this._resultsTableTbody.rows.length; i > 0;i--) {
				this._resultsTableTbody.deleteRow(i -1);
			}
            for (var i = (this._currentPage-1)*PAGE_SIZE; 
            	i < this._filteredComments.length && i < ((this._currentPage-1)*PAGE_SIZE+PAGE_SIZE); 
            	i++) {
                var comment = this._filteredComments[i];
	        	this._resultsTableTbody.appendChild(this.createResourceTableRow(comment));
            }
        },
        
        _addResourceEntriesAndTimeWindowEntries: function() {
            var todayAsDayBasedDate = this._createDayBasedDate(new Date());
            var lastResourceInfo = null;
            var timeWindowSection = null;
            var oldestDayDelta = this._maxDayDeltaThreshold;
            var i;
            var j;
            var comments=[];
            this._timeWindowSections.push(new com.ibm.rdm.web.search.results.TimeWindowSection(
                    {sectionTitle:this._messages.today, recentResourceResults:this}, null));
            this._timeWindowSections.push(new com.ibm.rdm.web.search.results.TimeWindowSection(
                    {sectionTitle:this._messages.yesterday, recentResourceResults:this}, null));
            this._timeWindowSections.push(new com.ibm.rdm.web.search.results.TimeWindowSection(
                    {sectionTitle:this._messages.pastweek, recentResourceResults:this}, null));  
            
            for (i = 0; i < this._comments.length; i++) {
                var comment = this._comments[i];
                var commentDayBasedDate = this._createDayBasedDate(comment.updated);
                var commentDayDelta = dojo.date.difference(commentDayBasedDate, todayAsDayBasedDate, "day");  
                if (commentDayDelta<oldestDayDelta){             	
                    var sectionTitle = this._getTimeWindowLabel(commentDayDelta, commentDayBasedDate.getDay());
                    for (j=0; j < this._timeWindowSections.length; j++) {
                    	timeWindowSection=this._timeWindowSections[j];
                    	if (sectionTitle==timeWindowSection.sectionTitle){
                    		timeWindowSection.addResourceInfo(comment);
                    		comments.push(comment);
                    	}
                    }
                }
            }

            this._comments=comments;
            
            for (i = 0; i < this._timeWindowSections.length; i++) {            	
                var timeWindowSection = this._timeWindowSections[i];
                timeWindowSection.startup();
                var title = null;
                var count = timeWindowSection.getNumberOfResources();
                if (count == 0) {
                    title = dojo.string.substitute(this._messages.noneShowing, [timeWindowSection.sectionTitle]);
                } else {
                    title = dojo.string.substitute(this._messages.nShowing, [timeWindowSection.sectionTitle, count]);
                }
                timeWindowSection.setTitle(title);
                this._createTimeWindowSectionTableRow(timeWindowSection);
            }
        },
        
        _showResultsInUserDashboardContext: function() {
	    	this._currentPage = 1;
            switch(this._activeFilter) {
				case com.ibm.rdm.web.search.results.RecentCommentResults.filterBy.MOST_RECENT_COMMENTS:
					var msg = dojo.string.substitute(this._messages.showingComments, [this._comments.length]);
					this._viewlet.getSite().setTitle(msg);
					break;
				case com.ibm.rdm.web.search.results.RecentCommentResults.filterBy.MY_RECENT_COMMENTS:
					var msg = dojo.string.substitute(this._messages.showingCommentsNofM, [this._filteredComments.length, this._comments.length]);
					this._viewlet.getSite().setTitle(msg);
					break;
				case com.ibm.rdm.web.search.results.RecentCommentResults.filterBy.RECENT_COMMENTS_FOR_ME:
					var msg = dojo.string.substitute(this._messages.showingCommentsNofM, [this._filteredComments.length, this._comments.length]);
					this._viewlet.getSite().setTitle(msg);
					break;
				default:
					console.error("Unrecognized recent comments filter type '" + this._activeFilter + "'");
            }
			//this._headerItemCount.innerHTML = dojo.string.substitute(this._messages.count, [this._filteredComments.length]);
	    	this._totalPages = Math.ceil(this._filteredComments.length/PAGE_SIZE);
	    	this._pageText.innerHTML = 
	        	dojo.string.substitute(this._messages.pageNOfM, [this._currentPage,this._totalPages]);
	    	if (this._totalPages == 0) {
	    		this._footer.style.visibility = "hidden";
	    	} else {
	    		this._footer.style.visibility = "";
        		if (!this._mouseOverHandle) {
        			this._mouseOverHandle = this.connect(this.domNode, "onmouseover", this._showNavOptions);
        		}
        		if (!this._mouseOutHandle) {
        			this._mouseOutHandle = this.connect(this.domNode, "onmouseout", this._hideNavOptions);
        		}
	    	}
	    	this._addResourceEntriesToUserDashboard();
        },
        
        _showResultsInSidebarContext: function() {
        	this._addResourceEntriesAndTimeWindowEntries();
            this._updateHeaderNumberShownInfo();
            this._expandFirstTimeWindowSection();
            this.applyOptions(this._showResolved, this._forUser, this._byUser, this._priority);
            if (this.displayContext
                == com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.SIDEBAR_SECTION) {
                this._resizeContentPaneIfNeeded();
            }
        },
        
        _showRecentCommentResults: function() {
        	var self = this;
            this._resultsTableTbody = document.createElement("tbody");
            this._resultsTable.appendChild(this._resultsTableTbody);
            if (this.displayContext
                == com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.USER_DASHBOARD) {
            	this._showResultsInUserDashboardContext();
            } else {
            	this._showResultsInSidebarContext();
            }
        },

		_showMostRecentCommentsClicked: function() {
            this._activeFilter = com.ibm.rdm.web.search.results.RecentCommentResults.filterBy.MOST_RECENT_COMMENTS;
            this._showRecentCommentsForMe.attr('checked', false);
            this._showMyRecentComments.attr('checked', false);
            this._filterComments();
            this._showResultsInUserDashboardContext();
		},
		
		_showMyRecentCommentsClicked: function() {
            this._activeFilter = com.ibm.rdm.web.search.results.RecentCommentResults.filterBy.MY_RECENT_COMMENTS;
            this._showRecentCommentsForMe.attr('checked', false);
            this._showMostRecentComments.attr('checked', false);
            this._filterComments();
            this._showResultsInUserDashboardContext();
		},
		       
		_showRecentCommentsForMeClicked: function() {
            this._activeFilter = com.ibm.rdm.web.search.results.RecentCommentResults.filterBy.RECENT_COMMENTS_FOR_ME;
            this._showMyRecentComments.attr('checked', false);
            this._showMostRecentComments.attr('checked', false);
            this._filterComments();
            this._showResultsInUserDashboardContext();
		},
		
        
        _filterComments: function() {
        	if (this._activeFilter == null) {
        		this._activeFilter = com.ibm.rdm.web.search.results.RecentCommentResults.filterBy.MOST_RECENT_COMMENTS;
        	}
        	this._filteredComments = new Array();
            for (var i = 0; i < this._comments.length; i++) {
            	var comment = this._comments[i];
            	switch(this._activeFilter) {
					case com.ibm.rdm.web.search.results.RecentCommentResults.filterBy.MOST_RECENT_COMMENTS:
						this._filteredComments.push(comment);
						break;
					case com.ibm.rdm.web.search.results.RecentCommentResults.filterBy.MY_RECENT_COMMENTS:
						if (comment.author == com.ibm.rdm.web.AUTHENTICATED_USER.userId) {
							this._filteredComments.push(comment);
						}
						break;
					case com.ibm.rdm.web.search.results.RecentCommentResults.filterBy.RECENT_COMMENTS_FOR_ME:
						if (comment.isDirectedTo(com.ibm.rdm.web.AUTHENTICATED_USER.userId)) {
							this._filteredComments.push(comment);
						}
						break;
					default:
						// show the comment by default
						this._filteredComments.push(comment);
	            }
            }
        }
    }
);

com.ibm.rdm.web.search.results.RecentCommentResults.dateComparator = function(comment1, comment2) {
	var nameComparator = function(comment1, comment2) {
	    var resourceName1 = comment1.title;
	    var resourceName2 = comment2.title;
	    return ((resourceName1 == resourceName2) ? 
	    		0 : ((resourceName1.toLowerCase() < resourceName2.toLowerCase() ) ? -1 : 1));
	};
	var resourceDate1 = comment1.updated;
	var resourceDate2 = comment2.updated;
	var compare = dojo.date.compare(resourceDate1, resourceDate2)
	if(compare == 0) {
		return nameComparator(comment1, comment2);
	} else {
		return compare;
	}
}

com.ibm.rdm.web.search.results.RecentCommentResults.filterBy = {
	RECENT_COMMENTS_FOR_ME: "recentForMe",
	MOST_RECENT_COMMENTS: "mostRecent",
	MY_RECENT_COMMENTS: "myRecent"
}

})();

