/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.results.RecentResourceResults");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.layout.ContentPane");
dojo.require("dojo.date");
dojo.require("dojo.date.stamp");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.search.results.RecentResourcesHeader");
dojo.require("com.ibm.rdm.web.search.results.TimeWindowSection");
dojo.require("com.ibm.rdm.web.search.searchutils");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.common.DateTimeHelper");
dojo.require("com.ibm.rdm.web.server.client.RecentFeedsClient");

dojo.requireLocalization("com.ibm.rdm.web.search.results", "RecentResourceResults");

var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var RecentFeedsClient = com.ibm.rdm.web.server.client.RecentFeedsClient;
var domutils = com.ibm.rdm.web.util.domutils;

var PAGE_SIZE = 7;


/**
 * Shows a presentation appropriate for "recent activity".  This presentation is
 * time ordered, most to least recent, and uses "time window sections" to
 * appropriately group resources that are older than today.
 *
 * Please note this class serves as a superclass for RecentCommentResults.
 */
dojo.declare(
    "com.ibm.rdm.web.search.results.RecentResourceResults",
    [dijit._Widget, dijit._Templated],
    {
        widgetsInTemplate: false,

        ///////////////////////////////////////////////////////////////////////////////////
        // Attach points

        _searchResultsContent: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Public fields
        /* Deprecated. Use scopingProjectURIs.
           When context is not pinned to a particular project, this will be set to "*".
           When context IS pinned to a particular project, this will be set to something
           like "Proj1/<star>".  Note: It's assumed that this path is already in URI format.
           For example, if project is "My Project", the setting would be "My%20Project/<star>"
        */
        // resourceURIPath: null,
        /*
           URIs of projects for which the recent artifacts will be shown (e.g. sidebar of a particular project)
           . If null all recent artifacts in all projects will be shown
           Arry of String 
        */
        scopingProjectURIs: null, 
        /* The value of this attribute must be one of the values found in the
           com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes
           property list. */
        viewType: null,
        /* The value of this attribute must be one of the values found in the
           com.ibm.rdm.web.search.results.RecentResourceResults.displayContext
           property list. */
        displayContext: null,
        
        
        ///////////////////////////////////////////////////////////////////////////////////
        // NLS field
        loading: null,
        
        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        // Set it to decide if _performQuery or _performXQuery should be executed and processed 
        // For support of old GET queries, in case they make a comeback. Default is XQuery
        _isGETQueryBased: null,  
        _messages: null,
        _maxDayDeltaThreshold: 28,
        _contentTypeFiltersObject: null,

        _resultsTable: null,
        _resultsTableTbody: null,
        _entryTables: null,
        _timeWindowSections: null,
        _resourceInfos: null,

        _atLeastOneRowDataRequestFailed: false,
        _numOutstandingPropertyRequests: 0,

        _currentPage: null,
        _totalPages: null,
        _isLoading: null,
        _mouseOverHandle: null,
        _mouseOutHandle: null,
        
        _header: null,
        _headerTitle: null,
        _contentPane: null,

        _beingDestroyed: false,
        _deferredArray: null,

        ///////////////////////////////////////////////////////////////////////////////////

		postCreate: function() {
			if (this._firstLinkImage) {
				this._firstLinkImage.src = dojo.moduleUrl("com.ibm.rdm.web", "common/images/icons/etool16/beginning.gif");
			}
			if (this._lastLinkImage) {
				this._lastLinkImage.src = dojo.moduleUrl("com.ibm.rdm.web", "common/images/icons/etool16/end.gif");
			}
		},
		
        startup: function() {
            // TODO Consider moving most of this DOM constuction into the .html file.

            if (this._viewlet) {
	            if (this.viewType == com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.SHOW_ARTIFACTS) {
            		this._viewlet.getSite().setTitle(dojo.string.substitute(this._messages.artifactsHeader, [""]));
	            } else if (this.viewType == com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.SHOW_REQUIREMENTS) {
            		this._viewlet.getSite().setTitle(dojo.string.substitute(this._messages.requirementsHeader, [""]));
	            }
            }
            
            if (this.displayContext == com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.USER_DASHBOARD) {
	            if (this.viewType == com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.SHOW_ARTIFACTS) {
	            	//this._headerImageNode.src = "com.ibm.rdm.web/common/images/icons/dtool16/recent-artifacts.gif"
	            } else if (this.viewType == com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.SHOW_REQUIREMENTS) {
	            	//this._headerImageNode.src = "com.ibm.rdm.web/common/images/recent-requirements.gif"
	            }
            }
            if (this.displayContext
                != com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.SIDEBAR_SECTION ||
                this.displayContext
                != com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.USER_DASHBOARD) {

            	// Hook up events for window resizing.
            	this._resizeEvent = jazz.util.ResizeNotifier.register({context: this, callback: this._handleWindowResize, force: true});
            }
            // Turn off the context menu (unless we're debugging).
            //if (!djConfig.isDebug) {
            //    this.connect(dojo.doc, "oncontextmenu", function(event) {dojo.stopEvent(event);return false;} );
            //}
            // Add the header for project dashboard.           
            if (this.displayContext != 
            	com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.USER_DASHBOARD) {
            	var header = this._createHeader();
                this._header = header;
            	this._searchResultsContent.appendChild(header.domNode);
            	header.informUserThatSearchIsInProgress();
            } 
            // Add a content pane, if needed.
            var contentPane = null;
            if (com.ibm.rdm.web.search.searchutils.isRCP()
                || this.displayContext == com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.SIDEBAR_SECTION) {
                var div = document.createElement("div");
                this._searchResultsContent.appendChild(div);
                contentPane = new dijit.layout.ContentPane({}, div);
                
                contentPane.startup();
            }
            // Create a table and parent it appropriately.
            var table = document.createElement("table");
            if (!dojo.isIE) {
            	table.setAttribute("cellSpacing", "0");
            }
            table.className = "search-results-table";
            if (contentPane) {
                contentPane.attr('content', table);
            } else {
                this._searchResultsContent.appendChild(table);
            }
            // Init some class attrs.
            this._resultsTable = table;
            this._contentPane = contentPane;
            this._entryTables = [];
            this._timeWindowSections = [];
            this._resourceInfos = [];
            this._beingDestroyed = false;
            this._deferredArray = null;
            // Set the initial height of the content pane.
            this._resizeContentPaneIfNeeded();
            // As our last step we perform the initial query.
            
            
            if (this._header) {
        		this._header.startup();
        	}
        	this.handleRefresh();
            /*if (!this._isGETQueryBased) {
                this.handleRefresh();
            } else {
                this._performQuery();
            }*/
            
            this._resizeContentPaneIfNeeded();
        },
        
        buildRendering: function() {
        	var templatePath;
        	if (this.displayContext == com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.USER_DASHBOARD) {
        		if (this.viewType == com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.RECENT_COMMENTS) {
        			templatePath = "search/results/templates/UserDashboardRecentComments.html";
        		} else {
        			templatePath = "search/results/templates/UserDashboardRecentResourceResults.html";
        		}
        	} 
        	else {
        		templatePath = "search/results/templates/RecentResourceResults.html";
        	}
        	this.templatePath = dojo.moduleUrl("com.ibm.rdm.web", templatePath);
        	this.inherited("buildRendering", arguments);
        },

        destroy: function() {
            this._beingDestroyed = true;
            jazz.util.ResizeNotifier.unregister(this._resizeEvent);
            this._clearLastResultsIfNeeded();
            if (this._deferredArray) {
                for (var i = 0; i < this._deferredArray.length; i++) {
                    var deferred = this._deferredArray[i];
                    if (deferred.fired == -1) {
                        deferred.cancel();
                    }
                }
            }
            if (this._header) {
                this._header.destroyRecursive();
                this._header = null;
            }
            if (this._contentPane) {
                this._contentPane.destroyRecursive();
                this._contentPane = null;
            }
            this.inherited("destroy", arguments);
        },

        postMixInProperties: function() {
            this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search.results", 
            		"RecentResourceResults");
            dojo.forEach(["loading", "next", "previous"], function(prop){
                this[prop] = this._messages[prop];
            }, this);
        },

        handleRefresh: function() {
            // summary:
            //    This function is called whenever the user clicks on the "global
            //    refresh" button.
        	if (!this._isLoading) {
        		this._isLoading = true;
	        	if (this._header) {
	        		this._header.informUserThatSearchIsInProgress();
	        	}
	            this._clearLastResultsIfNeeded();
	        	//this._showLoadingMessage();
	        	domutils.createHourGlassCursor();
	        	var recentArtifactsDeferred;
				var isPinnedToSingleProject =
					(this.scopingProjectURIs && this.scopingProjectURIs.length === 1);
				switch (this.viewType) {
				case com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.SHOW_ARTIFACTS:
					recentArtifactsDeferred = (
						isPinnedToSingleProject ?
						RecentFeedsClient.getRecentArtifacts(this.scopingProjectURIs[0]) :
						RecentFeedsClient.getRecentArtifacts()
					);
					break;
				case com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.SHOW_REQUIREMENTS:
					recentArtifactsDeferred = (
						isPinnedToSingleProject ?
						RecentFeedsClient.getRecentRequirements(this.scopingProjectURIs[0]) :
						RecentFeedsClient.getRecentRequirements()
					);
					break;
				}
	            recentArtifactsDeferred.addCallback(this, this._handlePerformQuerySuccessResponse);
	        	if (isPinnedToSingleProject) {
	        		recentArtifactsDeferred.addErrback(this, function(response) {
 						if (XhrUtils.getResponseCode(response) == 404) {
 							// TODO (LPK) ignore bogus 404 error from server
 							this._handlePerformQuerySuccessResponse([]);
 						} else {
							var errorSectionWidget = ErrorHandler.getErrorSectionWidget();
							if (errorSectionWidget) {
								errorSectionWidget.showErrorMessage(response);
							}
 						}
 						return response;
 					});
	        	} 
        	}
        },

        expandAll: function() {
            this._setExpansionState(true);
        },

        collapseAll: function() {
            this._setExpansionState(false);
        },
        
        createAndAppendResourceTableRow: function(resourceInfo, tbody) {
        	if (this instanceof com.ibm.rdm.web.search.results.RecentCommentResults){
        		// Recent Comments section on the side bar may require 
        		// tooltip dijits to display full comment text. The tooltip dijit
        		// can only be connected to an element in the DOM 
        		// tree, so we need pass 'tbody' in the row creation method.
        		// this modification concerns workitem 5676
        		if(this.scopingProjectURIs){
        			resourceInfo.projectURI = this.scopingProjectURIs[0];        			
        		}
        		this.createResourceTableRow(resourceInfo,tbody);
        	}else{
        		tbody.appendChild(this.createResourceTableRow(resourceInfo));
        	}
        },

        createResourceTableRow: function(resourceInfo) {
            var isVisible = (this._contentTypeFiltersObject ?
                this._contentTypeFiltersObject[resourceInfo.contentType] :
                true);
            // Add a new table row.
            var tr = document.createElement("tr");
            //this._resultsTableTbody.appendChild(tr);
            if (!isVisible) {
                tr.style.display = "none";
            }
            
            if (this.displayContext
                    != com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.USER_DASHBOARD) {
	            // Add a SingeLineEntryTable instance as the second td element.
            	var showImageAndSubSection;
            	if (this.viewType == com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.SHOW_REQUIREMENTS) {
            		showImageAndSubSection = false;
            	} else {
            		showImageAndSubSection = true;
            	}
	            var entryTable = new com.ibm.rdm.web.search.results.SingleLineEntryTable(
		            {resourceInfo:resourceInfo, doShowContentTypeImage:showImageAndSubSection,
		            doShowSubSection:showImageAndSubSection, displayContext:this.displayContext,
		            projectURI: this.scopingProjectURIs[0]},
		            null
	            );
	            tr.appendChild(com.ibm.rdm.web.search.domutils.createTD(entryTable.domNode));
	            entryTable.startup();
	            this._entryTables.push(entryTable);
            } else {
            	this._createUserDashResourceTableRow(tr, resourceInfo);
            }
            return tr;
        },
        
        _showLoadingMessage: function() {
        	if (this.displayContext == com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.USER_DASHBOARD) {
	        	this._loadingMessageNode.style.display = "";
	        	//this._headerItemCount.innerHTML = "";
	        	this._footer.style.visibility = "hidden";
        	}
        },
        
        _hideLoadingMessage: function() {
        	if (this.displayContext == com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.USER_DASHBOARD) {
            	this._loadingMessageNode.style.display = "none";
        	}        	
        },
        
        _createUserDashResourceTableRow: function(tr, resourceInfo) {
			var resourceURI = resourceInfo.resourceURI;
			var resourceName = resourceInfo.displayName;
			var imageSrc = com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(resourceInfo.contentType);			
			
            var imgNode = document.createElement("img");
            var link = ResourceLinksHandler.createResourceLink(resourceURI, resourceName, true);
			dojo.connect(link, "onclick", this, function(event) {
            	ResourceLinksHandler.handleAnchorClicked(event)});  
            var imgTD = document.createElement("td");
            var resourceNameTD = document.createElement("td");
    		var lastModifiedTD = document.createElement("td");
    		
			imgNode.setAttribute("src", imageSrc);

            dojo.addClass(imgTD, "resource-icon");
            dojo.addClass(resourceNameTD, "resource-name");
            dojo.addClass(lastModifiedTD, "last-modified-date");
            dojo.addClass(resourceNameTD, "resource-attribute");
            dojo.addClass(lastModifiedTD, "resource-attribute");
            
            imgTD.appendChild(imgNode);
            resourceNameTD.appendChild(link);
           
            lastModifiedTD.appendChild(document.createTextNode(
            	jazz.util.date.formatDif(new Date(resourceInfo.lastModifiedDate), false, true)));

            tr.appendChild(imgTD);
            tr.appendChild(resourceNameTD);
            tr.appendChild(lastModifiedTD);

        },

        shouldResourceBeVisible: function(resourceInfo) {
            return (
                this._contentTypeFiltersObject ?
                this._contentTypeFiltersObject[resourceInfo.contentType] != undefined :
                true
            );
        },

        applyContentTypeFilters: function(contentTypeFiltersObject) {
            var totalNumHidden = 0;
            this._contentTypeFiltersObject = contentTypeFiltersObject;
            var i;
            var entryTables = this._entryTables;
            for (i = 0; i < entryTables.length; i++) {
                var entryTable = entryTables[i];
                var isVisible = this.shouldResourceBeVisible(entryTable.resourceInfo);
                if (!isVisible) {
                    numHidden++;
                }
                this._setWidgetGrandparentVisibility(entryTable, isVisible);
            }
            for (i = 0; i < this._timeWindowSections.length; i++) {
                var section = this._timeWindowSections[i];
                var numTotal = section.getNumberOfResources();
                var numHidden = section.getTallyOfResourcesThatWouldBeHiddenViaFilters();
                var numShown = numTotal - numHidden;
                this._setWidgetGrandparentVisibility(section, (numShown > 0));
                totalNumHidden += numHidden;
            }
            // TODO fix this to pass the arg instead?
//             this._updateHeaderNumberShownInfo(totalNumHidden);
            this._updateHeaderNumberShownInfo();
        },

        removeContentTypeFilters: function() {
            this._contentTypeFiltersObject = null;
            var i;
            for (i = 0; i < this._entryTables.length; i++) {
                var entryTable = this._entryTables[i];
                this._setWidgetGrandparentVisibility(entryTable, true);
            }
            for (i = 0; i < this._timeWindowSections.length; i++) {
                var section = this._timeWindowSections[i];
                this._setWidgetGrandparentVisibility(section, true);
            }
            this._updateHeaderNumberShownInfo(0);
        },

        getContentTypeFiltersObject: function() {
            return this._contentTypeFiltersObject;
        },

        headerHasChangedHeight: function() {
            this._resizeContentPaneIfNeeded();
        },

        containerDimensionsHaveChanged: function() {
            this._resizeContentPaneIfNeeded();
        },

        ///////////////////////////////////////////////////////////////////////////////////
        // Helpers
		_showMostRecentCommentsClicked: function() {
		},
		
		_showMyRecentCommentsClicked: function() {
		},
		       
		_showRecentCommentsForMeClicked: function() {
		},
		
        _handleResourceContextClicked: function (event) {
        	ResourceLinksHandler.handleAnchorContextMenuClicked(event);
        },
        
        _showResultsInDashboardContext: function() {
        	this._currentPage = 1;
        	//this._headerItemCount.innerHTML = dojo.string.substitute(this._messages.count ,[this._resourceInfos.length]);
        	this._totalPages = Math.ceil(this._resourceInfos.length/PAGE_SIZE);
        	this._pageText.innerHTML = 
            	dojo.string.substitute(this._messages.pageNOfM, [this._currentPage,this._totalPages]);
        	if (this._totalPages == 0) {
        		this._footer.style.visibility = "hidden";
        	} else {
        		this._footer.style.visibility = "";
        		if (!this._mouseOverHandle) {
        			this._mouseOverHandle = this.connect(this.domNode, "onmouseover", this._showNavOptions);
        		}
        		if (!this._mouseOutHandle) {
        			this._mouseOutHandle = this.connect(this.domNode, "onmouseout", this._hideNavOptions);
        		}
        	}
        	this._addResourceEntriesToUserDashboard();
        },

        _showResultsInOtherContext: function() {
        	this._addResourceEntriesAndTimeWindowEntries();
            this._updateHeaderNumberShownInfo();
            this._expandFirstTimeWindowSection();
            if (this.displayContext
                == com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.SIDEBAR_SECTION) {
                this._resizeContentPaneIfNeeded();
            }
        },
        
        _showRecentResourceResults: function() {
        	var self = this;
        	if (!this._resultsTableTbody) {
        		this._resultsTableTbody = document.createElement("tbody");
        		this._resultsTable.appendChild(this._resultsTableTbody);
        	}
            if (this.displayContext
                == com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.USER_DASHBOARD) {
            	this._showResultsInDashboardContext();
            } else {
            	this._showResultsInOtherContext();
            }
        },
        
        _showNavOptions: function(event) {
        	if (event) {
        		event.preventDefault();
    			event.stopPropagation();
        	}
        	if (this._totalPages > 1) {
        		if (this._currentPage != this._totalPages) {
        			dojo.removeClass(this._next, "invisible");
        			dojo.removeClass(this._last, "invisible");
        		}
        		if (this._currentPage != 1) {
        			dojo.removeClass(this._first, "invisible");
                	dojo.removeClass(this._prev, "invisible");
        		}
        	}
        },
        
        _hideNavOptions: function(event) {
        	if (event) {
        		event.preventDefault();
    			event.stopPropagation();
        	}
        	dojo.addClass(this._first, "invisible");
        	dojo.addClass(this._last, "invisible");
        	dojo.addClass(this._prev, "invisible");
        	dojo.addClass(this._next, "invisible");        	
        },

        _handleWindowResize: function() {
            this._resizeContentPaneIfNeeded();
        },

        _resizeContentPaneIfNeeded: function() {
            var self = this;
            var windowBasedResizeContentPane = function() {
                var windowHeight = (window.innerHeight || document.body.clientHeight);
                var windowWidth = (window.innerWidth || document.body.clientWidth);
                if(this._contentPane && this._contentPane.domNode) {
                    var contentPaneTop = this._contentPane.domNode.offsetTop;
                }
                var scrollHeightToUse = (windowHeight - contentPaneTop);
                // TODO Resolve this fudging issue ...
                // The '2' fudge factor in the following statement is needed to
                // make this look right on IE ...
                if(this._contentPane && this._contentPane.domNode) {
                    this._contentPane.resize({ h: scrollHeightToUse, w: windowWidth-2 });
                }
            };
            var domBasedResizeContentPane = function() {
                if (!self.domNode.parentNode) { return; }
                var height = self.domNode.parentNode.clientHeight;
                var width = self.domNode.parentNode.clientWidth;
                var contentPaneTop = (
                    self._contentPane.domNode.offsetTop
                    - self._contentPane.domNode.parentNode.offsetTop);
                var scrollHeightToUse = (height - contentPaneTop);
                self._contentPane.resize({ h: scrollHeightToUse, w: width });
            };
            if (this._contentPane) {
                if (this.displayContext
                    == com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.SIDEBAR_SECTION) {
                    domBasedResizeContentPane();
                } else {
                    windowBasedResizeContentPane();
                }
            }
        },

        _doExcludeEntry: function(atomEntryNode) {
            var isRequirement = com.ibm.rdm.web.search.results.ResourceInfo.isRequirementResource(atomEntryNode);
            return (
                com.ibm.rdm.web.search.results.ResourceInfo.isCommentResource(atomEntryNode)
                || com.ibm.rdm.web.search.results.ResourceInfo.isAtomResource(atomEntryNode)
                || com.ibm.rdm.web.search.results.ResourceInfo.isDirectoryResource(atomEntryNode)
                || com.ibm.rdm.web.search.results.ResourceInfo.isAttributeGroupResource(atomEntryNode)
                || (this.viewType == com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.SHOW_ARTIFACTS
                    && isRequirement)
                || (this.viewType == com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.SHOW_REQUIREMENTS
                    && (!isRequirement))
            );
        },

        _setExpansionState: function(isExpanded) {
            for (var i = 0; i < this._timeWindowSections.length; i++) {
                var section = this._timeWindowSections[i];
                var isVisible = this._getWidgetGrandparentVisibility(section, true);
                if (isVisible) {
                    if (isExpanded) {
                        section.setExpanded();
                    } else {
                        section.setCollapsed();
                    }
                }
            }
        },

        _expandFirstTimeWindowSection: function() {
        	for (var i = 0; i < this._timeWindowSections.length; i++) {
        		var section = this._timeWindowSections[i];
	            if (i == 0) {
	                section.setExpanded();
	            } else {
	            	section.setCollapsed();
	            }
        	}
        },

        _updateHeaderNumberShownInfo: function(numCurrentlyHidden) {
            if (!numCurrentlyHidden) {
                numCurrentlyHidden = 0;
                for (var i = 0; i < this._resourceInfos.length; i++) {
                    if (!this.shouldResourceBeVisible(this._resourceInfos[i])) {
                        numCurrentlyHidden++;
                    }
                }
            }
            var numShown = (this._resourceInfos.length - numCurrentlyHidden);
            
            if (this._header) {
            	this._header.updateNumberShownInfo(numShown, this._resourceInfos.length);
            }
        },

        // TODO Look into a better approach for this (where we just muck with the widget
        // itself rather than the appropriate containing HTML element ...
        _setWidgetGrandparentVisibility: function(widget, isVisible) {
            widget.domNode.parentNode.parentNode.style.display = (isVisible ? "" : "none");
        },
        // Return true iff the widget is visible.
        _getWidgetGrandparentVisibility: function(widget) {
            return (widget.domNode.parentNode.parentNode.style.display == "");
        },
    
    	_destroyWidgets: function() {
    		if (this._entryTables) {
    			for (var i=0; i<this._entryTables.length; i++) {
    				var entry = this._entryTables[i];
    				entry.destroyRecursive();
    			}
    		}
 			this._entryTables = [];
     	},

        _clearLastResultsIfNeeded: function() {
            if (this._resultsTable && this._resultsTableTbody) {
                var list = dojo.query('[widgetId]', this._resultsTable);
                dojo.forEach(list.map(dijit.byNode), function(widget){ widget.destroyRecursive(); });
                com.ibm.rdm.web.util.domutils.removeChildNodes(this._resultsTableTbody);
                //this._resultsTable.removeChild(this._resultsTableTbody);
                this._entryTables = [];
                this._timeWindowSections = [];
                //this._resourceInfos = [];
            }
        },
        
        _clearLastAndShowCurrentPageResults: function() {
        	this._pageText.innerHTML = 
            	dojo.string.substitute(this._messages.pageNOfM, [this._currentPage, this._totalPages]);
            this._destroyWidgets();
        	com.ibm.rdm.web.util.domutils.removeChildNodes(this._resultsTableTbody);
            this._addResourceEntriesToUserDashboard();
        },

        _createHeader: function() {
        // No longer necessary, work item 13444
        //    var filterSection = null;
        //    if (this.viewType == com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.SHOW_ARTIFACTS) {
        //        filterSection = new com.ibm.rdm.web.search.results.RecentResourcesFilter(
        //            {
        //                recentResourceResults:this,
        //                contentTypes:contentTypes
        //            },
        //            null
        //        );
        //    }
            return (new com.ibm.rdm.web.search.results.RecentResourcesHeader(
                {
                    recentResourceResults:this,
                    filterSection:null
                },
                null
            ));
        },

        _performXQuery: function() {
            com.ibm.rdm.web.search.searchutils.showAsBusy(true);
            var today = new Date();
            today.setUTCHours(0, 0, 0, 0);
            var fourWeeksAgo = dojo.date.add(today, "week", -4);
            var modifiedSinceIsoDate = dojo.date.stamp.toISOString(
                fourWeeksAgo, { zulu:true, milliseconds:false }
            );
            var xquery = 
                "xquery \n"
                + "declare namespace dc = \"http://purl.org/dc/terms/\";\n"
                + "declare namespace ref = \"http://jazz.net/xmlns/alm/rm/Reference/v0.1#\";"
                + "declare namespace rdf = \"http://www.w3.org/1999/02/22-rdf-syntax-ns#\";\n"
                + "declare namespace resExt = \"http://com.ibm.rdm/resource/extension#\";"
                + "declare namespace atom = \"http://www.w3.org/2005/Atom#\";\n"
                + "declare namespace prop = \"http://jazz.net/xmlns/properties/v0.6#\";"
                + "for $index in collection('JRS_RDF_INDEX')/rdf:Description\n"
                //+ "where not($index/resExt:parent)\n" //added  
                + "where\n"
                + "    $index/prop:resource-collection/@rdf:resoure=\"/jazz/resources/rdm/extendedResources\"\n" //added
                + "    and $index/dc:modified > \"" + modifiedSinceIsoDate + "\"\n"   // must be ISO date, e.g. '2008-05-31T00:00:00Z'
                //+ "    and $index/prop:resource-collection/@rdf:resource=\"/jazz/resources/rdm/resources\"\n"
                + this._createProjectScopeXQueryPred("$index") + "\n"
                //+ "    and $index/fn:not(fn:starts-with(dc:format, 'application/x-com.ibm.rdm.attributegroupstyle+xml'))\n"
                + "    and ($index/resExt:format < 'application/atom+xml;type=entry' or $index/resExt:format > 'application/atom+xml;type=entry')\n" //added
                //+ "    and $index/fn:not(fn:starts-with(dc:format, 'application/atom+xml'))\n"
                + "    and $index//prop:archived=\"false\"\n"
                + "order by $index/dc:modified descending\n"
                + "return $index";
        },

        _createProjectScopeXQueryPred: function(indexParam) {
            var scopeToProjects = (this.scopingProjectURIs && this.scopingProjectURIs.length>0)?true:false;
            
            if (!scopeToProjects)
                return "";
                
            // Creates something like $childResources/<star>:owning-team/@rdf:resource="/jazz/projects/rdm/teams/Proj2" 
            var predTemplate = indexParam + "/prop:owning-team/@rdf:resource=";
            var pred = " and ("; 
            var len = this.scopingProjectURIs.length;
            for (var i=0; i<len-1; i++) {
                var u = this.scopingProjectURIs[i];
                pred += predTemplate + "'" + u + "'" + " or "; 
            }
            pred += predTemplate + "'" + this.scopingProjectURIs[len-1] + "') \n";
            return pred;
        },

        // deprecated, GET based
        _performQuery: function() {
            com.ibm.rdm.web.search.searchutils.showAsBusy(true);
            var today = new Date();
            today.setUTCHours(0, 0, 0, 0);
            var fourWeeksAgo = dojo.date.add(today, "week", -4);
            var modifiedSinceIsoDate = dojo.date.stamp.toISOString(
                fourWeeksAgo, { zulu:true, milliseconds:false }
            );
            var queryParams = (
                "http://jazz.net/xmlns/properties/v0.6%23resource-modified-since=" + modifiedSinceIsoDate
                + "&properties=http://schema.ibm.com/rdm/2008/Base%23name"
                + ",http://jazz.net/xmlns/alm/rm/BPMNExtensions/v0.1%23name"
                + ",http://jazz.net/xmlns/alm/rm/Base/v0.1%23name"
            );
        },

        /*
         * Callback for a successful query.
         */
        _handlePerformQuerySuccessResponse: function(resourceInfos) {
        	this._resourceInfos = resourceInfos;
        	// workaround work item 19808
			var filteredResourcesInfo = new Array();
			for (var i = 0; i < this._resourceInfos.length; i++) {
				var info = this._resourceInfos[i];
				if (info.contentType != com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_FILTER
					&& info.displayName 
					&& info.displayName != "") {
						filteredResourcesInfo.push(info);
				}
			}
			this._resourceInfos = filteredResourcesInfo;
        	com.ibm.rdm.web.search.searchutils.showAsBusy(false);
            this._hideLoadingMessage();
            domutils.createDefaultCursor();
            this._showRecentResourceResults();
            this._isLoading = false;
            return resourceInfos;
        },

        /*
         * Callback for an unsuccessful query.
         */
        _handlePerformQueryErrorResponse: function(error) {
            this._deferred = null;
            com.ibm.rdm.web.search.searchutils.showAsBusy(false);
            domutils.createDefaultCursor();
            if (! (error.dojoType && error.dojoType == "cancel")) {
                com.ibm.rdm.web.search.searchutils.reportError(
                      dojo.string.substitute(this._messages.errorAttemptingToPerformQuery,
                          [(error.description || error)]));
            }
        },

        _sortByMostToLeastRecent: function(resourceInfos) {
            return resourceInfos.sort(function(resourceInfo1, resourceInfo2) {
                var date1 = resourceInfo1.lastModifiedDate;
                var date2 = resourceInfo2.lastModifiedDate;
                var time1 = date1.getTime();
                var time2 = date2.getTime();
                return ((time1 == time2) ? 0 : ((time1 < time2) ? 1 : -1));
            });
        },
        
        /*
         * Note: It's expected that dayDelta is an int that its values range between
         * 1 and this._maxDayDeltaThreshold
         */                        
        _getTimeWindowLabel: function(dayDelta, dayOfWeekIndex) {
            var label = null;
            switch (dayDelta) {
            	case 0:
            		label = this._messages.today;
                    break;
                case 1:
                    label = this._messages.yesterday;
                    break;
                default:
                	if (dayDelta<7){
                		label = this._messages.pastweek;
                	}
                break;
            }
            return label;
        },

        _getDayDelta: function(nowAsDayBasedDate, resourceInfo) {
            var label = null;
            var resourceDayBasedDate = this._createDayBasedDate(resourceInfo.lastModifiedDate);
            var dayDelta = dojo.date.difference(resourceDayBasedDate, nowAsDayBasedDate, "day");
            return dayDelta;
        },

        _createDayBasedDate: function(date) {
        	var dayBasedDate = new Date();
        	dayBasedDate.setFullYear(date.getFullYear());
        	dayBasedDate.setMonth(date.getMonth());
        	dayBasedDate.setDate(date.getDate());
        	dayBasedDate.setHours(0, 0, 0, 0);
        	return dayBasedDate;
        },

        _createTimeWindowSectionTableRow: function(timeWindowSection) {
            var tr = document.createElement("tr");
            this._resultsTableTbody.appendChild(tr);
            tr.appendChild(com.ibm.rdm.web.search.domutils.createTD(timeWindowSection.domNode));
        },
        
        _addResourceEntriesToUserDashboard: function() {
     
            for (var i = (this._currentPage-1)*PAGE_SIZE; 
            	i < this._resourceInfos.length && i < ((this._currentPage-1)*PAGE_SIZE+PAGE_SIZE); 
            	i++) {
                var resourceInfo = this._resourceInfos[i];
                this._resultsTableTbody.appendChild(this.createResourceTableRow(resourceInfo));
            }
            
            if (this.displayContext == com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.USER_DASHBOARD){
               	switch(this.viewType) {
					case com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.SHOW_REQUIREMENTS:
            			this._viewlet.getSite().setTitle(dojo.string.substitute(this._messages.requirementsHeader, [this._resourceInfos.length]));
						break; 
					case com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.SHOW_ARTIFACTS:
            			this._viewlet.getSite().setTitle(dojo.string.substitute(this._messages.artifactsHeader, [this._resourceInfos.length]));
						break;            	
            	}
            }
        },

        _addResourceEntriesAndTimeWindowEntries: function() {
            var sameDay = function(resourceInfo1, resourceInfo2) {
                var date1 = resourceInfo1.lastModifiedDate;
                var date2 = resourceInfo2.lastModifiedDate;
                return (
                    date1.getDate() == date2.getDate()
                    && date1.getMonth() == date2.getMonth()
                    && date1.getFullYear() == date2.getFullYear()
                );
            }
            var todayAsDayBasedDate = this._createDayBasedDate(new Date());
            var lastResourceInfo = null;
            var timeWindowSection = null;
            var oldestDayDelta = this._maxDayDeltaThreshold;
            var i;
            var entries=[];
            
            // open three time sections: Today, Yesterday, PastWeek
            this._timeWindowSections.push(new com.ibm.rdm.web.search.results.TimeWindowSection(
                    {sectionTitle:this._messages.today, recentResourceResults:this}, null));
            this._timeWindowSections.push(new com.ibm.rdm.web.search.results.TimeWindowSection(
                    {sectionTitle:this._messages.yesterday, recentResourceResults:this}, null));
            this._timeWindowSections.push(new com.ibm.rdm.web.search.results.TimeWindowSection(
                    {sectionTitle:this._messages.pastweek, recentResourceResults:this}, null));  
            
            
            for (i = 0; i < this._resourceInfos.length; i++) {
                var resourceInfo = this._resourceInfos[i];
                var resourceDayBasedDate = this._createDayBasedDate(resourceInfo.lastModifiedDate);
                var resourceDayDelta = dojo.date.difference(resourceDayBasedDate, todayAsDayBasedDate, "day");                
                if (resourceDayDelta<oldestDayDelta){             	
                    var sectionTitle = this._getTimeWindowLabel(resourceDayDelta, resourceDayBasedDate.getDay());
                    for (var j=0; j < this._timeWindowSections.length; j++) {
                    	timeWindowSection=this._timeWindowSections[j];
                    	if (sectionTitle==timeWindowSection.sectionTitle){
                    		timeWindowSection.addResourceInfo(resourceInfo);
                    		entries.push(resourceInfo);
                    	}
                    }
                }
            }
           
            this._resourceInfos=entries;
            
            for (i = 0; i < this._timeWindowSections.length; i++) {
                var timeWindowSection = this._timeWindowSections[i];
                timeWindowSection.startup();   
                var timeSectionTitle = dojo.string.substitute(this._messages.timeSectionTitle, [timeWindowSection.sectionTitle, timeWindowSection.getNumberOfResources()]);
                timeWindowSection.setTitle(timeSectionTitle);
                this._createTimeWindowSectionTableRow(timeWindowSection);
            }
        },

        // Note: this function is currently only used by subclasses, but it's likely
        // to eventually be needed for use in this class as well.
        _incrementOutstandingRequestCount: function() {
            this._numOutstandingPropertyRequests++;
        },

        // Note: this function is currently only used by subclasses, but it's likely
        // to eventually be needed for use in this class as well.
        _decrementOutstandingRequestCount: function() {
            this._numOutstandingPropertyRequests--;
            if (this._numOutstandingPropertyRequests == 0) {
                if (this._atLeastOneRowDataRequestFailed) {
                    if (!this._beingDestroyed) {
                        com.ibm.rdm.web.search.searchutils.reportError(this._messages.errorAttemptingToRetrieveResourceInfo);
                    }
                    this._atLeastOneRowDataRequestFailed = false;
                }
            }
        },
        
        _handleFirstPage: function() {
        	this._currentPage = 1;
        	dojo.addClass(this._first, "invisible");
        	dojo.addClass(this._prev, "invisible");
        	dojo.removeClass(this._last, "invisible");
        	dojo.removeClass(this._next, "invisible");
        	this._clearLastAndShowCurrentPageResults();
        },
        
        _handlePreviousPage: function() {
        	if (this._currentPage > 1) {
        		this._currentPage--;
        		if (this._currentPage == 1) {
        			dojo.addClass(this._first, "invisible");
                	dojo.addClass(this._prev, "invisible");
        		}
        		dojo.removeClass(this._next, "invisible");
            	dojo.removeClass(this._last, "invisible");
        		this._clearLastAndShowCurrentPageResults();
        	}
        },
        
        _handleNextPage: function() {
        	if (this._currentPage < this._totalPages) {
        		this._currentPage++;
        		if (this._currentPage == this._totalPages) {
        			dojo.addClass(this._next, "invisible");
                	dojo.addClass(this._last, "invisible");		
        		}
        		dojo.removeClass(this._first, "invisible");
            	dojo.removeClass(this._prev, "invisible");
        		this._clearLastAndShowCurrentPageResults();
        	}
        },
        
        _handleLastPage: function() {
        	this._currentPage = this._totalPages;
    		dojo.addClass(this._next, "invisible");
        	dojo.addClass(this._last, "invisible");
        	dojo.removeClass(this._first, "invisible");
        	dojo.removeClass(this._prev, "invisible");
        	this._clearLastAndShowCurrentPageResults();
        }
    }
);


})();

com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes = {
    SHOW_ARTIFACTS: 0,
    SHOW_REQUIREMENTS: 1,
    RECENT_COMMENTS: 2
};

com.ibm.rdm.web.search.results.RecentResourceResults.displayContext = {
    MAIN_CONTENT_AREA: 0,
    SIDEBAR_SECTION: 1,
    OTHER: 2,
    USER_DASHBOARD: 3
};
