/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.rdm.web.search.results.ReqWithImplWorkItems");
	
	dojo.require("dijit._Templated");
	dojo.require("dijit._Widget");
	dojo.require("dojo.i18n");
	dojo.require("dojox.data.dom");
	dojo.require("jazz.app.ResourceLink");
	dojo.require("com.ibm.rdm.web.project.ProjectSection");
	dojo.require("dojo.data.ItemFileReadStore");
	dojo.require("dijit.form.ComboBox");
	dojo.require("com.ibm.rdm.web.common.ProjectCache");
	dojo.require("com.ibm.rdm.web.util.queryutils");
	dojo.require("com.ibm.rdm.web.common.MimeTypes");
	dojo.require("com.ibm.rdm.web.server.client.RDMClient");
	dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
	dojo.require("com.ibm.rdm.web.pages.WebUIinitializer");

	dojo.requireLocalization("com.ibm.rdm.web.search.results", "ReqWithImplWorkItems");

	var COMBOWIDTH = "100%";  	
	//Filter types
	var filterOpenItems = "openItems";
	var filterResolvedItems = "resolvedItems";
	var filterNone = "filterNone";
	var successfulTests = "successfulTests";
	var failingTests = "failingTests";
	var noTests = "noTests";
	var ResourceLink = jazz.app.ResourceLink;
	var collectionInfos = [];
	var sourceArtifactInfos = []; 
	//Keys used for persisting preferences
	var RDMProject ="rdmProject";
	var RDMCollection = "rdmCollection";
	var RDMCollectionFilter = "rdmFilter";
	//Section Labels
	var PROJECT_HEADING = "";
	var COLLECTION_HEADING = "";
	var FILTER_HEADING = "";
	var NORESULTS_HEADING = "";
	var LOADING_HEADING="";
	

	dojo.declare("com.ibm.rdm.web.search.results.ReqWithImplWorkItems", [dijit._Widget, dijit._Templated], {
		widgetsInTemplate: false,
		templatePath: dojo.moduleUrl("com.ibm.rdm.web", "search/results/templates/ReqWithImplWorkItems.html"),
		_projectStore:null,
		_collectionStore:null,
		_filterStore:null,
		_messages:null,
		_commonMessages: null,
		collectionSelectionCombo:null,
		projectSelectionCombo:null,
		filterSelectionCombo:null,
		_selectedProject:null,
		_selectedCollection:null,
		_selectedFilter:null,
		_projectsLoaded:false,
		_ignoreComboBoxChanges:false,

	startup: function() {

		//Initialize project properties
		var def = com.ibm.rdm.web.pages.WebUIinitializer.initializeJFSForConfigProperties();
		def.addCallback(this, this.start);		
	},
	start: function() {
			
		this._getSelectionsFromPreferences();

		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search.results", 
		"ReqWithImplWorkItems");
		
		this._commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");

		if (this._viewlet) {				
			//this.queryID can be of type "requirementsValidatedBy" or "requirementsImplementedBy")
			if(this.queryID == "requirementsImplementedBy"){
				this._headerTitle = this._messages.reqWithImplDevItems;
				this._viewlet.getSite().setTitle(this._headerTitle);
			}else if(this.queryID == "requirementsValidatedBy"){
				this._headerTitle = this._messages.reqWithPlanItems;
				this._viewlet.getSite().setTitle(this._headerTitle);
			}
		}		
		this._viewletData = {};
		//Create the Project, Collection and Filter sections
		PROJECT_HEADING = "-- " + this._messages.project + " --";
		COLLECTION_HEADING = "-- Collection --";
		FILTER_HEADING = "-- " + this._messages.filterby  + " --";
		NORESULTS_HEADING = "-- " +  this._messages.noResults + " --";
		LOADING_HEADING  = this._messages.loading;
		LOADING_MESSAGE = this._commonMessages.loading;
		this._createProjectSection();
		this._createCollectionSection();
		this._createFilterSection(this.queryID);
		this._createConnectionEvents();
	
	},	
	
	/////////////////////////////////
	//     UI WIDGETS              //
	/////////////////////////////////
	
	_createProjectSection:function(){
		
		var storedSelectedProject;
		var projectData = { identifier: 'index', items:[], label: 'name'};
		this._projectStore = new dojo.data.ItemFileReadStore( { data: projectData} );		
		this.projectSelectionCombo = new dijit.form.ComboBox({name:"projectSelection",
			autocomplete:true,
			searchAttr:"name",
			store:this._projectStore		                              
		});

		var deferred = com.ibm.rdm.web.common.ProjectCache.getUserMemberProjects();
		deferred.addCallback(this, function(projects){
			
			var counter=0;
			if(projects && projects.length != 0)
				projectData.items.push({index:counter,name:PROJECT_HEADING,url:null}); 
			for(var i in projects){
				var projectName = projects[i].name;
				var projectUrl = projects[i].url;
				if(this._selectedProject && (projectUrl==this._selectedProject)){//Check if there is a selected project in preferences 					
					storedSelectedProject =projectName;	
					this._selectedProject = projectUrl;
				}
				counter++;
				projectData.items.push({index:counter,name:projectName,url:projectUrl}); 
			}
			if(storedSelectedProject)
				this.projectSelectionCombo.attr("displayedValue", storedSelectedProject);				
			else
				this.projectSelectionCombo.attr("displayedValue", PROJECT_HEADING);				
			this._projectsLoaded=true;			
		}
		);		
		
    	this.projectSelectionCombo.domNode.style.width = COMBOWIDTH;
		this.projectSelectionCombo.domNode.style.border = "none";
		this._projectSelectionNode.appendChild(this.projectSelectionCombo.domNode);	
		
	},
	_createCollectionSection:function(){
		var collectionData = { identifier: "index", items:[], label: "name"};
		this._collectionStore = new dojo.data.ItemFileReadStore( { data: collectionData } );

		this.collectionSelectionCombo = new dijit.form.ComboBox({name:"collectionSelection",
			autocomplete:true,
			searchAttr:"name",
			store:this._collectionStore			                              
		});
		
		this.collectionSelectionCombo.attr("displayedValue", COLLECTION_HEADING);	
		this.collectionSelectionCombo.domNode.style.width= COMBOWIDTH;
		this.collectionSelectionCombo.domNode.style.border = "none";		
		this._collectionSelectionNode.appendChild(this.collectionSelectionCombo.domNode);
		
		if(this._projectsLoaded){
			this._populateCollectionSection();
			this._projectsLoaded=false;
		}
		
	},
	_createFilterSection : function(viewletType){		
		var filterData = { identifier: "index", items:[], label: "name"};		
		if(viewletType == "requirementsImplementedBy"){
			filterData.items.push({index:0, name:this._messages.allItems , value:filterNone});
			filterData.items.push({index:1, name:this._messages.openItems , value:filterOpenItems });
			filterData.items.push({index:2, name:this._messages.resolvedItems, value:filterResolvedItems });
			this._filterStore = new dojo.data.ItemFileReadStore( { data: filterData } );
			this.filterSelectionCombo = new dijit.form.ComboBox({name:"collectionSelection",
				autocomplete:true,
				searchAttr:"name",
				store:this._filterStore			                              
			});
			if(this._selectedFilter){
				if(this._selectedFilter == filterNone)
					this.filterSelectionCombo.attr("displayedValue",this._messages.allItems);	
				else if(this._selectedFilter == filterOpenItems)
					this.filterSelectionCombo.attr("displayedValue",this._messages.openItems);	
				else if(this._selectedFilter == filterResolvedItems)
					this.filterSelectionCombo.attr("displayedValue",this._messages.resolvedItems);				
			}			
			else{
				//Set default filter type to all.
				this._selectedFilter=filterNone;
				this.filterSelectionCombo.attr("displayedValue", this._messages.allItems);	
			}
		}
		else if(viewletType == "requirementsValidatedBy"){
			filterData.items.push({index:0, name:this._messages.allItems , value:filterNone});
			filterData.items.push({index:1, name:this._messages.noTests , value:noTests});
			filterData.items.push({index:2, name:this._messages.successfulTests , value:successfulTests });
			filterData.items.push({index:3, name:this._messages.failingTests, value:failingTests });
			this._filterStore = new dojo.data.ItemFileReadStore( { data: filterData } );
			this.filterSelectionCombo = new dijit.form.ComboBox({name:"collectionSelection",
				autocomplete:true,
				searchAttr:"name",
				store:this._filterStore			                              
			});
			if(this._selectedFilter){
				if(this._selectedFilter == filterNone)
					this.filterSelectionCombo.attr("displayedValue",this._messages.allItems);	
				else if(this._selectedFilter == noTests)
					this.filterSelectionCombo.attr("displayedValue", this._messages.noTests);	
				else if(this._selectedFilter == successfulTests)
					this.filterSelectionCombo.attr("displayedValue",this._messages.successfulTests);	
				else if(this._selectedFilter == failingTests)
					this.filterSelectionCombo.attr("displayedValue",this._messages.failingTests);	
			}			
			else{
				//Set default filter type to all.
				this._selectedFilter=filterNone;
				this.filterSelectionCombo.attr("displayedValue", this._messages.allItems);	
			}
		}
		this.filterSelectionCombo.domNode.style.width=COMBOWIDTH;
		this.filterSelectionCombo.domNode.style.border = "none";
		this._filterSelectionNode.appendChild(this.filterSelectionCombo.domNode);	
	},
	
	_populateCollectionSection: function(projects){		
		this._projectsLoaded = false;	
		//Get selected project
		if(projects)
		  this._selectedProject = projects[0].url + ""; 
		else if(!this._selectedProject) 
		  return;
		
		var query = this._composeCollectionPostQuery();
	     
		var deferred = com.ibm.rdm.web.common.ProjectCache.executeMultiFetch(query);
		var collectionData = { identifier: "index", items:[], label: "name"};	
	    this.collectionSelectionCombo.attr("displayedValue",LOADING_HEADING);	
	    this.setLoadingMsg();
		//Populate collection selection
		deferred.addCallback(this, function(collections){
            var firstCollection ={};	
			var prevStoredCollectionURI=null;
			var prevStoredDisplayName=null;	
			var counter=0;
			if(collections && collections.length > 0){
				collectionData.items.push({index:counter,name:COLLECTION_HEADING,value:null});
				for(var i in collections){						
					var displayName = collections[i].displayName;	
					var resourceURI = collections[i].resourceURI;
					counter++;
					collectionData.items.push({index:counter,name:displayName,value:resourceURI});
					if(counter == 1)
						firstCollection={label:displayName,uri:resourceURI};
					if(resourceURI == this._selectedCollection){ 
						prevStoredCollectionURI= resourceURI; 
						prevStoredDisplayName = displayName;
					}
				}	
				this._collectionStore = new dojo.data.ItemFileReadStore( { data: collectionData } );
				this.collectionSelectionCombo.store = this._collectionStore;
				if((prevStoredDisplayName != null) && (prevStoredCollectionURI !=null)){					
					this._selectedCollection = prevStoredCollectionURI;
					this._ignoreComboBoxChanges=true;
					this.collectionSelectionCombo.attr("displayedValue", prevStoredDisplayName);	
					this._doProcessArtifacts();
				}else{
					this._selectedCollection = firstCollection.uri;
					this._ignoreComboBoxChanges=true;
					this.collectionSelectionCombo.attr("displayedValue", firstCollection.label);	
					this._doProcessArtifacts();
				}
			}else{
				this._collectionStore = new dojo.data.ItemFileReadStore( { data: collectionData } );
				this.collectionSelectionCombo.store = this._collectionStore;
				this.collectionSelectionCombo.attr("displayedValue", NORESULTS_HEADING);
			}
					
			
		});				
	},	
	_createConnectionEvents: function(){		
		//Listen to selections in the project combobox
		dojo.connect(this.projectSelectionCombo, 'onChange', this, function(){			
	
			var value = this.projectSelectionCombo.value;
			//First item in the combo list has a null value, ignore this selection
			if(!value){
				this._selectedProject=null;
				return;
			}	
			this._projectStore.fetch({query: {name:value}, onComplete:  dojo.hitch(this,this._populateCollectionSection)});
		});
		//Listen to selections in the collection combobox
		dojo.connect(this.collectionSelectionCombo, 'onChange', this, function(){	
			if(this._ignoreComboBoxChanges){
				this._ignoreComboBoxChanges = false;
				return;
			}
			var value = this.collectionSelectionCombo.value;
			//Ignore user notifications and null values in the collection combo list
			if(!value || value == NORESULTS_HEADING || value == LOADING_HEADING){
				this._handleNoResults();
				return;				
			}
			this._collectionStore.fetch({query: {name:value}, onComplete:  dojo.hitch(this,this._getRequirementsFromCollection)});
		});
		//Listen to selections in the filter combobox
		dojo.connect(this.filterSelectionCombo, 'onChange', this, function(){			
			var value = this.filterSelectionCombo.value;
			this._filterStore.fetch({query: {name:value}, onComplete:  dojo.hitch(this,this._runFilteredSearch)});
		});
	},
	
	_getSelectionsFromPreferences: function(){
		this._selectedProject    = this._viewlet.getContributorState().get(RDMProject);
		this._selectedCollection = this._viewlet.getContributorState().get(RDMCollection);
		this._selectedFilter     = this._viewlet.getContributorState().get(RDMCollectionFilter);		
	},
	
	_persistUserSelections:function(){
		this._viewlet.getContributorState().set(RDMProject,this._selectedProject);
		this._viewlet.getContributorState().set(RDMCollection,this._selectedCollection);
		this._viewlet.getContributorState().set(RDMCollectionFilter,this._selectedFilter);
	},
	
	_getRequirementsFromCollection: function(collectionElement){
	   this.setLoadingMsg();
	   if(!collectionElement)return;
	   var collectionURI = (collectionElement.length > 0)? collectionElement[0].value + "":collectionElement.value + "";
	   this._selectedCollection = collectionURI;
	   this._doProcessArtifacts();
     },
	
	_runFilteredSearch: function(filter){
    	this.setLoadingMsg();
    	if(!this._selectedProject || !this._selectedCollection)return;    	
		this._selectedFilter=  filter[0].value + "";
		this._doProcessArtifacts();
	},
	
	_doProcessArtifacts: function(){
		if(!this._selectedCollection)return;
		var collectionURI = this._selectedCollection;
		
		var collectionArtifactDeferred = com.ibm.rdm.web.common.ProjectCache.getCollection(collectionURI);
		   
		 collectionArtifactDeferred.addCallback(this, function(collection){
			 if(collection.collectionArtifacts.length == 0){
		  			this._handleNoResults();
		  		}else{
		  			this._fetchDataFromCALMQueryService(collection.collectionArtifacts);
		  		}		        
		});		 
		 collectionArtifactDeferred.addErrback(function(response) {
	    		console.log("Error reading collection " + collectionURI);
	    }); 
	},
	
	_composeURIsClause: function(artifactURIs) {
    	var URIsClause = "";
    	var URIs =artifactURIs;
		if (URIs && URIs.length > 0) {
			URIsClause = 'resource in [';
			for (var i=0; i<URIs.length; i++) {
				URIsClause += (i==0?'':',') + '<' + URIs[i] + '>';
			}
			URIsClause += ']';
		}
        return URIsClause;
    },

	_fetchDataFromCALMQueryService: function(artifacts) {	   
    
       var projectURI = this._selectedProject;
       var filterBy = (this._selectedFilter)?this._selectedFilter:filterNone;     
       //Persist the user selections
       this._persistUserSelections();    
       var combinedURIS = "";      
       for(var i=0; i < artifacts.length; i++){   	   
    	   var resourceURI =  artifacts[i].resourceURI; 	
    	   sourceArtifactInfos[resourceURI]=artifacts[i];
    	   combinedURIS += resourceURI;
    	   if(i < artifacts.length -1)
    	      combinedURIS += ",";	   
       }       
       var query = "query=" + this.queryID;
       if(filterBy != filterNone) //Default is show all
    	  query +="&filter=" + dojo.trim(filterBy);
       
       query +="&resources=" + combinedURIS;
    
       var calmQueryServicedeferred = com.ibm.rdm.web.server.client.RDMClient.doPost({
			uri: com.ibm.rdm.web.FRONTING_SERVICE_CALM_QUERY,
	        postData: query,
	        projectURI: projectURI
		});
		
        calmQueryServicedeferred.addCallback(dojo.hitch(this, this._handleFetchDataResponse));      
        calmQueryServicedeferred.addErrback(dojo.hitch(this, this._handleFetchDataError));                       
	},

	_handleFetchDataResponse: function (response) {
		this._viewletUrls = this._calmLinkResponseParser(response);
		this._populateViewlet();
	},

	_handleFetchDataError: function (err) {

	},
	_calmLinkResponseParser: function(response) {		
		var viewletType = this.queryID;	
		var modifyLabel = function(originalLabel){   
			if(!originalLabel)
				return "...";
			var hasIpInLabel = /^\[[0-9]|[A-Z|a-z]+:[0-9]+\]/; //RQM send validated by labels beginning with an IP address. Remove this from the label.
			originalLabel =  dojo.trim(originalLabel);			
			if (hasIpInLabel.test(originalLabel))
			    return originalLabel.substring(originalLabel.indexOf("]")+ 1); //Trim the IP from the label
		    else
		    	return originalLabel;
		};				
		var viewletUrls=[]	    
		var linkNodes = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(response.doc, "rrm:Link"); 
		for (var i=0; i < linkNodes.length; i++){		
			var linkNode = linkNodes[i];
			var sourceNode = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(linkNode, "rrm:linkSource"); 
			var targetNode= com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(linkNode, "rrm:linkTarget"); 		
			var tempLabel= com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(linkNode, "rdfs:label"); 	
			tempLabel = (tempLabel[0].firstChild)?tempLabel[0].firstChild.nodeValue:null;
			var targetLabel = modifyLabel(tempLabel);		
			var targetSourceURI = com.ibm.rdm.web.util.domutils.getAttributeNS(targetNode[0], "resource", com.ibm.rdm.web.util.queryutils.RDF_NAMESPACE, com.ibm.rdm.web.util.queryutils.RDF_QUALIFIER);			
			var sourceUrl = com.ibm.rdm.web.util.domutils.getAttributeNS(sourceNode[0], "resource", com.ibm.rdm.web.util.queryutils.RDF_NAMESPACE, com.ibm.rdm.web.util.queryutils.RDF_QUALIFIER);			
			if(viewletUrls[sourceUrl])
				viewletUrls[sourceUrl].push({label:targetLabel,url:targetSourceURI});
			else{
				targets=[];
				targets.push({label:targetLabel,url:targetSourceURI});
				viewletUrls[sourceUrl] = targets;
			}
		}
		return viewletUrls;		
	},
	
	_populateViewlet: function(){
            var recordCounter = 0;
		    dojox.data.dom.removeChildren(this._searchResultsContent);
			for (var req in this._viewletUrls) {
				recordCounter++;
				if (!this._viewletData[req]) {
					this._viewletData[req] = {};
				}
				//Set the icon and the label for the artifacts
				this._viewletData[req].icon= com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(sourceArtifactInfos[req].contentType);
				var reqDiv = document.createElement("div");
				var reqImg = document.createElement("img");
				reqImg.style.marginLeft = "3px";
				reqImg.src = this._viewletData[req].icon;				
				reqDiv.appendChild(reqImg);
				
				var resourceURI = (
						sourceArtifactInfos[req].resourceURI
						+ (
								(sourceArtifactInfos[req].inSnapshot && sourceArtifactInfos[req].isCollectionArtifactAssociatedWithSnapshot) ?
										(
												"?revision=" + sourceArtifactInfos[req].etag
												+ "&rrc:baseline=" + encodeURIComponent(sourceArtifactInfos[req].projectSnapshotURI)
										) :
											""
						)
				);
				
				if(sourceArtifactInfos[req].inSnapshot && sourceArtifactInfos[req].isCollectionArtifactAssociatedWithSnapshot){
					var reqLink = com.ibm.rdm.web.common.ResourceLinksHandler.getResourceLink(
							resourceURI,
							sourceArtifactInfos[req].displayName,
							false
					);
				}
				else{	
					var reqLink = com.ibm.rdm.web.common.ResourceLinksHandler.getResourceLink(
							resourceURI,
							sourceArtifactInfos[req].displayName,
							true
					);
				}
				reqLink.style.marginLeft = "3px";
				reqDiv.appendChild(reqLink);
				this._searchResultsContent.appendChild(reqDiv);
			  

				for (var j=0; j < this._viewletUrls[req].length; j++){
					//Create a ResourceLink object for the work items that implement the requirement 
					var workItemURI = this._viewletUrls[req][j];
					var wiDiv = document.createElement("div");
					wiDiv.style.marginLeft = "15px";
					var resourceLinkAnchor = document.createElement("a");
					resourceLinkAnchor.style.marginLeft = "3px";
					var  resourceIconImage = document.createElement("span");
					resourceLinkAnchor.appendChild(document.createTextNode(workItemURI.label));

					var resourceLink = new jazz.app.ResourceLink({
						uri: workItemURI.url,
						iconNode: resourceIconImage
					}, resourceLinkAnchor);

					wiDiv.appendChild(resourceIconImage);
					wiDiv.appendChild(resourceLinkAnchor);
					this._searchResultsContent.appendChild(wiDiv);
				}			
			}
			if(recordCounter == 0){		
				this._handleNoResults();			
			}		
	},

	_handleNoResults:function(){
		dojox.data.dom.removeChildren(this._searchResultsContent);
		var messageHolder = document.createElement("p");
		var textEl = document.createTextNode(this._messages.noResults);
		messageHolder.appendChild(textEl);
		this._searchResultsContent.appendChild(messageHolder);	
		this._persistUserSelections();
	},
	setLoadingMsg: function(){
		dojox.data.dom.removeChildren(this._searchResultsContent);
		var messageHolder = document.createElement("p")
		messageHolder.style.color="#bbbbbb";
		var textEl = document.createTextNode(LOADING_MESSAGE);
		messageHolder.appendChild(textEl);
		this._searchResultsContent.appendChild(messageHolder);		
	},
	_composeCollectionPostQuery: function() {		
		if(!this._selectedProject)return null;	
		var queryInfo = null;
		var query = "contentType=\"x-com.ibm.rdm.collection/rdf+xml\""; 
		var namespaces = null;	 
		queryInfo = {
				query: query,
				namespaces: namespaces,
				projectURI: this._selectedProject,
				extendedQueryMode: true,
				cancelDeferreds: false
		};
		return queryInfo;
		
	}
	
	});
})();
