/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.results.ResourceAnchor");

dojo.require("com.ibm.rdm.web.search.searchutils");
dojo.require("com.ibm.rdm.web.search.SearchEventTopics");
dojo.require("com.ibm.rdm.web.search.searchutils");


dojo.declare(
    "com.ibm.rdm.web.search.results.ResourceAnchor",
    dijit._Widget,
    {
        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        anchorTagClassAttr: null, // the CSS class name that the "a" tag is to use
        href: null,               // the href to use in the "a" tag
        text: null,               // the text to display in the "a" tag

        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        _isRCP: false,

        ///////////////////////////////////////////////////////////////////////////////////
        // API

        postCreate: function() {
            if (com.ibm.rdm.web.search.searchutils.isRCP()) {
                this._isRCP = true;
                this.connect(this.domNode, "onclick", this._handleLinkClicked);
                this.connect(this.domNode, "onmouseover", this._handleLinkMouseOver);
                this.connect(this.domNode, "onmouseout", this._handleLinkMouseOut);
            }
        },

        buildRendering: function(){
            var link = document.createElement("a");
            link.className = this.anchorTagClassAttr;
            // Note: I needed to add the bogus span in the following
            // appendChild call due to the fact that IE prefers to display
            // using the 'href' attr instead of the text node if the text
            // node starts with "http" (as some of ours do ...).
            link.appendChild(com.ibm.rdm.web.search.domutils.createSpannedTextNode("", this.text));
            link.setAttribute("href", this.href);
            this.domNode = link;
        },

        destroy: function() {
            this.inherited(arguments);
        },

        ///////////////////////////////////////////////////////////////////////////////////
        // Helpers

        _handleLinkClicked: function(event) {
            if (this._isRCP) {
                this._sendStatus(event, com.ibm.rdm.web.search.SearchEventTopics.RESOURCE_SELECTED_TOPIC);
            } else {
                // TODO At some point we'll probably want to use this in the web ui as well ...
            }
        },

        _handleLinkMouseOver: function(event) {
            this._sendStatus(event, com.ibm.rdm.web.search.SearchEventTopics.RESOURCE_MOUSE_OVER);
        },

        _handleLinkMouseOut: function(event) {
            this._sendStatus(event, com.ibm.rdm.web.search.SearchEventTopics.RESOURCE_MOUSE_OUT);
        },

        /**
         * Send the status message to the embedded web browser inside the rich client.
         */
        _sendStatus: function(event, eventType) {
            event.preventDefault();
            event.stopPropagation();
            var href = this.domNode.href;
            var params = {};
            params[com.ibm.rdm.web.search.SearchEventConstants.JSEVENT_RESOURCE_ID] = href;
            com.ibm.rdm.web.search.searchutils.triggerEvent(eventType, params);
            return false;
        }
    }
);

})();
