/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.results.ResourceInfo");

dojo.require("dojo.date.stamp");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.search.searchutils");
dojo.require("com.ibm.rdm.web.search.results.SecondaryResourceEntriesTracker");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("dojo.date");
dojo.require("dojo.date.stamp");

var queryutils = com.ibm.rdm.web.util.queryutils;
var searchconstants = com.ibm.rdm.web.common.MimeTypes;
var domutils = com.ibm.rdm.web.util.domutils;

/**
 * This class keeps track of all JRS provided resource related information about
 * a particular resource.
 */
dojo.declare(
    "com.ibm.rdm.web.search.results.ResourceInfo",
    null,
    {
        ///////////////////////////////////////////////////////////////////////////////////
        // Public fields

        resourceName: null,               // E.g. "Doc1" (_not_ URI encoded)
        displayName: null,
        projectName: null,		          // E.g. "CD Classics"
        description: null,
        resourceURI: null,                // E.g. "/jazz/resources/rdm/resources/rrc80"
		wrappedResourceURI: null,		  // E.g. "/jazz/resources/rdm/wrapperResources/rrc_w9" (only set when 'this' represents a wrapper)
        lastModifiedBy: null,             // E.g. "/jazz/users/Bob"
        lastModifiedDate: null,           // A Date object
        etag: null,
        summary: null,                    // The contents of the 'summary' atom entry element (may be null)
        contentType: null,                // E.g. "application/x-com.ibm.rdm.richtext+xml"
        contentLength: null,              // E.g. "2134"
		isXQueryBased: true,			  // Whatever that was passes in keywordParameters.isXQuery in ctor
		isExtendedResource: null,
		reqProTag: null,
		linkToReqPro: null,
		modified: null,
		contributor: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        _documentTypes: null,
        /**
         * The value for each field in this object represents a "properties object".
         * Each of these "properties objects" has fields representing doctype field names
         * (and asking for the field value results in getting the appropriate doc type
         * field value).
         */
        _docTypeNamespaceToDocTypeProperties: null,
        _secondaryResourceLabelToArrayOfMatchEntryProps: null,
        _secondaryResourceEntriesTracker: null,
        _namespaceBasedQueryKeysToLabels: null,
        _matchLabelToValuesAreSentenceBased: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // API

        /**
         * keywordParameters.atomEntryNode ... an atom entry node from a "query" or "search"
         *     results feed
         * keywordParameters.baseURI ... uri links are relative to
         * keywordParameters.documentTypes ... a DocumentTypes object
         * keywordParameters.secondaryResourceEntriesTracker ... a
         *     SecondaryResourceEntriesTracker object
         * keywordParameters.secondaryResourceQueryParams ... an array of query params
         *     representing all query params that are for secondary resources
         * keywordParameters.isXQuery ... true iff the document containing the
         *     keywordParameters.atomEntryNode was created via a request made with POST
         *     (with XQuery post data) rather than a GET.
		 * keywordParameters.wrapperInfo ... if passed, we ignore the passed atomEntryNode
		 *     and instead pull the relevant info out of the this property list.
         *
         * The original query or search might have been something like this:
         *   http://localhost:9080/jazz/projects/rdm/query?resource-uri=Doc10
         * or
         *   http://localhost:9080/jazz/projects/rdm/search?q=custom
         * 
         * The passed keywordParameters.atomEntryNode will either have a "query" format:
// TODO Remove -- this is how things were before M6 JRS drop ...
//          *   <entry>
//          *     <title>rdm/Doc10</title>
//          *     <author><name>/jazz/users/Bob</name></author>
//          *     <summary></summary>
//          *     <id>_ZbmJhNDSEdyXQsYC1NlwXw</id>
//          *     <updated>2008-02-01T10:09:49-05:00</updated>
//          *     <link rel="alternate" href="/jazz/resources/rdm/Doc10"/>
//          *     <link rel="self" href="/jazz/resources/rdm/Doc10"/>
//          *     <link rel="http://jazz.net/xmlns/v0.6#parent" href="null"/>
//          *     <content type="application/x-com.ibm.rdm.richtext+xml" src="/jazz/resources/rdm/Doc10"/>
//          *   </entry>
         * in GET results format ...
         *   <entry>
         *     <title>rdm/resources/res.media_3</title>
         *     <author><name>/jazz/users/Bob</name></author>
         *     <summary></summary>
         *     <id>_60iWmSd2Ed21_NAL71EYPg</id>
         *     <updated>2008-05-23T10:35:44-04:00</updated>
         *     <link rel="alternate" href="/jazz/resources/rdm/resources/res.media_3"/>
         *     <link rel="self" href="/jazz/resources/rdm/resources/res.media_3"/>
         *     <link rel="http://jazz.net/xmlns/v0.6#parent" href="rdm/resources"/>
         *     <rdf:Description
         *       xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
         *       xmlns:jzindex="http://jazz.net/xmlns/properties/v0.6#"
         *       rdf:about="/jazz/resources/rdm/resources/res.media_3">
         *       <name xmlns="http://schema.ibm.com/rdm/2008/Base#">Sketch100</name>
         *     </rdf:Description>
         *     <content
         *       type="application/x-com.ibm.sid.sketch+xml;charset=UTF-8"
         *       src="/jazz/resources/rdm/resources/res.media_3"/>
         *   </entry>
         * or in POST results format (i.e. XQuery) ...
         *   <entry>
         *     <title>Row 3</title> 
         *     <author><name>System</name></author>
         *     <id>3</id> 
         *     <updated>2008-05-27T16:02:55-04:00</updated> 
         *     <content type="application/rdf+xml">
         *       <rdf:Description
         *           xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
         *           xmlns:dcterms="http://purl.org/dc/terms/"
         *           rdf:about="/jazz/resources/rdm/resources/res.media_3">
         *         <dcterms:contributor rdf:resource="/jazz/users/Bob" /> 
         *         <dcterms:creator rdf:resource="/jazz/users/Bob" /> 
         *         <resource-collection xmlns="http://jazz.net/xmlns/properties/v0.6#" rdf:resource="/jazz/resources/rdm/resources" /> 
         *         <resource-entry xmlns="http://jazz.net/xmlns/properties/v0.6#" rdf:resource="/jazz/resources/rdm/resources/res.entry_3" /> 
         *         <owning-team xmlns="http://jazz.net/xmlns/properties/v0.6#" rdf:resource="/jazz/projects/rdm/teams/Proj2" /> 
         *         <dcterms:format>application/x-com.ibm.sid.sketch+xml;charset=UTF-8</dcterms:format> 
         *         <archived xmlns="http://jazz.net/xmlns/properties/v0.6#" rdf:datatype="http://www.w3.org/2001/XMLSchema#boolean">false</archived> 
         *         <dcterms:modified rdf:datatype="http://www.w3.org/2001/XMLSchema#dateTime">2008-05-23T10:35:44-04:00</dcterms:modified> 
         *         <dcterms:created rdf:datatype="http://www.w3.org/2001/XMLSchema#dateTime">2008-05-21T16:45:59-04:00</dcterms:created> 
         *         <name xmlns="http://schema.ibm.com/rdm/2008/Base#">Sketch100</name> 
         *         <rdf:type rdf:resource="http://www.ibm.com/SID/Sketch/0.1#Document" /> 
         *       </rdf:Description>
         *     </content>
         *   </entry>
         * or it will have a "search" format:
         *   <entry>
         *     <title>rdm/Doc10</title>
         *     <author><name>/jazz/users/Bob</name></author>
         *     <summary></summary>
         *     <id>_ZbmJhNDSEdyXQsYC1NlwXw</id>
         *     <updated>2008-02-01T10:09:49-05:00</updated>
         *     <link rel="alternate" href="/jazz/resources/rdm/Doc10"/>
         *     <link rel="self" href="/jazz/resources/rdm/Doc10"/>
         *     <link rel="http://jazz.net/xmlns/v0.6#parent" href="null"/>
         *     <jrs:score xmlns:jrs="http://jazz.net/xmlns/v0.6">37735</jrs:score>
         *     <content type="application/x-com.ibm.rdm.richtext+xml" src="/jazz/resources/rdm/Doc10"/>
         *   </entry>
         */
	    constructor: function(/* Object */ keywordParameters) {
	        this.isXQueryBased = keywordParameters.isXQuery;
	        this.isExtendedResource = keywordParameters.isExtendedResource;

	        if (keywordParameters.isXQuery) {
	        	this._parseXQueryAtomEntryNode(keywordParameters.atomEntryNode);
	        } else {
	        	this._parseQueryOrSearchAtomEntryNode(keywordParameters.atomEntryNode, keywordParameters.baseURI);
	        }
	        
            this._documentTypes = keywordParameters.documentTypes;
            this._secondaryResourceEntriesTracker = keywordParameters.secondaryResourceEntriesTracker;
            this._namespaceBasedQueryKeysToLabels = new Object;
            this._matchLabelToValuesAreSentenceBased = new Object;
            if (keywordParameters.secondaryResourceQueryParams) {
                this._initSecondaryResourceAttributes(keywordParameters.secondaryResourceQueryParams);
            }
            this._docTypeNamespaceToDocTypeProperties = new Object;
        },

        addResourceHeadInfo: function(ioArgs) {
            this.contentLength = ioArgs.xhr.getResponseHeader("Content-Length");
        },

        /**
         * Parse the relevant content out of a resource's index properties response.
         * Here's an example of the content that might be parsed:
         *   <jzindex:properties xmlns:jzindex="http://jazz.net/xmlns/properties/v0.6">
         *       <jzindex:uriProperties>
         *           <jzindex:property name="resource-uri">/jazz/resources/rdm/S1</jzindex:property>
         *       </jzindex:uriProperties>
         *       <jzindex:stringProperties>
         *           <jzindex:property name="resource-content-type">application/x-com.ibm.sid.sketch+xml</jzindex:property>
         *           <jzindex:property name="resource-last-modified-by">/jazz/users/Bob</jzindex:property>
         *           <jzindex:property name="resource-root-element">http://www.ibm.com/SID/Sketch/0.1#Document</jzindex:property>
         *           <jzindex:property name="http://com.ibm.rdm/SUPP#Priority">Won't</jzindex:property>
         *           <jzindex:property name="http://com.ibm.rdm/SUPP#Stability">High</jzindex:property>
         *           <jzindex:property name="http://com.ibm.rdm/SUPP#Difficulty">High</jzindex:property>
         *           <jzindex:property name="http://com.ibm.rdm/SUPP#Status">Approved</jzindex:property>
         *           <jzindex:property name="http://com.ibm.rdm/UC#Priority">Could</jzindex:property>
         *           <jzindex:property name="http://com.ibm.rdm/UC#Property">Post Conditions</jzindex:property>
         *           <jzindex:property name="http://com.ibm.rdm/UC#AffectsArchitecture">false</jzindex:property>
         *           <jzindex:property name="http://com.ibm.rdm/UC#Difficulty">High</jzindex:property>
         *           <jzindex:property name="http://com.ibm.rdm/UC#Status">Incorporated</jzindex:property>
         *           <jzindex:property name="http://com.ibm.rdm/FEAT#Priority">Must</jzindex:property>
         *           <jzindex:property name="http://com.ibm.rdm/FEAT#Stability">Low</jzindex:property>
         *           <jzindex:property name="http://com.ibm.rdm/FEAT#Origin">Help Desk</jzindex:property>
         *           <jzindex:property name="http://com.ibm.rdm/FEAT#Difficulty">High</jzindex:property>
         *           <jzindex:property name="http://com.ibm.rdm/FEAT#Contact">bla</jzindex:property>
         *           <jzindex:property name="http://com.ibm.rdm/FEAT#Type">Non-functional</jzindex:property>
         *           <jzindex:property name="http://com.ibm.rdm/FEAT#Status">Proposed</jzindex:property>
         *           <jzindex:property name="http://com.ibm.rdm/REQPRO#ComposerLink">Whatever1</jzindex:property>
         *       </jzindex:stringProperties>
         *       <jzindex:booleanProperties/>
         *       <jzindex:integerProperties/>
         *       <jzindex:timestampProperties>
         *           <jzindex:property name="resource-last-modified">2008-01-28T09:58:09-05:00</jzindex:property>
         *       </jzindex:timestampProperties>
         *   </jzindex:properties>
         * If the resource contains secondary indices, some of the jzindex:stringProperties elements
         * might look like this:
         *           <jzindex:property
         *            name="http://schema.ibm.com/rdm/2007/Base#name"
         *            subject="/jazz/resources/rdm/Proj1/test/UC4#/document:UseCaseDocument/document:useCaseContext[0]/usecase:actor[0]">
         *               Arnold
         *           </jzindex:property>
         *           <jzindex:property
         *            name="http://schema.ibm.com/rdm/ba/2007/UseCase#actor"
         *            subject="/jazz/resources/rdm/Proj1/test/UC4#/document:UseCaseDocument/document:useCaseContext[0]/usecase:actor[0]">
         *               Arnold
         *           </jzindex:property>
         *           <jzindex:property
         *            name="http://schema.ibm.com/rdm/2007/Base#id"
         *            subject="/jazz/resources/rdm/Proj1/test/UC4#/document:UseCaseDocument/document:useCaseContext[0]/usecase:actor[0]">
         *               Actor_3
         *           </jzindex:property>
         */
        addResourceIndexPropertiesInfo: function(resourceIndexPropertiesDoc) {
            this._parseResourceIndexProperies(resourceIndexPropertiesDoc);
        },

        getDocTypeNamespacesObject: function() {
            return this._docTypeNamespaceToDocTypeProperties;
        },

        getDocTypePropertiesObject: function(docTypeNamespace) {
            return this._docTypeNamespaceToDocTypeProperties[docTypeNamespace];
        },

        /**
         * The properties of the returned object represent match category
         * display labels (e.g. "Actors") and accessing the value of a particular
         * property will results in returning an array of prop lists.  Here's
         * an example of what you might find in a particular prop list within
         * this array:
         *     {
         *         displayLabel: "Andy"
         *         labelIsSentenceBased: true
         *         href: "http://localhost:9080/jazz/resources/rdm/Proj1/test/UC5#Actor_3"
         *     }
         */
        getSecondaryResourceMatchesObject: function() {
            return this._secondaryResourceLabelToArrayOfMatchEntryProps;
        },

        ///////////////////////////////////////////////////////////////////////////////////
        // Helpers

        /**
         * Init the following:
         *     this._namespaceBasedQueryKeysToLabels
         *     this._matchLabelToValuesAreSentenceBased
         */
	    _initSecondaryResourceAttributes: function(secondaryResourceQueryParams) {
            // TODO Consider moving this functionality into the ContentTypes class instead
            // of defining it here ...
            for (var i = 0; i < secondaryResourceQueryParams.length; i++) {
                var queryParam = secondaryResourceQueryParams[i];
                var index = queryParam.indexOf("=");
                if (index != -1) {
                    queryParamKey = queryParam.substring(0, index);
                    // Only consider keys that are namespace-based by looking for an encoded "#"
                    if (queryParamKey.indexOf("%23")) {
                        var labelToUse = null;
                        decodedQueryParamKey = queryParamKey.replace("%23", "#");
                        var areValuesSentenceBased = false;
                        // TODO nls on all labelToUse in the following ...
                        switch (decodedQueryParamKey) {
                            // Use Case Diagram
                            case "http://schema.ibm.com/rdm/ba/2007/UseCase#actor":
                                labelToUse = "Actors";
                                break;
                            case "http://schema.ibm.com/rdm/ba/2007/UseCase#context":
                                labelToUse = "Contexts";
                                break;
                            case "http://schema.ibm.com/rdm/ba/2007/UseCase#usecase":
                                labelToUse = "Use Cases";
                                break;
                            // Business Process Sketch
                            case "http://schema.ibm.com/rdm/ba/2007/bpmn#pool":
                                labelToUse = "Pools";
                                break;
                            case "http://schema.ibm.com/rdm/ba/2007/bpmn#lane":
                                labelToUse = "Lanes";
                                break;
                            case "http://schema.ibm.com/rdm/ba/2007/bpmn#flowobject":
                                labelToUse = "Flow Objects";
                                break;
                            // Glossary
                            case "http://schema.ibm.com/rdm/ba/2007/Glossary#term":
                                labelToUse = "Terms";
                                break;
                            case "http://schema.ibm.com/rdm/ba/2007/Glossary#definition":
                                labelToUse = "Definitions";
                                areValuesSentenceBased = true;
                                break;
                            // Composite Document or Requirement
                            case "http://www.w3.org/TR/REC-html40#a":
                                labelToUse = "Link Descriptions";
                                break;
                            case "uri:http://www.w3.org/TR/REC-html40#href":
                                labelToUse = "Links";
                                break;
                        }
                        if (labelToUse) {
                            if ("uri:" == decodedQueryParamKey.substring(0, 4)) {
                                decodedQueryParamKey = decodedQueryParamKey.substring(4);
                            }
                            this._namespaceBasedQueryKeysToLabels[decodedQueryParamKey] = labelToUse;
                            if (areValuesSentenceBased) {
                                this._matchLabelToValuesAreSentenceBased[labelToUse] = true;
                            }
                        }
                    }
                }
            }
        },

		/**
		 * Here are some examples of extended resources that might be parsed by
		 * _parseQueryOrSearchAtomEntryNode ...
		 *
		 * A Storyboard ...
		 *   <extendedResource xmlns="http://com.ibm.rdm/resource/extension">
		 *     <reference>/jazz/resources/rdm/resources/rrc100</reference> 
		 *     <name>Story</name> 
		 *     <owning-team>/jazz/projects/rdm/teams/project%20starter</owning-team> 
		 *     <description /> 
		 *     <creator>/jazz/users/ankur</creator> 
		 *     <created>2008-09-09T14:49:42-04:00</created> 
		 *     <contributor>/jazz/users/ankur</contributor> 
		 *     <modified>2008-09-09T14:49:42-04:00</modified> 
		 *     <format>application/x-com.ibm.sid.storyboard+xml</format> 
		 *     <etag>"_EEz5En6gEd2d8pC1hZsETA"</etag> 
		 *     <commentCount>0</commentCount> 
		 *     <commentCount>0</commentCount> 
		 *     <tag tag-bookmark="/jazz/resources/rdm/publicBookmarkLists/2/rrc_bookmark13">/jazz/resources/rdm/publicTagLists/2/rrc_tag18</tag> 
		 *     <tag tag-bookmark="/jazz/resources/rdm/publicBookmarkLists/2/rrc_bookmark13">/jazz/resources/rdm/publicTagLists/2/rrc_tag20</tag> 
		 *     <parent>/jazz/resources/rdm/publicTagLists/2/rrc_tag4</parent> 
		 *     <public-bookmark public-bookmark-etag=""_EPUf836gEd2d8pC1hZsETA"">/jazz/resources/rdm/publicBookmarkLists/2/rrc_bookmark13</public-bookmark> 
		 *   </extendedResource>
		 *
		 * An uploaded Image ...
		 *   <extendedResource xmlns="http://com.ibm.rdm/resource/extension">
		 *     <reference>/jazz/resources/rdm/resources/rrc90</reference> 
		 *     <name>rich-text-doc_40.png</name> 
		 *     <owning-team>/jazz/projects/rdm/teams/Proj2</owning-team> 
		 *     <description /> 
		 *     <creator>/jazz/users/user1</creator> 
		 *     <created>2008-09-17T09:22:19-04:00</created> 
		 *     <contributor>/jazz/users/user1</contributor> 
		 *     <modified>2008-09-17T09:22:19-04:00</modified> 
		 *     <format>image/png</format> 
		 *     <etag>"_p0uMw4S7Ed27mYNMvfu9uQ"</etag> 
		 *     <commentCount>0</commentCount> 
		 *     <commentCount>0</commentCount> 
		 *     <public-bookmark public-bookmark-etag=""_p-1yFYS7Ed27mYNMvfu9uQ"">/jazz/resources/rdm/publicBookmarkLists/3/rrc_bookmark2</public-bookmark> 
		 *     <wrapper-resource wrapper-resource-etag=""_qRRl5YS7Ed27mYNMvfu9uQ"">/jazz/resources/rdm/wrapperResources/rrc_w11</wrapper-resource> 
		 *   </extendedResource>
		 */
        _parseQueryOrSearchAtomEntryNode: function(atomEntryNode, baseURI) {
            var contentNode;
            var summaryNode;
            var lastModifiedJRSDateString;
            var lastModifiedBy;
            var contentType;
            var resourceURI;
            var decodedResourceURI;
            var displayName;
            var projectName;
            var linkToReqPro;
            var reqProTag;
        	if (this.isExtendedResource) {
        		try{
                	var childNodes = atomEntryNode.childNodes;
                	var rdfDescriptionNode;
                	for (var i = 0; i < childNodes.length; i++) {
                		if (childNodes[i].tagName == "rdf:Description") {
                   			rdfDescriptionNode = childNodes[i];
                   			childNodes = childNodes[i].childNodes;
                   			break;
                		}
                	}
                	for (var i = 0; i < childNodes.length; i++) {
                		switch (childNodes[i].tagName) {
                		case "owning-team":
                			projectName = decodeURI(
                				com.ibm.rdm.web.util.queryutils.getLastPathSegment(
                    				com.ibm.rdm.web.util.domutils.getAttributeNS(
                    					dojo.isIE?(childNodes[i]):(childNodes[i]), "resource",
                    					"http://www.w3.org/1999/02/22-rdf-syntax-ns#", "rdf"
                    				)
                    			)
                    		);
                			break;
                		case "etag":
                			if (!this.etag) {
                				var etag = dojo.isIE?(childNodes[i].text):(childNodes[i].textContent);
                				this.etag = etag.substring(1, etag.length -1);
                			}
                			break;
                		case "contributor":
                			lastModifiedBy = com.ibm.rdm.web.util.domutils.getAttributeNS(
                					dojo.isIE?(childNodes[i]):(childNodes[i]), "resource",
                					"http://www.w3.org/1999/02/22-rdf-syntax-ns#", "rdf");
                					
                			break;
                		case "reference":
                			resourceURI = 
                				com.ibm.rdm.web.util.domutils.getAttributeNS(
                    					dojo.isIE?(childNodes[i]):(childNodes[i]), "resource",
                    					"http://www.w3.org/1999/02/22-rdf-syntax-ns#", "rdf");
                			decodedResourceURI = decodeURI(resourceURI);
                			break;
                		case "format": 
                			contentType = dojo.isIE?(childNodes[i].text):(childNodes[i].textContent);
                			break;
                		case "name":
                			displayName = dojo.isIE?(childNodes[i].text):(childNodes[i].textContent);
                			break;
                		case "modified":
                			lastModifiedJRSDateString = dojo.isIE?(childNodes[i].text):(childNodes[i].textContent);
                			break;
                		case "linkToReqPro":
                			linkToReqPro = dojo.isIE?(childNodes[i].text):(childNodes[i].textContent);
                			break;
                		case "reqProTag":
                			reqProTag = dojo.isIE?(childNodes[i].text):(childNodes[i].textContent);
                			break;
                		}
                	}
                	this.extendedResourceURI = com.ibm.rdm.web.util.domutils.getAttributeNS(
                			rdfDescriptionNode, "about",
                			"http://www.w3.org/1999/02/22-rdf-syntax-ns#", "rdf");
//		            lastModifiedJRSDateString = atomEntryNode.getElementsByTagName("modified")[0].firstChild.nodeValue;
//		            lastModifiedBy = atomEntryNode.getElementsByTagName("contributor")[0].firstChild.nodeValue;
//		            contentType = atomEntryNode.getElementsByTagName("format")[0].firstChild.nodeValue;
//		            resourceURI = atomEntryNode.getElementsByTagName("reference")[0].firstChild.nodeValue;
//		            decodedResourceURI = decodeURI(resourceURI);
//		            displayName = atomEntryNode.getElementsByTagName("name")[0].firstChild.nodeValue;
//		            projectName = atomEntryNode.getElementsByTagName("owning-team")[0].firstChild.nodeValue;
//		            var etag = atomEntryNode.getElementsByTagName("etag")[0].firstChild.nodeValue;
//		            this.etag = etag.substring(1, etag.length -1);
        		} catch (e) {
        			console.error(e.toString() + "\nError occured in parsing extended resource in ResourceInfo.js");
        			return;
        		}
        	} else {
// TODO JFS cleanup -- remove this commented out code ...
// 	            contentNode = atomEntryNode.getElementsByTagName("content")[0];
// 	            summaryNode = atomEntryNode.getElementsByTagName("summary")[0].firstChild;
// 	            lastModifiedJRSDateString = atomEntryNode.getElementsByTagName("updated")[0].firstChild.nodeValue;
// 	            lastModifiedBy =
// 	                atomEntryNode.getElementsByTagName("author")[0].getElementsByTagName("name")[0].firstChild.nodeValue;
// 	            contentType = contentNode.getAttribute("type");
// 	            resourceURI = contentNode.getAttribute("src");
// 	            if (com.ibm.rdm.web.search.searchutils.isSecondaryResource(resourceURI)) {
// 	                resourceURI = com.ibm.rdm.web.search.searchutils.getSecondaryResourceParentURI(resourceURI);
// 	            }
// 	            decodedResourceURI = decodeURI(resourceURI);
// 	            displayName = queryutils.getDisplayName(atomEntryNode);
// 	            projectName = decodeURI(com.ibm.rdm.web.util.queryutils.getLastPathSegment(
// 		                queryutils.getProjectNameFromQueryOrPropertiesEntry(atomEntryNode)));
	            contentNode = atomEntryNode.getElementsByTagName("content")[0];
				var artifactNode = contentNode.firstChild;
				// updated
	            var updatedNode = atomEntryNode.getElementsByTagName("updated")[0];
	            lastModifiedJRSDateString = updatedNode.firstChild ? updatedNode.firstChild.nodeValue : "";
	            // author
	            var authorNode = artifactNode.getElementsByTagName("contributor")[0];
	            lastModifiedBy = authorNode.firstChild ? authorNode.firstChild.nodeValue : "";
	            // format
	            var formatNode = artifactNode.getElementsByTagName("format")[0];
	            contentType = (formatNode.firstChild)?formatNode.firstChild.nodeValue:"";
	            // resourceURI
	            var resourceNode = artifactNode.getElementsByTagName("resourceUri")[0];
	            resourceURI = resourceNode.firstChild ? resourceNode.firstChild.nodeValue : "";
	            if ((resourceURI != "") && baseURI) {
        			resourceURI = com.ibm.rdm.web.util.queryutils.getFullyQualifiedUriFromRelativeLink(baseURI, resourceURI);
        		}
	            // name
				var nameNode = artifactNode.getElementsByTagName("name")[0];
				displayName = (nameNode.firstChild ? nameNode.firstChild.nodeValue : "");
        	}
        	this.projectName = projectName;
// TODO JFS cleanup -- remove this commented out code ...
//             this.resourceName = com.ibm.rdm.web.util.queryutils.getLastPathSegment(decodedResourceURI);
//             this.displayName = (displayName ? displayName : this.resourceName);
			if (baseURI && baseURI.indexOf(com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES) == 0) {
				// from the recent feeds
            	this.lastModifiedDate = new Date(dojo.date.stamp.fromISOString(lastModifiedJRSDateString));
			} else {
            	this.lastModifiedDate = new Date(lastModifiedJRSDateString);
			}
			this.displayName = displayName;
            this.resourceURI = resourceURI;
            this.contentType = contentType;
            this.lastModifiedBy = lastModifiedBy;
            this.summary = (summaryNode ? summaryNode.nodeValue : null);
            this.reqProTag = reqProTag;
            this.linkToReqPro = linkToReqPro;
        },

        /*
        <?xml version="1.0" encoding="UTF-8"?>
        <feed xmlns="http://www.w3.org/2005/Atom">
          <title>Query Results</title>
          <updated>2009-04-28T14:27:41-04:00</updated>
           <link rel="self" href="/jazz/query?query=PREFIX+nav%3A+%3Chttp%3A%2F%2Fcom.ibm.rdm%2Fnavigation%23%3E%0APREFIX+b%3A+%3Chttp%3A%2F%2Fjazz.net%2Fxmlns%2Falm%2Frm%2FBase%2Fv0.1%23%3E%0APREFIX+dc%3A+%3Chttp%3A%2F%2Fpurl.org%2Fdc%2Fterms%2F%3E%0ASELECT+%3Fresource+%3Fformat+%3Fname+%3Fcontributor+%3Fcreated+%0AWHERE+%7B%0A++++%3Fresource+dc%3Aformat+%3Fformat+%3B%0A++++++b%3Aname+%3Fname+%3B%0A++++++dc%3Acontributor+%3Fcontributor+%3B%0A++++++dc%3Acreated+%3Fcreated+%3B%0A++++++nav%3Aparent+%22https%3A%2F%2Flocalhost%3A9443%2Frdm%2Fprojects%2Frdm%2Fteams%2Fproject%25201%22+%3B%0A%7D%0A" type="application/atom+xml;type=feed" />
          <generator>Jazz Resource Services</generator>
          <entry>
            <title>Result 0</title>

            <author><name>System</name></author>
            <id></id>
            <updated>2009-04-28T14:27:41-04:00</updated>
            <content type="application/sparql-results+xml">
            <result xmlns="http://www.w3.org/2005/sparql-results#">
        <binding name="resource"><uri>https://localhost:9443/rdm/resources/_DpeA0DQOEd6IesyJ4mj1Dw</uri></binding>
        <binding name="contentType"><literal>application/x-com.ibm.rdm.requirement+xml</literal></binding>
        <binding name="name"><literal>RRRRRRR</literal></binding>

        <binding name="lastModifiedBy"><uri>https://localhost:9443/jazz/users/TestJazzAdmin1</uri></binding>
        <binding name="created"><literal datatype="http://www.w3.org/2001/XMLSchema#dateTime">2009-04-28T16:03:04.669Z</literal></binding>
        </result>
            </content>
          </entry>
          <entry>
            <title>Result 1</title>
            <author><name>System</name></author>

            <id></id>
            <updated>2009-04-28T14:27:41-04:00</updated>
            <content type="application/sparql-results+xml">
            <result xmlns="http://www.w3.org/2005/sparql-results#">
        <binding name="resource"><uri>https://localhost:9443/rdm/resources/_GtucADQOEd6IesyJ4mj1Dw</uri></binding>
        <binding name="contentType"><literal>application/x-com.ibm.rdm.requirement+xml</literal></binding>
        <binding name="name"><literal>RRRRR2</literal></binding>
        <binding name="lastModifiedBy"><uri>https://localhost:9443/jazz/users/TestJazzAdmin1</uri></binding>

        <binding name="created"><literal datatype="http://www.w3.org/2001/XMLSchema#dateTime">2009-04-28T16:03:25.248Z</literal></binding>
        </result>
            </content>
          </entry>
        </feed>
		*/
        _parseXQueryAtomEntryNode: function(atomEntryNode) {
            // TODO: This is not the best place to add comment-only parsing info
            /*try {
                var nodes = domutils.getElementsByTagNameNS(atomEntryNode, "resourceUri", "http://com.ibm.rdm/commenting#");
                this.commentingNsResourceUri  = nodes.item(0).firstChild.nodeValue;
            }
            catch (e) {
            }*/
            var contentType = "";
            var resourceURI = "";
            var decodedResourceURI = "";
            var description = "";
            var lastModifiedBy = "";
            var lastModifiedDate;
            var displayName = "";
            var projectName = "";
            var linkToReqPro;
            var reqProTag;
            var etag;
            var contributor = "";
            var modified = "";
            var reference = "";
            if (this.isExtendedResource) {
    			projectName = domutils.getEntryBindingValueByName(atomEntryNode, "owning-team");
    			etag = domutils.getEntryBindingValueByName(atomEntryNode, "etag");
    			lastModifiedBy = domutils.getEntryBindingValueByName(atomEntryNode, "lastModifiedBy");
    			resourceURI = domutils.getEntryBindingValueByName(atomEntryNode, "resource");
    			contentType = domutils.getEntryBindingValueByName(atomEntryNode, "contentType");
    			displayName = domutils.getEntryBindingValueByName(atomEntryNode, "name");
    			lastModifiedDate = dojo.date.stamp.fromISOString(domutils.getEntryBindingValueByName(atomEntryNode, "lastModified"));
    			linkToReqPro = domutils.getEntryBindingValueByName(atomEntryNode, "linkToReqPro");
    			reqProTag = domutils.getEntryBindingValueByName(atomEntryNode, "reqProTag");
    			contributor = domutils.getEntryBindingValueByName(atomEntryNode, "contributor");
    			modified = domutils.getEntryBindingValueByName(atomEntryNode, "modified");
    			this.extendedResourceURI = "";

            } else {
	            contentType = queryutils.getFormat(atomEntryNode);
	            resourceURI = queryutils.getResourceURI(atomEntryNode);
	            lastModifiedDate = queryutils.getModifiedDate(atomEntryNode);
	            lastModifiedBy = queryutils.getContributor(atomEntryNode);
	            if (com.ibm.rdm.web.search.searchutils.isSecondaryResource(resourceURI)) {
	                resourceURI = com.ibm.rdm.web.search.searchutils.getSecondaryResourceParentURI(resourceURI);
	            }
	            decodedResourceURI = decodeURI(resourceURI);
	            description = queryutils.getDescriptionFromQueryOrPropertiesEntry(atomEntryNode);
	            if (description) {
	                description = dojo.trim(description);
	            }
	            displayName = queryutils.getDisplayName(atomEntryNode);
	            projectName = decodeURI(com.ibm.rdm.web.util.queryutils.getLastPathSegment(
		                queryutils.getProjectNameFromQueryOrPropertiesEntry(atomEntryNode)));
            }
            this.lastModifiedDate = lastModifiedDate;
            this.resourceName = com.ibm.rdm.web.util.queryutils.getLastPathSegment(decodedResourceURI);
            this.displayName = (displayName ? displayName : this.resourceName);
            this.projectName = projectName;
            this.description = description;
            this.resourceURI = resourceURI;
            this.contentType = contentType;
            this.lastModifiedBy = lastModifiedBy;
            this.summary = null;
            this.linkToReqPro = linkToReqPro;
            this.reqProTag = reqProTag;
            this.modified = modified;
            this.contributor = contributor;
        },

        _parseResourceIndexProperies: function(resourceIndexPropertiesDoc) {
            var doctypePropNodes = this._getDocTypePropertyNodes(resourceIndexPropertiesDoc);
            this._initSecondaryResourceLabelToArrayOfMatchEntryProps(resourceIndexPropertiesDoc);
            if (doctypePropNodes.length > 0) {
                var lastDocType = null;
                var props = null;
                for (var i = 0; i < doctypePropNodes.length; i++) {
                    var docPropNode = doctypePropNodes[i];
                    var nameAttr = docPropNode.getAttribute("name");
                    var hashIndex = nameAttr.lastIndexOf("#");
                    var docType = nameAttr.substring(0, hashIndex);
                    var docTypeAttrName = nameAttr.substring(hashIndex+1);
                    var firstChild = docPropNode.firstChild;
                    var docTypeAttrValue = (
                        firstChild ? firstChild.nodeValue : "");
                    if (lastDocType == null || docType != lastDocType) {
                        if (lastDocType != null) {
                            this._addDocTypeProperties(lastDocType, props);
                        }
                        lastDocType = docType;
                        props = new Array;
                        props[docTypeAttrName] = docTypeAttrValue;
                    } else {
                        props[docTypeAttrName] = docTypeAttrValue;
                    }
                    if (i == doctypePropNodes.length-1) {
                        this._addDocTypeProperties(docType, props);
                    }
                }
            }
            // Note: In the case of full text searches, there is currently no way
            // to specify that we'd like the returned atom entries to include the
            // resource display name.  This is due to the fact that our full
            // text queries must be of the following format (where in this case
            // we're seaching for "end"):
            //     http://localhost:9080/jazz/projects/rdm/search?q=end
            // when we'd prefer to use something like this instead:
            //     http://localhost:9080/jazz/projects/rdm/search?q=end&properties=http://schema.ibm.com/rdm/2008/Base%23name
            // in order to make sure that the display name is available without having to
            // issue a subsequent query such as:
            //     http://localhost:9080/jazz/resources/rdm/resources/res.media_4?properties
            // for each returned entry.  Due to the fact that the subsequent query
            // for each entry _is_ currently needed, this function
            // (_parseResourceIndexProperies) accounts for resetting the
            // 'displayName' attribute and the caller patches things up
            // appropriately so that the search results show the appropriate
            // resource name (i.e. the display name instead of the raw JRS resource
            // name).
            var displayName = queryutils.getDisplayName(resourceIndexPropertiesDoc);
            if (displayName && this.displayName != displayName) {
                this.displayName = displayName;
            }
        },

        _addDocTypeProperties: function(docTypeNamespace, propertiesObject) {
            this._docTypeNamespaceToDocTypeProperties[docTypeNamespace] = propertiesObject;
        },

        _getDocTypePropertyNodes: function(resourceIndexPropertiesDoc) {
            var doctypePropertyNodes = [];
            // Note: IE doesn't have a getElementsByTagNameNS and I'm not sure what Safari has
            // so as a work around we'll get all descendants and check their tagNames ...
            var descendantNodes = resourceIndexPropertiesDoc.getElementsByTagName("*");
            for (var i = 0; i < descendantNodes.length; i++) {
                var node = descendantNodes[i];
                if (node.nodeName == "jzindex:property") {
                    var name = node.getAttribute("name");
                    if (this._documentTypes.isDocTypeProperty(name)) {
                        doctypePropertyNodes.push(node);
                    }
                }
            }
            return doctypePropertyNodes;
        },

        _initSecondaryResourceLabelToArrayOfMatchEntryProps: function(resourceIndexPropertiesDoc) {
            // The following var contains fields representing all the resourceURI of all
            // matched secondary resources from the original query's feed results.
            var matchedSecondaryResourceURIsObject =
                this._secondaryResourceEntriesTracker.getMatchedSecondaryResourceURIsObject();
            var secondaryResourceLabelToArrayOfMatchEntryProps = new Object;
            var mapSecondaryResourceURIToAnchorNode = new Object;
            var mapSecondaryResourceURIToType = new Object;
            var matchEntries = [];
            // Note: IE doesn't have a getElementsByTagNameNS and I'm not sure what Safari has
            // so as a work around we'll get all descendants and check their tagNames ...
            var descendantNodes = resourceIndexPropertiesDoc.getElementsByTagName("*");
            for (var i = 0; i < descendantNodes.length; i++) {
                var node = descendantNodes[i];
                // Note we do the node.firstChild check to avoid the following index property case that
                // occurs when a Glossary contains no terms ...
                //   <jzindex:property name="http://schema.ibm.com/rdm/ba/2007/Glossary#term"
                //     subject="/jazz/resources/rdm/Proj2/My%20Stuff/G17#/document:GlossaryDocument/document:glossary[0]/glossary:term[0]"> </jzindex:property>
                // Here's an example of an "empty" Glossary which, for unknown reasons, contains the following term:
                //   <document:glossary base:id="Glossary_2" base:name="G17">
                //     <glossary:term base:id="Term_3" base:name=" "/>
                //   </document:glossary>
                if (node.nodeName == "jzindex:property" && node.firstChild) {
                    var name = node.getAttribute("name");
                    var labelToUse = this._namespaceBasedQueryKeysToLabels[name];
                    var subject = node.getAttribute("subject");
                    if (labelToUse && matchedSecondaryResourceURIsObject[subject]) {
                        var array = secondaryResourceLabelToArrayOfMatchEntryProps[labelToUse];
                        var valuesAreSentenceBased = this._matchLabelToValuesAreSentenceBased[labelToUse];
                        if (!array) {
                            array = new Array;
                            secondaryResourceLabelToArrayOfMatchEntryProps[labelToUse] = array;
                        }
                        var matchEntryProps = null;
                        matchEntryProps = {
                            displayLabel:node.firstChild.nodeValue,
                            labelIsSentenceBased:valuesAreSentenceBased,
                            secondaryResourceURI:subject,
                            href:null
                        };
                        matchEntries.push(matchEntryProps);
                        array.push(matchEntryProps);
                    }
                    if (name == "http://schema.ibm.com/rdm/2007/Base#id" && matchedSecondaryResourceURIsObject[subject]) {
//                         console.log("Adding mapping name:" + subject + " => value:" + node.firstChild.nodeValue);
                        mapSecondaryResourceURIToAnchorNode[subject] =
                            node.firstChild.nodeValue;
                    } else if (name == "http://www.w3.org/TR/REC-html40#a" && matchedSecondaryResourceURIsObject[subject]) {
                        mapSecondaryResourceURIToAnchorNode[subject] =
                            node.firstChild.nodeValue;
                    } else if (name == "http://www.w3.org/2001/XMLSchema-instance#type") {
                        mapSecondaryResourceURIToType[subject] =
                            node.firstChild.nodeValue;
                    }
                }
            }
            for (var i = 0; i < matchEntries.length; i++) {
                var matchEntryProps = matchEntries[i];
                var secondaryResourceURI = matchEntryProps.secondaryResourceURI;
                var anchor = mapSecondaryResourceURIToAnchorNode[secondaryResourceURI];
                var parentResourceURI = com.ibm.rdm.web.search.searchutils.getSecondaryResourceParentURI(secondaryResourceURI);
                // Here are a couple examples of what the href might be set to:
                // - http://localhost:9080/jazz/resources/rdm/Proj2/Req3#Yahoo
                // - http://localhost:9080/jazz/resources/rdm/Proj2/UC#Andrew
                matchEntryProps.setAttribute("href", parentResourceURI);
            }
            this._secondaryResourceLabelToArrayOfMatchEntryProps =
                this._replaceFlowObjectsWithAppropriateTypedObjects(
                    secondaryResourceLabelToArrayOfMatchEntryProps,
                    mapSecondaryResourceURIToType);
        },

        _replaceFlowObjectsWithAppropriateTypedObjects: function(secondaryResourceLabelToArrayOfMatchEntryProps, mapSecondaryResourceURIToType) {
            var flowObjectsArray = secondaryResourceLabelToArrayOfMatchEntryProps["Flow Objects"];
            var tasks = [];
            var startEvents = [];
            var intermediateEvents = [];
            var endEvents = [];
            var subProcesses = [];
            var gateways = [];
            if (flowObjectsArray) {
                for (var i = 0; i < flowObjectsArray.length; i++) {
                    var matchEntryProps = flowObjectsArray[i];
                    var secondaryResourceURI = matchEntryProps.secondaryResourceURI;
                    switch (mapSecondaryResourceURIToType[secondaryResourceURI]) {
                        case "bpmn:Task":
                            tasks.push(matchEntryProps);
                            break;
                        case "bpmn:StartEvent":
                            startEvents.push(matchEntryProps);
                            break;
                        case "bpmn:IntermediateEvent":
                            intermediateEvents.push(matchEntryProps);
                            break;
                        case "bpmn:EndEvent":
                            endEvents.push(matchEntryProps);
                            break;
                        case "bpmn:SubProcess":
                            subProcesses.push(matchEntryProps);
                            break;
                        case "bpmn:DataBasedExclusiveGateway":
                        case "bpmn:ParallelGateway":
                        case "bpmn:InclusiveGateway":
                            gateways.push(matchEntryProps);
                            break;
                    }
                }
            }
            delete secondaryResourceLabelToArrayOfMatchEntryProps["Flow Objects"];
            if (tasks.length > 0) {
                secondaryResourceLabelToArrayOfMatchEntryProps["Tasks"] = tasks;
            }
            if (startEvents.length > 0) {
                secondaryResourceLabelToArrayOfMatchEntryProps["Start Events"] = startEvents;
            }
            if (intermediateEvents.length > 0) {
                secondaryResourceLabelToArrayOfMatchEntryProps["Intermediate Events"] = intermediateEvents;
            }
            if (endEvents.length > 0) {
                secondaryResourceLabelToArrayOfMatchEntryProps["End Events"] = endEvents;
            }
            if (subProcesses.length > 0) {
                secondaryResourceLabelToArrayOfMatchEntryProps["Sub-Processes"] = subProcesses;
            }
            if (gateways.length > 0) {
                secondaryResourceLabelToArrayOfMatchEntryProps["Gateways"] = gateways;
            }
            return secondaryResourceLabelToArrayOfMatchEntryProps;
        }
    }
);

com.ibm.rdm.web.search.results.ResourceInfo.isCommentResource = function(atomEntryNode) {
    var isComment = false;
    // TODO Shouldn't these comment resources have their own special content type?
    // As it stands right now, I've got to check the resource name as well as the
    // content type to be confident that the entry REALLY represents a comment ...
    var looksLikeCommentResourceName = function(atomEntryNode) {
        var doesLookLikeCommentResourceName = false;
        var title = atomEntryNode.getElementsByTagName("title")[0].firstChild.nodeValue;
        var lastSlashIndex = title.lastIndexOf("/");
        if (lastSlashIndex != -1) {
            var segmentStartIndex = lastSlashIndex+1;
            doesLookLikeCommentResourceName =
                (segmentStartIndex == title.indexOf("comment", segmentStartIndex));
        }
        return doesLookLikeCommentResourceName;
    };
    if (looksLikeCommentResourceName(atomEntryNode)) {
        var contentNode = atomEntryNode.getElementsByTagName("content")[0];
        var typeAttr = contentNode.getAttribute("type");
        if (0 == typeAttr.indexOf("application/atom+xml")) {
            isComment = true;
        }
    }
    return isComment;
};

com.ibm.rdm.web.search.results.ResourceInfo.isAtomResource = function(atomEntryNode) {
    var contentNode = atomEntryNode.getElementsByTagName("content")[0];
    var typeAttr = contentNode.getAttribute("type");
    return (0 == typeAttr.indexOf("application/atom+xml"))
};

com.ibm.rdm.web.search.results.ResourceInfo.isDirectoryResource = function(atomEntryNode) {
    var contentNode = atomEntryNode.getElementsByTagName("content")[0];
    var contentType = contentNode.getAttribute("type");
    return (
        contentType == searchconstants.CONTENT_TYPE_FOLDER
        || contentType == searchconstants.CONTENT_TYPE_FOLDER_PROJECT
    );
};

com.ibm.rdm.web.search.results.ResourceInfo.isAttributeGroupResource = function(atomEntryNode) {
    var contentNode = atomEntryNode.getElementsByTagName("content")[0];
    var contentType = contentNode.getAttribute("type");
    return (
//         contentType == searchconstants.CONTENT_TYPE_DOCTYPE
//         || contentType == searchconstants.CONTENT_TYPE_REQ_DOCTYPE
           contentType == searchconstants.ATTRIBUTE_GROUP
    );
};

/**
 * Return true iff the atomEntryNode "matches" the userQueryString.
 * For example, if the userQueryString is "My UC*" we'd return true for such
 * entry node titles as:
 *   - "rdm/ProjectA/My%20UC1"
 *   - "rdm/ProjectB/test/My%20UC2"
 * or, in the case of a secondary-resource query:
 *   - "rdm/ProjectB/test/My%20UC2#/document:UseCaseDocument/document:useCaseContext[0]/usecase:actor[0]
 */
com.ibm.rdm.web.search.results.ResourceInfo.isResourceNameMatch = function(atomEntryNode, userQueryString) {
    var decodedTitle = decodeURI(atomEntryNode.getElementsByTagName("title")[0].firstChild.nodeValue);
    var endIndex = decodedTitle.lastIndexOf("#"); // We do this to handle secondary-resources
    if (endIndex == -1) { endIndex = decodedTitle.length };
    var lastSlashIndex = decodedTitle.lastIndexOf("/", endIndex);
    var name = (lastSlashIndex != -1 ?
        decodedTitle.substring(lastSlashIndex+1, endIndex) :
        decodedTitle.substring(0, endIndex));
    var wildcardIndex = userQueryString.lastIndexOf("*");
    if (wildcardIndex != -1) {
        var rootNameToMatch = userQueryString.substring(0, wildcardIndex);
        return (name.substring(0, rootNameToMatch.length).toLowerCase() == rootNameToMatch.toLowerCase());
    } else {
        return (name.toLowerCase() == userQueryString.toLowerCase());
    }
};

com.ibm.rdm.web.search.results.ResourceInfo.isRequirementResource = function(atomEntryNode) {
		var isReq = false;
		// try XQuery
		isReq = (searchconstants.CONTENT_TYPE_REQUIREMENT == queryutils.getFormat(atomEntryNode));
		if (isReq)
			return isReq;
		
		// try GET based query 
	    var contentNode = atomEntryNode.getElementsByTagName("content")[0];
	    var contentType = contentNode.getAttribute("type");
	    isReq = (contentType == searchconstants.CONTENT_TYPE_REQUIREMENT);
	    return isReq;
	    
};

com.ibm.rdm.web.search.results.ResourceInfo.isWrappedContentType = function(atomEntryNode) {
	var contentType = queryutils.getContent(atomEntryNode);
	for (var i = 0; i < com.ibm.rdm.web.common.MimeTypes.WRAPPERS.length; i++) {
		if (com.ibm.rdm.web.common.MimeTypes.WRAPPERS[i].contentType === contentType) {
			return true;
		}
	}
	return false;
};

com.ibm.rdm.web.search.results.ResourceInfo.isWrapper = function(atomEntryNode) {
	var contentType = queryutils.getContent(atomEntryNode);
	return (contentType === com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_UPLOADED);
};

})();
