/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.results.SearchHeader");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rdm.web.search.domutils");
dojo.require("com.ibm.rdm.web.search.searchutils");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.common", "common");
dojo.requireLocalization("com.ibm.rdm.web.search", "common");
dojo.requireLocalization("com.ibm.rdm.web.search.results", "SearchHeader");

dojo.declare(
    "com.ibm.rdm.web.search.results.SearchHeader",
    [dijit._Widget, dijit._Templated],
    {
        templatePath: dojo.moduleUrl("com.ibm.rdm.web", "search/results/templates/SearchHeader.html"),
        widgetsInTemplate: true,

        ///////////////////////////////////////////////////////////////////////////////////
        // Attach points

        _summaryTitleSpan: null,
        _showDetailsCheckBox: null,
        _sortBySelect: null,
        _viewTypeSelect: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Public fields

        doShowDetails: true,
        searchResults: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        SORTBY_DOC_NAME_SELECT_VALUE: "docName",
        SORTBY_LAST_MODIFIED_SELECT_VALUE: "lastModified",
        SORTBY_TYPE_SELECT_VALUE: "type",

        VIEWTYPE_LIST: "listViewType",
        VIEWTYPE_DETAILS: "detailsViewType",

        _messages: null,
        _commonMessages: null,
        _commonSearchMessages: null,
        _resultsTable: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // NLS fields requiring translation

        sortBy: null,
        showDetails: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // API

        postCreate: function() {
            this._populateSortBySelect();
            this._populateViewTypeSelect();
//             this._showDetailsCheckBox.setChecked(this.doShowDetails);
            this.connect(this._sortBySelect, "onchange", this._handleSortByValueChange);
            this.connect(this._viewTypeSelect, "onchange", this._handleViewTypeValueChange);
//             this.connect(this._showDetailsCheckBox, "onClick", this._handleShowDetailsCheckBoxClick);
        },

        postMixInProperties: function() {
            this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search.results", "SearchHeader");
			this._commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
            this._commonSearchMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.search", "common");
            dojo.forEach(["sortBy"], function(prop){
                this[prop] = this._commonSearchMessages[prop];
            }, this);
            dojo.forEach(["showDetails"], function(prop){
                this[prop] = this._messages[prop];
            }, this);
        },

        informUserThatSearchIsInProgress: function() {
            var span = com.ibm.rdm.web.search.domutils.createSpannedTextNode(
                "search-results-search-in-progress-label",
                this._messages.searching);
            this._updateSummaryTitleSpan(span);
        },
        
		informUserNoSearchCriterion: function() {
            var span = com.ibm.rdm.web.search.domutils.createSpannedTextNode(
                "search-results-summary-label",
                this._messages.noSearchCriterion);
            this._updateSummaryTitleSpan(span);
		},
		
        updateSummary: function(summaryTitle) {
            var textNode = document.createTextNode(summaryTitle);
            this._updateSummaryTitleSpan(textNode);
        },

        setResultsTable: function(resultsTable) {
            this._resultsTable = resultsTable;
        },

        handleTableContentsHasChanged: function() {
            this._performSort();
        },
        
        getSortOrder: function() {
        	var sortBy = null;
        	var selection = this._sortBySelect.options[this._sortBySelect.selectedIndex].value;
            if (selection == this.SORTBY_LAST_MODIFIED_SELECT_VALUE) {
            	sortBy = com.ibm.rdm.web.search.SearchQuery.sortBy.DATE;
            } else if (selection == this.SORTBY_DOC_NAME_SELECT_VALUE) {
            	sortBy = com.ibm.rdm.web.search.SearchQuery.sortBy.NAME;
        	} else if (selection == this.SORTBY_TYPE_SELECT_VALUE) {
            	sortBy = com.ibm.rdm.web.search.SearchQuery.sortBy.TYPE;
        	}
            return sortBy;
        },

        ///////////////////////////////////////////////////////////////////////////////////
        // Helpers

        _updateSummaryTitleSpan: function(node) {
            var span = this._summaryTitleSpan;
            if (span.firstChild) {
                span.removeChild(span.firstChild);
            }
            span.appendChild(node);
        },

        _populateSortBySelect: function() {
            var selectNode = this._sortBySelect;
            var sortBySelectOptions = [
                { value: this.SORTBY_DOC_NAME_SELECT_VALUE,      name: this._commonSearchMessages.artifactName },
                { value: this.SORTBY_LAST_MODIFIED_SELECT_VALUE, name: this._messages.lastModified },
                { value: this.SORTBY_TYPE_SELECT_VALUE,          name: this._messages.type }
            ];
            for (var i = 0; i < sortBySelectOptions.length; i++) {
                var optionProps = sortBySelectOptions[i];
                var option = document.createElement("option");
                option.value = optionProps.value;
                option.appendChild(document.createTextNode(optionProps.name));
                selectNode.appendChild(option);
            }
        },

        _populateViewTypeSelect: function() {
            var selectNode = this._viewTypeSelect;
            var viewType = this.searchResults.getViewType();
            var viewTypeSelectOptions = [
                                       // TODO nls (need to get real viewtype label names first)
                { value:this.VIEWTYPE_LIST,    name:"List",    selected:(viewType == this.searchResults.viewTypes.LIST) },
                { value:this.VIEWTYPE_DETAILS, name:"Details", selected:(viewType == this.searchResults.viewTypes.DETAILS) }
            ];
            for (var i = 0; i < viewTypeSelectOptions.length; i++) {
                var optionProps = viewTypeSelectOptions[i];
                var option = document.createElement("option");
                option.value = optionProps.value;
                option.appendChild(document.createTextNode(optionProps.name));
                option.selected = optionProps.selected;
                selectNode.appendChild(option);
            }
        },

        _handleSortByValueChange: function(e) {
            com.ibm.rdm.web.search.searchutils.showAsBusy(true);
            this._performSort();
            com.ibm.rdm.web.search.searchutils.showAsBusy(false);
        },

        _handleViewTypeValueChange: function(e) {
            com.ibm.rdm.web.search.searchutils.showAsBusy(true);
            var selectedOption = this._viewTypeSelect.options[this._viewTypeSelect.selectedIndex];
            var viewTypeValue = selectedOption.value;
            switch (viewTypeValue) {
                case this.VIEWTYPE_DETAILS:
                    this.searchResults.switchViewType(this.searchResults.viewTypes.DETAILS);
                    break;
                case this.VIEWTYPE_LIST:
                    this.searchResults.switchViewType(this.searchResults.viewTypes.LIST);
                    break;
            }
            com.ibm.rdm.web.search.searchutils.showAsBusy(false);
        },

        _performSort: function() {
            var selectedOption = this._sortBySelect.options[this._sortBySelect.selectedIndex];
            var sortByValue = selectedOption.value;
            switch (sortByValue) {
                case this.SORTBY_LAST_MODIFIED_SELECT_VALUE:
                	if (this.searchResults.isFullTextSearch) {
                		this._sortResultsTableByLastModifiedDate();
                		this.searchResults.handleSortHasBeenPerformed();
                	} else {
	                	this.searchResults.handleSort(
	                			com.ibm.rdm.web.search.SearchQuery.sortBy.DATE
	                	);
            		}
                    break;
                case this.SORTBY_DOC_NAME_SELECT_VALUE:
                	if (this.searchResults.isFullTextSearch) {
                		this._sortResultsTableByResourceName();
                		this.searchResults.handleSortHasBeenPerformed();
                	} else {
	                	this.searchResults.handleSort(
	                			com.ibm.rdm.web.search.SearchQuery.sortBy.NAME
	                	);
                	}
                    break;
                case this.SORTBY_TYPE_SELECT_VALUE:
                	if (this.searchResults.isFullTextSearch) {
                		this._sortResultsTableByType();
                		this.searchResults.handleSortHasBeenPerformed();
                	} else {                	
	                	this.searchResults.handleSort(
	                			com.ibm.rdm.web.search.SearchQuery.sortBy.TYPE
	                	);
                	}
                    break;
                case this.SIZE_SELECT_VALUE:
                	if (this.searchResults.isFullTextSearch) {
                		this._sortResultsTableBySize();
                		this.searchResults.handleSortHasBeenPerformed();
                	} else {                	
                		this.searchResults.handleSort();
                	}
                    break;
            }
        },

        _sortResultsTableByResourceName: function() {
            var tbody = this._resultsTable.firstChild;
            // TODO Is there a better way to copy the array than dojo.map?
            var trNodesToSort = dojo.map(tbody.childNodes, function(tr) {return tr;});
            trNodesToSort = trNodesToSort.sort(
                com.ibm.rdm.web.search.results.SearchHeader._compareResourceName);
            this._reorderResultSetTableRows(trNodesToSort);
        },

        _sortResultsTableByLastModifiedDate: function() {
            var tbody = this._resultsTable.firstChild;
            // TODO Is there a better way to copy the array than dojo.map?
            var trNodesToSort = dojo.map(tbody.childNodes, function(tr) {return tr;});
            trNodesToSort.sort(
                com.ibm.rdm.web.search.results.SearchHeader._compareLastModified);
            this._reorderResultSetTableRows(trNodesToSort);
        },

        _sortResultsTableByType: function() {
			var commonMessages = this._commonMessages;
			var typeComparator = function(tr1, tr2) {
				var getContentTypeDisplayName = function(contentType) {
				    switch (contentType) {
				        case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_RICH_TEXT:
							return commonMessages["richTextDocument"];
				        case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_GLOSSARY:
							return commonMessages["glossary"];
				        case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_PART:
							return commonMessages["part"];
				        case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_PROCESS:
							return commonMessages["process"];
				        case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_SCREEN_FLOW:
							return commonMessages["screenFlow"];
				        case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_SKETCH:
							return commonMessages["sketch"];
				        case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_STORY_BOARD:
							return commonMessages["storyBoard"];
				        case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_USE_CASE:
							return commonMessages["useCase"];
				        case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_USE_CASE_DIAGRAM:
							return commonMessages["useCaseDiagram"];
				        case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_ACTOR:
							return commonMessages["actor"];
				        case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_TERM:
							return commonMessages["term"];
				        case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REQUIREMENT:
							return commonMessages["requirement"];
				        default:
				            // Note: We tack on ZZZ_ to make sure it ends up AFTER the artifact types
				            return "ZZZ_" + contentType;
				    }
				};
			    var entryTable1 = com.ibm.rdm.web.search.searchutils.getEntryTable(tr1);
			    var entryTable2 = com.ibm.rdm.web.search.searchutils.getEntryTable(tr2);
			    var resourceInfo1 = entryTable1.getResourceInfo();
			    var resourceInfo2 = entryTable2.getResourceInfo();
			    var contentType1 = resourceInfo1.contentType;
			    var contentType2 = resourceInfo2.contentType;
			    if (contentType1 == contentType2) {
			        return com.ibm.rdm.web.search.results.SearchHeader._compareResourceName(tr1, tr2);
			    } else {
			        var sortName1 = getContentTypeDisplayName(contentType1);
			        var sortName2 = getContentTypeDisplayName(contentType2);
			        return ((sortName1 < sortName2) ? -1 : 1);
			    }
			};
            var tbody = this._resultsTable.firstChild;
            // TODO Is there a better way to copy the array than dojo.map?
            var trNodesToSort = dojo.map(tbody.childNodes, function(tr) {return tr;});
            trNodesToSort.sort(typeComparator);
            this._reorderResultSetTableRows(trNodesToSort);
        },

        _sortResultsTableBySize: function() {
            // TODO Implement
        },

        _reorderResultSetTableRows: function(sortedTRNodes) {
            var tbody = this._resultsTable.firstChild;
            var i;
            var len = tbody.childNodes.length;
            for (i = 0; i < len; i++) {
                tbody.removeChild(sortedTRNodes[i]);
            }
            for (i = 0; i < len; i++) {
                tbody.appendChild(sortedTRNodes[i]);
            }
        },

        _handleShowDetailsCheckBoxClick: function(e) {
            this.searchResults.showDetails(this._showDetailsCheckBox.checked);
        }
    }
);


com.ibm.rdm.web.search.results.SearchHeader._compareResourceName = function(tr1, tr2) {
    var getTableRowResourceInfo = function(tr) {
        var entryTable = com.ibm.rdm.web.search.searchutils.getEntryTable(tr);
        return entryTable.getResourceInfo();
    };
    var resourceName1 = getTableRowResourceInfo(tr1).displayName;
    var resourceName2 = getTableRowResourceInfo(tr2).displayName;
    return ((resourceName1 == resourceName2) ?
        0 :
        ((resourceName1 < resourceName2) ? -1 : 1));
};

com.ibm.rdm.web.search.results.SearchHeader._compareLastModified = function(tr1, tr2) {
    var entryTable1 = com.ibm.rdm.web.search.searchutils.getEntryTable(tr1);
    var entryTable2 = com.ibm.rdm.web.search.searchutils.getEntryTable(tr2);
    var date1 = entryTable1.getLastModifiedDate();
    var date2 = entryTable2.getLastModifiedDate();
    var time1 = date1.getTime();
    var time2 = date2.getTime();
    return ((time1 == time2) ? 0 : ((time1 < time2) ? 1 : -1));
};

})();
