/*
 * Copyright (c) 2007, 2009. IBM Corporation
 * All rights reserved.
 */
(function() {
dojo.provide("com.ibm.rdm.web.search.searchutils");

dojo.require("dojo.date.locale");
dojo.require("com.ibm.rdm.web.search.SearchEventConstants");


com.ibm.rdm.web.search.searchutils = {
    _busyCursorTally: 0,

//     getResourceName: function(jrsResourcePath) {
//         var lastSlashIndex = jrsResourcePath.lastIndexOf("/");
//         return (lastSlashIndex != -1 ? jrsResourcePath.substring(lastSlashIndex+1) : jrsResourcePath);
//     },

    getParentResourceName: function(resourceURI) {
        var lastSlashIndex = resourceURI.lastIndexOf("/");
        var nextToLastSlashIndex = resourceURI.lastIndexOf("/", lastSlashIndex-1);
        return (decodeURI(resourceURI.substring(nextToLastSlashIndex+1, lastSlashIndex)));
    },

    /**
     * E.g.
     *   getProjectResourcePath("/jazz/resources/rdm/Proj1/My%20Doc")
     *   ==> "/Proj1/My Doc"
     */
    getProjectResourcePath: function(resourceURI) {
    	return resourceURI;
    },

    /**
     * Does not work after permalink implementation
     * E.g.
     *   getFolder("/Proj1/My Doc"
     *   ==> "/Proj1/"
     */
    getFolder: function(projectResourcePath) {
    	console.error("searchutils.getFolder() does not work with permalink implementation of resources");
    	return "";
    	
        var index = projectResourcePath.lastIndexOf("/") + 1;
        return projectResourcePath.substring(0, index);
    },

    getDetailsTable: function(tr) {
        var td = tr.lastChild.firstChild;
        var widgetId = td.getAttribute("widgetId");
        return dijit.byId(widgetId);
    },

    getEntryTable: function(tr) {
        var td = tr.childNodes[1].firstChild;
        var widgetId = td.getAttribute("widgetId");
        return dijit.byId(widgetId);
    },

    createDateLabel: function(date) {
        return dojo.date.locale.format(date, { formatLength: "short" });
    },

    getNamedIndexProperty: function(atomFeedPropertiesNode, propertyAttrName, docAsString) {
        // TODO Use a better approach for this ...
        // Work around the following bug:
        // https://jazz.net/jazz/web/projects/Jazz%20Project#action=com.ibm.team.workitem.viewWorkItem&id=40287
        if (atomFeedPropertiesNode.firstChild != null && atomFeedPropertiesNode.firstChild.tagName != "parseerror") {
            try {
                var firstPropertyChild = atomFeedPropertiesNode.firstChild.firstChild;
                for (var child = firstPropertyChild; child != null; child = child.nextSibling) {
                    // TODO Read the default namespace to avoid hardcoded "jzindex" NS in the following?
                    if (child.nodeName == "jzindex:property" && child.getAttribute("name") == propertyAttrName) {
                        return child.firstChild.nodeValue;
                    }
                }
            } catch(e) {
                console.log("Parse error encountered.  Using work around ...");
            }
        } else {
            console.log("Parse error encountered.  Using work around ...");
        }
        // Work around the following bug:
        // https://jazz.net/jazz/web/projects/Jazz%20Project#action=com.ibm.team.workitem.viewWorkItem&id=40287
        var index = docAsString.indexOf('<jzindex:property name="' + propertyAttrName + '"');
        var textStartIndex = docAsString.indexOf(">", index) + 1;
        var textEndIndex = docAsString.indexOf("<", textStartIndex);
        return docAsString.substring(textStartIndex, textEndIndex);
    },

    showAsBusy: function(isBusy) {
        if (isBusy) {
            this._busyCursorTally++;
            if (this._busyCursorTally == 1) {
                document.body.style.cursor = 'progress';
            }
        } else {
            this._busyCursorTally--;
            if (this._busyCursorTally == 0) {
                document.body.style.cursor = 'auto';
            }
        }
    },

    isBusy: function() {
        return (this._busyCursorTally != 0);
    },

    // Note: This function was originally lifted from annotations.html.
    triggerEvent: function(eventType, params) {
        params[com.ibm.rdm.web.search.SearchEventConstants.JSEVENT_EVENT_TYPE] = eventType;
        var qrystr = dojo.objectToQuery(params);
        var eventText = com.ibm.rdm.web.search.SearchEventConstants.JSEVENT_PREFIX + qrystr;
        var clearEventText = (
            com.ibm.rdm.web.search.SearchEventConstants.JSEVENT_PREFIX
            + com.ibm.rdm.web.search.SearchEventConstants.JSEVENT_EVENT_TYPE
            + "="
            + com.ibm.rdm.web.search.SearchEventConstants.JSEVENT_CLEAR_EVENT_CACHE
        );
        window.status = clearEventText;
        window.status = eventText;
    },

    reportError: function(errorMessage) {
        if (com.ibm.rdm.web.search.searchutils.isRCP()) {
            // Send it to the RCP Java code.
            var params = {};
            params[com.ibm.rdm.web.search.SearchEventConstants.JSEVENT_ERROR_MESSAGE] = errorMessage;
            com.ibm.rdm.web.search.searchutils.triggerEvent(
                com.ibm.rdm.web.search.SearchEventTopics.REPORT_ERROR,
                params);
        } else {
            // We're in our Web UI ... so use an alert instead.
            alert(errorMessage);
        }
    },

    isRCP: function() {
        var searchContainerNode = dojo.byId("searchContainer");
        return (searchContainerNode && searchContainerNode.getAttribute("source") == "RCP");
    },

    /**
     * This function performs special treatment on "+" and "=" chars in content type:
     * Example1 ... Use:
     *         "application/x-com.ibm.sid.part%2bxml"
     *     instead of:
     *         "application/x-com.ibm.sid.part+xml"
     * Example2 ... Use:
     *         "application/x-com.ibm.rdm.ba.usecase%2bxml;charset%3DUTF-8"
     *     instead of:
     *         "application/x-com.ibm.rdm.ba.usecase+xml;charset=UTF-8"
     */
    encodeContentType: function(contentType) {
        return contentType.replace("+", "%2b").replace("=", "%3D");
    },

    isSecondaryResource: function(resourceURI) {
        return (resourceURI.indexOf("#") != -1);
    },

    getSecondaryResourceParentURI: function(resourceURI) {
        var index = resourceURI.indexOf("#");
        return resourceURI.substring(0, index);
    },

    stripLeadingSlash: function(path) {
        return (path.charAt(0) == "/" ? path.substring(1) : path);
    }
};

})();
