/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

(function(){
dojo.provide("com.ibm.rdm.web.server.client.RecentFeedsClient");
dojo.require("com.ibm.rdm.web.search.results.ResourceInfo");
dojo.require("com.ibm.rdm.web.server.XhrUtils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.commenting.Comment");
dojo.require("com.ibm.rdm.web.links.LinksWidget");

var ResourceInfo = com.ibm.rdm.web.search.results.ResourceInfo;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var XhrUtils = com.ibm.rdm.web.server.XhrUtils;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;

var MAX_RECENT_ARTIFACTS = 100;
var RECENT_COMMENTS = "/Recent%20Comments";
var RECENT_ARTIFACTS = "/Recent%20Artifacts";
var RECENT_REQUIREMENTS = "/Recent%20Requirements";

com.ibm.rdm.web.server.client.RecentFeedsClient = {
	///////////////////////////////////////////////////////////////////////////////////
    // PUBLIC FIELDS
   
    ///////////////////////////////////////////////////////////////////////////////////
    // API
    
    getRecentArtifacts:function(projectURI) {
    	return this._getRecentFeed(projectURI, com.ibm.rdm.web.FRONTING_SERVICE_RECENT_FEEDS + RECENT_ARTIFACTS, this.recentArtifactsParser);
    },
    getRecentRequirements: function(projectURI) {
    	return this._getRecentFeed(projectURI, com.ibm.rdm.web.FRONTING_SERVICE_RECENT_FEEDS + RECENT_REQUIREMENTS, this.recentArtifactsParser);
    },
    getRecentComments: function(projectURI, sortComparator) {
    	return this._getRecentFeed(projectURI, com.ibm.rdm.web.FRONTING_SERVICE_RECENT_FEEDS + RECENT_COMMENTS, this.recentCommentsParser, sortComparator);
    },
    _getRecentFeed: function(projectURI, feedURI, feedParser, sortComparator) {
		// summary:
		//    Get all recent artifacts for the given projectURI or if projectURI is null
		// 	  get all recent artifacts that the current user should be able to see
		//    (across all projects that the user is a member of).
		// returns: array
		//    Array of ResourceInfo objects representing the appropriate content for a
		//    recent artifacts view.
 		// summary:
 		//    Use this method to get all recent artifacts for a project or the 
		//	  given User in the session.
 		//	  It returns a deferred that contains an array of Resource objects in
 		// 	  its response
 		var recentArtifactsDeferred = new dojo.Deferred();
 		
 		var processRecentFeedResponse = dojo.hitch(this, function(projectsMap) {
 				var multiGetArgs = [];
 				for (var userProjectURI in projectsMap) {
 					multiGetArgs.push
 					({
						uri: feedURI + "?resourceContext=" + encodeURIComponent(userProjectURI),
						parser: dojo.hitch(this, feedParser, userProjectURI)
					});
 				}
 				var multiGetDeferred = ProjectCache.multiGet(multiGetArgs);					
				multiGetDeferred.addCallback(this, function(multiGetResponse){
					var allArtifacts = [];
					for (var i=0; i<multiGetResponse.length; i++) {
						allArtifacts = allArtifacts.concat(multiGetResponse[i]);
					}
					if (sortComparator) {
						allArtifacts.sort(sortComparator);
					} else {
						allArtifacts.sort(com.ibm.rdm.web.links.LinksWidget.dateComparator);
					}
					allArtifacts.reverse();
					allArtifacts.splice(MAX_RECENT_ARTIFACTS, allArtifacts.length - MAX_RECENT_ARTIFACTS);
					recentArtifactsDeferred.callback(allArtifacts);
				});
				multiGetDeferred.addErrback(this, function(response) {
	 				var errorSectionWidget = ErrorHandler.getErrorSectionWidget();
					if (errorSectionWidget) {
						errorSectionWidget.showErrorMessage(response);
					}
					recentArtifactsDeferred.errback(response);
	 			});
 			}
 		);
 		
 		if (projectURI) {
 			var singleProjMap = {};
 			singleProjMap[projectURI] = projectURI;
			processRecentFeedResponse(singleProjMap);
 		} 
 		// otherwise gather and sort the artifacts for each project
 		else {
 			var deferred = ProjectCache.getUserMemberProjects();
 			deferred.addCallback(this, processRecentFeedResponse);
 			deferred.addErrback(this, function(response) {
 				var errorSectionWidget = ErrorHandler.getErrorSectionWidget();
				if (errorSectionWidget) {
					errorSectionWidget.showErrorMessage(response);
				}
				recentArtifactsDeferred.errback(response);
 			});
 		}
 		return recentArtifactsDeferred;
	},
	
	///////////////////////////////////////////////////////////////////////////////////
    // PARSERS
    
    recentArtifactsParser: function(projectURI, response) {
    	
		var entryNodes = response.doc.getElementsByTagName("entry");
		return dojo.map(
			entryNodes,
			function(entryNode) {
				// TODO (LPK) - think we should be passing recent feeds URL here, but currently server not
				// returning resourceURI relative to recent artifacts feed
				return new ResourceInfo({
	                atomEntryNode: entryNode,
	                baseURI: com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES,
	                projectURI: projectURI
				});
			}
		);
	},
	
	recentCommentsParser: function(projectURI, response) {
		// summary:
		//    Parses the recent comments feed and creates a comment object out
		// 	  of each entry
		if (response.doc.firstChild && response.doc.firstChild.tagName == "parsererror") {
			console.error("Error Parsing XML Feed for Recent Comments in RecentCommentsParser");
		}
		
		var recentComments = [];
		var entries = response.doc.getElementsByTagName("entry");
		for (var i = 0; i < entries.length; i++) {
			var comment = new com.ibm.rdm.web.commenting.Comment();
			comment.parseFeedEntry(entries[i], com.ibm.rdm.web.FRONTING_SERVICE_RECENT_FEEDS + RECENT_COMMENTS);
			comment.projectURI = projectURI;
			recentComments.push(comment);
		}
		return recentComments;
	}
};

})();
