/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.user.UserDashboard");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.layout.ContentPane");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.user", "UserDashboard");


dojo.declare(
    "com.ibm.rdm.web.user.UserDashboard",
    [dijit._Widget, dijit._Templated],
{
    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "user/templates/UserDashboard.html"),

  	widgetsInTemplate: true,
  	
  	recentlyViewed: null,
  	yourProjects: null,
  	welcome: null,
  	productName: null,
  	productDescription: null,
  	overviewMessage: null,
  	tutorialMessage: null,
  	productTourMessage: null,
  	sampleGalleryMessage: null,
  	helpMessage:null,
  	 
    _widgets: null,
    _recentArtifacts: null,
    _recentRequirements: null,
    _recentComments: null,
    _isStarted: null,
    	
    postCreate: function() {
        this._recentArtifacts = new com.ibm.rdm.web.search.results.RecentResourceResults({
            viewType:com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.SHOW_ARTIFACTS,
            displayContext:com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.USER_DASHBOARD,
            _headerTitle: this._messages.recentArtifacts
        }, this._recentlyModifiedSectionNode);
        this._recentComments = new com.ibm.rdm.web.search.results.RecentCommentResults({
            viewType:com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.RECENT_COMMENTS,
            displayContext:com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.USER_DASHBOARD,
            _headerTitle: this._messages.recentComments
        }, this._recentlyCommentedSectionNode);
        this._recentRequirements = new com.ibm.rdm.web.search.results.RecentResourceResults({
            viewType:com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.SHOW_REQUIREMENTS,
            displayContext:com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.USER_DASHBOARD,
            _headerTitle: this._messages.recentRequirements
        }, this._recentRequirementsSectionNode);
        this._projectSection = new com.ibm.rdm.web.project.ProjectSection({
        }, this._projectSectionNode);
        this._recentlyViewedSection = new com.ibm.rdm.web.viewer.RecentlyViewedSection({
        }, this._recentlyViewedSectionNode);
        
        dojo.addClass(this._projectSection.domNode, "main-content");
		dojo.addClass(this._recentlyViewedSection.domNode, "main-content");

        this._widgets = [];
        this._widgets.push(this._recentArtifacts);
        this._widgets.push(this._recentComments);
        this._widgets.push(this._recentRequirements);
        this._widgets.push(this._projectSection);
        this._widgets.push(this._recentlyViewedSection);
        this.inherited("postCreate", arguments);
    },
    
    startup: function() {
    	this._isStarted = true;
    	this._recentArtifacts.startup();
    	this._recentRequirements.startup();
    	this._recentComments.startup();
    	this._projectSection.startup();
    	this.inherited("startup", arguments);
    },
    
    postMixInProperties: function() {
        this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.user", 
        												"UserDashboard");
        
        dojo.forEach(["recentlyViewed", "yourProjects", "welcome", "productName", 
                      "productDescription", "overviewMessage", "tutorialMessage", 
                      "productTourMessage", "sampleGalleryMessage", "helpMessage"], function(prop){
            this[prop] = this._messages[prop];
        }, this);
    },
    
    destroy: function() {
    	this._recentArtifacts.destroyRecursive();
        this._recentRequirements.destroyRecursive();
        this._recentComments.destroyRecursive();
        this._projectSection.destroyRecursive();
        this._recentlyViewedSection.destroyRecursive();
        this.inherited(arguments);
    },
    
    show: function() {
    	if (!this._isStarted) {
    		this.startup();
    	} else {
    		this.handleRefresh();
    	}
    },
    
    handleRefresh: function() {
    	dojo.forEach(this._widgets, function(widget){
    		if (widget.handleRefresh) {
    			widget.handleRefresh();
    		}
    	});
    },
    
    getRecentlyViewedSection: function() {
    	return this._recentlyViewedSection;
    }

});

})();
