/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.web.util.domutils");

com.ibm.rdm.web.util.domutils = {
    
    createTr: function(trClass, childNode) {
        var tr = document.createElement("tr");
        if (trClass)
            tr.setAttribute("class", trClass);
        if (childNode)
            tr.appendChild(childNode);
        return tr;
    },

    createTd: function(tdClass, childNode) {
        var td = document.createElement("td");
        if (tdClass)
            td.setAttribute("class", tdClass);
        if (childNode)
            td.appendChild(childNode);
        return td;
    },
    
    createTrTd: function(trClass, tdClass, childNode) {
        var td = this.createTd(tdClass, childNode);
        var tr = this.createTr(trClass, td);
        return tr;
    },

    createSpannedTextNode: function(spanClassAttr, text) {
        var span = document.createElement("span");
        span.className = spanClassAttr;
        var text = document.createTextNode(text);
        span.appendChild(text);
        return span;
    },

    createAnchor: function(label, href, /*Optional*/cssClassName) {
        var link = document.createElement("a");
        link.setAttribute("target", "_blank");
        link.setAttribute("href", href);
		if (cssClassName) {
			link.className = cssClassName;
		}
        link.appendChild(document.createTextNode(label));
        return link;
    },
    
    createMailTo: function(emailAddress, subject, userName, body, /*Optional*/cssClassName) {
    	// Create A MailTo that includes address, subject, body, and a new line
    	// 17702 emailAddress is being returned as 'mailto:email address'. No need for the extra mailto.
        var link = document.createElement("a");
        // truncate title due to work item 17549
//        var shortenedSubject = subject;
//        if (shortenedSubject.length > 40) {
//        	shortenedSubject = shortenedSubject.substring(0,40);
//        	// TODO - should be externalized
//        	shortenedSubject = shortenedSubject + "...";
//        }
        var linkText = emailAddress + "?body=" + body + "%0d";
        // TODO - issues with UTF-8 characters and Notes, for now do not send subject as it may be in non-ascii format
//        var linkText = emailAddress + "?subject=?UTF-8?" + subject + "?body=" + body + "%0d";
        link.setAttribute("href", linkText);
        link.innerHTML = userName;
		if (cssClassName) {
			link.className = cssClassName;
		}
        dojo.connect(link, "onclick", dojo.hitch(this, this._mailToClick));
        return link;
    },
    
    _mailToClick: function(event) {
    	dojo.stopEvent(event);
		var anchor = event.currentTarget;
		var href = anchor.attributes.getNamedItem("href").value;
		window.location.href = href;
    },
    
	getXMLEncodedText: function(text) {
    	if (!text) return "";
      	var encodedText = text.toString();
      	encodedText = encodedText.replace(/&/g, "&amp;");
      	encodedText = encodedText.replace(/</g, "&lt;");
      	encodedText = encodedText.replace(/>/g, "&gt;");
      	encodedText = encodedText.replace(/"/g, "&quot;");
      	encodedText = encodedText.replace(/'/g, "&#039;");
		return encodedText;
    },
    
	getHTMLEncodedText: function(text) {
        if (!text) return "";
        var encodedText = text;
		var pre = document.createElement("pre");
		// use a text area so that linebreaks are preserved
        var textNode = document.createElement("textarea");
        if (dojo.isIE) {textNode.innerText = encodedText}
        else {textNode.textContent = encodedText};
        pre.appendChild(textNode);
		encodedText = textNode.innerHTML;
		delete pre;
		delete textNode;
        return encodedText;
    },

    getElementsByTagNameNS: function(domNode, unqualifiedTagName, namespace, qualifier) {
        // summary:
        //     E.g. assuming you've got a domNode like this ...
        //       <entry>
        //         ...
        //         <rdf:Description
        //             xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
        //             xmlns:dcterms="http://purl.org/dc/terms/"
        //             rdf:about="/jazz/resources/rdm/resources/res.media_1">
        //         ...
        //         </rdf:Description>
        //         ...
        //       </entry>
        //     You can use:
        //         com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
        //             atomEntryNode, "Description",
        //             "http://www.w3.org/1999/02/22-rdf-syntax-ns#", "rdf");
        //     to get the appropriate "rdf:Description" node.
        //     
        //     If no qualifier is speficied, xmlns attribute is used to determine the namespace of the tags
        //     E.g. to get <resourceUri xmlns="http://com.ibm.rdm.commenting#">/jazz/example</resourceUri>:
        //         com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
        //             atomEntryNode, "resourceUri",
        //             "http://com.ibm.rdm.commenting#");
        //
        
        if (!qualifier) {
        	var nodes = domNode.getElementsByTagName(unqualifiedTagName);
        	var nodeArray = new Array();
        	for (var i=0; i<nodes.length; i++) {
        		var n = nodes.item(i);
        		var xmlns = n.getAttribute("xmlns");
        		if (xmlns==namespace)
        			nodeArray.push(n);
        	}
        	/*
        	var NodeList = function (array Array of Node) {
        	   this.nodes = array;
        	   this.length = array.length;
        	}
            // Make sure it is a NodeList with item() method
        	NodeList.prototype.item = function(i) {
                return this.nodes[i];
            }
        	return new NodeList(nodeArray);
        	*/
        	nodeArray.item = function(i) {
        		return nodeArray[i];
        	}
        	return nodeArray;
        }
        
        return (
            dojo.isIE ?
            domNode.getElementsByTagName(qualifier + ":" + unqualifiedTagName) :
            domNode.getElementsByTagNameNS(namespace, unqualifiedTagName));
    },

    getAttributeNS: function(domNode, unqualifiedAttributeName, namespace, qualifier) {
        // summary:
        //     E.g. assuming you've got a domNode like this ...
        //         <rdf:Description
        //             xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
        //             xmlns:dcterms="http://purl.org/dc/terms/"
        //             rdf:about="/jazz/resources/rdm/resources/res.media_1">
        //         ...
        //         </rdf:Description>
        //     You can use:
        //         com.ibm.rdm.web.util.domutils.getAttributeNS(
        //             descriptionNode, "about",
        //             "http://www.w3.org/1999/02/22-rdf-syntax-ns#", "rdf");
        //     to get at the "rdf:about" attribute value:
        //             "/jazz/resources/rdm/resources/res.media_1"
        if (!domNode)
        	return "";
        return (
            dojo.isIE ?
            domNode.getAttribute(qualifier + ":" + unqualifiedAttributeName) :
            domNode.getAttributeNS(namespace, unqualifiedAttributeName));
    },
    
    
    getEntryBindingValueByName: function(entryNode, bindingNameAttribute) {
    	var bindingNodes = entryNode.getElementsByTagName("binding");
        for (var i=0; i<bindingNodes.length; i++) {
        	var bindingAttribute = bindingNodes[i].getAttribute("name");
        	if (bindingAttribute == bindingNameAttribute) {
        		if (dojo.isIE) 
        			return bindingNodes[i].text;
        		else
        			return bindingNodes[i].textContent;
        	}
        }
        return "";
    },
    
        
    //////////////////////////////////////////////////////////////////////////////////////////
    // Removes all of the nodes in a given element
    removeChildNodes: function(element) {    	
    	while( element.childNodes && element.childNodes.length > 0 ) {
    		element.removeChild( element.firstChild );
       	}
    },

	getCssPropertyValue: function(node, cssPropertyName) {
		// summary:
		//    Gets a CSS property value for a particular DOM node.
		// example:
		//    In CSS we might have the following defined:
		// |.com-ibm-rdm-web-header .bannerRight {
		// |    "position: absolute;"
		// |...
		//    In javascript we could get access to this value via the following
		//    (in this example, rightDiv has CSS class "bannerRight"):
		// |var rightDiv = dojo.byId("header-banner-right");
		// |var position = domutils.getCssPropertyValue(rightDiv, "position"));
		//    The end result would be that position would contain the string
		//    value "absolute".
		var value = null;
        var convertToCamelCase = function (cssProperty){
			// e.g. convertToCamelCase("margin-top") ==> "marginTop"
            for (
                var exp = /-([a-z])/;
                exp.test(cssProperty);
                cssProperty = cssProperty.replace(exp, RegExp.$1.toUpperCase())
            );
            return cssProperty;
        };
		if (dojo.isIE) {
			domProp = convertToCamelCase(cssPropertyName);
			value = node.currentStyle[domProp];
		} else {
			var css = document.defaultView.getComputedStyle(node, "");
			value = css.getPropertyValue(cssPropertyName);
		}
		return value;
	},

	getCssIntPropertyValue: function(node, cssPropertyName, /*Integer?*/defaultValue) {
		// summary:
		//    Gets a CSS Integer property value for a particular DOM node.
		// returns:
		//    An integer value for the passed CSS property cssPropertyName in
		//    the passed node.  This returned value MAY be NaN, unless
		//    defaultValue is passed, in which case it will return that value
		//    instead of NaN if an appropriate integer value cannot be found.
		// example:
		//    In CSS we might have the following defined:
		// |.com-ibm-rdm-web-header .bannerRight {
		// |    "max-width: 608px;"
		// |...
		//    In javascript we could get access to this value via the following
		//    (in this example, rightDiv has CSS class "bannerRight"):
		// |var rightDiv = dojo.byId("header-banner-right");
		// |var maxWidth = domutils.getCssIntPropertyValue(rightDiv, "max-width");
	    //    The end result would be that maxWidth would contain the int
		//    value 608.
		var value = this.getCssPropertyValue(node, cssPropertyName);
		// Note by SPC: I experimented to see what would happen if "width" was NOT
		// specified in the CSS ... in this case if we called getCssPropertyValue
		// for the "width" as the 'cssPropertyName' the 'value' [from previous
		// code line] would be set to "auto" for IE.  In Firefox, however, it would
		// be set to some reasonable value (like "86px), but the MDC shows that it
		// might be set to something like "" depending on which property we were
		// asking for.  Therefore calling parseInt in either case would return NaN.
		var valueAsInt = parseInt(value, 10);
		return ((defaultValue && isNaN(valueAsInt))? defaultValue: valueAsInt);
	},
	
	createHourGlassCursor: function() {
		// summary: Create an hourglass cursor
		window.document.body.style.cursor = 'wait';
	},
	
	createDefaultCursor: function() {
		// summary: set the default cursor
		window.document.body.style.cursor = 'default';
	}
};
