/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.viewer.EmbeddedCollectionViewer");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.viewer.EmbeddedResourceViewer");
dojo.require("com.ibm.rdm.web.viewer.viewerutils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.collections.CollectionViewer");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.requireLocalization("com.ibm.rdm.web.project", "ResultSetView");
dojo.requireLocalization("com.ibm.rdm.web.viewer", "ResourceViewer");

var UriHelper = com.ibm.rdm.web.common.UriHelper;
var constants = com.ibm.rdm.web.common.MimeTypes;
var viewerutils = com.ibm.rdm.web.viewer.viewerutils;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;

dojo.declare(
	"com.ibm.rdm.web.viewer.EmbeddedCollectionViewer",
	[com.ibm.rdm.web.viewer.EmbeddedResourceViewer],
{
	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.project", "ResultSetView");
		this._messages2 = dojo.i18n.getLocalization("com.ibm.rdm.web.viewer", "ResourceViewer");
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	// Superclass override functions ...

	_populateContents: function() {
		// Note: this is an override of a superclass function.
		var collectionDeferred = ProjectCache.getCollection(this.resourceURI, this.revision, this.projectSnapshotURI);
		collectionDeferred.addCallback(this, function(collection){
			this._URIs = collection.collectionArtifactURIs;
			this.projectSnapshotURI = collection.associatedSnapshotURI;
								
			if (collection.associatedSnapshotURI) {
				var projectSnapshotsDeferred = ProjectCache.getProjectSnapshot(collection.associatedSnapshotURI);
				projectSnapshotsDeferred.addCallback(this, function (projectSnapshot){
					if (projectSnapshot) {
						this._artifactNameSpan.appendChild(document.createTextNode(" (" + this._messages2.snapshotInfoLabel + projectSnapshot.name + ")"));
					}
				});
			}

			var nameComparator = function(resource1, resource2) {
			    return ((resource1.name == resource2.name) ? 0 : 
			    	((resource1.name.toLowerCase() < resource2.name.toLowerCase()) ? -1 : 1));
			};
			var collectionArtifacts = collection.collectionArtifacts.sort(nameComparator);
			this._handleMultiFetchSuccessResponse(collectionArtifacts);
		});
		
	},

	_handleMultiFetchSuccessResponse: function(collectionResources) {
		this._loadingDiv.innerHTML = "";
		for (var i=0; i<collectionResources.length; i++) {
			var resourceInfo = collectionResources[i];
			var div = document.createElement("DIV");
			
			var imageURI = com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(resourceInfo.contentType);
	        if (imageURI) {
	            var img = document.createElement("img");
	            img.setAttribute("src", imageURI);
	            img.style.verticalAlign = "middle";
	            img.style.paddingRight = "5px";
	            div.appendChild(img);
	        }
	        var resourceURI = (
				resourceInfo.resourceURI
				+ (
					(resourceInfo.inSnapshot && resourceInfo.isCollectionArtifactAssociatedWithSnapshot) ?
					(
						"?revision=" + resourceInfo.etag
						+ "&rrc:baseline=" + encodeURIComponent(resourceInfo.projectSnapshotURI)
					) :
					""
				  )
			);
	        var link = ResourceLinksHandler.getResourceLink(
				resourceURI,
	        	resourceInfo.displayName
				+ ((!resourceInfo.inSnapshot && resourceInfo.isCollectionArtifactAssociatedWithSnapshot) ?
				   this._messages.artifactNotInSnapshot :
				   ""),
	        	true
	        );
			
			div.appendChild(link);
			this._artifactContentDiv.appendChild(div);
		}
	},

	_composeURIsClause: function() {
    	var URIsClause = "";
    	var URIs = this._URIs;
		if (URIs && URIs.length > 0) {
			URIsClause = 'resource in [';
			for (var i=0; i<URIs.length; i++) {
				URIsClause += (i==0?'':',') + '<' + URIs[i] + '>';
			}
			URIsClause += ']';
		}
        return URIsClause;
    }
});


})();
