/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.viewer.EmbeddedResourceViewer");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.viewer.viewerutils");

dojo.requireLocalization("com.ibm.rdm.web.common", "common");

var queryutils = com.ibm.rdm.web.util.queryutils;
var viewerutils = com.ibm.rdm.web.viewer.viewerutils;
var constants = com.ibm.rdm.web.common.MimeTypes;


// TODO Implement functionality to make these embedded resource viewers
// resizable.  Perhaps clicking on the title pane should result in toggling
// the size (switching between full size and 1/4 size) or perhaps we should
// extend FloatingPane instead of _Widget.  Not sure ... need UX direction
// on this.

dojo.declare(
	"com.ibm.rdm.web.viewer.EmbeddedResourceViewer",
	[dijit._Widget, dijit._Templated],
{
    // summary:
    //    Abstract base class for embedded resource viewers.
	// description:
	//    This widget provides the "container" in which embedded artifact
	//    content lives.  Different artifact type renderings are achieved by
	//    subclassing this widget and overriding the _populateContents
	//    function (and optionally the _populateTitle function).

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "viewer/templates/EmbeddedResourceViewer.html"),
	widgetsInTemplate: false,

	///////////////////////////////////////////////////////////////////////////
	// Attach points

	_artifactDiv: null,
	_titleDiv: null,
	_titleSpan: null,
	_artifactTypeImage: null,
	_artifactNameSpan: null,
	_loadingDiv: null,
	_artifactContentDiv: null,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	// resourceURI: String
	//    For example, this might be set to
	//    "/jazz/resources/rdm/resources/rrc96".
	resourceURI: null,
	// contentType: String
	//	  For example, this might be set to
	//    "application/x-com.ibm.sid.sketch+xml".
	contentType: null,
	// revision: String
	//	  The etag associated with the resource.
	revision: null,
	// docAsString: String
	//    The raw string content of the resourceURI at the passed revision
	//    level.
	docAsString: null,
	// docAsXml: XML
	//    The Document content of the resourceURI at the passed revision level.
	docAsXml: null,
	// reviewParser: Object
	//    If this embedded resource is being viewed in the context of a
	//    review, this field will be set to an appropriate ReviewParser object.
	reviewParser: null,
	// projectSnapshotURI: String
	//    If this embedded resource is being displayed in the context of a
	//    snapshot in the history view, this field will be set to the URI that
	//    represents the project snapshot.
	projectSnapshotURI: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	// TODO Compute these by reading in CSS info ...
	_contentPaddingLeft: 6,
	_contentPaddingRight: 6,

    ///////////////////////////////////////////////////////////////////////////
    // NLS fields requiring translation

    loading: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this._populateTitle();
		this._populateContents();
	},

    postMixInProperties: function() {
		var messages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
		this.loading = messages["loading"];
    },

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_populateTitle: function() {
		// summary:
		//    Fill in the content that is to appear in the title area.
		// description:
		//    Fill in the title information including an icon for the artifact type
		//    as well as the artifact name.
		//    Subclasses _may_ wish to override this function if they need additional
		//    content to appear in the title area.
		this._artifactNameSpan.appendChild(
			document.createTextNode(
				queryutils.getArtifactNameFromArtifactContent(this.docAsXml, this.contentType)));
		this._artifactTypeImage.src =
			com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(this.contentType);
	},

	_populateContents: function() {
		// summary:
		//    Populate the "content area" that appears below the title.
		// description:
		//    This function MUST be overriden by subclasses.  This function will
		//    be called to fill in the "content area".
		throw new Error('Unimplemented API: com.ibm.rdm.web.viewer.EmbeddedResourceViewer._populateContents');
	}
});


})();
