/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.viewer.ScreenFlowUIActivityLoader");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.server.XhrUtils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.viewer.viewerutils");

dojo.requireLocalization("com.ibm.rdm.web.viewer", "ScreenFlowUIActivityLoader");

var XhrUtils = com.ibm.rdm.web.server.XhrUtils;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var constants = com.ibm.rdm.web.common.MimeTypes;
var queryutils = com.ibm.rdm.web.util.queryutils;
var viewerutils = com.ibm.rdm.web.viewer.viewerutils;


dojo.declare("com.ibm.rdm.web.viewer.ScreenFlowUIActivityLoader", null, {
	// summary:
	//    Mixin for widgets that need to load the "thumbnails" that appear in
	//    Screen Flow artifacts.

	_loadUIActivities: function(domNode, /*ReviewParser?*/reviewParser, /*String?*/projectSnapshotURI) {
		// summary:
		//    Fill in the content for the "Loading..." thumbnails that are
		//    currently being displayed in a rendered Screen Flow artifact.
		// description:
		//    The Screen Flow image that is currently displayed contains
		//    thumbnails for all the UIActivities in which a "Loading..."
		//    label appears.  This function is called to replace all of
		//    these thumbnails with the appropriate images.  We do so by making
		//    use of the resourceURI attribute that's stored as an attribute of
		//    every DIV whose controlType attribute is "User Interface
		//    Activity".  For each of these resourceURI's we get the image and
		//    then call '_addScreenFlowThumbnailContent' to make sure the
		//    appropriate thumbnail image appears in the DOM.
		// reviewParser: Object
		//    The reviewParser is only passed if the viewer is being shown in
		//    the context of a review.
		var messages = dojo.i18n.getLocalization(
			"com.ibm.rdm.web.viewer", "ScreenFlowUIActivityLoader");
		var self = this;
		var loadUIActivity = function(parentDiv, resourceURI) {
			var onHead = function(headResponse) {
				var onGet = function(response) {
					var htmlContent = response.ioArgs.xhr.responseText;
					var imageSrc = viewerutils.getUrlToUseToForBackgroundImage(htmlContent);
					self._addScreenFlowThumbnailContent(
						domNode, parentDiv, messages, imageSrc, resourceURI);
					return response;
				};
				var onError = function(response) {
					self._addScreenFlowThumbnailContent(
						domNode, parentDiv, messages, null, resourceURI, response);
					return response;
				};
				var normalizedContentType =
					queryutils.getNormalizedContentType(headResponse.ioArgs.xhr.getResponseHeader("Content-Type"));
				// Note: we need to account for the border, including the shadow
				// that appears to right/bottom of the activity.  For this
				// reason we're not using the commented out attribute values for
				// width/height in the following ...
				var thumbnailWidth = 75;//parentDiv.getAttribute("thumbnailWidth");
				var thumbnailHeight = 89;//parentDiv.getAttribute("thumbnailHeight");
				var urlToUse = (
					UriHelper.getGefDiagramConversionUri()
					+ "?resourceURL=" + encodeURIComponent(resourceURI) + ".html"
					+ "&width=" + thumbnailWidth
					+ "&height=" + thumbnailHeight
					+ "&doSuppressCommentingDivCreation=true"
					+ "&contentType=" + normalizedContentType.replace("+", "%2b")
					+ "&revision=" + XhrUtils.getResponseEtag(headResponse)
					+ UriHelper.getReviewQueryParams(reviewParser)
					+ UriHelper.getBaselineQueryParams(projectSnapshotURI)
				);
				var deferredGetRequest = RDMClient.doGet({
					uri: urlToUse,
					skipErrors: true
				});
				deferredGetRequest.addCallback(onGet);
				deferredGetRequest.addErrback(onError);
			}
			var deferredHeadRequest = RDMClient.doHead({uri:resourceURI});
			deferredHeadRequest.addCallback(onHead);
		}
		var urlToToResourceURI = function(url) {
			return url;
		}
		var isExternalResource = function(path) {
			var isExternalResource = false;
			if (path) {
				var value = path.toLowerCase();
				isExternalResource =
					(0 == value.indexOf("http:") || 0 == value.indexOf("https:"));
			}
			return isExternalResource;
		};
		var loadAppropriateUIActivities = function(response) {
			// Note: This function takes care to ONLY call loadUIActivity for UI Activities
			// that are sourced from the same repo as the one our webserver talks to.
			var mapKeysToLocalPaths = new Object;
			var uiActivityNodes = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
				response.doc, "UIActivity",
				"http://jazz.net/xmlns/alm/rm/ScreenFlow/v0.1", "flow");
			for (var i = 0; i < uiActivityNodes.length; i++) {
				var path = uiActivityNodes[i].getAttribute("path");
				var key = uiActivityNodes[i].getAttribute("key");
				if (! isExternalResource(path)) {
					mapKeysToLocalPaths[key] = path;
				}
			}
			var divNodes = domNode.getElementsByTagName("div");
			var unresolveableUIActivityDivs = [];
			for (var i = 0; i < divNodes.length; i++) {
				var div = divNodes[i];
				if (div.getAttribute("controlType") === "User Interface Activity") {
					var resourceURI = div.getAttribute("resourceURI");
					if (resourceURI) {
						var key = div.id;
						if (mapKeysToLocalPaths[key]) {
							loadUIActivity(div, urlToToResourceURI(resourceURI));
							// TODO Remove commented out code
							self._navigibleUIActivityIsLoading(div, resourceURI);
						} else {
							unresolveableUIActivityDivs.push(div);
						}
					}
				}
			}
			// Note: divNodes is a live list ... and this is why we take care to
			// call _addScreenFlowThumbnailContent here in a separate for loop.
			for (var i = 0; i < unresolveableUIActivityDivs.length; i++) {
				self._addScreenFlowThumbnailContent(
					domNode, unresolveableUIActivityDivs[i], messages);
			}
		};
		var deferred = RDMClient.doGet({
			uri:this.resourceURI
		});
		deferred.addCallback(loadAppropriateUIActivities);
	},

	_addScreenFlowThumbnailContent: function(
			domNode, uiActivityDiv, messages,
			/*String?*/imageSrc, /*String?*/resourceURI,
			/*Optional*/errorResponse
		) {
		// summary:
		//    Add content for a Screen Flow thumbnail to the DOM.
		// description: 
		//    We "sprinkle in" our thumbnail images as the requests return from
		//    the server.
		//    Please note that we DO NOT use an IMG based approach here, but
		//    instead make use of a DIV in which we specifiy a backgroundImage.
		//    The reason we do this is to avoid the "jiggling" that would occur
		//    when the parent DIV becomes selected or unselected (e.g. when
		//    parent DIV becomes selected a 2px wide margin appear around it
		//    and this would cause the image to move 2px to the right/bottom).
		// domNode:
		//    The DOM node that represents the element to which we adding Screen
		//    Flow content.
		// uiActivityDiv:
		//    A div element whose controlType attribute is set to "User Interface
		 //   Activity".
		// messages:
		//    Contains the nls message properties associated with this source file.
		// imageSrc:
		//    The URI that we can make use of to obtain the appropriate
		//    image graphics.  If not passed, this function assumes that the
		//    thumbnail is based on a Sketch/ScreenFlow from a different
		//    repository, in which case we overlay the thumbnail's "Loading..."
		//    image with a DIV in which we provide a message to clue the user
		//    into the reason why the thumbnail graphics could not be provided.
		var rootDiv = uiActivityDiv.parentNode;
		var div = document.createElement("div");
		var paddingLeft =
			domNode.style.paddingLeft === "" ? 0 : parseInt(domNode.style.paddingLeft, 10);
		var paddingTop =
			domNode.style.paddingTop === "" ? 0 : parseInt(domNode.style.paddingTop, 10);
		// Note we're adding '2' to the following to account for the margin that
		// appears in the native client when displaying this UI activity.
		var thumbnailOffsetX =
			parseInt(uiActivityDiv.getAttribute("thumbnailOffsetX"), 10)
			+ paddingLeft + 2;
		var thumbnailOffsetY =
			parseInt(uiActivityDiv.getAttribute("thumbnailOffsetY"), 10)
			+ paddingTop + 2;
		var absoluteY =
			parseInt(uiActivityDiv.style.top, 10) + thumbnailOffsetY;
		var absoluteX =
			parseInt(uiActivityDiv.style.left, 10) + thumbnailOffsetX;
		if (imageSrc) {
			// Show the thumbnail.
			div.style.backgroundImage =
				"url(" + imageSrc + ")";
			div.style.backgroundRepeat = "no-repeat";
			div.style.backgroundPosition = absoluteX + "px " + absoluteY + "px";
			div.style.width = rootDiv.clientWidth + "px";
			div.style.height = rootDiv.clientHeight + "px";
			div.style.position = "absolute";
			div.style.left = "0px";
			div.style.top = "0px";
			this._contentForUIActivityHasBeenRendered(uiActivityDiv, true, resourceURI);
		} else {
			// Show a div containing a span that says the thumbnail can't be loaded.
			// TODO Move this CSS style info into CSS rather than hardcoding it in here.
			var msg = "";
			if (errorResponse) {
				// The most common case for this is that 403 came back and we're
				// unable to view the artifact (as a dependent resource is
				// found in a project the user isn't a member of).
				msg = com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().getErrorMessage(errorResponse);
			} else {
				// This artifact comes from a different repository.
				msg = messages["thumbnailUnavailable"];
			}
			var thumbnailWidth = parseInt(uiActivityDiv.getAttribute("thumbnailWidth"), 10) - 4;
			var thumbnailHeight = parseInt(uiActivityDiv.getAttribute("thumbnailHeight"), 10) - 4;
 			div.style.background = "#FFDEDE";
			div.style.position = "absolute";
			div.style.left = absoluteX-1 + "px";
			div.style.top = absoluteY-1 + "px";
			div.style.width = thumbnailWidth + "px";
			div.style.height = thumbnailHeight + "px";
			div.style.border = "1px solid";
			div.style.textAlign = "center";
			div.style.fontSize = "9pt";
			div.style.color = "#999999";
			div.innerHTML = (
				"<span>"
				+ "&lt;" + msg + "&gt;"
				+ "</span>"
			);
			this._contentForUIActivityHasBeenRendered(uiActivityDiv, false);
		}
		rootDiv.insertBefore(div, rootDiv.firstChild);
	},

	_navigibleUIActivityIsLoading: function(uiActivityDiv, resourceURI) {
		// summary:
		//    A callback for noticing when a thumbnail has started to load.
		// description:
		//    Widgets that mixin this class may wish to override this function
		//    to be notified when a particular thumbnail's DIV element is
		//    starting to load.
	},

	_contentForUIActivityHasBeenRendered: function(
			uiActivityDiv, isRenderedAsArtifactImage, /*String?*/resourceURI) {
		// summary:
		//    A callback to be notified when thumbnail content has been
		//    rendered.
		// description:
		//    Widgets that mixin this class may wish to override this function
		//    to be notified when a particular thumbnail's DIV element has been
		//    populatated.
		// uiActivityDiv:
		//    A DIV element whose content has been newly populated.
		// isRenderedAsArtifactImage:
		//    If the thumbnail represented a Sketch that could not be accessed,
		//    this will be set to false.  Otherwise it's set to true.
		// resourceURI:
		//    The resource URI of the artifact that appears within a thumbnail.
		//    Will only contain a non-null value if isRenderedAsArtifactImage
		//    is set to true.
	}
});


})();
