/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.web.viewer.embeddedResourceFactory");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.viewer.EmbeddedDiagramViewer");
dojo.require("com.ibm.rdm.web.viewer.EmbeddedStoryboardViewer");
dojo.require("com.ibm.rdm.web.viewer.EmbeddedCollectionViewer");
dojo.require("com.ibm.rdm.web.server.XhrUtils");

var XhrUtils = com.ibm.rdm.web.server.XhrUtils;
var EmbeddedDiagramViewer = com.ibm.rdm.web.viewer.EmbeddedDiagramViewer;
var EmbeddedStoryboardViewer = com.ibm.rdm.web.viewer.EmbeddedStoryboardViewer;
var EmbeddedCollectionViewer = com.ibm.rdm.web.viewer.EmbeddedCollectionViewer;
var queryutils = com.ibm.rdm.web.util.queryutils;


// Note by SPC: I see such names as simpleFetch.js, consoleMessages.js,
// mouseOver.js, callbackPollTransport.js in dojo sources ... so I'm calling
// this "embeddedResourceFactory" instead of "embeddedresourcefactory".

com.ibm.rdm.web.viewer.embeddedResourceFactory = {

	create: function(resourceURI, response, /*DomNode?*/nodeToReplace, /*ReviewParser?*/reviewParser, projectSnapshotURI) {
		// summary:
		//    Create an embedded resource viewer that is capable of displaying
		//    artifact represented by the passed parameters.
		// returns:
		//    An instance of a class derived from EmbeddedResourceViewer.
		// resourceURI: String
		//    For example, this might be set to "/jazz/resources/rdm/resources/rrc96".
		// response: Object
		//    The response returned by a call to RDMClient.doGet(...).
		// nodeToReplace: DomNode
		//    Optional DOM node to replace when the appropriate widget is constructed.
		var normalizedContentType =
			queryutils.getNormalizedContentType(
				response.ioArgs.xhr.getResponseHeader("Content-Type"));
		var revision = XhrUtils.getResponseEtag(response);
		var embeddedResourceViewer = null;
		switch (normalizedContentType) {
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_SKETCH:
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_PART:
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_SCREEN_FLOW:
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_PROCESS:
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_USE_CASE_DIAGRAM:
				embeddedResourceViewer = new EmbeddedDiagramViewer(
					{
						resourceURI: resourceURI,
						contentType: normalizedContentType,
						revision: revision,
						docAsString: response.ioArgs.xhr.responseText,
						docAsXml: response.doc,
						reviewParser: reviewParser,
						projectSnapshotURI: projectSnapshotURI
					},
					nodeToReplace);
				break;
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_STORY_BOARD:
				embeddedResourceViewer = new EmbeddedStoryboardViewer(
					{
						resourceURI: resourceURI,
						contentType: normalizedContentType,
						revision: revision,
						docAsString: response.ioArgs.xhr.responseText,
						docAsXml: response.doc,
						reviewParser: reviewParser,
						projectSnapshotURI: projectSnapshotURI
					},
					nodeToReplace);
				break;
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_COLLECTION:
				embeddedResourceViewer = new EmbeddedCollectionViewer(
					{
						resourceURI: resourceURI,
						contentType: normalizedContentType,
						revision: revision,
						docAsString: response.ioArgs.xhr.responseText,
						docAsXml: response.doc,
						reviewParser: reviewParser,
						projectSnapshotURI: projectSnapshotURI
					},
					nodeToReplace);
				break;
			default:
				// TODO Handle all remaining (non-GEF based) artifact types ...
				// Need the following:
				//   EmbeddedRichTextViewer (for Term/Actor/Document/Requirement)
				//   EmbeddedGlossaryViewer
				//   EmbeddedWrappedResourceViewer
				console.error(
					"embeddedResourceFactory doesn't know how to handle content type "
					+ normalizedContentType);
		}
		return embeddedResourceViewer;
	}
};
