/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009-2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.web.viewer.viewerutils");

dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.history.revisionsHelper");
dojo.require("com.ibm.rdm.web.review.ReviewCache");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.util.queryutils");


var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var ReviewCache = com.ibm.rdm.web.review.ReviewCache;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var queryutils = com.ibm.rdm.web.util.queryutils;
var revisionsHelper = com.ibm.rdm.web.history.revisionsHelper;


com.ibm.rdm.web.viewer.viewerutils = {

	calculateTextWidth: function(label, /*String?*/fontFamily, /*String?*/fontSize, /*String?*/fontWeight) {
		// summary:
		//    Returns the offsetWidth of a span containing the passed label.
		// description:
		//    Some TVT bugs require that a DOM node's width be set to the exact
		//    width needed to display a label using the the current language
		//    setting.  This function can be used for this purpose.
		var span = document.createElement("span");
		if (fontFamily) {
			span.style.fontFamily = fontFamily;
		}
		if (fontSize) {
			span.style.fontSize = fontSize;
		}
		if (fontWeight) {
			span.style.fontWeight = fontWeight;
		}
		document.body.appendChild(span);
		span.innerHTML = label;
		var offsetWidth = parseInt(span.offsetWidth, 10);
		document.body.removeChild(span);
		return offsetWidth;
	},

    transformImageGenerationDocument: function(/*String*/doc) {
        // summary:
		//    Massage the passed [GEF artfict HTML] doc into the content
		//    that would be appropriate for setting as an innerHTML.
		// description:
        //    The passed doc string is in a format appropriate for enabling
        //    easy viewing of the cache files for testing purposes.  We need to
        //    transform them a little bit to enable viewing in our widget.
        //    More specifically, we need to tweak the DIV containing a background
		//    image from something like this:
        //        <div style="background-image:url(./jazz_resources_rdm_resources_rrc53_ZeSYEWinEd2DZPXOx8bXBg.png); background-repeat:no-repeat; width:411px; height:423px;">
        //    to something like this:
        //        <div style="background-image:url(/converter/htmlgen/jazz_resources_rdm_resources_rrc53_ZeSYEWinEd2DZPXOx8bXBg.png?oauthToken=9431354f4d6c4dacbae7c71fcde86b1a&requestTokenSecret=Q6Dk4nQQZe%2BFjzu80x1zWIv8J1FovXwtXuHiy4ExExU%3D&preventPNGCache=1233582844243); background-repeat:no-repeat; width:411px; height:423px;">
        //    and we also have to remove everything from the document start to
        //    the end of the <body> tag and everything from the </body> tag to
        //    the end of the document (as this HTML content doesn't live in an
		//    iframe).
		// returns:
		//    A massaged version of the document that is appropriate for setting
		//    as an element's innerHTML.
		var regexp = /(.*)background-image:url\(\.(.*)\)(.*)/;
		doc = doc.replace(
			regexp,
			("$1background-image:url("
				+ com.ibm.rdm.web.common.UriHelper.getGefDiagramConversionUri()
				+ "$2"
			 	+ "?" + UriHelper.getOAuthQueryParams()
			 	+ "&" + this.createPNGCacheQueryParamAssignment()
				+ ")$3")
		);
        var bodyStartIndex = doc.indexOf("&lt;body&gt;") + 12; // Note: 12 is length of "&lt;body&gt;"
        var bodyEndIndex = doc.lastIndexOf("&lt;/body&gt;");
		var bodyContentString = doc.substring(bodyStartIndex, bodyEndIndex);
		// Note: The doc currently contains HTML encoded text which was
		// originally generated by a call to:
		//     StringEscapeUtils.escapeHtml(content)
		// back in the converter servlet.  This means we need to change over
		// fragments such as "&lt;div&gt;" into their corresponding decoded
		// text (e.g. "<div>").  We do so here by using setting the innerHTML
		// on a dummy node and then asking for its textual content ...
		var div = document.createElement("div");
		div.innerHTML = bodyContentString;
		var decodedContentString = dojo.isIE ? div.innerText : div.textContent;
		return decodedContentString;
    },

	createPNGCacheQueryParamAssignment: function() {
		// summary:
		//    Compose a unique query param to tack onto URLs to avoid hits in
		//    the browser cache.
		// description:
		//    Whenever we make image requests for images that were originally
		//    created via the GEF artifact converter, we call this function to
		//    make sure that we include a query param so that the browser cache
		//    does not come into play.
		return ("preventPNGCache=" + new Date().valueOf());
	},

	getBackgroundImageDiv: function(/*DomNode*/ node) {
		var divNodes = node.getElementsByTagName("div");
		for (var i = 0; i < divNodes.length; i++) {
			if (divNodes[i].style.backgroundImage) {
				return divNodes[i];
			}
		}
	},

	appendDivWithHtmlContent: function(nodeToAppendTo, htmlContentToAdd) {
		// summary:
		//    Work around for IE issue.
		// description:
		//    For some reason IE won't let us just do:
		//        nodeToAppendTo.innerHTML = htmlContentToAdd
		//    to get around this fact we're creating a new div for this assignment
		//    and then we use appendChild to add this new div to the DOM.
		var div = document.createElement("div");
		div.innerHTML = htmlContentToAdd;
		nodeToAppendTo.appendChild(div);
		return div;
	},

    openResourceUsingAppropriateViewer: function(
			resourceURI, /*String?*/reviewId, /*String?*/projectSnapshotURI) {
		// summary:
		//    If the 'resourceURI' represents an internal resource, make an
		//    appropriate action call to either showArtifact,
		//    showArtifactInReviewContext or showArtifactHistory.  If it's
		//    NOT an internal resource use window.open to open it.
		// description:
		//    If the resource viewer is NOT displaying an artifact under review
		//    we show the clicked resource using showArtifact.  On the other
		//    hand, if the resource viewer IS showing an artifact under review
		//    we'll use one of the following:
		//        showArtifactInReviewContext
		//            Called when the clicked link represents a resource that
		//            is found in the same review as the review pertaining to
		//            the artifact under review.
		//        showArtifactHistory
		//            Called when the clicked link represents a resource that
		//            is NOT found in the same review as the review pertaining
		//            to the artifact under review.  In this case the UX folks
		//            have asked that the "appropriate version" of this
		//            resource appear using our Open History display.  The
		//            "appropriate version" is the version that is older but
		//            closest to the start timestamp found within the review
		//            itself (i.e. the review that is associated with the
		//            artifact that is currently being shown in the resource
		//            viewer).
		// reviewId: String
		//    A string such as "_lckswDjhEd67TdjgK4pk9g".
		// projectSnapshotURI: String
		//    A string such as "https://localhost:9443/jazz/baselines/_LF3vkK-iEd6eqqbI97jrPQ"
		//
		var isInternalResource = ResourceLinksHandler.isLinkInternal(resourceURI);
		if (isInternalResource && reviewId) {
			var onReviewLoaded = function(response) {
				var reviewParser = response.doc;
				var artifactInfo = reviewParser.getArtifactInfo(resourceURI);
				if (artifactInfo) {
					// Open the artifact using the showArtifactInReviewContext action.
					var href = UriHelper.getResourceInReviewContextJafLink(
						artifactInfo.resourceURI,
						artifactInfo.revision,
						reviewId
					);
					window.location.href = href;
				} else if (reviewParser.isFormal()) {
					// Open the artifact using the showArtifactHistory action.
					var onNewestPastRevisionByDateComplete = function(revisionsInfo) {
						var revisionToUse = revisionsInfo.revisionToUse;
						var href = UriHelper.getResourceHistoryJafLink(
							resourceURI, revisionToUse);
						window.location.href = href;
					}
					var deferred = revisionsHelper.getNewestPastRevisionByDate(
						resourceURI, reviewParser.getStartTimestampDate());
					deferred.addCallback(onNewestPastRevisionByDateComplete);
				} else {
					// Open the artifact using the showArtifact action.
					var href = UriHelper.getResourceJafLink(resourceURI);
					window.location.href = href;
				}
			}
			var deferred = ReviewCache.getReview(reviewId);
			deferred.addCallback(onReviewLoaded);
		} else {
			if (isInternalResource) {
				// Open the artifact using the showArtifact or showArtifactHistory action.
				if (projectSnapshotURI) {
					// In the context of a snapshot, we use the history viewer ...
					this._showHistoryViewerAndSelectSpecificSnapshot(
						resourceURI, projectSnapshotURI);
				} else {
					// ... otherwise we use the normal artifact viewer.
					ResourceLinksHandler.showInternalResource(
						resourceURI, true, false);
				}
			} else {
				// Open the resource using the winow.open function.
				ResourceLinksHandler.showExternalResource(resourceURI);
			}
		}
    },

	getNativeClientUrlForOpenArtifact: function(resourceURI) {
		// summary:
		//    Return a URL appropriate for opening an artifact in the native
		//    client.
		// returns: String
		//    For most artifacts the returned URL will be a string such as:
		//        "rpc://localhost:9443/rdm/resources/rrc105"
		//    For Reviews, the returned URL will be a string such as:
		//        "rpc://localhost:9443/rdm/reviews/_u6GAsJmEEd6WuJFixEAwCQ"
		return (
			"rpc"
			+ resourceURI.substring(resourceURI.indexOf(":"), resourceURI.length)
		);
	},

	getNativeClientUrlForOpenHistory: function(resourceURI, revision, projectSnapshotURI) {
		// summary:
		//    Return a URL appropriate for opening an artifact's history view in
		//    the native client.
		// returns: String
		//    A URL string such as:
		//        "rpc://localhost:9443/rdm/resources/rrc105?revision=_Y9jCAZihEd6qEbbZSstz8g"
		var snapshotId = (
			projectSnapshotURI ?
			queryutils.getLastPathSegment(projectSnapshotURI) :
			null
		);
		return (
			this.getNativeClientUrlForOpenArtifact(resourceURI)
			+ "?revision=" + revision
			+ (snapshotId ? ("&rrc:baseline=" + snapshotId) : "")
		);
	},

	getNativeClientUrlForOpenArtifactInReviewContext: function(resourceURI, reviewId, /*String?*/revision) {
		// summary:
		//    Return a URL appropriate for opening an artifact under review in
		//    the native client.
		// reviewId: String
		//    The shortname for a review (e.g. "_ReMN4ZupEd692I6BIgt2Kw").
		// revision: String
		//    This parameter is only passed when the reviewId pertains to a
		//    "formal" review.
		// returns: String
		//    If the passed reviewId pertains to a "formal" review the returned
		//    URL string would be similar to:
		//        "rpc://localhost:9443/rdm/resources/rrc106?revision=_Y5rOoZihEd6qEbbZSstz8g&rrc:review=_ReMN4ZupEd692I6BIgt2Kw"
		//    and if the passed reviewId pertains to an "informal" review the
		//    returned URL string would be similar to:
		//        "rpc://localhost:9443/rdm/resources/rrc178?rrc:review=_Olq-QZlcEd6jA6FWoMJxnA"
		return (
			this.getNativeClientUrlForOpenArtifact(resourceURI)
			+ (revision ? ("?revision=" + revision) : "")
			+ "&rrc:review=" + reviewId
		);
	},

	getNativeClientUrlForOpenProject: function(projectURI) {
		// summary:
		//    Return a URL appropriate for opening a project in the native
		//    client.
		// returns: String
		//    A URL string such as the following:
		//        "rpc://localhost:9443/jazz/process/project-areas/_Ox_M8ZihEd6qEbbZSstz8g"
		return (
			"rpc"
			+ projectURI.substring(projectURI.indexOf(":"), projectURI.length)
		);
	},

	getUrlToUseToForBackgroundImage: function(htmlContent) {
		var startFragment = "background-image:url(./";
		var endFragment = ");";
		var startIndex = htmlContent.indexOf(startFragment) + startFragment.length;
		var endIndex = htmlContent.indexOf(endFragment);
		return (
			UriHelper.getGefDiagramConversionUri()
			+ "/" + htmlContent.substring(startIndex, endIndex)
			+ "?" + UriHelper.getOAuthQueryParams()
			+ "&" + this.createPNGCacheQueryParamAssignment()
		);
	},

	_showHistoryViewerAndSelectSpecificSnapshot: function(resourceURI, projectSnapshotURI) {
		// We first need to run a query to get the exact revion of the artifact in the snapshot.
		var resourceDescriptionInSnapshotDeferred =
			ProjectCache.getResourceDescriptionMultiFetch(resourceURI, projectSnapshotURI);
		resourceDescriptionInSnapshotDeferred.addCallback(this, function(resourceDescriptionInSnapshot){
			var resourceInSnapshotRevision = resourceDescriptionInSnapshot.etag;
			ResourceLinksHandler.showInternalResource(
				(resourceURI + "?revision=" + resourceInSnapshotRevision),
				true, true, projectSnapshotURI
			);
			return resourceDescriptionInSnapshot;
		});
	}
};
