/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
dojo.provide("com.ibm.rdm.web.wrapper.WrappedResourceViewer");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.history.RevisionsCache");
dojo.require("com.ibm.rdm.web.viewer.viewerutils");

dojo.requireLocalization("com.ibm.rdm.web.wrapper", "WrappedResourceViewer");

var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var RevisionsCache = com.ibm.rdm.web.history.RevisionsCache;
var viewerutils = com.ibm.rdm.web.viewer.viewerutils;
/**
 * Main Term widget. 
*/
dojo.declare(
	"com.ibm.rdm.web.wrapper.WrappedResourceViewer",
	[dijit._Widget, dijit._Templated],
{
    // summary:
    //    Provides a viewer that's able to display wrapped resources.
	//    If the wrapped resource represents an image, the image is displayed
	//    in the main content area.  Otherwise, if it's not an image, appropriate
	//    links are displayed to enable the user to open in an external
	//    application or directly in Rational Requirements Composer.

	templateString: "<div class=\"rdm-wrapped-resource-viewer\"><div class=\"contentPane\" dojoType=\"dijit.layout.ContentPane\" dojoAttachPoint=\"_contentPane\"></div></div>",
	widgetsInTemplate: true,

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields
	REFERENCE_NAMESPACE: "http://jazz.net/xmlns/alm/rm/Reference/v0.1",
	REFERENCE_QUALIFIER: "reference",

	///////////////////////////////////////////////////////////////////////////////////
	// Private fields
	_widthToUse: null,
	_heightToUse: null,
	_messages: null,
	_contentPane: null,
	idNodesMap: [],
	resourceViewer: null,
	
	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited("postCreate", arguments);
		this.wrapperResourceURI = this.resourceURI.replace(com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES, com.ibm.rdm.web.FRONTING_SERVICE_WRAPPER_RESOURCES);
	},

	startup: function() {
		this.inherited("startup", arguments);
		this._contentPane.startup();
	},
	
	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization(
			"com.ibm.rdm.web.wrapper", "WrappedResourceViewer");
	},
	
	destroy: function() {
		this.inherited("destroy", arguments);
	},
	
	loadResource: function(contentType, revision) {
		this.contentType = contentType;
		this.revision = revision;
		var deferred = new dojo.Deferred();
		
		var wrappedResourceRevisionsDeferred = RevisionsCache.getRevisions(this.resourceURI).addCallback(this, 
			function(wrappedResponse){
				var wrappedResourceRevisions = wrappedResponse.doc._revisionInfos;
				var wrapperResourceRevisionsDeferred = RevisionsCache.getRevisions(this.wrapperResourceURI).addCallback(this, 
					function(wrapperResponse){
						var wrapperResourceRevisions = wrapperResponse.doc._revisionInfos;
						//remove all revisions that are associated with snapshots
						var wrappedRevisionsWithNoSnpashots = [];
						for (var i=0; i<wrappedResourceRevisions.length; i++) {
							if (!wrappedResourceRevisions[i].snapshotRevisionId) {
								wrappedRevisionsWithNoSnpashots.push(wrappedResourceRevisions[i]);
							}
						}
						//find index of requested wrapped revision
						var matchingRevisionIndex = null;
						for (var i=0; i<wrappedRevisionsWithNoSnpashots.length; i++) {
							if (wrappedRevisionsWithNoSnpashots[i].revisionId == revision) {
								matchingRevisionIndex = i;
								break;
							}
						}
						var wrappedResourceRevision = wrappedRevisionsWithNoSnpashots[matchingRevisionIndex];
						var wrapperResourceRevision = wrapperResourceRevisions[matchingRevisionIndex];
						if (!wrapperResourceRevision) {
							//if there is not matching revision, show head
							wrapperResourceRevision = wrapperResourceRevisions[wrapperResourceRevisions.length-1]
						}
						var wrapperDeferred = RDMClient.doGet({
							uri: this.wrapperResourceURI,
							revision: wrapperResourceRevision.revisionId,
		        			projectSnapshotURI: this.resourceViewer.projectSnapshotURI
						});
						wrapperDeferred.addCallback(this, this._handleLoadResponse);
						wrapperDeferred.addCallback(this, function(wrapperDeferredResponse){
							deferred.callback(wrapperDeferredResponse)
						});
						wrapperDeferred.addErrback(this, function(wrapperDeferredErrorResponse){
							deferred.errback(wrapperDeferredErrorResponse)
						});
					}
				);
				wrapperResourceRevisionsDeferred.addErrback(this, function(response){
					deferred.errback(response);
				});
			}
		);
		wrappedResourceRevisionsDeferred.addErrback(this, function(response){
			deferred.errback(response);
		});
		
		return deferred;
	},
	
	setDimensions: function(width, height) {
		// Note: width and height are optional.
		if (width && height) {
			this._contentPane.resize({
				h: height,
				w: width
			});
		}
	},

	setHtmlContent: function(htmlContent) {
		// summary:
		//    Reset the displayed HTML content to be the passed htmlContent.
		//    This method is normally called during initial opening or
		//    subsequent refreshing of the artifact under view.
		this._contentPane.setContent(htmlContent);
	},

	// TODO This is needed as ResourceViewer calls this function ... but we should
	// probably refactor ResourceViewer to NOT always make this call.
	highlightComments: function() {
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_handleLoadResponse: function(response) {
		var doc = response.doc;
		var wrappedResourceURI = this.resourceURI;
		switch (this.contentType) {
			case com.ibm.rdm.web.common.MimeTypes.JPG.contentType:
			case com.ibm.rdm.web.common.MimeTypes.JPE.contentType:
			case com.ibm.rdm.web.common.MimeTypes.BMP.contentType:
			case com.ibm.rdm.web.common.MimeTypes.JPEG.contentType:
			case com.ibm.rdm.web.common.MimeTypes.GIF.contentType:
			case com.ibm.rdm.web.common.MimeTypes.PNG.contentType:
			case com.ibm.rdm.web.common.MimeTypes.ICO.contentType:
				this._showImage(wrappedResourceURI + UriHelper.getResourceRevisionQueryAttribute(this.revision, true) + "&accept=none");
				break;
			default:
				this._showUnableToDisplayContentType(wrappedResourceURI);
		}
		this._documentHasBeenLoaded = true;
		return response;
	},

	_showImage: function(wrappedResourceURI) {
		this._contentPane.attr('content', 
			"<img src=\"" + 
			wrappedResourceURI + 
			"\"/>"
		);
	},

	_showUnableToDisplayContentType: function(wrappedResourceURI) {
		var nativeURI = viewerutils.getNativeClientUrlForOpenArtifact(wrappedResourceURI);
		if (this.resourceViewer.viewType == com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactHistory) {
			nativeURI = viewerutils.getNativeClientUrlForOpenHistory(wrappedResourceURI, this.revision, this.resourceViewer.projectSnapshotURI);
		}
		var buf = (
			"<br><div class=\"content-text\">" + this._messages.unableToDisplayContentType + "<br><br>"			
			+ dojo.string.substitute(this._messages.openInExternalApp, ['<a href="' + 
														wrappedResourceURI +
														UriHelper.getResourceRevisionQueryAttribute(this.revision, true) + 
														(this.resourceViewer.projectSnapshotURI?"&rrc:baseline=" + queryutils.getLastPathSegment(this.resourceViewer.projectSnapshotURI):"") +
														'&accept=none" target="_blank">', "</a>"])
			+ "<br>"
			+ dojo.string.substitute(this._messages.openInComp, ["<a href=" + nativeURI + ">", "</a>"])
			+ "<br><div>"			
//			+ this._messages.click + " <a href=" + externalAppURI + ">" + this._messages.here + "</a> "
//			+ this._messages.openInExternalApplication + "<br>"
//			+ this._messages.click + " <a href=" + nativeURI + ">" + this._messages.here + "</a> "
//			+ this._messages.openInComposer + "<br><div>"
		);
		this._contentPane.attr('content', buf);
	}
});

})();
